@@ -23,4 +23,4 @@ __copyright__ = '(C) 2012, Salvatore Larosa'
 # This will get replaced with a git SHA1 when you do a git archive
 __revision__ = '$Format:%H$'
 
-from console import show_console
+from .console import show_console  # NOQA
