@@ -18,13 +18,10 @@ email                : lrssvtml (at) gmail (dot) com
  ***************************************************************************/
 Some portions of code were taken from https://code.google.com/p/pydee/
 """
-
-from PyQt4.QtCore import Qt, QObject, QEvent, QSettings, QCoreApplication, QFileInfo, QSize, QDir, SIGNAL
-from PyQt4.QtGui import QFont, QFontMetrics, QColor, QShortcut, QKeySequence, QMenu, QApplication, QCursor, QWidget, QGridLayout, QSpacerItem, QSizePolicy, QFileDialog, QTabWidget, QTreeWidgetItem, QFrame, QLabel, QToolButton, QMessageBox
-from PyQt4.Qsci import (QsciScintilla,
-                        QsciLexerPython,
-                        QsciAPIs,
-                        QsciStyle)
+from qgis.PyQt.QtCore import Qt, QObject, QEvent, QSettings, QCoreApplication, QFileInfo, QSize
+from qgis.PyQt.QtGui import QFont, QFontMetrics, QColor, QKeySequence, QCursor
+from qgis.PyQt.QtWidgets import QShortcut, QMenu, QApplication, QWidget, QGridLayout, QSpacerItem, QSizePolicy, QFileDialog, QTabWidget, QTreeWidgetItem, QFrame, QLabel, QToolButton, QMessageBox
+from qgis.PyQt.Qsci import QsciScintilla, QsciLexerPython, QsciAPIs, QsciStyle
 from qgis.core import QgsApplication
 from qgis.gui import QgsMessageBar
 import sys
@@ -50,7 +47,7 @@ class KeyFilter(QObject):
         self.window = window
         self.tab = tab
         self._handlers = {}
-        for shortcut, handler in KeyFilter.SHORTCUTS.iteritems():
+        for shortcut, handler in KeyFilter.SHORTCUTS.items():
             modifiers = shortcut[0]
             if not isinstance(modifiers, list):
                 modifiers = [modifiers]
@@ -268,7 +265,7 @@ class Editor(QsciScintilla):
         else:
             apiPath = self.settings.value("pythonConsole/userAPI", [])
             for i in range(0, len(apiPath)):
-                self.api.load(unicode(apiPath[i]))
+                self.api.load(apiPath[i])
             self.api.prepare()
             self.lexer.setAPIs(self.api)
 
@@ -297,13 +294,13 @@ class Editor(QsciScintilla):
         iconFind = QgsApplication.getThemeIcon("console/iconSearchEditorConsole.png")
         iconSyntaxCk = QgsApplication.getThemeIcon("console/iconSyntaxErrorConsole.png")
         iconObjInsp = QgsApplication.getThemeIcon("console/iconClassBrowserConsole.png")
-        iconCut = QgsApplication.getThemeIcon("mActionEditCut.png")
-        iconCopy = QgsApplication.getThemeIcon("mActionEditCopy.png")
-        iconPaste = QgsApplication.getThemeIcon("mActionEditPaste.png")
+        iconCut = QgsApplication.getThemeIcon("mActionEditCut.svg")
+        iconCopy = QgsApplication.getThemeIcon("mActionEditCopy.svg")
+        iconPaste = QgsApplication.getThemeIcon("mActionEditPaste.svg")
         menu.addAction(
             QCoreApplication.translate("PythonConsole", "Hide Editor"),
             self.hideEditor)
-        menu.addSeparator() # ------------------------------
+        menu.addSeparator()  # ------------------------------
         syntaxCheck = menu.addAction(iconSyntaxCk,
                                      QCoreApplication.translate("PythonConsole", "Check Syntax"),
                                      self.syntaxCheck, 'Ctrl+4')
@@ -323,7 +320,7 @@ class Editor(QsciScintilla):
         menu.addSeparator()
         menu.addAction(iconFind,
                        QCoreApplication.translate("PythonConsole", "Find Text"),
-                       self.showFindWidget)
+                       self.openFindWidget)
         cutAction = menu.addAction(iconCut,
                                    QCoreApplication.translate("PythonConsole", "Cut"),
                                    self.cut, QKeySequence.Cut)
@@ -383,9 +380,13 @@ class Editor(QsciScintilla):
             showCodeInspection.setEnabled(True)
         menu.exec_(self.mapToGlobal(e.pos()))
 
-    def findText(self, forward):
+    def findText(self, forward, showMessage=True, findFirst=False):
         lineFrom, indexFrom, lineTo, indexTo = self.getSelection()
-        line, index = self.getCursorPosition()
+        if findFirst:
+            line = 0
+            index = 0
+        else:
+            line, index = self.getCursorPosition()
         text = self.parent.pc.lineEditFind.text()
         re = False
         wrap = self.parent.pc.wrapAround.isChecked()
@@ -403,13 +404,20 @@ class Editor(QsciScintilla):
             if notFound:
                 styleError = 'QLineEdit {background-color: #d65253; \
                                         color: #ffffff;}'
-                msgText = QCoreApplication.translate('PythonConsole',
-                                                     '<b>"{0}"</b> was not found.').format(text)
-                self.parent.pc.callWidgetMessageBarEditor(msgText, 0, True)
+                if showMessage:
+                    msgText = QCoreApplication.translate('PythonConsole',
+                                                         '<b>"{0}"</b> was not found.').format(text)
+                    self.parent.pc.callWidgetMessageBarEditor(msgText, 0, True)
             else:
                 styleError = ''
             self.parent.pc.lineEditFind.setStyleSheet(styleError)
 
+    def findNext(self):
+        self.findText(True)
+
+    def findPrevious(self):
+        self.findText(False)
+
     def objectListEditor(self):
         listObj = self.parent.pc.listClassMethod
         if listObj.isVisible():
@@ -420,19 +428,20 @@ class Editor(QsciScintilla):
             self.parent.pc.objectListButton.setChecked(True)
 
     def codepad(self):
-        import urllib2
-        import urllib
+        import urllib.request
+        import urllib.parse
+        import urllib.error
         listText = self.selectedText().split('\n')
         getCmd = []
         for strLine in listText:
-            getCmd.append(unicode(strLine))
+            getCmd.append(strLine)
         pasteText = u"\n".join(getCmd)
         url = 'http://codepad.org'
         values = {'lang': 'Python',
                   'code': pasteText,
                   'submit': 'Submit'}
         try:
-            response = urllib2.urlopen(url, urllib.urlencode(values))
+            response = urllib.request.urlopen(url, urllib.parse.urlencode(values))
             url = response.read()
             for href in url.split("</a>"):
                 if "Link:" in href:
@@ -445,22 +454,33 @@ class Editor(QsciScintilla):
                 QApplication.clipboard().setText(link)
                 msgText = QCoreApplication.translate('PythonConsole', 'URL copied to clipboard.')
                 self.parent.pc.callWidgetMessageBarEditor(msgText, 0, True)
-        except urllib2.URLError as e:
+        except urllib.error.URLError as e:
             msgText = QCoreApplication.translate('PythonConsole', 'Connection error: ')
-            self.parent.pc.callWidgetMessageBarEditor(msgText + str(e.args), 0, True)
+            self.parent.pc.callWidgetMessageBarEditor(msgText + repr(e.args), 0, True)
 
     def hideEditor(self):
         self.parent.pc.splitterObj.hide()
         self.parent.pc.showEditorButton.setChecked(False)
 
-    def showFindWidget(self):
+    def openFindWidget(self):
+        wF = self.parent.pc.widgetFind
+        wF.show()
+        if self.hasSelectedText():
+            self.parent.pc.lineEditFind.setText(self.selectedText().strip())
+        self.parent.pc.lineEditFind.setFocus()
+        self.parent.pc.findTextButton.setChecked(True)
+
+    def closeFindWidget(self):
+        wF = self.parent.pc.widgetFind
+        wF.hide()
+        self.parent.pc.findTextButton.setChecked(False)
+
+    def toggleFindWidget(self):
         wF = self.parent.pc.widgetFind
         if wF.isVisible():
-            wF.hide()
-            self.parent.pc.findTextButton.setChecked(False)
+            self.closeFindWidget()
         else:
-            wF.show()
-            self.parent.pc.findTextButton.setChecked(True)
+            self.openFindWidget()
 
     def commentEditorCode(self, commentCheck):
         self.beginUndoAction()
@@ -508,10 +528,10 @@ class Editor(QsciScintilla):
         try:
             ## set creationflags for running command without shell window
             if sys.platform.startswith('win'):
-                p = subprocess.Popen(['python', unicode(filename)], shell=False, stdin=subprocess.PIPE,
+                p = subprocess.Popen(['python', filename], shell=False, stdin=subprocess.PIPE,
                                      stderr=subprocess.PIPE, stdout=subprocess.PIPE, creationflags=0x08000000)
             else:
-                p = subprocess.Popen(['python', unicode(filename)], shell=False, stdin=subprocess.PIPE,
+                p = subprocess.Popen(['python', filename], shell=False, stdin=subprocess.PIPE,
                                      stderr=subprocess.PIPE, stdout=subprocess.PIPE)
             out, _traceback = p.communicate()
 
@@ -531,15 +551,15 @@ class Editor(QsciScintilla):
                 file = file + tmpFileTr
             if _traceback:
                 msgTraceTr = QCoreApplication.translate('PythonConsole', '## Script error: {0}').format(file)
-                print "## %s" % datetime.datetime.now()
-                print unicode(msgTraceTr)
+                print("## {}".format(datetime.datetime.now()))
+                print(msgTraceTr)
                 sys.stderr.write(_traceback)
                 p.stderr.close()
             else:
                 msgSuccessTr = QCoreApplication.translate('PythonConsole',
                                                           '## Script executed successfully: {0}').format(file)
-                print "## %s" % datetime.datetime.now()
-                print unicode(msgSuccessTr)
+                print("## {}".format(datetime.datetime.now()))
+                print(msgSuccessTr)
                 sys.stdout.write(out)
                 p.stdout.close()
             del p
@@ -549,10 +569,10 @@ class Editor(QsciScintilla):
             IOErrorTr = QCoreApplication.translate('PythonConsole',
                                                    'Cannot execute file {0}. Error: {1}\n').format(filename,
                                                                                                    error.strerror)
-            print '## Error: ' + IOErrorTr
+            print('## Error: ' + IOErrorTr)
         except:
             s = traceback.format_exc()
-            print '## Error: '
+            print('## Error: ')
             sys.stderr.write(s)
 
     def runScriptCode(self):
@@ -561,8 +581,6 @@ class Editor(QsciScintilla):
         filename = tabWidget.path
         msgEditorBlank = QCoreApplication.translate('PythonConsole',
                                                     'Hey, type something to run!')
-        msgEditorUnsaved = QCoreApplication.translate('PythonConsole',
-                                                      'You have to save the file before running it.')
         if filename is None:
             if not self.isModified():
                 self.parent.pc.callWidgetMessageBarEditor(msgEditorBlank, 0, True)
@@ -606,7 +624,7 @@ class Editor(QsciScintilla):
         eline = None
         ecolumn = 0
         edescr = ''
-        source = unicode(self.text())
+        source = self.text()
         try:
             if not filename:
                 filename = self.parent.tw.currentWidget().path
@@ -615,7 +633,8 @@ class Editor(QsciScintilla):
                 source = source.encode('utf-8')
             if isinstance(filename, type(u"")):
                 filename = filename.encode('utf-8')
-            compile(source, str(filename), 'exec')
+            if filename:
+                compile(source, filename, 'exec')
         except SyntaxError as detail:
             eline = detail.lineno and detail.lineno or 1
             ecolumn = detail.offset and detail.offset or 1
@@ -648,7 +667,7 @@ class Editor(QsciScintilla):
             return True
 
     def keyPressEvent(self, e):
-        t = unicode(e.text())
+        t = e.text()
         startLine, _, endLine, endPos = self.getSelection()
         line, pos = self.getCursorPosition()
         self.autoCloseBracket = self.settings.value("pythonConsole/autoCloseBracketEditor", False, type=bool)
@@ -765,7 +784,7 @@ class EditorTab(QWidget):
 
     def loadFile(self, filename, modified):
         self.newEditor.lastModified = QFileInfo(filename).lastModified()
-        fn = codecs.open(unicode(filename), "rb", encoding='utf-8')
+        fn = codecs.open(filename, "rb", encoding='utf-8')
         txt = fn.read()
         fn.close()
         QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
@@ -783,10 +802,10 @@ class EditorTab(QWidget):
         if self.path is None:
             saveTr = QCoreApplication.translate('PythonConsole',
                                                 'Python Console: Save file')
-            self.path = str(QFileDialog().getSaveFileName(self,
-                                                          saveTr,
-                                                          self.tw.tabText(index) + '.py',
-                                                          "Script file (*.py)"))
+            self.path = QFileDialog().getSaveFileName(self,
+                                                      saveTr,
+                                                      self.tw.tabText(index) + '.py',
+                                                      "Script file (*.py)")
             # If the user didn't select a file, abort the save operation
             if len(self.path) == 0:
                 self.path = None
@@ -796,7 +815,7 @@ class EditorTab(QWidget):
                                                  'Script was correctly saved.')
             self.pc.callWidgetMessageBarEditor(msgText, 0, True)
         # Rename the original file, if it exists
-        path = unicode(self.path)
+        path = self.path
         overwrite = QFileInfo(path).exists()
         if overwrite:
             try:
@@ -880,7 +899,7 @@ class EditorTabWidget(QTabWidget):
         toolTipClose = QCoreApplication.translate("PythonConsole",
                                                   "Close")
         self.clButton.setToolTip(toolTipClose)
-        self.clButton.setIcon(QgsApplication.getThemeIcon("mIconClose.png"))
+        self.clButton.setIcon(QgsApplication.getThemeIcon("/mIconClose.svg"))
         self.clButton.setIconSize(QSize(18, 18))
         self.clButton.setCursor(Qt.PointingHandCursor)
         self.clButton.setStyleSheet('QToolButton:hover{border: none } \
@@ -895,8 +914,8 @@ class EditorTabWidget(QTabWidget):
         self.layoutTopFrame2.addWidget(self.clButton, 0, 2, 1, 1)
 
         self.topFrame.hide()
-        self.connect(self.restoreTabsButton, SIGNAL('clicked()'), self.restoreTabs)
-        self.connect(self.clButton, SIGNAL('clicked()'), self.closeRestore)
+        self.restoreTabsButton.clicked.connect(self.restoreTabs)
+        self.clButton.clicked.connect(self.closeRestore)
 
         ## Fixes #7653
         if sys.platform != 'darwin':
@@ -908,10 +927,8 @@ class EditorTabWidget(QTabWidget):
 
         # Menu button list tabs
         self.fileTabMenu = QMenu()
-        self.connect(self.fileTabMenu, SIGNAL("aboutToShow()"),
-                     self.showFileTabMenu)
-        self.connect(self.fileTabMenu, SIGNAL("triggered(QAction*)"),
-                     self.showFileTabMenuTriggered)
+        self.fileTabMenu.aboutToShow.connect(self.showFileTabMenu)
+        self.fileTabMenu.triggered.connect(self.showFileTabMenuTriggered)
         self.fileTabButton = QToolButton()
         txtToolTipMenuFile = QCoreApplication.translate("PythonConsole",
                                                         "List all tabs")
@@ -922,8 +939,8 @@ class EditorTabWidget(QTabWidget):
         self.fileTabButton.setPopupMode(QToolButton.InstantPopup)
         self.fileTabButton.setMenu(self.fileTabMenu)
         self.setCornerWidget(self.fileTabButton, Qt.TopRightCorner)
-        self.connect(self, SIGNAL("tabCloseRequested(int)"), self._removeTab)
-        self.connect(self, SIGNAL('currentChanged(int)'), self._currentWidgetChanged)
+        self.tabCloseRequested.connect(self._removeTab)
+        self.currentChanged.connect(self._currentWidgetChanged)
 
         # New Editor button
         self.newTabButton = QToolButton()
@@ -934,7 +951,7 @@ class EditorTabWidget(QTabWidget):
         self.newTabButton.setIcon(QgsApplication.getThemeIcon("console/iconNewTabEditorConsole.png"))
         self.newTabButton.setIconSize(QSize(24, 24))
         self.setCornerWidget(self.newTabButton, Qt.TopLeftCorner)
-        self.connect(self.newTabButton, SIGNAL('clicked()'), self.newTabEditor)
+        self.newTabButton.clicked.connect(self.newTabEditor)
 
     def _currentWidgetChanged(self, tab):
         if self.settings.value("pythonConsole/enableObjectInsp",
@@ -985,7 +1002,7 @@ class EditorTabWidget(QTabWidget):
     def closeOthers(self):
         idx = self.idx
         countTab = self.count()
-        for i in range(countTab - 1, idx, -1) + range(idx - 1, -1, -1):
+        for i in list(range(countTab - 1, idx, -1)) + list(range(idx - 1, -1, -1)):
             self._removeTab(i)
 
     def closeAll(self):
@@ -1015,14 +1032,14 @@ class EditorTabWidget(QTabWidget):
         if filename:
             readOnly = not QFileInfo(filename).isWritable()
             try:
-                fn = codecs.open(unicode(filename), "rb", encoding='utf-8')
+                fn = codecs.open(filename, "rb", encoding='utf-8')
                 fn.read()
                 fn.close()
             except IOError as error:
                 IOErrorTr = QCoreApplication.translate('PythonConsole',
                                                        'The file {0} could not be opened. Error: {1}\n').format(filename,
                                                                                                                 error.strerror)
-                print '## Error: '
+                print('## Error: ')
                 sys.stderr.write(IOErrorTr)
                 return
 
@@ -1034,7 +1051,7 @@ class EditorTabWidget(QTabWidget):
         self.addTab(self.tab, self.iconTab, tabName + ' (ro)' if readOnly else tabName)
         self.setCurrentWidget(self.tab)
         if filename:
-            self.setTabToolTip(self.currentIndex(), unicode(filename))
+            self.setTabToolTip(self.currentIndex(), filename)
         else:
             self.setTabToolTip(self.currentIndex(), tabName)
 
@@ -1114,14 +1131,14 @@ class EditorTabWidget(QTabWidget):
 
     def restoreTabs(self):
         for script in self.restoreTabList:
-            pathFile = unicode(script)
+            pathFile = script
             if QFileInfo(pathFile).exists():
                 tabName = pathFile.split('/')[-1]
                 self.newTabEditor(tabName, pathFile)
             else:
                 errOnRestore = QCoreApplication.translate("PythonConsole",
                                                           "Unable to restore the file: \n{0}\n").format(pathFile)
-                print '## Error: '
+                print('## Error: ')
                 s = errOnRestore
                 sys.stderr.write(s)
                 self.parent.updateTabListScript(pathFile, action='remove')
@@ -1156,7 +1173,7 @@ class EditorTabWidget(QTabWidget):
             tabWidget = self.widget(tab)
         if tabWidget:
             if tabWidget.path:
-                pathFile, file = os.path.split(unicode(tabWidget.path))
+                pathFile, file = os.path.split(tabWidget.path)
                 module, ext = os.path.splitext(file)
                 found = False
                 if pathFile not in sys.path:
@@ -1167,8 +1184,8 @@ class EditorTabWidget(QTabWidget):
                     dictObject = {}
                     readModule = pyclbr.readmodule(module)
                     readModuleFunction = pyclbr.readmodule_ex(module)
-                    for name, class_data in sorted(readModule.items(), key=lambda x: x[1].lineno):
-                        if os.path.normpath(str(class_data.file)) == os.path.normpath(str(tabWidget.path)):
+                    for name, class_data in sorted(list(readModule.items()), key=lambda x: x[1].lineno):
+                        if os.path.normpath(class_data.file) == os.path.normpath(tabWidget.path):
                             superClassName = []
                             for superClass in class_data.super:
                                 if superClass == 'object':
@@ -1191,7 +1208,7 @@ class EditorTabWidget(QTabWidget):
                             iconClass = QgsApplication.getThemeIcon("console/iconClassTreeWidgetConsole.png")
                             classItem.setIcon(0, iconClass)
                             dictObject[name] = class_data.lineno
-                            for meth, lineno in sorted(class_data.methods.items(), key=itemgetter(1)):
+                            for meth, lineno in sorted(list(class_data.methods.items()), key=itemgetter(1)):
                                 methodItem = QTreeWidgetItem()
                                 methodItem.setText(0, meth + ' ')
                                 methodItem.setText(1, str(lineno))
@@ -1203,9 +1220,9 @@ class EditorTabWidget(QTabWidget):
                                 classItem.addChild(methodItem)
                                 dictObject[meth] = lineno
                             self.parent.listClassMethod.addTopLevelItem(classItem)
-                    for func_name, data in sorted(readModuleFunction.items(), key=lambda x: x[1].lineno):
+                    for func_name, data in sorted(list(readModuleFunction.items()), key=lambda x: x[1].lineno):
                         if isinstance(data, pyclbr.Function) and \
-                           os.path.normpath(str(data.file)) == os.path.normpath(str(tabWidget.path)):
+                           os.path.normpath(data.file) == os.path.normpath(tabWidget.path):
                             funcItem = QTreeWidgetItem()
                             funcItem.setText(0, func_name + ' ')
                             funcItem.setText(1, str(data.lineno))
