@@ -9,11 +9,23 @@
 
 """ CSW request and response processor """
 
+from __future__ import (absolute_import, division, print_function)
+
+import inspect
 import warnings
-import StringIO
+import six
+try:
+    from StringIO import StringIO as BytesIO  # Python 2
+except ImportError:
+    from io import BytesIO  # Python 3
 import random
-from urllib import urlencode
-from urllib2 import urlopen
+try:                    # Python 3
+    from urllib.parse import urlencode
+except ImportError:     # Python 2
+    from urllib import urlencode
+
+from owslib.util import OrderedDict
+
 from owslib.etree import etree
 from owslib import fes
 from owslib import util
@@ -21,8 +33,9 @@ from owslib import ows
 from owslib.iso import MD_Metadata
 from owslib.fgdc import Metadata
 from owslib.dif import DIF
+from owslib.gm03 import GM03
 from owslib.namespaces import Namespaces
-from owslib.util import cleanup_namespaces, bind_url
+from owslib.util import cleanup_namespaces, bind_url, add_namespaces, openURL
 
 # default variables
 outputformat = 'application/xml'
@@ -34,9 +47,10 @@ namespaces = get_namespaces()
 schema = 'http://schemas.opengis.net/csw/2.0.2/CSW-discovery.xsd'
 schema_location = '%s %s' % (namespaces['csw'], schema)
 
-class CatalogueServiceWeb:
+class CatalogueServiceWeb(object):
     """ csw request class """
-    def __init__(self, url, lang='en-US', version='2.0.2', timeout=10, skip_caps=False):
+    def __init__(self, url, lang='en-US', version='2.0.2', timeout=10, skip_caps=False,
+                 username=None, password=None):
         """
 
         Construct and process a GetCapabilities request
@@ -49,6 +63,8 @@ class CatalogueServiceWeb:
         - version: version (default is '2.0.2')
         - timeout: timeout in seconds
         - skip_caps: whether to skip GetCapabilities processing on init (default is False)
+        - username: username for HTTP basic authentication
+        - password: password for HTTP basic authentication
 
         """
 
@@ -56,6 +72,8 @@ class CatalogueServiceWeb:
         self.lang = lang
         self.version = version
         self.timeout = timeout
+        self.username = username
+        self.password = password
         self.service = 'CSW'
         self.exceptionreport = None
         self.owscommon = ows.OwsCommon('1.0.0')
@@ -65,25 +83,31 @@ class CatalogueServiceWeb:
 
             data = {'service': self.service, 'version': self.version, 'request': 'GetCapabilities'}
 
-            self.request = '%s%s' % (bind_url(self.url), urlencode(data))
+            self.request = urlencode(data)
     
             self._invoke()
     
             if self.exceptionreport is None:
                 # ServiceIdentification
                 val = self._exml.find(util.nspath_eval('ows:ServiceIdentification', namespaces))
-                self.identification=ows.ServiceIdentification(val,self.owscommon.namespace)
+                if val is not None:
+                  self.identification = ows.ServiceIdentification(val,self.owscommon.namespace)
+                else:
+                  self.identification = None
                 # ServiceProvider
                 val = self._exml.find(util.nspath_eval('ows:ServiceProvider', namespaces))
-                self.provider=ows.ServiceProvider(val,self.owscommon.namespace)
+                if val is not None:
+                    self.provider = ows.ServiceProvider(val,self.owscommon.namespace)
+                else:
+                  self.provider = None
                 # ServiceOperations metadata 
-                self.operations=[]
+                self.operations = []
                 for elem in self._exml.findall(util.nspath_eval('ows:OperationsMetadata/ows:Operation', namespaces)):
                     self.operations.append(ows.OperationsMetadata(elem, self.owscommon.namespace))
         
                 # FilterCapabilities
                 val = self._exml.find(util.nspath_eval('ogc:Filter_Capabilities', namespaces))
-                self.filters=fes.FilterCapabilities(val)
+                self.filters = fes.FilterCapabilities(val)
  
     def describerecord(self, typename='csw:Record', format=outputformat):
         """
@@ -234,7 +258,7 @@ class CatalogueServiceWeb:
             self.results['nextrecord'] = int(util.testXMLValue(val, True))
     
             # process list of matching records
-            self.records = {}
+            self.records = OrderedDict()
 
             self._parserecords(outputschema, esn)
 
@@ -253,7 +277,7 @@ class CatalogueServiceWeb:
 
         """
 
-        # construct request 
+        # construct request
         data = {
             'service': self.service,
             'version': self.version,
@@ -261,16 +285,16 @@ class CatalogueServiceWeb:
             'outputFormat': format,
             'outputSchema': outputschema,
             'elementsetname': esn,
-            'id': '',
+            'id': ','.join(id),
         }
 
-        self.request = '%s%s%s' % (bind_url(self.url), urlencode(data), ','.join(id))
+        self.request = urlencode(data)
 
         self._invoke()
- 
+
         if self.exceptionreport is None:
             self.results = {}
-            self.records = {}
+            self.records = OrderedDict()
             self._parserecords(outputschema, esn)
 
     def getrecords2(self, constraints=[], sortby=None, typenames='csw:Record', esn='summary', outputschema=namespaces['csw'], format=outputformat, startposition=0, maxrecords=10, cql=None, xml=None, resulttype='results'):
@@ -300,6 +324,9 @@ class CatalogueServiceWeb:
             val = self.request.find(util.nspath_eval('csw:Query/csw:ElementSetName', namespaces))
             if val is not None:
                 esn = util.testXMLValue(val)
+            val = self.request.attrib.get('outputSchema')
+            if val is not None:
+                outputschema = util.testXMLValue(val, True)
         else:
             # construct request
             node0 = self._setrootelement('csw:GetRecords')
@@ -359,7 +386,7 @@ class CatalogueServiceWeb:
                 self.results['nextrecord'] = None
 
             # process list of matching records
-            self.records = {}
+            self.records = OrderedDict()
 
             self._parserecords(outputschema, esn)
 
@@ -392,7 +419,7 @@ class CatalogueServiceWeb:
         validtransactions = ['insert', 'update', 'delete']
 
         if ttype not in validtransactions:  # invalid transaction
-            raise RuntimeError, 'Invalid transaction \'%s\'.' % ttype
+            raise RuntimeError('Invalid transaction \'%s\'.' % ttype)
 
         node1 = etree.SubElement(node0, util.nspath_eval('csw:%s' % ttype.capitalize(), namespaces))
 
@@ -401,7 +428,7 @@ class CatalogueServiceWeb:
 
         if ttype == 'insert':
             if record is None:
-                raise RuntimeError, 'Nothing to insert.'
+                raise RuntimeError('Nothing to insert.')
             node1.append(etree.fromstring(record))
  
         if ttype == 'update':
@@ -472,6 +499,13 @@ class CatalogueServiceWeb:
                 self._parsetransactionsummary()
                 self._parseinsertresult()
 
+    def get_operation_by_name(self, name):
+        """Return a named operation"""
+        for item in self.operations:
+            if item.name.lower() == name.lower():
+                return item
+        raise KeyError("No operation named %s" % name)
+
     def getService_urls(self, service_string=None):
         """
 
@@ -485,7 +519,7 @@ class CatalogueServiceWeb:
         """
         
         urls=[]
-        for key,rec in self.records.iteritems():
+        for key,rec in six.iteritems(self.records):
             #create a generator object, and iterate through it until the match is found
             #if not found, gets the default value (here "none")
             url = next((d['url'] for d in rec.references if d['scheme'] == service_string), None)
@@ -495,13 +529,13 @@ class CatalogueServiceWeb:
 
     def _parseinsertresult(self):
         self.results['insertresults'] = []
-        for i in self._exml.findall(util.nspath_eval('csw:InsertResult', namespaces)):
+        for i in self._exml.findall('.//'+util.nspath_eval('csw:InsertResult', namespaces)):
             for j in i.findall(util.nspath_eval('csw:BriefRecord/dc:identifier', namespaces)):
                 self.results['insertresults'].append(util.testXMLValue(j))
 
     def _parserecords(self, outputschema, esn):
         if outputschema == namespaces['gmd']: # iso 19139
-            for i in self._exml.findall('.//'+util.nspath_eval('gmd:MD_Metadata', namespaces)):
+            for i in self._exml.findall('.//'+util.nspath_eval('gmd:MD_Metadata', namespaces)) or self._exml.findall('.//'+util.nspath_eval('gmi:MI_Metadata', namespaces)):
                 val = i.find(util.nspath_eval('gmd:fileIdentifier/gco:CharacterString', namespaces))
                 identifier = self._setidentifierkey(util.testXMLValue(val))
                 self.records[identifier] = MD_Metadata(i)
@@ -515,6 +549,11 @@ class CatalogueServiceWeb:
                 val = i.find(util.nspath_eval('dif:Entry_ID', namespaces))
                 identifier = self._setidentifierkey(util.testXMLValue(val))
                 self.records[identifier] = DIF(i)
+        elif outputschema == namespaces['gm03']: # GM03
+            for i in self._exml.findall('.//'+util.nspath_eval('gm03:TRANSFER', namespaces)):
+                val = i.find(util.nspath_eval('gm03:fileIdentifier', namespaces))
+                identifier = self._setidentifierkey(util.testXMLValue(val))
+                self.records[identifier] = GM03(i)
         else: # process default
             for i in self._exml.findall('.//'+util.nspath_eval('csw:%s' % self._setesnel(esn), namespaces)):
                 val = i.find(util.nspath_eval('dc:identifier', namespaces))
@@ -522,7 +561,7 @@ class CatalogueServiceWeb:
                 self.records[identifier] = CswRecord(i)
 
     def _parsetransactionsummary(self):
-        val = self._exml.find(util.nspath_eval('csw:TransactionSummary', namespaces))
+        val = self._exml.find(util.nspath_eval('csw:TransactionResponse/csw:TransactionSummary', namespaces))
         if val is not None:
             rid = val.attrib.get('requestId')
             self.results['requestid'] = util.testXMLValue(rid, True)
@@ -573,16 +612,60 @@ class CatalogueServiceWeb:
     def _invoke(self):
         # do HTTP request
 
-        if isinstance(self.request, basestring):  # GET KVP
-            self.response = urlopen(self.request, timeout=self.timeout).read()
+        request_url = self.url
+
+        # Get correct URL based on Operation list.
+
+        # If skip_caps=True, then self.operations has not been set, so use
+        # default URL.
+        if hasattr(self, 'operations'):
+            caller = inspect.stack()[1][3]
+            if caller == 'getrecords2': caller = 'getrecords'
+            try:
+                op = self.get_operation_by_name(caller)
+                if isinstance(self.request, six.string_types):  # GET KVP
+                    get_verbs = [x for x in op.methods if x.get('type').lower() == 'get']
+                    request_url = get_verbs[0].get('url')
+                else:
+                    post_verbs = [x for x in op.methods if x.get('type').lower() == 'post']
+                    if len(post_verbs) > 1:
+                        # Filter by constraints.  We must match a PostEncoding of "XML"
+                        for pv in post_verbs:
+                            for const in pv.get('constraints'):
+                                if const.name.lower() == 'postencoding':
+                                    values = [v.lower() for v in const.values]
+                                    if 'xml' in values:
+                                        request_url = pv.get('url')
+                                        break
+                        else:
+                            # Well, just use the first one.
+                            request_url = post_verbs[0].get('url')
+                    elif len(post_verbs) == 1:
+                        request_post_url = post_verbs[0].get('url')
+            except:  # no such luck, just go with request_url
+                pass
+
+        if isinstance(self.request, six.string_types):  # GET KVP
+            self.request = '%s%s' % (bind_url(request_url), self.request)
+            self.response = openURL(self.request, None, 'Get', username=self.username, password=self.password, timeout=self.timeout).read()
         else:
             self.request = cleanup_namespaces(self.request)
-            self.request = util.xml2string(etree.tostring(self.request))
+            # Add any namespaces used in the "typeNames" attribute of the
+            # csw:Query element to the query's xml namespaces.
+            for query in self.request.findall(util.nspath_eval('csw:Query', namespaces)):
+                ns = query.get("typeNames", None)
+                if ns is not None:
+                    # Pull out "gmd" from something like "gmd:MD_Metadata" from the list
+                    # of typenames
+                    ns_keys = [x.split(':')[0] for x in ns.split(' ')]
+                    self.request = add_namespaces(self.request, ns_keys)
 
-            self.response = util.http_post(self.url, self.request, self.lang, self.timeout)
+            self.request = util.element_to_string(self.request, encoding='utf-8')
+
+            self.response = util.http_post(request_url, self.request, self.lang, self.timeout, self.username, self.password)
 
         # parse result see if it's XML
-        self._exml = etree.parse(StringIO.StringIO(self.response))
+        self._exml = etree.parse(BytesIO(self.response))
 
         # it's XML.  Attempt to decipher whether the XML response is CSW-ish """
         valid_xpaths = [
@@ -597,7 +680,7 @@ class CatalogueServiceWeb:
         ]
 
         if self._exml.getroot().tag not in valid_xpaths:
-            raise RuntimeError, 'Document is XML, but not CSW-ish'
+            raise RuntimeError('Document is XML, but not CSW-ish')
 
         # check if it's an OGC Exception
         val = self._exml.find(util.nspath_eval('ows:Exception', namespaces))
@@ -664,7 +747,7 @@ class CswRecord(object):
         val = record.find(util.nspath_eval('dc:relation', namespaces))
         self.relation = util.testXMLValue(val)
 
-        val = record.find(util.nspath_eval('dc:temporal', namespaces))
+        val = record.find(util.nspath_eval('dct:temporal', namespaces))
         self.temporal = util.testXMLValue(val)
 
         self.uris = []  # list of dicts
