@@ -31,10 +31,9 @@ import json
 import os.path
 from urllib2 import build_opener, install_opener, ProxyHandler
 
-from PyQt4.QtCore import QSettings, Qt, SIGNAL, SLOT
-from PyQt4.QtGui import (QApplication, QColor, QCursor, QDialog,
-                         QDialogButtonBox, QMessageBox, QTreeWidgetItem,
-                         QWidget)
+from qgis.PyQt.QtCore import QSettings, Qt
+from qgis.PyQt.QtWidgets import QApplication, QDialog, QDialogButtonBox, QMessageBox, QTreeWidgetItem, QWidget
+from qgis.PyQt.QtGui import QColor, QCursor
 
 from qgis.core import (QgsApplication, QgsCoordinateReferenceSystem,
                        QgsCoordinateTransform, QgsGeometry, QgsPoint,
@@ -438,6 +437,8 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.timeout = self.spnTimeout.value()
 
         # bbox
+        # CRS is WGS84 with axis order longitude, latitude
+        # defined by 'urn:ogc:def:crs:OGC:1.3:CRS84'
         minx = self.leWest.text()
         miny = self.leSouth.text()
         maxx = self.leEast.text()
@@ -448,7 +449,8 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         # even for a global bbox, if a spatial filter is applied, then
         # the CSW server will skip records without a bbox
         if bbox != ['-180', '-90', '180', '90']:
-            self.constraints.append(BBox(bbox))
+            self.constraints.append(BBox(bbox,
+                                         crs='urn:ogc:def:crs:OGC:1.3:CRS84'))
 
         # keywords
         if self.leKeywords.text():
@@ -730,25 +732,15 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         # connect dialog signals to iface slots
         if service_type == 'OGC:WMS/OGC:WMTS':
-            ows_provider.connect(
-                ows_provider,
-                SIGNAL('addRasterLayer(QString, QString, QString)'),
-                self.iface, SLOT('addRasterLayer(QString, QString, QString)'))
+            ows_provider.addRasterLayer.connect(self.iface.addRasterLayer)
             conn_cmb = ows_provider.findChild(QWidget, 'cmbConnections')
             connect = 'on_btnConnect_clicked'
         elif service_type == 'OGC:WFS':
-            ows_provider.connect(
-                ows_provider,
-                SIGNAL('addWfsLayer(QString, QString)'),
-                self.iface.mainWindow(),
-                SLOT('addWfsLayer(QString, QString)'))
+            ows_provider.addWfsLayer.connect(self.iface.mainWindow().addWfsLayer)
             conn_cmb = ows_provider.findChild(QWidget, 'cmbConnections')
             connect = 'connectToServer'
         elif service_type == 'OGC:WCS':
-            ows_provider.connect(
-                ows_provider,
-                SIGNAL('addRasterLayer(QString, QString, QString)'),
-                self.iface, SLOT('addRasterLayer(QString, QString, QString)'))
+            ows_provider.addRasterLayer.connect(self.iface.addRasterLayer)
             conn_cmb = ows_provider.findChild(QWidget, 'mConnectionsComboBox')
             connect = 'on_mConnectButton_clicked'
         ows_provider.setModal(False)
