@@ -23,16 +23,14 @@ The content of this file is based on
  ***************************************************************************/
 """
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-from PyQt4.QtSql import QSqlDatabase
+from qgis.PyQt.QtSql import QSqlDatabase
 
 from ..connector import DBConnector
 from ..plugin import ConnectionError, DbError, Table
 
 import os
-from qgis.core import QGis, QgsApplication
-import QtSqlDB
+from qgis.core import QGis, QgsApplication, NULL
+from . import QtSqlDB
 import sqlite3
 
 
@@ -104,8 +102,7 @@ class OracleDBConnector(DBConnector):
         if (os.path.isfile(sqlite_cache_file)):
             try:
                 self.cache_connection = sqlite3.connect(sqlite_cache_file)
-            except sqlite3.Error as e:
-
+            except sqlite3.Error:
                 self.cache_connection = False
 
         # Find if there is cache for our connection:
@@ -120,7 +117,7 @@ class OracleDBConnector(DBConnector):
                 if not has_cached:
                     self.cache_connection = False
 
-            except sqlite3.Error as e:
+            except sqlite3.Error:
                 self.cache_connection = False
 
         self._checkSpatial()
@@ -192,7 +189,7 @@ class OracleDBConnector(DBConnector):
         return self.has_spatial
 
     def hasRasterSupport(self):
-        """No raster support for the moment !"""
+        """No raster support for the moment!"""
         # return self.has_raster
         return False
 
@@ -242,8 +239,6 @@ class OracleDBConnector(DBConnector):
         user.
         """
         result = [False, False, False, False]
-        if owner != self.user:
-            prefix = u"USER"
         # Inspect in all tab privs
         sql = u"""
         SELECT DISTINCT PRIVILEGE
@@ -342,7 +337,6 @@ class OracleDBConnector(DBConnector):
             # get all non geographic tables and views
             prefix = u"ALL"
             owner = u"o.owner"
-            metatable = u"tab_columns"
             where = u""
             if self.userTablesOnly:
                 prefix = u"USER"
@@ -618,7 +612,7 @@ class OracleDBConnector(DBConnector):
         for i, tbl in enumerate(lst_tables):
             item = list(tbl)
             detectedSrid = item.pop()
-            if isinstance(detectedSrid, QPyNullVariant):
+            if detectedSrid == NULL:
                 detectedSrid = u"-1"
             else:
                 detectedSrid = int(detectedSrid)
@@ -772,7 +766,7 @@ class OracleDBConnector(DBConnector):
 
         try:
             c = self._execute(None, query)
-        except DbError as e:  # handle error views or other problems
+        except DbError:  # handle error views or other problems
             return [QGis.WKBUnknown], [-1]
 
         rows = self._fetchall(c)
@@ -786,11 +780,11 @@ class OracleDBConnector(DBConnector):
         geomtypes = []
         srids = []
         for row in rows:
-            if isinstance(row[1], QPyNullVariant):
+            if row[1] == NULL:
                 srids.append(-1)
             else:
                 srids.append(int(row[1]))
-            if int(row[0]) in OracleDBConnector.ORGeomTypes.keys():
+            if int(row[0]) in list(OracleDBConnector.ORGeomTypes.keys()):
                 geomtypes.append(OracleDBConnector.ORGeomTypes[int(row[0])])
             else:
                 geomtypes.append(QGis.WKBUnknown)
@@ -838,7 +832,7 @@ class OracleDBConnector(DBConnector):
         res = self._fetchone(c)
         c.close()
 
-        if not res or isinstance(res[0], QPyNullVariant):
+        if not res or res[0] == NULL:
             return 0
         else:
             return int(res[0])
@@ -1077,7 +1071,7 @@ class OracleDBConnector(DBConnector):
 
         try:
             c = self._execute(None, sql)
-        except DbError as e:  # no spatial index on table, try aggregation
+        except DbError:  # no spatial index on table, try aggregation
             return None
 
         res = self._fetchone(c)
@@ -1112,7 +1106,7 @@ class OracleDBConnector(DBConnector):
             sql = request.format(where, dimension)
             try:
                 c = self._execute(None, sql)
-            except DbError as e:  # no statistics for the current table
+            except DbError:  # no statistics for the current table
                 return None
 
             res_d = self._fetchone(c)
@@ -1120,7 +1114,7 @@ class OracleDBConnector(DBConnector):
 
             if not res_d or len(res_d) < 2:
                 return None
-            elif isinstance(res_d[0], QPyNullVariant):
+            elif res_d[0] == NULL:
                 return None
             else:
                 res.extend(res_d)
@@ -1166,7 +1160,7 @@ class OracleDBConnector(DBConnector):
                 None,
                 (u"SELECT CS_NAME FROM MDSYS.CS_SRS WHERE"
                  u" SRID = {}".format(srid)))
-        except DbError as e:
+        except DbError:
             return
         sr = self._fetchone(c)
         c.close()
@@ -1218,8 +1212,6 @@ class OracleDBConnector(DBConnector):
         """Delete table and its reference in sdo_geom_metadata."""
 
         schema, tablename = self.getSchemaTableName(table)
-        schema_part = u"AND owner = {} ".format(
-            self.quoteString(schema)) if schema else ""
 
         if self.isVectorTable(table):
             self.deleteMetadata(table)
@@ -1289,8 +1281,6 @@ class OracleDBConnector(DBConnector):
     def deleteView(self, view):
         """Delete a view."""
         schema, tablename = self.getSchemaTableName(view)
-        schema_part = u"AND owner = {} ".format(
-            self.quoteString(schema)) if schema else ""
 
         if self.isVectorTable(view):
             self.deleteMetadata(view)
@@ -1651,7 +1641,7 @@ class OracleDBConnector(DBConnector):
             if c:
                 c.close()
 
-        except self.error_types() as e:
+        except self.error_types():
             pass
 
         return
@@ -1688,11 +1678,6 @@ class OracleDBConnector(DBConnector):
     # def _get_cursor_columns(self, c):
     #     pass
 
-    def getQueryBuilderDictionary(self):
-        from .sql_dictionary import getQueryBuilderDictionary
-
-        return getQueryBuilderDictionary()
-
     def getSqlDictionary(self):
         """Returns the dictionary for SQL dialog."""
         from .sql_dictionary import getSqlDictionary
@@ -1756,5 +1741,4 @@ class OracleDBConnector(DBConnector):
 
     def getQueryBuilderDictionary(self):
         from .sql_dictionary import getQueryBuilderDictionary
-
         return getQueryBuilderDictionary()
