@@ -20,8 +20,9 @@ email                : brush.tyler@gmail.com
  ***************************************************************************/
 """
 
-from PyQt4.QtCore import Qt, QObject, QSettings, SIGNAL
-from PyQt4.QtGui import QApplication, QAction, QKeySequence, QIcon, QMenu, QInputDialog, QMessageBox
+from qgis.PyQt.QtCore import Qt, QObject, QSettings, pyqtSignal
+from qgis.PyQt.QtWidgets import QApplication, QAction, QMenu, QInputDialog, QMessageBox
+from qgis.PyQt.QtGui import QKeySequence, QIcon
 
 from qgis.gui import QgsMessageBar
 from ..db_plugins import createDbPlugin
@@ -38,7 +39,7 @@ class BaseError(Exception):
             msg = e
 
         if not isinstance(msg, unicode):
-            msg = unicode(msg, 'utf-8', 'replace') # convert from utf8 and replace errors (if any)
+            msg = unicode(msg, 'utf-8', 'replace')  # convert from utf8 and replace errors (if any)
 
         self.msg = msg
         Exception.__init__(self, msg)
@@ -75,6 +76,9 @@ class DbError(BaseError):
 
 
 class DBPlugin(QObject):
+    deleted = pyqtSignal()
+    changed = pyqtSignal()
+    aboutToChange = pyqtSignal()
 
     def __init__(self, conn_name, parent=None):
         QObject.__init__(self, parent)
@@ -116,7 +120,7 @@ class DBPlugin(QObject):
         settings = QSettings()
         settings.beginGroup(u"/%s/%s" % (self.connectionSettingsKey(), self.connectionName()))
         settings.remove("")
-        self.emit(SIGNAL('deleted'))
+        self.deleted.emit()
         return True
 
     @classmethod
@@ -181,6 +185,9 @@ class DBPlugin(QObject):
 
 
 class DbItemObject(QObject):
+    changed = pyqtSignal()
+    aboutToChange = pyqtSignal()
+    deleted = pyqtSignal()
 
     def __init__(self, parent=None):
         QObject.__init__(self, parent)
@@ -189,10 +196,7 @@ class DbItemObject(QObject):
         return None
 
     def refresh(self):
-        self.emit(SIGNAL('changed'))  # refresh the item data reading them from the db
-
-    def aboutToChange(self):
-        self.emit(SIGNAL('aboutToChange'))
+        self.changed.emit()  # refresh the item data reading them from the db
 
     def info(self):
         pass
@@ -233,10 +237,10 @@ class Database(DbItemObject):
         return self.connector.publicUri()
 
     def delete(self):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         ret = self.connection().remove()
         if ret is not False:
-            self.emit(SIGNAL('deleted'))
+            self.deleted.emit()
         return ret
 
     def info(self):
@@ -289,11 +293,11 @@ class Database(DbItemObject):
     def registerSubPluginActions(self, mainWindow):
         # load plugins!
         try:
-            exec (u"from .%s.plugins import load" % self.dbplugin().typeName())
+            exec (u"from .%s.plugins import load" % self.dbplugin().typeName(), globals())
         except ImportError:
             pass
         else:
-            load(self, mainWindow)
+            load(self, mainWindow)  # NOQA
 
     def registerDatabaseActions(self, mainWindow):
         action = QAction(QApplication.translate("DBManagerPlugin", "&Re-connect"), self)
@@ -330,8 +334,11 @@ class Database(DbItemObject):
         if self.schemas() is not None:
             action = QAction(QApplication.translate("DBManagerPlugin", "&Move to schema"), self)
             action.setMenu(QMenu(mainWindow))
-            invoke_callback = lambda: mainWindow.invokeCallback(self.prepareMenuMoveTableToSchemaActionSlot)
-            QObject.connect(action.menu(), SIGNAL("aboutToShow()"), invoke_callback)
+
+            def invoke_callback():
+                return mainWindow.invokeCallback(self.prepareMenuMoveTableToSchemaActionSlot)
+
+            action.menu().aboutToShow.connect(invoke_callback)
             mainWindow.registerAction(action, QApplication.translate("DBManagerPlugin", "&Table"))
 
     def reconnectActionSlot(self, item, action, parent):
@@ -392,7 +399,7 @@ class Database(DbItemObject):
     def schemas(self):
         schemas = self.connector.getSchemas()
         if schemas is not None:
-            schemas = map(lambda x: self.schemasFactory(x, self), schemas)
+            schemas = [self.schemasFactory(x, self) for x in schemas]
         return schemas
 
     def createSchema(self, name):
@@ -463,7 +470,8 @@ class Database(DbItemObject):
 
     def prepareMenuMoveTableToSchemaActionSlot(self, item, menu, mainWindow):
         """ populate menu with schemas """
-        slot = lambda x: lambda: mainWindow.invokeCallback(self.moveTableToSchemaActionSlot, x)
+        def slot(x):
+            return lambda: mainWindow.invokeCallback(self.moveTableToSchemaActionSlot, x)
 
         menu.clear()
         for schema in self.schemas():
@@ -501,12 +509,12 @@ class Database(DbItemObject):
     def tables(self, schema=None, sys_tables=False):
         tables = self.connector.getTables(schema.name if schema else None, sys_tables)
         if tables is not None:
-            tables = map(lambda x: self.tablesFactory(x, self, schema), tables)
+            tables = [self.tablesFactory(x, self, schema) for x in tables]
         return tables
 
     def createTable(self, table, fields, schema=None):
-        field_defs = map(lambda x: x.definition(), fields)
-        pkeys = filter(lambda x: x.primaryKey, fields)
+        field_defs = [x.definition() for x in fields]
+        pkeys = [x for x in fields if x.primaryKey]
         pk_name = pkeys[0].name if len(pkeys) > 0 else None
 
         ret = self.connector.createTable((schema, table), field_defs, pk_name)
@@ -567,14 +575,14 @@ class Schema(DbItemObject):
         return self.database().tables(self)
 
     def delete(self):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         ret = self.database().connector.deleteSchema(self.name)
         if ret is not False:
-            self.emit(SIGNAL('deleted'))
+            self.deleted.emit()
         return ret
 
     def rename(self, new_name):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         ret = self.database().connector.renameSchema(self.name, new_name)
         if ret is not False:
             self.name = new_name
@@ -622,17 +630,17 @@ class Table(DbItemObject):
         return self.database().connector.quoteId((self.schemaName(), self.name))
 
     def delete(self):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         if self.isView:
             ret = self.database().connector.deleteView((self.schemaName(), self.name))
         else:
             ret = self.database().connector.deleteTable((self.schemaName(), self.name))
         if ret is not False:
-            self.emit(SIGNAL('deleted'))
+            self.deleted.emit()
         return ret
 
     def rename(self, new_name):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         ret = self.database().connector.renameTable((self.schemaName(), self.name), new_name)
         if ret is not False:
             self.name = new_name
@@ -640,14 +648,14 @@ class Table(DbItemObject):
         return ret
 
     def empty(self):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         ret = self.database().connector.emptyTable((self.schemaName(), self.name))
         if ret is not False:
             self.refreshRowCount()
         return ret
 
     def moveToSchema(self, schema):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         if self.schema() == schema:
             return True
         ret = self.database().connector.moveTableToSchema((self.schemaName(), self.name), schema.name)
@@ -690,7 +698,7 @@ class Table(DbItemObject):
         ret = []
 
         # add the pk
-        pkcols = filter(lambda x: x.primaryKey, self.fields())
+        pkcols = [x for x in self.fields() if x.primaryKey]
         if len(pkcols) == 1:
             ret.append(pkcols[0])
 
@@ -722,7 +730,7 @@ class Table(DbItemObject):
         if self._fields is None:
             fields = self.database().connector.getTableFields((self.schemaName(), self.name))
             if fields is not None:
-                self._fields = map(lambda x: self.tableFieldsFactory(x, self), fields)
+                self._fields = [self.tableFieldsFactory(x, self) for x in fields]
         return self._fields
 
     def refreshFields(self):
@@ -730,14 +738,14 @@ class Table(DbItemObject):
         self.refresh()
 
     def addField(self, fld):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         ret = self.database().connector.addTableColumn((self.schemaName(), self.name), fld.definition())
         if ret is not False:
             self.refreshFields()
         return ret
 
     def deleteField(self, fld):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         ret = self.database().connector.deleteTableColumn((self.schemaName(), self.name), fld.name)
         if ret is not False:
             self.refreshFields()
@@ -746,7 +754,7 @@ class Table(DbItemObject):
         return ret
 
     def addGeometryColumn(self, geomCol, geomType, srid, dim, createSpatialIndex=False):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         ret = self.database().connector.addGeometryColumn((self.schemaName(), self.name), geomCol, geomType, srid, dim)
         if not ret:
             return False
@@ -768,7 +776,7 @@ class Table(DbItemObject):
         if self._constraints is None:
             constraints = self.database().connector.getTableConstraints((self.schemaName(), self.name))
             if constraints is not None:
-                self._constraints = map(lambda x: self.tableConstraintsFactory(x, self), constraints)
+                self._constraints = [self.tableConstraintsFactory(x, self) for x in constraints]
         return self._constraints
 
     def refreshConstraints(self):
@@ -776,7 +784,7 @@ class Table(DbItemObject):
         self.refresh()
 
     def addConstraint(self, constr):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         if constr.type == TableConstraint.TypePrimaryKey:
             ret = self.database().connector.addTablePrimaryKey((self.schemaName(), self.name),
                                                                constr.fields()[constr.columns[0]].name)
@@ -790,7 +798,7 @@ class Table(DbItemObject):
         return ret
 
     def deleteConstraint(self, constr):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         ret = self.database().connector.deleteTableConstraint((self.schemaName(), self.name), constr.name)
         if ret is not False:
             self.refreshConstraints()
@@ -803,7 +811,7 @@ class Table(DbItemObject):
         if self._indexes is None:
             indexes = self.database().connector.getTableIndexes((self.schemaName(), self.name))
             if indexes is not None:
-                self._indexes = map(lambda x: self.tableIndexesFactory(x, self), indexes)
+                self._indexes = [self.tableIndexesFactory(x, self) for x in indexes]
         return self._indexes
 
     def refreshIndexes(self):
@@ -811,7 +819,7 @@ class Table(DbItemObject):
         self.refresh()
 
     def addIndex(self, idx):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         ret = self.database().connector.createTableIndex((self.schemaName(), self.name), idx.name,
                                                          idx.fields()[idx.columns[0]].name)
         if ret is not False:
@@ -819,7 +827,7 @@ class Table(DbItemObject):
         return ret
 
     def deleteIndex(self, idx):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         ret = self.database().connector.deleteTableIndex((self.schemaName(), self.name), idx.name)
         if ret is not False:
             self.refreshIndexes()
@@ -832,7 +840,7 @@ class Table(DbItemObject):
         if self._triggers is None:
             triggers = self.database().connector.getTableTriggers((self.schemaName(), self.name))
             if triggers is not None:
-                self._triggers = map(lambda x: self.tableTriggersFactory(x, self), triggers)
+                self._triggers = [self.tableTriggersFactory(x, self) for x in triggers]
         return self._triggers
 
     def refreshTriggers(self):
@@ -846,7 +854,7 @@ class Table(DbItemObject):
         if self._rules is None:
             rules = self.database().connector.getTableRules((self.schemaName(), self.name))
             if rules is not None:
-                self._rules = map(lambda x: self.tableRulesFactory(x, self), rules)
+                self._rules = [self.tableRulesFactory(x, self) for x in rules]
         return self._rules
 
     def refreshRules(self):
@@ -854,7 +862,7 @@ class Table(DbItemObject):
         self.refresh()
 
     def refreshRowCount(self):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         prevRowCount = self.rowCount
         try:
             self.rowCount = self.database().connector.getTableRowCount((self.schemaName(), self.name))
@@ -887,7 +895,7 @@ class Table(DbItemObject):
 
             if trigger_action == "enable" or trigger_action == "disable":
                 enable = trigger_action == "enable"
-                self.aboutToChange()
+                self.aboutToChange.emit()
                 self.database().connector.enableAllTableTriggers(enable, (self.schemaName(), self.name))
                 self.refreshTriggers()
                 return True
@@ -908,14 +916,14 @@ class Table(DbItemObject):
                 QApplication.setOverrideCursor(Qt.WaitCursor)
 
             if trigger_action == "delete":
-                self.aboutToChange()
+                self.aboutToChange.emit()
                 self.database().connector.deleteTableTrigger(trigger_name, (self.schemaName(), self.name))
                 self.refreshTriggers()
                 return True
 
             elif trigger_action == "enable" or trigger_action == "disable":
                 enable = trigger_action == "enable"
-                self.aboutToChange()
+                self.aboutToChange.emit()
                 self.database().connector.enableTableTrigger(trigger_name, enable, (self.schemaName(), self.name))
                 self.refreshTriggers()
                 return True
@@ -952,7 +960,7 @@ class VectorTable(Table):
         return False
 
     def createSpatialIndex(self, geom_column=None):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         geom_column = geom_column if geom_column is not None else self.geomColumn
         ret = self.database().connector.createSpatialIndex((self.schemaName(), self.name), geom_column)
         if ret is not False:
@@ -960,7 +968,7 @@ class VectorTable(Table):
         return ret
 
     def deleteSpatialIndex(self, geom_column=None):
-        self.aboutToChange()
+        self.aboutToChange.emit()
         geom_column = geom_column if geom_column is not None else self.geomColumn
         ret = self.database().connector.deleteSpatialIndex((self.schemaName(), self.name), geom_column)
         if ret is not False:
@@ -1086,7 +1094,7 @@ class TableField(TableSubItemObject):
         return self.update(new_name)
 
     def update(self, new_name, new_type_str=None, new_not_null=None, new_default_str=None):
-        self.table().aboutToChange()
+        self.table().aboutToChange.emit()
         if self.name == new_name:
             new_name = None
         if self.type2String() == new_type_str:
