@@ -147,10 +147,10 @@ def getSqlDictionary(spatial=True):
 def getQueryBuilderDictionary():
     # concat functions
     def ff(l):
-        return filter(lambda s: s[0] != '*', l)
+        return [s for s in l if s[0] != '*']
 
     def add_paren(l):
-        return map(lambda s: s + "(", l)
+        return [s + "(" for s in l]
     foo = sorted(add_paren(ff(list(set.union(set(functions), set(spatialite_functions))))))
     m = sorted(add_paren(ff(math_functions)))
     agg = sorted(add_paren(ff(aggregate_functions)))
