@@ -25,12 +25,12 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 __revision__ = '$Format:%H$'
 
-from processing.tools.dataobjects import *
-from processing.tools.general import *
-from processing.tools.vector import *
-from processing.tools.raster import *
-from processing.tools.system import *
-from processing.tests.TestData import loadTestData
+from processing.tools.dataobjects import *          # NOQA
+from processing.tools.general import *              # NOQA
+from processing.tools.vector import *               # NOQA
+from processing.tools.raster import *               # NOQA
+from processing.tools.system import *               # NOQA
+from processing.tests.TestData import loadTestData  # NOQA
 
 
 def classFactory(iface):
