@@ -25,7 +25,7 @@ __copyright__ = '(C) 2013, Victor Olaya'
 
 __revision__ = '$Format:%H$'
 
-from PyQt4.QtCore import QSettings
+from qgis.PyQt.QtCore import QSettings
 from qgis.core import QgsVectorFileWriter
 
 from processing.core.GeoAlgorithm import GeoAlgorithm
@@ -35,6 +35,7 @@ from processing.tools import dataobjects, vector
 
 
 class ExampleAlgorithm(GeoAlgorithm):
+
     """This is an example algorithm that takes a vector layer and
     creates a new one just with just those features of the input
     layer that are selected.
@@ -60,19 +61,19 @@ class ExampleAlgorithm(GeoAlgorithm):
         """
 
         # The name that the user will see in the toolbox
-        self.name = 'Create copy of layer'
+        self.name, self.i18n_name = self.trAlgorithm('Create copy of layer')
 
         # The branch of the toolbox under which the algorithm will appear
-        self.group = 'Algorithms for vector layers'
+        self.group, self.i18n_group = self.trAlgorithm('Algorithms for vector layers')
 
         # We add the input vector layer. It can have any kind of geometry
         # It is a mandatory (not optional) one, hence the False argument
         self.addParameter(ParameterVector(self.INPUT_LAYER,
-            self.tr('Input layer'), [ParameterVector.VECTOR_TYPE_ANY], False))
+                                          self.tr('Input layer'), [ParameterVector.VECTOR_TYPE_ANY], False))
 
         # We add a vector layer as output
         self.addOutput(OutputVector(self.OUTPUT_LAYER,
-            self.tr('Output layer with selected features')))
+                                    self.tr('Output layer with selected features')))
 
     def processAlgorithm(self, progress):
         """Here is where the processing itself takes place."""
