@@ -25,6 +25,10 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 __revision__ = '$Format:%H$'
 
+import os
+
+from qgis.PyQt.QtGui import QIcon
+
 from processing.algs.gdal.GdalAlgorithm import GdalAlgorithm
 from processing.core.parameters import ParameterRaster
 from processing.core.parameters import ParameterBoolean
@@ -34,6 +38,8 @@ from processing.core.outputs import OutputRaster
 
 from processing.algs.gdal.GdalUtils import GdalUtils
 
+pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
+
 
 class gdaladdo(GdalAlgorithm):
 
@@ -57,25 +63,28 @@ class gdaladdo(GdalAlgorithm):
     FORMATS = ['Internal (if possible)', 'External (GTiff .ovr)',
                'External (ERDAS Imagine .aux)']
 
+    def getIcon(self):
+        return QIcon(os.path.join(pluginPath, 'images', 'gdaltools', 'raster-overview.png'))
+
     def commandLineName(self):
         return "gdalogr:overviews"
 
     def defineCharacteristics(self):
-        self.name = 'Build overviews (pyramids)'
-        self.group = '[GDAL] Miscellaneous'
+        self.name, self.i18n_name = self.trAlgorithm('Build overviews (pyramids)')
+        self.group, self.i18n_group = self.trAlgorithm('[GDAL] Miscellaneous')
         self.addParameter(ParameterRaster(
             self.INPUT, self.tr('Input layer'), False))
         self.addParameter(ParameterString(self.LEVELS,
-            self.tr('Overview levels'), '2 4 8 16'))
+                                          self.tr('Overview levels'), '2 4 8 16'))
         self.addParameter(ParameterBoolean(self.CLEAN,
-            self.tr('Remove all existing overviews'), False))
+                                           self.tr('Remove all existing overviews'), False))
         self.addParameter(ParameterSelection(self.RESAMPLING_METHOD,
-            self.tr('Resampling method'), self.METHODS, 0))
+                                             self.tr('Resampling method'), self.METHODS, 0))
         self.addParameter(ParameterSelection(self.FORMAT,
-            self.tr('Overview format'), self.FORMATS, 0))
+                                             self.tr('Overview format'), self.FORMATS, 0))
         self.addOutput(OutputRaster(self.OUTPUT, self.tr('Pyramidized'), True))
 
-    def processAlgorithm(self, progress):
+    def getConsoleCommands(self):
         inFile = self.getParameterValue(self.INPUT)
         clearOverviews = self.getParameterValue(self.CLEAN)
         ovrFormat = self.getParameterValue(self.FORMAT)
@@ -97,5 +106,4 @@ class gdaladdo(GdalAlgorithm):
         arguments.extend(self.getParameterValue(self.LEVELS).split(' '))
         self.setOutputValue(self.OUTPUT, inFile)
 
-        GdalUtils.runGdal(['gdaladdo', GdalUtils.escapeAndJoin(arguments)],
-                          progress)
+        return ['gdaladdo', GdalUtils.escapeAndJoin(arguments)]
