@@ -25,42 +25,57 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 __revision__ = '$Format:%H$'
 
+import os
+
+from qgis.PyQt.QtGui import QIcon
 
 from processing.algs.gdal.GdalAlgorithm import GdalAlgorithm
 from processing.core.outputs import OutputRaster
 from processing.core.parameters import ParameterBoolean
 from processing.core.parameters import ParameterMultipleInput
 from processing.core.parameters import ParameterSelection
+from processing.core.parameters import ParameterNumber
 from processing.tools.system import isWindows
 from processing.algs.gdal.GdalUtils import GdalUtils
 
+pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
+
 
 class merge(GdalAlgorithm):
 
     INPUT = 'INPUT'
     OUTPUT = 'OUTPUT'
     PCT = 'PCT'
+    NODATA = 'NODATA'
     SEPARATE = 'SEPARATE'
     RTYPE = 'RTYPE'
 
     TYPE = ['Byte', 'Int16', 'UInt16', 'UInt32', 'Int32', 'Float32', 'Float64']
 
+    def getIcon(self):
+        return QIcon(os.path.join(pluginPath, 'images', 'gdaltools', 'merge.png'))
+
     def defineCharacteristics(self):
-        self.name = 'Merge'
-        self.group = '[GDAL] Miscellaneous'
+        self.name, self.i18n_name = self.trAlgorithm('Merge')
+        self.group, self.i18n_group = self.trAlgorithm('[GDAL] Miscellaneous')
         self.addParameter(ParameterMultipleInput(merge.INPUT,
-            self.tr('Input layers'), ParameterMultipleInput.TYPE_RASTER))
+                                                 self.tr('Input layers'), ParameterMultipleInput.TYPE_RASTER))
         self.addParameter(ParameterBoolean(merge.PCT,
-            self.tr('Grab pseudocolor table from first layer'), False))
+                                           self.tr('Grab pseudocolor table from first layer'), False))
         self.addParameter(ParameterBoolean(merge.SEPARATE,
-            self.tr('Layer stack'), False))
+                                           self.tr('Place each input file into a separate band'), False))
+        self.addParameter(ParameterNumber(self.NODATA,
+                                          self.tr('Assign a specified nodata value to output bands'), None, None, -9999))
         self.addParameter(ParameterSelection(self.RTYPE,
-            self.tr('Output raster type'), self.TYPE, 5))
+                                             self.tr('Output raster type'), self.TYPE, 5))
 
         self.addOutput(OutputRaster(merge.OUTPUT, self.tr('Merged')))
 
-    def processAlgorithm(self, progress):
+    def getConsoleCommands(self):
         arguments = []
+        if self.getParameterValue(merge.NODATA):
+            arguments.append('-a_nodata')
+            arguments.append(unicode(self.getParameterValue(merge.NODATA)))
         arguments.append('-ot')
         arguments.append(self.TYPE[self.getParameterValue(self.RTYPE)])
         if self.getParameterValue(merge.SEPARATE):
@@ -81,4 +96,4 @@ class merge(GdalAlgorithm):
         else:
             commands = ['gdal_merge.py', GdalUtils.escapeAndJoin(arguments)]
 
-        GdalUtils.runGdal(commands, progress)
+        return commands
