@@ -25,6 +25,10 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 __revision__ = '$Format:%H$'
 
+import os
+
+from qgis.PyQt.QtGui import QIcon
+
 from processing.algs.gdal.GdalAlgorithm import GdalAlgorithm
 from processing.core.parameters import ParameterRaster
 from processing.core.parameters import ParameterNumber
@@ -32,6 +36,8 @@ from processing.core.parameters import ParameterBoolean
 from processing.core.outputs import OutputRaster
 from processing.algs.gdal.GdalUtils import GdalUtils
 
+pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
+
 
 class nearblack(GdalAlgorithm):
 
@@ -40,26 +46,31 @@ class nearblack(GdalAlgorithm):
     NEAR = 'NEAR'
     WHITE = 'WHITE'
 
-    def defineCharacteristics(self):
-        self.name = 'Near black'
-        self.group = '[GDAL] Analysis'
-        self.addParameter(ParameterRaster(nearblack.INPUT,
-           self.tr('Input layer'), False))
-        self.addParameter(ParameterNumber(nearblack.NEAR,
-            self.tr('How far from black (white)'), 0, None, 15))
-        self.addParameter(ParameterBoolean(nearblack.WHITE,
-            self.tr('Search for nearly white pixels instead of nearly black'),
-            False))
-        self.addOutput(OutputRaster(nearblack.OUTPUT, self.tr('Nearblack')))
+    def getIcon(self):
+        return QIcon(os.path.join(pluginPath, 'images', 'gdaltools', 'nearblack.png'))
 
-    def processAlgorithm(self, progress):
+    def defineCharacteristics(self):
+        self.name, self.i18n_name = self.trAlgorithm('Near black')
+        self.group, self.i18n_group = self.trAlgorithm('[GDAL] Analysis')
+        self.addParameter(ParameterRaster(self.INPUT,
+                                          self.tr('Input layer'), False))
+        self.addParameter(ParameterNumber(self.NEAR,
+                                          self.tr('How far from black (white)'), 0, None, 15))
+        self.addParameter(ParameterBoolean(self.WHITE,
+                                           self.tr('Search for nearly white pixels instead of nearly black'),
+                                           False))
+        self.addOutput(OutputRaster(self.OUTPUT, self.tr('Nearblack')))
+
+    def getConsoleCommands(self):
         arguments = []
         arguments.append('-o')
-        arguments.append(self.getOutputValue(nearblack.OUTPUT))
+        output = unicode(self.getOutputValue(self.OUTPUT))
+        arguments.append(output)
+        arguments.append('-of')
+        arguments.append(GdalUtils.getFormatShortNameFromFilename(output))
         arguments.append('-near')
-        arguments.append(str(self.getParameterValue(nearblack.NEAR)))
-        if self.getParameterValue(nearblack.WHITE):
+        arguments.append(unicode(self.getParameterValue(self.NEAR)))
+        if self.getParameterValue(self.WHITE):
             arguments.append('-white')
-        arguments.append(self.getParameterValue(nearblack.INPUT))
-        GdalUtils.runGdal(['nearblack', GdalUtils.escapeAndJoin(arguments)],
-                          progress)
+        arguments.append(self.getParameterValue(self.INPUT))
+        return ['nearblack', GdalUtils.escapeAndJoin(arguments)]
