@@ -25,6 +25,9 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 __revision__ = '$Format:%H$'
 
+import os
+
+from qgis.PyQt.QtGui import QIcon
 
 from processing.algs.gdal.GdalAlgorithm import GdalAlgorithm
 
@@ -37,6 +40,8 @@ from processing.tools.system import isWindows
 
 from processing.algs.gdal.GdalUtils import GdalUtils
 
+pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
+
 
 class sieve(GdalAlgorithm):
 
@@ -47,27 +52,30 @@ class sieve(GdalAlgorithm):
 
     PIXEL_CONNECTIONS = ['4', '8']
 
+    def getIcon(self):
+        return QIcon(os.path.join(pluginPath, 'images', 'gdaltools', 'sieve.png'))
+
     def defineCharacteristics(self):
-        self.name = 'Sieve'
-        self.group = '[GDAL] Analysis'
+        self.name, self.i18n_name = self.trAlgorithm('Sieve')
+        self.group, self.i18n_group = self.trAlgorithm('[GDAL] Analysis')
         self.addParameter(ParameterRaster(self.INPUT, self.tr('Input layer'), False))
         self.addParameter(ParameterNumber(self.THRESHOLD,
-            self.tr('Threshold'), 0, 9999, 2))
+                                          self.tr('Threshold'), 0, 9999, 2))
         self.addParameter(ParameterSelection(self.CONNECTIONS,
-            self.tr('Pixel connection'), self.PIXEL_CONNECTIONS, 0))
+                                             self.tr('Pixel connection'), self.PIXEL_CONNECTIONS, 0))
 
         self.addOutput(OutputRaster(self.OUTPUT, self.tr('Sieved')))
 
-    def processAlgorithm(self, progress):
+    def getConsoleCommands(self):
         output = self.getOutputValue(self.OUTPUT)
 
         arguments = []
         arguments.append('-st')
-        arguments.append(str(self.getParameterValue(self.THRESHOLD)))
+        arguments.append(unicode(self.getParameterValue(self.THRESHOLD)))
 
         arguments.append('-' +
-            self.PIXEL_CONNECTIONS[self.getParameterValue(
-                self.CONNECTIONS)])
+                         self.PIXEL_CONNECTIONS[self.getParameterValue(
+                                                self.CONNECTIONS)])
 
         arguments.append('-of')
         arguments.append(GdalUtils.getFormatShortNameFromFilename(output))
@@ -82,4 +90,4 @@ class sieve(GdalAlgorithm):
         else:
             commands = ['gdal_sieve.py', GdalUtils.escapeAndJoin(arguments)]
 
-        GdalUtils.runGdal(commands, progress)
+        return commands
