@@ -30,8 +30,8 @@ __revision__ = '$Format:%H$'
 import os
 from processing.core.parameters import ParameterFile
 from processing.core.outputs import OutputFile
-from FusionUtils import FusionUtils
-from FusionAlgorithm import FusionAlgorithm
+from .FusionUtils import FusionUtils
+from .FusionAlgorithm import FusionAlgorithm
 from processing.core.parameters import ParameterString
 from processing.core.parameters import ParameterBoolean
 
@@ -46,12 +46,12 @@ class CloudMetrics(FusionAlgorithm):
     HTMIN = 'HTMIN'
 
     def defineCharacteristics(self):
-        self.name = 'Cloud Metrics'
-        self.group = 'Points'
+        self.name, self.i18n_name = self.trAlgorithm('Cloud Metrics')
+        self.group, self.i18n_group = self.trAlgorithm('Points')
         self.addParameter(ParameterFile(
-            self.INPUT, self.tr('Input las layer')))
+            self.INPUT, self.tr('Input LAS layer')))
         self.addOutput(OutputFile(
-            self.OUTPUT, self.tr('Output file with tabular metric information'), 'dtm'))
+            self.OUTPUT, self.tr('Output file with tabular metric information'), 'csv'))
         above = ParameterString(self.ABOVE, self.tr('Above'), '', False)
         above.isAdvanced = True
         self.addParameter(above)
@@ -71,17 +71,17 @@ class CloudMetrics(FusionAlgorithm):
         commands = [os.path.join(FusionUtils.FusionPath(), 'CloudMetrics.exe')]
         commands.append('/verbose')
         above = self.getParameterValue(self.ABOVE)
-        if str(above).strip() != '':
-            commands.append('/above:' + str(above))
+        if unicode(above).strip() != '':
+            commands.append('/above:' + unicode(above))
         firstImpulse = self.getParameterValue(self.FIRSTIMPULSE)
         if firstImpulse:
-            commands.append('/firstinpulse:' + firstImpulse)
+            commands.append('/firstinpulse')
         firstReturn = self.getParameterValue(self.FIRSTRETURN)
         if firstReturn:
-            commands.append('/firstreturn:' + firstReturn)
+            commands.append('/firstreturn')
         htmin = self.getParameterValue(self.HTMIN)
-        if str(htmin).strip() != '':
-            commands.append('/minht:' + str(htmin))
+        if unicode(htmin).strip() != '':
+            commands.append('/minht:' + unicode(htmin))
         files = self.getParameterValue(self.INPUT).split(';')
         if len(files) == 1:
             commands.append(self.getParameterValue(self.INPUT))
