@@ -27,8 +27,8 @@ import os
 from processing.core.parameters import ParameterFile
 from processing.core.parameters import ParameterString
 from processing.core.outputs import OutputFile
-from FusionAlgorithm import FusionAlgorithm
-from FusionUtils import FusionUtils
+from .FusionAlgorithm import FusionAlgorithm
+from .FusionUtils import FusionUtils
 
 
 class Csv2Grid(FusionAlgorithm):
