@@ -29,8 +29,8 @@ import os
 from processing.core.parameters import ParameterFile
 from processing.core.parameters import ParameterBoolean
 from processing.core.outputs import OutputFile
-from FusionAlgorithm import FusionAlgorithm
-from FusionUtils import FusionUtils
+from .FusionAlgorithm import FusionAlgorithm
+from .FusionUtils import FusionUtils
 
 
 class FirstLastReturn(FusionAlgorithm):
