@@ -26,10 +26,10 @@ __copyright__ = '(C) 2012, Victor Olaya'
 __revision__ = '$Format:%H$'
 
 import os
-from PyQt4 import QtGui
+from qgis.PyQt.QtGui import QIcon
 from processing.core.GeoAlgorithm import GeoAlgorithm
 from processing.core.parameters import ParameterString
-from FusionUtils import FusionUtils
+from .FusionUtils import FusionUtils
 
 
 class FusionAlgorithm(GeoAlgorithm):
@@ -38,7 +38,7 @@ class FusionAlgorithm(GeoAlgorithm):
 
     def getIcon(self):
         filepath = os.path.dirname(__file__) + '/../../../images/tool.png'
-        return QtGui.QIcon(filepath)
+        return QIcon(filepath)
 
     def checkBeforeOpeningParametersDialog(self):
         path = FusionUtils.FusionPath()
