@@ -33,8 +33,8 @@ from processing.core.parameters import ParameterNumber
 from processing.core.parameters import ParameterBoolean
 from processing.core.parameters import ParameterSelection
 from processing.core.outputs import OutputFile
-from FusionAlgorithm import FusionAlgorithm
-from FusionUtils import FusionUtils
+from .FusionAlgorithm import FusionAlgorithm
+from .FusionUtils import FusionUtils
 from processing.core.parameters import ParameterString
 
 
@@ -55,10 +55,10 @@ class GridSurfaceCreate(FusionAlgorithm):
     ADVANCED_MODIFIERS = 'ADVANCED_MODIFIERS'
 
     def defineCharacteristics(self):
-        self.name = 'Grid Surface Create'
-        self.group = 'Surface'
+        self.name, self.i18n_name = self.trAlgorithm('Grid Surface Create')
+        self.group, self.i18n_group = self.trAlgorithm('Surface')
         self.addParameter(ParameterFile(
-            self.INPUT, self.tr('Input las layer')))
+            self.INPUT, self.tr('Input LAS layer')))
         self.addParameter(ParameterNumber(
             self.CELLSIZE, self.tr('Cellsize'), 0, None, 10.0))
         self.addParameter(ParameterSelection(
@@ -100,28 +100,28 @@ class GridSurfaceCreate(FusionAlgorithm):
         commands = [os.path.join(FusionUtils.FusionPath(), 'GridSurfaceCreate.exe')]
         commands.append('/verbose')
         spike = self.getParameterValue(self.SPIKE)
-        if str(spike).strip():
-            commands.append('/spike:' + str(spike))
+        if unicode(spike).strip():
+            commands.append('/spike:' + unicode(spike))
         median = self.getParameterValue(self.MEDIAN)
-        if str(median).strip():
-            commands.append('/median:' + str(median))
+        if unicode(median).strip():
+            commands.append('/median:' + unicode(median))
         smooth = self.getParameterValue(self.SMOOTH)
-        if str(smooth).strip():
-            commands.append('/smooth:' + str(smooth))
+        if unicode(smooth).strip():
+            commands.append('/smooth:' + unicode(smooth))
         slope = self.getParameterValue(self.SLOPE)
-        if str(slope).strip():
-            commands.append('/slope:' + str(slope))
+        if unicode(slope).strip():
+            commands.append('/slope:' + unicode(slope))
         minimum = self.getParameterValue(self.MINIMUM)
-        if str(minimum).strip():
-            commands.append('/minimum:' + str(minimum))
+        if unicode(minimum).strip():
+            commands.append('/minimum:' + unicode(minimum))
         class_var = self.getParameterValue(self.CLASS)
-        if str(class_var).strip():
-            commands.append('/class:' + str(class_var))
-        advance_modifiers = str(self.getParameterValue(self.ADVANCED_MODIFIERS)).strip()
+        if unicode(class_var).strip():
+            commands.append('/class:' + unicode(class_var))
+        advance_modifiers = unicode(self.getParameterValue(self.ADVANCED_MODIFIERS)).strip()
         if advance_modifiers:
             commands.append(advance_modifiers)
         commands.append(self.getOutputValue(self.OUTPUT_DTM))
-        commands.append(str(self.getParameterValue(self.CELLSIZE)))
+        commands.append(unicode(self.getParameterValue(self.CELLSIZE)))
         commands.append(self.UNITS[self.getParameterValue(self.XYUNITS)][0])
         commands.append(self.UNITS[self.getParameterValue(self.ZUNITS)][0])
         commands.append('0')
