@@ -29,12 +29,13 @@ __revision__ = '$Format:%H$'
 
 import os
 
-from LAStoolsUtils import LAStoolsUtils
-from LAStoolsAlgorithm import LAStoolsAlgorithm
+from .LAStoolsUtils import LAStoolsUtils
+from .LAStoolsAlgorithm import LAStoolsAlgorithm
 
 from processing.core.parameters import ParameterSelection
 from processing.core.parameters import ParameterBoolean
 
+
 class las2dem(LAStoolsAlgorithm):
 
     ATTRIBUTE = "ATTRIBUTE"
@@ -44,18 +45,18 @@ class las2dem(LAStoolsAlgorithm):
     USE_TILE_BB = "USE_TILE_BB"
 
     def defineCharacteristics(self):
-        self.name = "las2dem"
-        self.group = "LAStools"
+        self.name, self.i18n_name = self.trAlgorithm('las2dem')
+        self.group, self.i18n_group = self.trAlgorithm('LAStools')
         self.addParametersVerboseGUI()
         self.addParametersPointInputGUI()
         self.addParametersFilter1ReturnClassFlagsGUI()
         self.addParametersStepGUI()
         self.addParameter(ParameterSelection(las2dem.ATTRIBUTE,
-            self.tr("Attribute"), las2dem.ATTRIBUTES, 0))
+                                             self.tr("Attribute"), las2dem.ATTRIBUTES, 0))
         self.addParameter(ParameterSelection(las2dem.PRODUCT,
-            self.tr("Product"), las2dem.PRODUCTS, 0))
+                                             self.tr("Product"), las2dem.PRODUCTS, 0))
         self.addParameter(ParameterBoolean(las2dem.USE_TILE_BB,
-            self.tr("use tile bounding box (after tiling with buffer)"), False))
+                                           self.tr("use tile bounding box (after tiling with buffer)"), False))
         self.addParametersRasterOutputGUI()
         self.addParametersAdditionalGUI()
 
