@@ -17,6 +17,7 @@
 ***************************************************************************
 """
 
+
 __author__ = 'Martin Isenburg'
 __date__ = 'September 2013'
 __copyright__ = '(C) 2013, Martin Isenburg'
@@ -24,12 +25,13 @@ __copyright__ = '(C) 2013, Martin Isenburg'
 __revision__ = '$Format:%H$'
 
 import os
-from LAStoolsUtils import LAStoolsUtils
-from LAStoolsAlgorithm import LAStoolsAlgorithm
+from .LAStoolsUtils import LAStoolsUtils
+from .LAStoolsAlgorithm import LAStoolsAlgorithm
 
 from processing.core.parameters import ParameterBoolean
 from processing.core.parameters import ParameterNumber
-from processing.core.outputs import OutputFile
+from processing.core.outputs import OutputVector
+
 
 class las2shp(LAStoolsAlgorithm):
 
@@ -38,16 +40,16 @@ class las2shp(LAStoolsAlgorithm):
     OUTPUT = "OUTPUT"
 
     def defineCharacteristics(self):
-        self.name = "las2shp"
-        self.group = "LAStools"
+        self.name, self.i18n_name = self.trAlgorithm('las2shp')
+        self.group, self.i18n_group = self.trAlgorithm('LAStools')
         self.addParametersVerboseGUI()
         self.addParametersPointInputGUI()
         self.addParameter(ParameterBoolean(las2shp.POINT_Z,
-            self.tr("use PointZ instead of MultiPointZ"), False))
+                                           self.tr("use PointZ instead of MultiPointZ"), False))
         self.addParameter(ParameterNumber(las2shp.RECORD_SIZE,
-            self.tr("number of points per record"), 0, None, 1024))
-        self.addOutput(OutputFile(las2shp.OUTPUT,
-            self.tr("Output SHP file")))
+                                          self.tr("number of points per record"), 0, None, 1024))
+        self.addOutput(OutputVector(las2shp.OUTPUT,
+                                    self.tr("Output SHP file")))
         self.addParametersAdditionalGUI()
 
     def processAlgorithm(self, progress):
@@ -59,9 +61,8 @@ class las2shp(LAStoolsAlgorithm):
         record_size = self.getParameterValue(las2shp.RECORD_SIZE)
         if record_size != 1024:
             commands.append("-record_size")
-            commands.append(str(record_size))
+            commands.append(unicode(record_size))
         commands.append("-o")
         commands.append(self.getOutputValue(las2shp.OUTPUT))
         self.addParametersAdditionalCommands(commands)
-
         LAStoolsUtils.runLAStools(commands, progress)
