@@ -24,12 +24,13 @@ __copyright__ = '(C) 2013, Martin Isenburg'
 __revision__ = '$Format:%H$'
 
 import os
-from LAStoolsUtils import LAStoolsUtils
-from LAStoolsAlgorithm import LAStoolsAlgorithm
+from .LAStoolsUtils import LAStoolsUtils
+from .LAStoolsAlgorithm import LAStoolsAlgorithm
 
 from processing.core.parameters import ParameterBoolean
 from processing.core.parameters import ParameterFile
 
+
 class lasduplicate(LAStoolsAlgorithm):
 
     LOWEST_Z = "LOWEST_Z"
@@ -38,18 +39,18 @@ class lasduplicate(LAStoolsAlgorithm):
     RECORD_REMOVED = "RECORD_REMOVED"
 
     def defineCharacteristics(self):
-        self.name = "lasduplicate"
-        self.group = "LAStools"
+        self.name, self.i18n_name = self.trAlgorithm('lasduplicate')
+        self.group, self.i18n_group = self.trAlgorithm('LAStools')
         self.addParametersVerboseGUI()
         self.addParametersPointInputGUI()
         self.addParameter(ParameterBoolean(lasduplicate.LOWEST_Z,
-            self.tr("keep duplicate with lowest z coordinate"), False))
+                                           self.tr("keep duplicate with lowest z coordinate"), False))
         self.addParameter(ParameterBoolean(lasduplicate.UNIQUE_XYZ,
-            self.tr("only remove duplicates in x y and z"), False))
+                                           self.tr("only remove duplicates in x y and z"), False))
         self.addParameter(ParameterBoolean(lasduplicate.SINGLE_RETURNS,
-            self.tr("mark surviving duplicate as single return"), False))
+                                           self.tr("mark surviving duplicate as single return"), False))
         self.addParameter(ParameterFile(lasduplicate.RECORD_REMOVED,
-            self.tr("record removed duplicates to LAS/LAZ file")))
+                                        self.tr("record removed duplicates to LAS/LAZ file")))
         self.addParametersPointOutputGUI()
         self.addParametersAdditionalGUI()
 
