@@ -24,33 +24,33 @@ __copyright__ = '(C) 2013, Martin Isenburg'
 __revision__ = '$Format:%H$'
 
 import os
-from LAStoolsUtils import LAStoolsUtils
-from LAStoolsAlgorithm import LAStoolsAlgorithm
+from .LAStoolsUtils import LAStoolsUtils
+from .LAStoolsAlgorithm import LAStoolsAlgorithm
 
 from processing.core.parameters import ParameterNumber
 from processing.core.parameters import ParameterSelection
 
+
 class lasoverage(LAStoolsAlgorithm):
 
     CHECK_STEP = "CHECK_STEP"
     OPERATION = "OPERATION"
-    OPERATIONS= ["classify as overlap", "flag as withheld", "remove from output"]
+    OPERATIONS = ["classify as overlap", "flag as withheld", "remove from output"]
 
     def defineCharacteristics(self):
-        self.name = "lasoverage"
-        self.group = "LAStools"
+        self.name, self.i18n_name = self.trAlgorithm('lasoverage')
+        self.group, self.i18n_group = self.trAlgorithm('LAStools')
         self.addParametersVerboseGUI()
         self.addParametersPointInputGUI()
         self.addParametersHorizontalFeetGUI()
         self.addParametersFilesAreFlightlinesGUI()
         self.addParameter(ParameterNumber(lasoverage.CHECK_STEP,
-            self.tr("size of grid used for scan angle check"), 0, None, 1.0))
+                                          self.tr("size of grid used for scan angle check"), 0, None, 1.0))
         self.addParameter(ParameterSelection(lasoverage.OPERATION,
-            self.tr("mode of operation"), lasoverage.OPERATIONS, 0))
+                                             self.tr("mode of operation"), lasoverage.OPERATIONS, 0))
         self.addParametersPointOutputGUI()
         self.addParametersAdditionalGUI()
 
-
     def processAlgorithm(self, progress):
         commands = [os.path.join(LAStoolsUtils.LAStoolsPath(), "bin", "lasoverage")]
         self.addParametersVerboseCommands(commands)
@@ -60,7 +60,7 @@ class lasoverage(LAStoolsAlgorithm):
         step = self.getParameterValue(lasoverage.CHECK_STEP)
         if step != 1.0:
             commands.append("-step")
-            commands.append(str(step))
+            commands.append(unicode(step))
         operation = self.getParameterValue(lasoverage.OPERATION)
         if operation == 1:
             commands.append("-flag_as_withheld")
