@@ -24,12 +24,13 @@ __copyright__ = '(C) 2014, Martin Isenburg'
 __revision__ = '$Format:%H$'
 
 import os
-from LAStoolsUtils import LAStoolsUtils
-from LAStoolsAlgorithm import LAStoolsAlgorithm
+from .LAStoolsUtils import LAStoolsUtils
+from .LAStoolsAlgorithm import LAStoolsAlgorithm
 
 from processing.core.parameters import ParameterNumber
 from processing.core.parameters import ParameterSelection
 
+
 class lassplit(LAStoolsAlgorithm):
 
     DIGITS = "DIGITS"
@@ -38,16 +39,16 @@ class lassplit(LAStoolsAlgorithm):
     INTERVAL = "INTERVAL"
 
     def defineCharacteristics(self):
-        self.name = "lassplit"
-        self.group = "LAStools"
+        self.name, self.i18n_name = self.trAlgorithm('lassplit')
+        self.group, self.i18n_group = self.trAlgorithm('LAStools')
         self.addParametersVerboseGUI()
         self.addParametersPointInputGUI()
         self.addParameter(ParameterNumber(lassplit.DIGITS,
-            self.tr("number of digits for file names"), 0, None, 5))
+                                          self.tr("number of digits for file names"), 0, None, 5))
         self.addParameter(ParameterSelection(lassplit.OPERATION,
-            self.tr("how to split"), lassplit.OPERATIONS, 0))
+                                             self.tr("how to split"), lassplit.OPERATIONS, 0))
         self.addParameter(ParameterNumber(lassplit.INTERVAL,
-            self.tr("interval or number"), 0, None, 5))
+                                          self.tr("interval or number"), 0, None, 5))
         self.addParametersPointOutputGUI()
         self.addParametersAdditionalGUI()
 
@@ -58,7 +59,7 @@ class lassplit(LAStoolsAlgorithm):
         digits = self.getParameterValue(lassplit.DIGITS)
         if digits != 5:
             commands.append("-digits")
-            commands.append(str(digits))
+            commands.append(unicode(digits))
         operation = self.getParameterValue(lassplit.OPERATION)
         if operation != 0:
             if operation == 9:
@@ -67,7 +68,7 @@ class lassplit(LAStoolsAlgorithm):
                 commands.append("-" + lassplit.OPERATIONS[operation])
         if operation > 1 and operation < 10:
             interval = self.getParameterValue(lassplit.INTERVAL)
-            commands.append(str(interval))
+            commands.append(unicode(interval))
         self.addParametersPointOutputCommands(commands)
         self.addParametersAdditionalCommands(commands)
 
