@@ -25,7 +25,11 @@ __copyright__ = '(C) 2015, Arnaud Morvan'
 
 __revision__ = '$Format:%H$'
 
-from PyQt4 import QtCore
+import os
+
+from qgis.PyQt.QtGui import QIcon
+from qgis.PyQt.QtCore import QSettings, QVariant
+
 from qgis.core import QGis, QgsGeometry, QgsFeature, QgsField
 from processing.core.GeoAlgorithm import GeoAlgorithm
 from processing.core.parameters import ParameterVector
@@ -34,6 +38,7 @@ from processing.core.outputs import OutputVector
 from processing.tools import dataobjects, vector
 
 settings_method_key = "/qgis/digitizing/validate_geometries"
+pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
 
 
 class CheckValidity(GeoAlgorithm):
@@ -44,6 +49,9 @@ class CheckValidity(GeoAlgorithm):
     INVALID_OUTPUT = 'INVALID_OUTPUT'
     ERROR_OUTPUT = 'ERROR_OUTPUT'
 
+    def getIcon(self):
+        return QIcon(os.path.join(pluginPath, 'images', 'ftools', 'check_geometry.png'))
+
     def defineCharacteristics(self):
         self.name, self.i18n_name = self.trAlgorithm('Check validity')
         self.group, self.i18n_group = self.trAlgorithm('Vector geometry tools')
@@ -75,7 +83,7 @@ class CheckValidity(GeoAlgorithm):
             self.tr('Error output')))
 
     def processAlgorithm(self, progress):
-        settings = QtCore.QSettings()
+        settings = QSettings()
         initial_method_setting = settings.value(settings_method_key, 1)
 
         method = self.getParameterValue(self.METHOD)
@@ -90,7 +98,7 @@ class CheckValidity(GeoAlgorithm):
         layer = dataobjects.getObjectFromUri(
             self.getParameterValue(self.INPUT_LAYER))
 
-        settings = QtCore.QSettings()
+        settings = QSettings()
         method = int(settings.value(settings_method_key, 1))
 
         valid_ouput = self.getOutputFromName(self.VALID_OUTPUT)
@@ -104,7 +112,7 @@ class CheckValidity(GeoAlgorithm):
         invalid_ouput = self.getOutputFromName(self.INVALID_OUTPUT)
         invalid_fields = layer.fields().toList() + [
             QgsField(name='_errors',
-                     type=QtCore.QVariant.String,
+                     type=QVariant.String,
                      len=255)]
         invalid_writer = invalid_ouput.getVectorWriter(
             invalid_fields,
@@ -115,7 +123,7 @@ class CheckValidity(GeoAlgorithm):
         error_ouput = self.getOutputFromName(self.ERROR_OUTPUT)
         error_fields = [
             QgsField(name='message',
-                     type=QtCore.QVariant.String,
+                     type=QVariant.String,
                      len=255)]
         error_writer = error_ouput.getVectorWriter(
             error_fields,
@@ -124,7 +132,7 @@ class CheckValidity(GeoAlgorithm):
         error_count = 0
 
         features = vector.features(layer)
-        total = 100.0 / len(features)
+        total = 100.0 / len(features) if len(features) > 0 else 1
         for current, inFeat in enumerate(features):
             geom = QgsGeometry(inFeat.geometry())
             attrs = inFeat.attributes()
