@@ -61,7 +61,7 @@ class DeleteColumn(GeoAlgorithm):
                                                                      layer.wkbType(), layer.crs())
 
         features = vector.features(layer)
-        total = 100.0 / len(features)
+        total = 100.0 / len(features) if len(features) > 0 else 1
 
         feat = QgsFeature()
         for current, f in enumerate(features):
