@@ -27,7 +27,7 @@ __revision__ = '$Format:%H$'
 
 import sys
 
-from PyQt4.QtCore import QVariant
+from qgis.PyQt.QtCore import QVariant
 from qgis.core import QgsFeature, QgsField
 from processing.core.GeoAlgorithm import GeoAlgorithm
 from processing.core.GeoAlgorithmExecutionException import GeoAlgorithmExecutionException
@@ -51,27 +51,31 @@ class FieldsPyculator(GeoAlgorithm):
     OUTPUT_LAYER = 'OUTPUT_LAYER'
     RESULT_VAR_NAME = 'value'
 
-    TYPE_NAMES = ['Integer', 'Float', 'String']
     TYPES = [QVariant.Int, QVariant.Double, QVariant.String]
 
     def defineCharacteristics(self):
-        self.name = 'Advanced Python field calculator'
-        self.group = 'Vector table tools'
+        self.name, self.i18n_name = self.trAlgorithm('Advanced Python field calculator')
+        self.group, self.i18n_group = self.trAlgorithm('Vector table tools')
+
+        self.type_names = [self.tr('Integer'),
+                           self.tr('Float'),
+                           self.tr('String')]
+
         self.addParameter(ParameterVector(self.INPUT_LAYER,
-            self.tr('Input layer'), [ParameterVector.VECTOR_TYPE_ANY], False))
+                                          self.tr('Input layer'), [ParameterVector.VECTOR_TYPE_ANY], False))
         self.addParameter(ParameterString(self.FIELD_NAME,
-            self.tr('Result field name'), 'NewField'))
+                                          self.tr('Result field name'), 'NewField'))
         self.addParameter(ParameterSelection(self.FIELD_TYPE,
-            self.tr('Field type'), self.TYPE_NAMES))
+                                             self.tr('Field type'), self.type_names))
         self.addParameter(ParameterNumber(self.FIELD_LENGTH,
-            self.tr('Field length'), 1, 255, 10))
+                                          self.tr('Field length'), 1, 255, 10))
         self.addParameter(ParameterNumber(self.FIELD_PRECISION,
-            self.tr('Field precision'), 0, 10, 0))
+                                          self.tr('Field precision'), 0, 10, 0))
         self.addParameter(ParameterString(self.GLOBAL,
-            self.tr('Global expression'), multiline=True, optional=True))
+                                          self.tr('Global expression'), multiline=True, optional=True))
         self.addParameter(ParameterString(self.FORMULA,
-            self.tr('Formula'), 'value = ', multiline=True))
-        self.addOutput(OutputVector(self.OUTPUT_LAYER, self.tr('Output layer')))
+                                          self.tr('Formula'), 'value = ', multiline=True))
+        self.addOutput(OutputVector(self.OUTPUT_LAYER, self.tr('Calculated')))
 
     def processAlgorithm(self, progress):
         fieldName = self.getParameterValue(self.FIELD_NAME)
@@ -84,12 +88,11 @@ class FieldsPyculator(GeoAlgorithm):
 
         layer = dataobjects.getObjectFromUri(
             self.getParameterValue(self.INPUT_LAYER))
-        provider = layer.dataProvider()
-        fields = provider.fields()
+        fields = layer.fields()
         fields.append(QgsField(fieldName, self.TYPES[fieldType], '',
-                      fieldLength, fieldPrecision))
-        writer = output.getVectorWriter(fields, provider.geometryType(),
-                layer.crs())
+                               fieldLength, fieldPrecision))
+        writer = output.getVectorWriter(fields, layer.wkbType(),
+                                        layer.crs())
         outFeat = QgsFeature()
         new_ns = {}
 
@@ -97,17 +100,17 @@ class FieldsPyculator(GeoAlgorithm):
         if globalExpression.strip() != '':
             try:
                 bytecode = compile(globalExpression, '<string>', 'exec')
-                exec bytecode in new_ns
+                exec(bytecode, new_ns)
             except:
                 raise GeoAlgorithmExecutionException(
-                    self.tr("FieldPyculator code execute error.Global code block can't be executed!\n%s\n%s" % (unicode(sys.exc_info()[0].__name__), unicode(sys.exc_info()[1]))))
+                    self.tr("FieldPyculator code execute error.Global code block can't be executed!\n%s\n%s") % (unicode(sys.exc_info()[0].__name__), unicode(sys.exc_info()[1])))
 
         # Replace all fields tags
-        fields = provider.fields()
+        fields = layer.fields()
         num = 0
         for field in fields:
             field_name = unicode(field.name())
-            replval = '__attr[' + str(num) + ']'
+            replval = '__attr[' + unicode(num) + ']'
             code = code.replace('<' + field_name + '>', replval)
             num += 1
 
@@ -123,14 +126,13 @@ class FieldsPyculator(GeoAlgorithm):
             bytecode = compile(code, '<string>', 'exec')
         except:
             raise GeoAlgorithmExecutionException(
-                self.tr("FieldPyculator code execute error.Field code block can't be executed!\n%s\n%s" % (unicode(sys.exc_info()[0].__name__), unicode(sys.exc_info()[1]))))
+                self.tr("FieldPyculator code execute error.Field code block can't be executed!\n%s\n%s") % (unicode(sys.exc_info()[0].__name__), unicode(sys.exc_info()[1])))
 
         # Run
         features = vector.features(layer)
-        nFeatures = len(features)
-        nElement = 1
-        for feat in features:
-            progress.setPercentage(int(100 * nElement / nFeatures))
+        total = 100.0 / len(features) if len(features) > 0 else 1
+        for current, feat in enumerate(features):
+            progress.setPercentage(int(current * total))
             attrs = feat.attributes()
             feat_id = feat.id()
 
@@ -151,17 +153,16 @@ class FieldsPyculator(GeoAlgorithm):
                 del new_ns[self.RESULT_VAR_NAME]
 
             # Exec
-            exec bytecode in new_ns
+            exec(bytecode, new_ns)
 
             # Check result
             if self.RESULT_VAR_NAME not in new_ns:
                 raise GeoAlgorithmExecutionException(
                     self.tr("FieldPyculator code execute error\n"
                             "Field code block does not return '%s1' variable! "
-                            "Please declare this variable in your code!" % self.RESULT_VAR_NAME))
+                            "Please declare this variable in your code!") % self.RESULT_VAR_NAME)
 
             # Write feature
-            nElement += 1
             outFeat.setGeometry(feat.geometry())
             attrs.append(new_ns[self.RESULT_VAR_NAME])
             outFeat.setAttributes(attrs)
