@@ -25,8 +25,18 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 __revision__ = '$Format:%H$'
 
-from PyQt4.QtCore import QVariant
-from qgis.core import QgsExpression, QgsExpressionContext, QgsExpressionContextUtils, QgsFeature, QgsField, QgsDistanceArea, QgsProject, GEO_NONE
+from qgis.PyQt.QtCore import QVariant
+from qgis.core import (
+    QgsExpression,
+    QgsExpressionContext,
+    QgsExpressionContextUtils,
+    QgsFeature,
+    QgsField,
+    QgsDistanceArea,
+    QgsProject,
+    QgsMapLayerRegistry,
+    GEO_NONE
+)
 from qgis.utils import iface
 from processing.core.GeoAlgorithm import GeoAlgorithm
 from processing.core.GeoAlgorithmExecutionException import GeoAlgorithmExecutionException
@@ -122,6 +132,14 @@ class FieldsCalculator(GeoAlgorithm):
             raise GeoAlgorithmExecutionException(
                 self.tr('Evaluation error: %s' % exp.evalErrorString()))
 
+        # add layer to registry to fix https://issues.qgis.org/issues/17300
+        # it is necessary only for aggregate expressions that verify that layer
+        # is registered
+        removeRegistryAfterEvaluation = False
+        if not QgsMapLayerRegistry.instance().mapLayer(layer.id()):
+            removeRegistryAfterEvaluation = True
+            QgsMapLayerRegistry.instance().addMapLayer(layer, addToLegend=False)
+
         outFeature = QgsFeature()
         outFeature.initAttributes(len(fields))
         outFeature.setFields(fields)
@@ -130,7 +148,7 @@ class FieldsCalculator(GeoAlgorithm):
         calculationSuccess = True
 
         features = vector.features(layer)
-        total = 100.0 / len(features)
+        total = 100.0 / len(features) if len(features) > 0 else 1
 
         rownum = 1
         for current, f in enumerate(features):
@@ -152,6 +170,11 @@ class FieldsCalculator(GeoAlgorithm):
             progress.setPercentage(int(current * total))
         del writer
 
+        # remove from registry if added for expression requirement
+        # see above comment about fix #17300
+        if removeRegistryAfterEvaluation:
+            QgsMapLayerRegistry.instance().removeMapLayer(layer)
+
         if not calculationSuccess:
             raise GeoAlgorithmExecutionException(
                 self.tr('An error occurred while evaluating the calculation '
