@@ -68,7 +68,7 @@ class Gridify(GeoAlgorithm):
             layer.pendingFields(), layer.wkbType(), layer.crs())
 
         features = vector.features(layer)
-        total = 100.0 / len(features)
+        total = 100.0 / len(features) if len(features) > 0 else 1
 
         for current, f in enumerate(features):
             geom = f.geometry()
