@@ -27,7 +27,7 @@ __revision__ = '$Format:%H$'
 
 from osgeo import gdal
 from qgis.core import QGis, QgsFields, QgsField, QgsFeature, QgsPoint, QgsGeometry
-from PyQt4.QtCore import QVariant
+from qgis.PyQt.QtCore import QVariant
 from processing.core.GeoAlgorithm import GeoAlgorithm
 from processing.core.parameters import ParameterRaster
 from processing.core.parameters import ParameterVector
@@ -78,7 +78,7 @@ class PointsFromPolygons(GeoAlgorithm):
         pointId = 0
 
         features = vector.features(layer)
-        total = 100.0 / len(features)
+        total = 100.0 / len(features) if len(features) > 0 else 1
         for current, f in enumerate(features):
             geom = f.geometry()
             bbox = geom.boundingBox()
