@@ -25,9 +25,11 @@ __copyright__ = '(C) 2014, Alexander Bruy'
 
 __revision__ = '$Format:%H$'
 
+import os
 import random
 
-from PyQt4.QtCore import QVariant
+from qgis.PyQt.QtGui import QIcon
+from qgis.PyQt.QtCore import QVariant
 from qgis.core import (QGis, QgsGeometry, QgsRectangle, QgsFeature, QgsFields,
                        QgsField, QgsSpatialIndex, QgsPoint)
 from qgis.utils import iface
@@ -39,6 +41,8 @@ from processing.core.parameters import ParameterNumber
 from processing.core.outputs import OutputVector
 from processing.tools import vector
 
+pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
+
 
 class RandomPointsExtent(GeoAlgorithm):
 
@@ -47,6 +51,9 @@ class RandomPointsExtent(GeoAlgorithm):
     MIN_DISTANCE = 'MIN_DISTANCE'
     OUTPUT = 'OUTPUT'
 
+    def getIcon(self):
+        return QIcon(os.path.join(pluginPath, 'images', 'ftools', 'random_points.png'))
+
     def defineCharacteristics(self):
         self.name, self.i18n_name = self.trAlgorithm('Random points in extent')
         self.group, self.i18n_group = self.trAlgorithm('Vector creation tools')
@@ -80,7 +87,7 @@ class RandomPointsExtent(GeoAlgorithm):
         nPoints = 0
         nIterations = 0
         maxIterations = pointCount * 200
-        total = 100.0 / pointCount
+        total = 100.0 / pointCount if pointCount > 0 else 1
 
         index = QgsSpatialIndex()
         points = dict()
