@@ -25,9 +25,11 @@ __copyright__ = '(C) 2014, Alexander Bruy'
 
 __revision__ = '$Format:%H$'
 
+import os
 import random
 
-from PyQt4.QtCore import QVariant
+from qgis.PyQt.QtGui import QIcon
+from qgis.PyQt.QtCore import QVariant
 from qgis.core import (QGis, QgsGeometry, QgsFields, QgsField, QgsSpatialIndex,
                        QgsPoint, QgsFeature, QgsFeatureRequest)
 
@@ -38,6 +40,8 @@ from processing.core.parameters import ParameterNumber
 from processing.core.outputs import OutputVector
 from processing.tools import dataobjects, vector
 
+pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
+
 
 class RandomPointsLayer(GeoAlgorithm):
 
@@ -46,6 +50,9 @@ class RandomPointsLayer(GeoAlgorithm):
     MIN_DISTANCE = 'MIN_DISTANCE'
     OUTPUT = 'OUTPUT'
 
+    def getIcon(self):
+        return QIcon(os.path.join(pluginPath, 'images', 'ftools', 'random_points.png'))
+
     def defineCharacteristics(self):
         self.name, self.i18n_name = self.trAlgorithm('Random points in layer bounds')
         self.group, self.i18n_group = self.trAlgorithm('Vector creation tools')
@@ -75,7 +82,7 @@ class RandomPointsLayer(GeoAlgorithm):
         nPoints = 0
         nIterations = 0
         maxIterations = pointCount * 200
-        total = 100.0 / pointCount
+        total = 100.0 / pointCount if pointCount > 0 else 1
 
         index = QgsSpatialIndex()
         points = dict()
