@@ -25,9 +25,11 @@ __copyright__ = '(C) 2014, Alexander Bruy'
 
 __revision__ = '$Format:%H$'
 
+import os
 import random
 
-from PyQt4.QtCore import QVariant
+from qgis.PyQt.QtGui import QIcon
+from qgis.PyQt.QtCore import QVariant
 from qgis.core import (QGis, QgsFields, QgsField, QgsDistanceArea, QgsGeometry,
                        QgsSpatialIndex, QgsPoint, QgsFeature)
 
@@ -39,6 +41,8 @@ from processing.core.parameters import ParameterSelection
 from processing.core.outputs import OutputVector
 from processing.tools import dataobjects, vector
 
+pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
+
 
 class RandomPointsPolygonsFixed(GeoAlgorithm):
 
@@ -48,6 +52,9 @@ class RandomPointsPolygonsFixed(GeoAlgorithm):
     STRATEGY = 'STRATEGY'
     OUTPUT = 'OUTPUT'
 
+    def getIcon(self):
+        return QIcon(os.path.join(pluginPath, 'images', 'ftools', 'random_points.png'))
+
     def defineCharacteristics(self):
         self.name, self.i18n_name = self.trAlgorithm('Random points inside polygons (fixed)')
         self.group, self.i18n_group = self.trAlgorithm('Vector creation tools')
@@ -95,7 +102,7 @@ class RandomPointsPolygonsFixed(GeoAlgorithm):
             nPoints = 0
             nIterations = 0
             maxIterations = pointCount * 200
-            total = 100.0 / pointCount
+            total = 100.0 / pointCount if pointCount > 0 else 1
 
             random.seed()
 
