@@ -25,8 +25,11 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 __revision__ = '$Format:%H$'
 
+import os
 import random
 
+from qgis.PyQt.QtGui import QIcon
+
 from processing.core.GeoAlgorithm import GeoAlgorithm
 from processing.core.GeoAlgorithmExecutionException import GeoAlgorithmExecutionException
 from processing.core.parameters import ParameterSelection
@@ -35,6 +38,8 @@ from processing.core.parameters import ParameterNumber
 from processing.core.outputs import OutputVector
 from processing.tools import dataobjects
 
+pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
+
 
 class RandomSelection(GeoAlgorithm):
 
@@ -43,6 +48,9 @@ class RandomSelection(GeoAlgorithm):
     METHOD = 'METHOD'
     NUMBER = 'NUMBER'
 
+    def getIcon(self):
+        return QIcon(os.path.join(pluginPath, 'images', 'ftools', 'random_selection.png'))
+
     def defineCharacteristics(self):
         self.allowOnlyOpenedLayers = True
         self.name, self.i18n_name = self.trAlgorithm('Random selection')
@@ -81,7 +89,7 @@ class RandomSelection(GeoAlgorithm):
                             "different value and try again."))
             value = int(round(value / 100.0, 4) * featureCount)
 
-        selran = random.sample(xrange(0, featureCount), value)
+        selran = random.sample(xrange(featureCount), value)
 
         layer.setSelectedFeatures(selran)
         self.setOutputValue(self.OUTPUT, filename)
