@@ -25,9 +25,12 @@ __copyright__ = '(C) 2014, Alexander Bruy'
 
 __revision__ = '$Format:%H$'
 
+import os
 import math
 
-from PyQt4.QtCore import QVariant
+from qgis.PyQt.QtGui import QIcon
+from qgis.PyQt.QtCore import QVariant
+
 from qgis.core import QGis, QgsRectangle, QgsFields, QgsField, QgsFeature, QgsGeometry, QgsPoint
 from qgis.utils import iface
 
@@ -37,6 +40,8 @@ from processing.core.parameters import ParameterNumber
 from processing.core.parameters import ParameterSelection
 from processing.core.outputs import OutputVector
 
+pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
+
 
 class VectorGrid(GeoAlgorithm):
 
@@ -46,21 +51,24 @@ class VectorGrid(GeoAlgorithm):
     TYPE = 'TYPE'
     OUTPUT = 'OUTPUT'
 
-    TYPES = ['Output grid as polygons',
-             'Output grid as lines'
-             ]
+    def getIcon(self):
+        return QIcon(os.path.join(pluginPath, 'images', 'ftools', 'vector_grid.png'))
 
     def defineCharacteristics(self):
-        self.name = 'Vector grid'
-        self.group = 'Vector creation tools'
+        self.name, self.i18n_name = self.trAlgorithm('Vector grid')
+        self.group, self.i18n_group = self.trAlgorithm('Vector creation tools')
+
+        self.types = [self.tr('Output grid as polygons'),
+                      self.tr('Output grid as lines')]
+
         self.addParameter(ParameterExtent(self.EXTENT,
-            self.tr('Grid extent')))
+                                          self.tr('Grid extent')))
         self.addParameter(ParameterNumber(self.STEP_X,
-            self.tr('X spacing'), 0.0, 1000000000.0, 0.0001))
+                                          self.tr('X spacing'), 0.0, 1000000000.0, 0.0001))
         self.addParameter(ParameterNumber(self.STEP_Y,
-            self.tr('Y spacing'), 0.0, 1000000000.0, 0.0001))
+                                          self.tr('Y spacing'), 0.0, 1000000000.0, 0.0001))
         self.addParameter(ParameterSelection(self.TYPE,
-            self.tr('Grid type'), self.TYPES))
+                                             self.tr('Grid type'), self.types))
 
         self.addOutput(OutputVector(self.OUTPUT, self.tr('Grid')))
 
@@ -117,7 +125,7 @@ class VectorGrid(GeoAlgorithm):
                 if int(math.fmod(count, count_update)) == 0:
                     progress.setPercentage(int(count / count_max * 50))
 
-            progress.setPercentage( 50 )
+            progress.setPercentage(50)
             # counters for progressbar - update every 5%
             count = 0
             count_max = (bbox.xMaximum() - bbox.xMinimum()) / xSpace
