@@ -27,6 +27,8 @@ __revision__ = '$Format:%H$'
 
 import os
 
+from qgis.PyQt.QtGui import QIcon
+
 from processing.core.GeoAlgorithm import GeoAlgorithm
 from processing.core.parameters import ParameterVector
 from processing.core.parameters import ParameterTableField
@@ -34,6 +36,8 @@ from processing.core.outputs import OutputDirectory
 from processing.tools import dataobjects, vector
 from processing.tools.system import mkdir
 
+pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
+
 
 class VectorSplit(GeoAlgorithm):
 
@@ -41,6 +45,9 @@ class VectorSplit(GeoAlgorithm):
     FIELD = 'FIELD'
     OUTPUT = 'OUTPUT'
 
+    def getIcon(self):
+        return QIcon(os.path.join(pluginPath, 'images', 'ftools', 'split_layer.png'))
+
     def defineCharacteristics(self):
         self.name, self.i18n_name = self.trAlgorithm('Split vector layer')
         self.group, self.i18n_group = self.trAlgorithm('Vector general tools')
@@ -67,7 +74,7 @@ class VectorSplit(GeoAlgorithm):
         crs = layer.crs()
         geomType = layer.wkbType()
 
-        total = 100.0 / len(uniqueValues)
+        total = 100.0 / len(uniqueValues) if len(uniqueValues) > 0 else 1
 
         for current, i in enumerate(uniqueValues):
             fName = u'{0}_{1}.shp'.format(baseName, unicode(i).strip())
