@@ -27,35 +27,35 @@ __revision__ = '$Format:%H$'
 
 import os
 
-from PyQt4.QtGui import QIcon
+from qgis.PyQt.QtGui import QIcon
 
 from processing.core.AlgorithmProvider import AlgorithmProvider
 from processing.core.ProcessingConfig import ProcessingConfig, Setting
 from processing.core.ProcessingLog import ProcessingLog
 
-from TauDEMAlgorithm import TauDEMAlgorithm
-from TauDEMMultifileAlgorithm import TauDEMMultifileAlgorithm
-from TauDEMUtils import TauDEMUtils
+from .TauDEMAlgorithm import TauDEMAlgorithm
+from .TauDEMMultifileAlgorithm import TauDEMMultifileAlgorithm
+from .TauDEMUtils import TauDEMUtils
 
-from peukerdouglas import PeukerDouglas
-from slopearea import SlopeArea
-from lengtharea import LengthArea
-from dropanalysis import DropAnalysis
-from dinfdistdown import DinfDistDown
-from dinfdistup import DinfDistUp
-from gridnet import GridNet
-from dinftranslimaccum import DinfTransLimAccum
-from dinftranslimaccum2 import DinfTransLimAccum2
+from .peukerdouglas import PeukerDouglas
+from .slopearea import SlopeArea
+from .lengtharea import LengthArea
+from .dropanalysis import DropAnalysis
+from .dinfdistdown import DinfDistDown
+from .dinfdistup import DinfDistUp
+from .gridnet import GridNet
+from .dinftranslimaccum import DinfTransLimAccum
+from .dinftranslimaccum2 import DinfTransLimAccum2
 
-from peukerdouglas_multi import PeukerDouglasMulti
-from slopearea_multi import SlopeAreaMulti
-from lengtharea_multi import LengthAreaMulti
-from dropanalysis_multi import DropAnalysisMulti
-from dinfdistdown_multi import DinfDistDownMulti
-from dinfdistup_multi import DinfDistUpMulti
-from gridnet_multi import GridNetMulti
-from dinftranslimaccum_multi import DinfTransLimAccumMulti
-from dinftranslimaccum2_multi import DinfTransLimAccum2Multi
+from .peukerdouglas_multi import PeukerDouglasMulti
+from .slopearea_multi import SlopeAreaMulti
+from .lengtharea_multi import LengthAreaMulti
+from .dropanalysis_multi import DropAnalysisMulti
+from .dinfdistdown_multi import DinfDistDownMulti
+from .dinfdistup_multi import DinfDistUpMulti
+from .gridnet_multi import GridNetMulti
+from .dinftranslimaccum_multi import DinfTransLimAccumMulti
+from .dinftranslimaccum2_multi import DinfTransLimAccum2Multi
 
 
 class TauDEMAlgorithmProvider(AlgorithmProvider):
@@ -71,32 +71,32 @@ class TauDEMAlgorithmProvider(AlgorithmProvider):
         return 'taudem'
 
     def getIcon(self):
-        return QIcon(os.path.dirname(__file__) + '/../../images/taudem.png')
+        return QIcon(os.path.dirname(__file__) + '/../../images/taudem.svg')
 
     def initializeSettings(self):
         AlgorithmProvider.initializeSettings(self)
 
         ProcessingConfig.addSetting(Setting(self.getDescription(),
-            TauDEMUtils.TAUDEM_FOLDER,
-            self.tr('TauDEM command line tools folder'),
-            TauDEMUtils.taudemPath()))
+                                            TauDEMUtils.TAUDEM_FOLDER,
+                                            self.tr('TauDEM command line tools folder'),
+                                            TauDEMUtils.taudemPath(), valuetype=Setting.FOLDER))
         ProcessingConfig.addSetting(Setting(self.getDescription(),
-            TauDEMUtils.TAUDEM_MULTIFILE_FOLDER,
-            self.tr('TauDEM multifile command line tools folder'),
-            TauDEMUtils.taudemMultifilePath()))
+                                            TauDEMUtils.TAUDEM_MULTIFILE_FOLDER,
+                                            self.tr('TauDEM multifile command line tools folder'),
+                                            TauDEMUtils.taudemMultifilePath(), valuetype=Setting.FOLDER))
         ProcessingConfig.addSetting(Setting(self.getDescription(),
-            TauDEMUtils.TAUDEM_USE_SINGLEFILE,
-            self.tr('Enable singlefile TauDEM tools'), True))
+                                            TauDEMUtils.TAUDEM_USE_SINGLEFILE,
+                                            self.tr('Enable singlefile TauDEM tools'), True))
         ProcessingConfig.addSetting(Setting(self.getDescription(),
-            TauDEMUtils.TAUDEM_USE_MULTIFILE,
-            self.tr('Enable multifile TauDEM tools'), False))
+                                            TauDEMUtils.TAUDEM_USE_MULTIFILE,
+                                            self.tr('Enable multifile TauDEM tools'), False))
         ProcessingConfig.addSetting(Setting(self.getDescription(),
-            TauDEMUtils.MPIEXEC_FOLDER,
-            self.tr('MPICH2/OpenMPI bin directory'),
-            TauDEMUtils.mpiexecPath()))
+                                            TauDEMUtils.MPIEXEC_FOLDER,
+                                            self.tr('MPICH2/OpenMPI bin directory'),
+                                            TauDEMUtils.mpiexecPath(), valuetype=Setting.FOLDER))
         ProcessingConfig.addSetting(Setting(self.getDescription(),
-            TauDEMUtils.MPI_PROCESSES,
-            self.tr('Number of MPI parallel processes to use'), 2))
+                                            TauDEMUtils.MPI_PROCESSES,
+                                            self.tr('Number of MPI parallel processes to use'), 2))
 
     def unload(self):
         AlgorithmProvider.unload(self)
@@ -158,7 +158,7 @@ class TauDEMAlgorithmProvider(AlgorithmProvider):
                 self.algs.append(alg)
             else:
                 ProcessingLog.addToLog(ProcessingLog.LOG_ERROR,
-                    self.tr('Could not open TauDEM algorithm: %s' % descriptionFile))
-        except Exception, e:
+                                       self.tr('Could not open TauDEM algorithm: %s' % descriptionFile))
+        except Exception as e:
             ProcessingLog.addToLog(ProcessingLog.LOG_ERROR,
-                self.tr('Could not open TauDEM algorithm %s:\n%s' % (descriptionFile, str(e))))
+                                   self.tr('Could not open TauDEM algorithm %s:\n%s' % (descriptionFile, unicode(e))))
