@@ -27,7 +27,7 @@ __revision__ = '$Format:%H$'
 
 import os
 
-from PyQt4.QtGui import QIcon
+from qgis.PyQt.QtGui import QIcon
 
 from processing.core.GeoAlgorithm import GeoAlgorithm
 from processing.core.ProcessingConfig import ProcessingConfig
@@ -40,7 +40,7 @@ from processing.core.parameters import ParameterNumber
 from processing.core.parameters import ParameterSelection
 from processing.core.outputs import OutputFile
 
-from TauDEMUtils import TauDEMUtils
+from .TauDEMUtils import TauDEMUtils
 
 
 class DropAnalysis(GeoAlgorithm):
@@ -60,34 +60,34 @@ class DropAnalysis(GeoAlgorithm):
     STEPS = ['Logarithmic', 'Linear']
 
     def getIcon(self):
-        return QIcon(os.path.dirname(__file__) + '/../../images/taudem.png')
+        return QIcon(os.path.dirname(__file__) + '/../../images/taudem.svg')
 
     def defineCharacteristics(self):
-        self.name = 'Stream Drop Analysis'
+        self.name, self.i18n_name = self.trAlgorithm('Stream Drop Analysis')
         self.cmdName = 'dropanalysis'
-        self.group = 'Stream Network Analysis tools'
+        self.group, self.i18n_group = self.trAlgorithm('Stream Network Analysis tools')
 
         self.addParameter(ParameterRaster(self.D8_CONTRIB_AREA_GRID,
-            self.tr('D8 Contributing Area Grid'), False))
+                                          self.tr('D8 Contributing Area Grid'), False))
         self.addParameter(ParameterRaster(self.D8_FLOW_DIR_GRID,
-            self.tr('D8 Flow Direction Grid'), False))
+                                          self.tr('D8 Flow Direction Grid'), False))
         self.addParameter(ParameterRaster(self.PIT_FILLED_GRID,
-            self.tr('Pit Filled Elevation Grid'), False))
+                                          self.tr('Pit Filled Elevation Grid'), False))
         self.addParameter(ParameterRaster(self.ACCUM_STREAM_SOURCE_GRID,
-            self.tr('Accumulated Stream Source Grid'), False))
+                                          self.tr('Accumulated Stream Source Grid'), False))
         self.addParameter(ParameterVector(self.OUTLETS_SHAPE,
-            self.tr('Outlets Shapefile'),
-            [ParameterVector.VECTOR_TYPE_POINT], False))
+                                          self.tr('Outlets Shapefile'),
+                                          [ParameterVector.VECTOR_TYPE_POINT], False))
         self.addParameter(ParameterNumber(self.MIN_TRESHOLD,
-            self.tr('Minimum Threshold'), 0, None, 5))
+                                          self.tr('Minimum Threshold'), 0, None, 5))
         self.addParameter(ParameterNumber(self.MAX_THRESHOLD,
-            self.tr('Maximum Threshold'), 0, None, 500))
+                                          self.tr('Maximum Threshold'), 0, None, 500))
         self.addParameter(ParameterNumber(self.TRESHOLD_NUM,
-            self.tr('Number of Threshold Values'), 0, None, 10))
+                                          self.tr('Number of Threshold Values'), 0, None, 10))
         self.addParameter(ParameterSelection(self.STEP_TYPE,
-            self.tr('Spacing for Threshold Values'), self.STEPS, 0))
+                                             self.tr('Spacing for Threshold Values'), self.STEPS, 0))
         self.addOutput(OutputFile(self.DROP_ANALYSIS_FILE,
-            self.tr('D-Infinity Drop to Stream Grid')))
+                                  self.tr('D-Infinity Drop to Stream Grid')))
 
     def processAlgorithm(self, progress):
         commands = []
@@ -100,7 +100,7 @@ class DropAnalysis(GeoAlgorithm):
                         'correct number before running TauDEM algorithms.'))
 
         commands.append('-n')
-        commands.append(str(processNum))
+        commands.append(unicode(processNum))
         commands.append(os.path.join(TauDEMUtils.taudemPath(), self.cmdName))
         commands.append('-ad8')
         commands.append(self.getParameterValue(self.D8_CONTRIB_AREA_GRID))
@@ -113,10 +113,10 @@ class DropAnalysis(GeoAlgorithm):
         commands.append('-o')
         commands.append(self.getParameterValue(self.OUTLETS_SHAPE))
         commands.append('-par')
-        commands.append(str(self.getParameterValue(self.MIN_TRESHOLD)))
-        commands.append(str(self.getParameterValue(self.MAX_THRESHOLD)))
-        commands.append(str(self.getParameterValue(self.TRESHOLD_NUM)))
-        commands.append(str(self.getParameterValue(self.STEPS)))
+        commands.append(unicode(self.getParameterValue(self.MIN_TRESHOLD)))
+        commands.append(unicode(self.getParameterValue(self.MAX_THRESHOLD)))
+        commands.append(unicode(self.getParameterValue(self.TRESHOLD_NUM)))
+        commands.append(unicode(self.getParameterValue(self.STEPS)))
         commands.append('-drp')
         commands.append(self.getOutputValue(self.DROP_ANALYSIS_FILE))
 
