@@ -26,7 +26,7 @@ __copyright__ = '(C) 2012, Alexander Bruy'
 __revision__ = '$Format:%H$'
 
 import os
-from PyQt4.QtGui import QIcon
+from qgis.PyQt.QtGui import QIcon
 from processing.core.GeoAlgorithm import GeoAlgorithm
 from processing.core.ProcessingConfig import ProcessingConfig
 from processing.core.GeoAlgorithmExecutionException import \
@@ -35,7 +35,7 @@ from processing.core.parameters import ParameterRaster
 from processing.core.parameters import ParameterNumber
 from processing.core.outputs import OutputRaster
 
-from TauDEMUtils import TauDEMUtils
+from .TauDEMUtils import TauDEMUtils
 
 
 class PeukerDouglas(GeoAlgorithm):
@@ -48,24 +48,24 @@ class PeukerDouglas(GeoAlgorithm):
     STREAM_SOURCE_GRID = 'STREAM_SOURCE_GRID'
 
     def getIcon(self):
-        return QIcon(os.path.dirname(__file__) + '/../../images/taudem.png')
+        return QIcon(os.path.dirname(__file__) + '/../../images/taudem.svg')
 
     def defineCharacteristics(self):
-        self.name = 'Peuker Douglas'
+        self.name, self.i18n_name = self.trAlgorithm('Peuker Douglas')
         self.cmdName = 'peukerdouglas'
-        self.group = 'Stream Network Analysis tools'
+        self.group, self.i18n_group = self.trAlgorithm('Stream Network Analysis tools')
 
         self.addParameter(ParameterRaster(self.ELEVATION_GRID,
-            self.tr('Elevation Grid'), False))
+                                          self.tr('Elevation Grid'), False))
         self.addParameter(ParameterNumber(self.CENTER_WEIGHT,
-            self.tr('Center Smoothing Weight'), 0, None, 0.4))
+                                          self.tr('Center Smoothing Weight'), 0, None, 0.4))
         self.addParameter(ParameterNumber(self.SIDE_WEIGHT,
-            self.tr('Side Smoothing Weight'), 0, None, 0.1))
+                                          self.tr('Side Smoothing Weight'), 0, None, 0.1))
         self.addParameter(ParameterNumber(self.DIAGONAL_WEIGHT,
-            self.tr('Diagonal Smoothing Weight'), 0, None, 0.05))
+                                          self.tr('Diagonal Smoothing Weight'), 0, None, 0.05))
 
         self.addOutput(OutputRaster(self.STREAM_SOURCE_GRID,
-            self.tr('Stream Source Grid')))
+                                    self.tr('Stream Source Grid')))
 
     def processAlgorithm(self, progress):
         commands = []
@@ -78,14 +78,14 @@ class PeukerDouglas(GeoAlgorithm):
                         'correct number before running TauDEM algorithms.'))
 
         commands.append('-n')
-        commands.append(str(processNum))
+        commands.append(unicode(processNum))
         commands.append(os.path.join(TauDEMUtils.taudemPath(), self.cmdName))
         commands.append('-fel')
         commands.append(self.getParameterValue(self.ELEVATION_GRID))
         commands.append('-par')
-        commands.append(str(self.getParameterValue(self.CENTER_WEIGHT)))
-        commands.append(str(self.getParameterValue(self.SIDE_WEIGHT)))
-        commands.append(str(self.getParameterValue(self.DIAGONAL_WEIGHT)))
+        commands.append(unicode(self.getParameterValue(self.CENTER_WEIGHT)))
+        commands.append(unicode(self.getParameterValue(self.SIDE_WEIGHT)))
+        commands.append(unicode(self.getParameterValue(self.DIAGONAL_WEIGHT)))
         commands.append('-ss')
         commands.append(self.getOutputValue(self.STREAM_SOURCE_GRID))
 
