@@ -28,6 +28,21 @@ __revision__ = '$Format:%H$'
 
 class GeoAlgorithmExecutionException(Exception):
 
-    def __init__(self, msg):
+    def __init__(self, msg, stack=None, cause=None):
         Exception.__init__(self)
         self.msg = msg
+        self.stack = stack
+        self.cause = cause
+
+    def __str__(self):
+        msg = self.msg.split(u'\n')
+        msg = u'  | ' + u'\n  | '.join(msg)
+
+        try:
+            stack = u'\n'.join(self.stack)
+        except TypeError:
+            stack = repr(self.stack)
+        stack = stack.split(u'\n')
+        stack = u'    ' + u'\n    '.join(stack)
+
+        return u'\n\n Message:\n{}\n\n Stack:\n\n{}'.format(msg, stack)
