@@ -20,10 +20,13 @@
 __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
+
 # This will get replaced with a git SHA1 when you do a git archive
+
 __revision__ = '$Format:%H$'
 
-class ProcessingResults():
+
+class ProcessingResults:
 
     results = []
 
@@ -36,13 +39,8 @@ class ProcessingResults():
         return ProcessingResults.results
 
 
-class Result():
+class Result:
 
     def __init__(self, name, filename):
         self.name = name
         self.filename = filename
-
-
-
-
-
