@@ -20,10 +20,16 @@
 __author__ = 'Victor Olaya'
 __date__ = 'April 2013'
 __copyright__ = '(C) 2013, Victor Olaya'
+
 # This will get replaced with a git SHA1 when you do a git archive
+
 __revision__ = '$Format:%H$'
 
-class SilentProgress():
+
+class SilentProgress(object):
+
+    def error(self, msg):
+        print msg
 
     def setText(self, text):
         pass
@@ -31,14 +37,17 @@ class SilentProgress():
     def setPercentage(self, i):
         pass
 
-    def setInfo(self, _):
+    def setInfo(self, msg):
         pass
 
-    def setCommand(self, _):
+    def setCommand(self, msg):
         pass
 
-    def setDebugInfo(self, _):
+    def setDebugInfo(self, msg):
         pass
 
-    def setConsoleInfo(self, _):
+    def setConsoleInfo(self, msg):
+        pass
+
+    def close(self):
         pass
