@@ -25,8 +25,13 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 __revision__ = '$Format:%H$'
 
-from PyQt4.QtGui import QWidget, QMenu, QAction, QCursor, QInputDialog
+import os
 
+from qgis.PyQt import uic
+from qgis.PyQt.QtWidgets import QMenu, QAction, QInputDialog
+from qgis.PyQt.QtGui import QCursor
+
+from qgis.gui import QgsMessageBar
 from qgis.core import QgsRasterLayer, QgsVectorLayer
 from qgis.utils import iface
 
@@ -34,14 +39,18 @@ from processing.gui.RectangleMapTool import RectangleMapTool
 from processing.core.parameters import ParameterRaster
 from processing.core.parameters import ParameterVector
 from processing.core.parameters import ParameterMultipleInput
+from processing.core.ProcessingConfig import ProcessingConfig
 from processing.tools import dataobjects
 
-from processing.ui.ui_widgetBaseSelector import Ui_Form
+pluginPath = os.path.split(os.path.dirname(__file__))[0]
+WIDGET, BASE = uic.loadUiType(
+    os.path.join(pluginPath, 'ui', 'widgetBaseSelector.ui'))
 
-class ExtentSelectionPanel(QWidget, Ui_Form):
 
-    def __init__(self, dialog, alg, default):
-        QWidget.__init__(self)
+class ExtentSelectionPanel(BASE, WIDGET):
+
+    def __init__(self, dialog, alg, default=None):
+        super(ExtentSelectionPanel, self).__init__(None)
         self.setupUi(self)
 
         self.dialog = dialog
@@ -58,6 +67,18 @@ class ExtentSelectionPanel(QWidget, Ui_Form):
         self.tool = RectangleMapTool(canvas)
         self.tool.rectangleCreated.connect(self.updateExtent)
 
+        if default:
+            tokens = unicode(default).split(',')
+            if len(tokens) == 4:
+                try:
+                    float(tokens[0])
+                    float(tokens[1])
+                    float(tokens[2])
+                    float(tokens[3])
+                    self.leText.setText(unicode(default))
+                except:
+                    pass
+
     def canUseAutoExtent(self):
         for param in self.params:
             if isinstance(param, (ParameterRaster, ParameterVector)):
@@ -100,10 +121,10 @@ class ExtentSelectionPanel(QWidget, Ui_Form):
             if param.value:
                 if isinstance(param, (ParameterRaster, ParameterVector)):
                     if isinstance(param.value, (QgsRasterLayer,
-                                  QgsVectorLayer)):
+                                                QgsVectorLayer)):
                         layer = param.value
                     else:
-                        layer = dataobjects.getObjectFromUri(param.value)
+                        layer = dataobjects.getObject(param.value)
                     if layer:
                         found = True
                         self.addToRegion(layer, first)
@@ -111,7 +132,7 @@ class ExtentSelectionPanel(QWidget, Ui_Form):
                 elif isinstance(param, ParameterMultipleInput):
                     layers = param.value.split(';')
                     for layername in layers:
-                        layer = dataobjects.getObjectFromUri(layername, first)
+                        layer = dataobjects.getObject(layername)
                         if layer:
                             found = True
                             self.addToRegion(layer, first)
@@ -140,16 +161,27 @@ class ExtentSelectionPanel(QWidget, Ui_Form):
     def useLayerExtent(self):
         CANVAS_KEY = 'Use canvas extent'
         extentsDict = {}
-        extentsDict[CANVAS_KEY] = iface.mapCanvas().extent()
+        extentsDict[CANVAS_KEY] = {"extent": iface.mapCanvas().extent(),
+                                   "authid": iface.mapCanvas().mapSettings().destinationCrs().authid()}
         extents = [CANVAS_KEY]
         layers = dataobjects.getAllLayers()
         for layer in layers:
-            extents.append(layer.name())
-            extentsDict[layer.name()] = layer.extent()
+            authid = layer.crs().authid()
+            if ProcessingConfig.getSetting(ProcessingConfig.SHOW_CRS_DEF) \
+                    and authid is not None:
+                layerName = u'{} [{}]'.format(layer.name(), authid)
+            else:
+                layerName = layer.name()
+            extents.append(layerName)
+            extentsDict[layerName] = {"extent": layer.extent(), "authid": authid}
         (item, ok) = QInputDialog.getItem(self, self.tr('Select extent'),
-                self.tr('Use extent from'), extents, False)
+                                          self.tr('Use extent from'), extents, False)
         if ok:
-            self.setValueFromRect(extentsDict[item])
+            self.setValueFromRect(extentsDict[item]["extent"])
+            if extentsDict[item]["authid"] != iface.mapCanvas().mapSettings().destinationCrs().authid():
+                iface.messageBar().pushMessage(self.tr("Warning"),
+                                               self.tr("The projection of the chosen layer is not the same as canvas projection! The selected extent might not be what was intended."),
+                                               QgsMessageBar.WARNING, 8)
 
     def selectOnCanvas(self):
         canvas = iface.mapCanvas()
@@ -173,7 +205,7 @@ class ExtentSelectionPanel(QWidget, Ui_Form):
         self.dialog.activateWindow()
 
     def getValue(self):
-        if str(self.leText.text()).strip() != '':
+        if unicode(self.leText.text()).strip() != '':
             return unicode(self.leText.text())
         else:
             return self.getMinCoveringExtent()
