@@ -26,7 +26,7 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 __revision__ = '$Format:%H$'
 
-from PyQt4.QtCore import QCoreApplication
+from qgis.PyQt.QtCore import QCoreApplication
 import os
 import re
 import json
@@ -40,9 +40,10 @@ exps = [(r"\*(.*?)\*", r"<i>\1</i>"),
         ("``(.*?)``", r'<FONT FACE="courier">\1</FONT>'),
         ("(.*?)\n==+\n+?", r'<h2>\1</h2>'),
         ("(.*?)\n--+\n+?", r'<h3>\1</h3>'),
-        (r"::(\s*\n(\s*\n)*((\s+).*?\n)(((\4).*?\n)|(\s*\n))*)",r"<pre>\1</pre>"),
+        (r"::(\s*\n(\s*\n)*((\s+).*?\n)(((\4).*?\n)|(\s*\n))*)", r"<pre>\1</pre>"),
         ("\n+", "</p><p>")]
 
+
 def getHtmlFromRstFile(rst):
     if not os.path.exists(rst):
         return None
@@ -54,6 +55,7 @@ def getHtmlFromRstFile(rst):
         s = p.sub(replace, s)
     return s
 
+
 def getHtmlFromHelpFile(alg, helpFile):
     if not os.path.exists(helpFile):
         return None
@@ -64,6 +66,7 @@ def getHtmlFromHelpFile(alg, helpFile):
     except:
         return None
 
+
 def getHtmlFromDescriptionsDict(alg, descriptions):
     s = tr('<html><body><h2>Algorithm description</h2>\n')
     s += '<p>' + getDescription(ALG_DESC, descriptions) + '</p>\n'
@@ -82,11 +85,13 @@ def getHtmlFromDescriptionsDict(alg, descriptions):
     s += '</body></html>'
     return s
 
+
 def getDescription(name, descriptions):
     if name in descriptions:
         return unicode(descriptions[name]).replace("\n", "<br>")
     else:
         return ''
 
+
 def tr(string):
     return QCoreApplication.translate('Help2Html', string)
