@@ -27,21 +27,25 @@ __revision__ = '$Format:%H$'
 
 import os
 
-from PyQt4.QtCore import QSettings
-from PyQt4.QtGui import QWidget, QIcon, QFileDialog
+from qgis.PyQt import uic
+from qgis.PyQt.QtCore import QSettings
+from qgis.PyQt.QtGui import QIcon
+from qgis.PyQt.QtWidgets import QFileDialog
 from processing.tools import dataobjects
 
-from processing.ui.ui_widgetLayerSelector import Ui_Form
+pluginPath = os.path.split(os.path.dirname(__file__))[0]
+WIDGET, BASE = uic.loadUiType(
+    os.path.join(pluginPath, 'ui', 'widgetLayerSelector.ui'))
 
 
-class InputLayerSelectorPanel(QWidget, Ui_Form):
+class InputLayerSelectorPanel(BASE, WIDGET):
 
     def __init__(self, options, param):
-        QWidget.__init__(self)
+        super(InputLayerSelectorPanel, self).__init__(None)
         self.setupUi(self)
 
         self.btnIterate.setIcon(
-            QIcon(os.path.dirname(__file__) + '/../images/iterate.png'))
+            QIcon(os.path.join(pluginPath, 'images', 'iterate.png')))
         self.btnIterate.hide()
 
         self.param = param
@@ -64,7 +68,7 @@ class InputLayerSelectorPanel(QWidget, Ui_Form):
             path = ''
 
         filename = QFileDialog.getOpenFileName(self, self.tr('Select file'),
-            path, self.tr('All files (*.*);;') + self.param.getFileFilter())
+                                               path, self.tr('All files (*.*);;') + self.param.getFileFilter())
         if filename:
             settings.setValue('/Processing/LastInputPath',
                               os.path.dirname(unicode(filename)))
@@ -72,6 +76,5 @@ class InputLayerSelectorPanel(QWidget, Ui_Form):
             self.cmbText.addItem(filename, filename)
             self.cmbText.setCurrentIndex(self.cmbText.count() - 1)
 
-
     def getValue(self):
         return self.cmbText.itemData(self.cmbText.currentIndex())
