@@ -10,8 +10,7 @@
     Email                : volayaf at gmail dot com
                            otb at c-s dot fr (CS SI)
     Contributors         : Victor Olaya
-                           Alexia Mondot (CS SI) - managing the new parameter
-                           ParameterMultipleExternalInput
+
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -33,8 +32,12 @@ __revision__ = '$Format:%H$'
 import os
 import locale
 
-from PyQt4.QtCore import QCoreApplication, QVariant
-from PyQt4.QtGui import QWidget, QLayout, QVBoxLayout, QHBoxLayout, QToolButton, QIcon, QLabel, QCheckBox, QComboBox, QLineEdit, QPlainTextEdit
+from qgis.core import QgsMapLayerRegistry, QgsMapLayer
+
+from qgis.PyQt import uic
+from qgis.PyQt.QtCore import QCoreApplication, QVariant
+from qgis.PyQt.QtWidgets import QWidget, QLayout, QVBoxLayout, QHBoxLayout, QToolButton, QLabel, QCheckBox, QComboBox, QLineEdit, QPlainTextEdit
+from qgis.PyQt.QtGui import QIcon
 
 from processing.core.ProcessingConfig import ProcessingConfig
 
@@ -47,14 +50,17 @@ from processing.gui.NumberInputPanel import NumberInputPanel
 from processing.gui.ExtentSelectionPanel import ExtentSelectionPanel
 from processing.gui.FileSelectionPanel import FileSelectionPanel
 from processing.gui.CrsSelectionPanel import CrsSelectionPanel
+from processing.gui.PointSelectionPanel import PointSelectionPanel
 from processing.gui.GeometryPredicateSelectionPanel import \
     GeometryPredicateSelectionPanel
+from processing.gui.ListMultiselectWidget import ListMultiSelectWidget
 
 from processing.core.parameters import ParameterRaster
 from processing.core.parameters import ParameterVector
 from processing.core.parameters import ParameterTable
 from processing.core.parameters import ParameterBoolean
 from processing.core.parameters import ParameterTableField
+from processing.core.parameters import ParameterTableMultipleField
 from processing.core.parameters import ParameterSelection
 from processing.core.parameters import ParameterFixedTable
 from processing.core.parameters import ParameterRange
@@ -64,6 +70,7 @@ from processing.core.parameters import ParameterExtent
 from processing.core.parameters import ParameterFile
 from processing.core.parameters import ParameterCrs
 from processing.core.parameters import ParameterString
+from processing.core.parameters import ParameterPoint
 from processing.core.parameters import ParameterGeometryPredicate
 
 from processing.core.outputs import OutputRaster
@@ -72,15 +79,17 @@ from processing.core.outputs import OutputVector
 
 from processing.tools import dataobjects
 
-from processing.ui.ui_widgetParametersPanel import Ui_Form
+pluginPath = os.path.split(os.path.dirname(__file__))[0]
+WIDGET, BASE = uic.loadUiType(
+    os.path.join(pluginPath, 'ui', 'widgetParametersPanel.ui'))
 
 
-class ParametersPanel(QWidget, Ui_Form):
+class ParametersPanel(BASE, WIDGET):
 
     NOT_SELECTED = QCoreApplication.translate('ParametersPanel', '[Not selected]')
 
     def __init__(self, parent, alg):
-        QWidget.__init__(self)
+        super(ParametersPanel, self).__init__(None)
         self.setupUi(self)
 
         self.grpAdvanced.hide()
@@ -99,15 +108,83 @@ class ParametersPanel(QWidget, Ui_Form):
 
         self.initWidgets()
 
-    def initWidgets(self):
-        #tooltips = self.alg.getParameterDescriptions()
+    def layerAdded(self, layer):
+        if layer.type() == QgsMapLayer.VectorLayer:
+            for param in self.alg.parameters:
+                if param.hidden:
+                    continue
+                if isinstance(param, ParameterVector):
+                    if dataobjects.canUseVectorLayer(layer, param.shapetype):
+                        widget = self.valueItems[param.name]
+                        if isinstance(widget, InputLayerSelectorPanel) and hasattr(widget.cmbText, 'addItem'):
+                            widget = widget.cmbText
+                        widget.addItem(self.getExtendedLayerName(layer), layer)
+        elif layer.type() == QgsMapLayer.RasterLayer and dataobjects.canUseRasterLayer(layer):
+            for param in self.alg.parameters:
+                if param.hidden:
+                    continue
+                if isinstance(param, ParameterRaster):
+                    widget = self.valueItems[param.name].cmbText
+                    widget.addItem(self.getExtendedLayerName(layer), layer)
 
+        self.updateMultipleInputs()
+
+    def layersWillBeRemoved(self, layers):
+        for layer in layers:
+            self.layerRemoved(layer)
+
+    def layerRemoved(self, layer):
+        layer = QgsMapLayerRegistry.instance().mapLayer(layer)
+        widget = None
+        if layer.type() == QgsMapLayer.VectorLayer:
+            for param in self.alg.parameters:
+                if param.hidden:
+                    continue
+                if isinstance(param, ParameterVector):
+                    widget = self.valueItems[param.name]
+                    if isinstance(widget, InputLayerSelectorPanel):
+                        widget = widget.cmbText
+
+                        if widget is not None:
+                            idx = widget.findData(layer)
+                            if idx != -1:
+                                widget.removeItem(idx)
+                            widget = None
+
+        elif layer.type() == QgsMapLayer.RasterLayer:
+            for param in self.alg.parameters:
+                if param.hidden:
+                    continue
+                if isinstance(param, ParameterRaster):
+                    widget = self.valueItems[param.name].cmbText
+
+                    if widget is not None:
+                        idx = widget.findData(layer)
+                        if idx != -1:
+                            widget.removeItem(idx)
+                        widget = None
+
+        self.updateMultipleInputs()
+
+    def updateMultipleInputs(self):
+        for param in self.alg.parameters:
+            if isinstance(param, ParameterMultipleInput) and param.datatype != ParameterMultipleInput.TYPE_FILE:
+                if param.datatype == ParameterMultipleInput.TYPE_RASTER:
+                    options = dataobjects.getRasterLayers(sorting=False)
+                elif param.datatype == ParameterMultipleInput.TYPE_VECTOR_ANY:
+                    options = dataobjects.getVectorLayers(sorting=False)
+                else:
+                    options = dataobjects.getVectorLayers([param.datatype], sorting=False)
+                opts = [self.getExtendedLayerName(opt) for opt in options]
+                widget = self.valueItems[param.name]
+                widget.updateForOptions(opts)
+
+    def initWidgets(self):
         # If there are advanced parameters — show corresponding groupbox
         for param in self.alg.parameters:
             if param.isAdvanced:
                 self.grpAdvanced.show()
                 break
-
         # Create widgets and put them in layouts
         for param in self.alg.parameters:
             if param.hidden:
@@ -116,6 +193,8 @@ class ParametersPanel(QWidget, Ui_Form):
             desc = param.description
             if isinstance(param, ParameterExtent):
                 desc += self.tr(' (xmin, xmax, ymin, ymax)')
+            if isinstance(param, ParameterPoint):
+                desc += self.tr(' (x, y)')
             try:
                 if param.optional:
                     desc += self.tr(' [optional]')
@@ -132,8 +211,7 @@ class ParametersPanel(QWidget, Ui_Form):
                 layout.setMargin(0)
                 layout.addWidget(widget)
                 button = QToolButton()
-                icon = QIcon(
-                    os.path.dirname(__file__) + '/../images/iterate.png')
+                icon = QIcon(os.path.join(pluginPath, 'images', 'iterate.png'))
                 button.setIcon(icon)
                 button.setToolTip(self.tr('Iterate over this layer'))
                 button.setCheckable(True)
@@ -143,11 +221,8 @@ class ParametersPanel(QWidget, Ui_Form):
                 widget = QWidget()
                 widget.setLayout(layout)
 
-            #~ if param.name in tooltips.keys():
-                #~ tooltip = tooltips[param.name]
-            #~ else:
-                #~ tooltip = param.description
-            #~ widget.setToolTip(tooltip)
+            tooltips = self.alg.getParameterDescriptions()
+            widget.setToolTip(tooltips.get(param.name, param.description))
 
             if isinstance(param, ParameterBoolean):
                 widget.setText(desc)
@@ -187,6 +262,22 @@ class ParametersPanel(QWidget, Ui_Form):
                 self.checkBoxes[output.name] = check
             self.valueItems[output.name] = widget
 
+            if isinstance(output, OutputVector):
+                if output.base_input in self.dependentItems:
+                    items = self.dependentItems[output.base_input]
+                else:
+                    items = []
+                    self.dependentItems[output.base_input] = items
+                items.append(output)
+
+                base_input = self.alg.getParameterFromName(output.base_input)
+                if isinstance(base_input, ParameterVector):
+                    layers = dataobjects.getVectorLayers(base_input.shapetype)
+                else:
+                    layers = dataobjects.getTables()
+                if len(layers) > 0:
+                    output.base_layer = layers[0]
+
     def buttonToggled(self, value):
         if value:
             sender = self.sender()
@@ -232,7 +323,7 @@ class ParametersPanel(QWidget, Ui_Form):
                 for layer in layers:
                     items.append((self.getExtendedLayerName(layer), layer))
                 # if already set, put first in list
-                for i,(name,layer) in enumerate(items):
+                for i, (name, layer) in enumerate(items):
                     if layer and layer.source() == param.value:
                         items.insert(0, items.pop(i))
                 item = InputLayerSelectorPanel(items, param)
@@ -254,7 +345,7 @@ class ParametersPanel(QWidget, Ui_Form):
                 for layer in layers:
                     items.append((layer.name(), layer))
                 # if already set, put first in list
-                for i,(name,layer) in enumerate(items):
+                for i, (name, layer) in enumerate(items):
                     if layer and layer.source() == param.value:
                         items.insert(0, items.pop(i))
                 item = InputLayerSelectorPanel(items, param)
@@ -264,27 +355,31 @@ class ParametersPanel(QWidget, Ui_Form):
                 item.setChecked(True)
             else:
                 item.setChecked(False)
-        elif isinstance(param, ParameterTableField):
-            item = QComboBox()
+        elif isinstance(param, ParameterTableField) or isinstance(param, ParameterTableMultipleField):
+            if isinstance(param, ParameterTableMultipleField):
+                item = ListMultiSelectWidget()
+            else:
+                item = QComboBox()
             if param.parent in self.dependentItems:
                 items = self.dependentItems[param.parent]
             else:
                 items = []
                 self.dependentItems[param.parent] = items
-            items.append(param.name)
+            items.append(param)
             parent = self.alg.getParameterFromName(param.parent)
             if isinstance(parent, ParameterVector):
                 layers = dataobjects.getVectorLayers(parent.shapetype)
             else:
                 layers = dataobjects.getTables()
             if len(layers) > 0:
-                if param.optional:
+                if param.optional and isinstance(param, ParameterTableField):
                     item.addItem(self.tr('[not set]'))
                 item.addItems(self.getFields(layers[0], param.datatype))
         elif isinstance(param, ParameterSelection):
             item = QComboBox()
             item.addItems(param.options)
-            item.setCurrentIndex(param.default)
+            if param.default:
+                item.setCurrentIndex(param.default)
         elif isinstance(param, ParameterFixedTable):
             item = FixedTablePanel(param)
         elif isinstance(param, ParameterRange):
@@ -301,15 +396,15 @@ class ParametersPanel(QWidget, Ui_Form):
                     options = dataobjects.getVectorLayers(sorting=False)
                 else:
                     options = dataobjects.getVectorLayers([param.datatype], sorting=False)
-                opts = []
-                for opt in options:
-                    opts.append(self.getExtendedLayerName(opt))
+                opts = [self.getExtendedLayerName(opt) for opt in options]
                 item = MultipleInputPanel(opts)
         elif isinstance(param, ParameterNumber):
             item = NumberInputPanel(param.default, param.min, param.max,
                                     param.isInteger)
         elif isinstance(param, ParameterExtent):
             item = ExtentSelectionPanel(self.parent, self.alg, param.default)
+        elif isinstance(param, ParameterPoint):
+            item = PointSelectionPanel(self.parent, param.default)
         elif isinstance(param, ParameterCrs):
             item = CrsSelectionPanel(param.default)
         elif isinstance(param, ParameterString):
@@ -318,12 +413,14 @@ class ParametersPanel(QWidget, Ui_Form):
                 verticalLayout.setSizeConstraint(
                     QLayout.SetDefaultConstraint)
                 textEdit = QPlainTextEdit()
-                textEdit.setPlainText(param.default)
+                if param.default:
+                    textEdit.setPlainText(param.default)
                 verticalLayout.addWidget(textEdit)
                 item = textEdit
             else:
                 item = QLineEdit()
-                item.setText(str(param.default))
+                if param.default:
+                    item.setText(unicode(param.default))
         elif isinstance(param, ParameterGeometryPredicate):
             item = GeometryPredicateSelectionPanel(param.enabledPredicates)
             if param.left:
@@ -339,10 +436,12 @@ class ParametersPanel(QWidget, Ui_Form):
                 widget.currentIndexChanged.connect(item.onRightLayerChange)
                 item.rightLayer = widget.itemData(widget.currentIndex())
             item.updatePredicates()
-            item.setValue(param.default)
+            if param.default:
+                item.setValue(param.default)
         else:
             item = QLineEdit()
-            item.setText(str(param.default))
+            if param.default:
+                item.setText(unicode(param.default))
 
         return item
 
@@ -353,21 +452,30 @@ class ParametersPanel(QWidget, Ui_Form):
         if sender.name not in self.dependentItems:
             return
         layer = sender.itemData(sender.currentIndex())
+        if not layer:
+            return
         children = self.dependentItems[sender.name]
         for child in children:
-            widget = self.valueItems[child]
-            widget.clear()
-            if self.alg.getParameterFromName(child).optional:
-                widget.addItem(self.tr('[not set]'))
-            widget.addItems(self.getFields(layer,
-                            self.alg.getParameterFromName(child).datatype))
+            if (isinstance(child, ParameterTableField) or isinstance(
+                    child, ParameterTableMultipleField)):
+                widget = self.valueItems[child.name]
+                widget.clear()
+                if (self.alg.getParameterFromName(child.name).optional and
+                        not isinstance(child, ParameterTableMultipleField)):
+                    widget.addItem(self.tr('[not set]'))
+                widget.addItems(
+                    self.getFields(layer, self.alg.getParameterFromName(
+                                   child.name).datatype))
+            if isinstance(child, OutputVector):
+                child.base_layer = layer
 
     def getFields(self, layer, datatype):
         fieldTypes = []
         if datatype == ParameterTableField.DATA_TYPE_STRING:
             fieldTypes = [QVariant.String]
         elif datatype == ParameterTableField.DATA_TYPE_NUMBER:
-            fieldTypes = [QVariant.Int, QVariant.Double]
+            fieldTypes = [QVariant.Int, QVariant.Double, QVariant.LongLong,
+                          QVariant.UInt, QVariant.ULongLong]
 
         fieldNames = set()
         for field in layer.pendingFields():
@@ -377,7 +485,12 @@ class ParametersPanel(QWidget, Ui_Form):
 
     def somethingDependsOnThisParameter(self, parent):
         for param in self.alg.parameters:
-            if isinstance(param, ParameterTableField):
+            if isinstance(param, (ParameterTableField,
+                                  ParameterTableMultipleField)):
                 if param.parent == parent.name:
                     return True
+        for output in self.alg.outputs:
+            if isinstance(output, OutputVector):
+                if output.base_layer == parent.name:
+                    return True
         return False
