@@ -17,6 +17,7 @@
 ***************************************************************************
 """
 
+
 __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
@@ -26,17 +27,18 @@ __copyright__ = '(C) 2012, Victor Olaya'
 __revision__ = '$Format:%H$'
 
 import os
-from PyQt4.QtGui import QApplication
-from PyQt4.QtCore import QCoreApplication
+import traceback
+from qgis.PyQt.QtWidgets import QApplication
+from qgis.PyQt.QtCore import QCoreApplication
 from qgis.core import QgsMapLayerRegistry
 
 from processing.core.ProcessingConfig import ProcessingConfig
 from processing.core.ProcessingResults import ProcessingResults
+from processing.core.ProcessingLog import ProcessingLog
+from processing.core.SilentProgress import SilentProgress
 
 from processing.gui.ResultsDialog import ResultsDialog
 from processing.gui.RenderingStyles import RenderingStyles
-from processing.gui.MessageDialog import MessageDialog
-from processing.gui.SilentProgress import SilentProgress
 
 from processing.core.outputs import OutputRaster
 from processing.core.outputs import OutputVector
@@ -45,6 +47,7 @@ from processing.core.outputs import OutputHTML
 
 from processing.tools import dataobjects
 
+
 def handleAlgorithmResults(alg, progress=None, showResults=True):
     wrongLayers = []
     htmlResults = False
@@ -58,32 +61,38 @@ def handleAlgorithmResults(alg, progress=None, showResults=True):
             continue
         if isinstance(out, (OutputRaster, OutputVector, OutputTable)):
             try:
-                if out.value.startswith('memory:'):
-                    layer = out.memoryLayer
-                    QgsMapLayerRegistry.instance().addMapLayers([layer])
+                if hasattr(out, "layer") and out.layer is not None:
+                    out.layer.setLayerName(out.description)
+                    QgsMapLayerRegistry.instance().addMapLayers([out.layer])
                 else:
                     if ProcessingConfig.getSetting(
                             ProcessingConfig.USE_FILENAME_AS_LAYER_NAME):
                         name = os.path.basename(out.value)
                     else:
                         name = out.description
+
+                    isRaster = True if isinstance(out, OutputRaster) else False
                     dataobjects.load(out.value, name, alg.crs,
-                            RenderingStyles.getStyle(alg.commandLineName(),
-                            out.name))
-            except Exception, e:
-                wrongLayers.append(out)
+                                     RenderingStyles.getStyle(alg.commandLineName(), out.name),
+                                     isRaster)
+            except Exception:
+                ProcessingLog.addToLog(ProcessingLog.LOG_ERROR,
+                                       "Error loading result layer:\n" + traceback.format_exc())
+                wrongLayers.append(out.description)
         elif isinstance(out, OutputHTML):
             ProcessingResults.addResult(out.description, out.value)
             htmlResults = True
         i += 1
+
+    QApplication.restoreOverrideCursor()
     if wrongLayers:
-        QApplication.restoreOverrideCursor()
-        dlg = MessageDialog()
-        dlg.setTitle(QCoreApplication.translate('Postprocessing', 'Problem loading output layers'))
-        dlg.setMessage(alg.getPostProcessingErrorMessage(wrongLayers))
-        dlg.exec_()
+        msg = "The following layers were not correctly generated.<ul>"
+        msg += "".join(["<li>%s</li>" % lay for lay in wrongLayers]) + "</ul>"
+        msg += "You can check the log messages to find more information about the execution of the algorithm"
+        progress.error(msg)
 
     if showResults and htmlResults and not wrongLayers:
-        QApplication.restoreOverrideCursor()
         dlg = ResultsDialog()
         dlg.exec_()
+
+    return len(wrongLayers) == 0
