@@ -28,9 +28,10 @@ __revision__ = '$Format:%H$'
 import os
 import codecs
 
-from PyQt4 import uic
-from PyQt4.QtCore import QUrl
-from PyQt4.QtGui import QIcon, QStyle, QTreeWidgetItem
+from qgis.PyQt import uic
+from qgis.PyQt.QtCore import QUrl
+from qgis.PyQt.QtGui import QIcon
+from qgis.PyQt.QtWidgets import QStyle, QTreeWidgetItem
 
 from processing.core.ProcessingResults import ProcessingResults
 
