@@ -27,11 +27,12 @@ __revision__ = '$Format:%H$'
 
 import os
 
-from PyQt4.QtCore import Qt, QSettings
-from PyQt4.QtGui import QFont, QColor, QShortcut, QKeySequence
+from qgis.PyQt.QtCore import Qt, QSettings
+from qgis.PyQt.QtGui import QFont, QColor, QKeySequence
+from qgis.PyQt.QtWidgets import QShortcut
 from qgis.core import QgsApplication
 
-from PyQt4.Qsci import QsciScintilla, QsciLexerPython, QsciAPIs
+from qgis.PyQt.Qsci import QsciScintilla, QsciLexerPython, QsciAPIs
 
 from processing.gui.LexerR import LexerR
 
@@ -59,7 +60,7 @@ class ScriptEdit(QsciScintilla):
         font = QFont()
         font.setFamily('Courier')
         font.setFixedPitch(True)
-        font.setPointSize(10)
+        font.setPointSize(20)
         self.setFont(font)
         self.setMarginsFont(font)
 
@@ -105,10 +106,14 @@ class ScriptEdit(QsciScintilla):
         self.setAutoCompletionThreshold(2)
         self.setAutoCompletionSource(QsciScintilla.AcsAPIs)
 
+        self.setFonts(10)
+
+    def setFonts(self, size):
+
         # Load font from Python console settings
         settings = QSettings()
         fontName = settings.value('pythonConsole/fontfamilytext', 'Monospace')
-        fontSize = int(settings.value('pythonConsole/fontsize', 10))
+        fontSize = int(settings.value('pythonConsole/fontsize', size))
 
         self.defaultFont = QFont(fontName)
         self.defaultFont.setFixedPitch(True)
@@ -142,7 +147,7 @@ class ScriptEdit(QsciScintilla):
 
         # Use Ctrl+Space for autocompletion
         self.shortcutAutocomplete = QShortcut(QKeySequence(Qt.CTRL
-                + Qt.Key_Space), self)
+                                                           + Qt.Key_Space), self)
         self.shortcutAutocomplete.setContext(Qt.WidgetShortcut)
         self.shortcutAutocomplete.activated.connect(self.autoComplete)
 
@@ -188,12 +193,12 @@ class ScriptEdit(QsciScintilla):
 
             settings = QSettings()
             useDefaultAPI = bool(settings.value('pythonConsole/preloadAPI',
-                                 True))
+                                                True))
             if useDefaultAPI:
                 # Load QGIS API shipped with Python console
                 self.api.loadPrepared(
                     os.path.join(QgsApplication.pkgDataPath(),
-                    'python', 'qsci_apis', 'pyqgis.pap'))
+                                 'python', 'qsci_apis', 'pyqgis.pap'))
             else:
                 # Load user-defined API files
                 apiPaths = settings.value('pythonConsole/userAPI', [])
