@@ -26,21 +26,25 @@ __copyright__ = '(C) 2012, Victor Olaya'
 __revision__ = '$Format:%H$'
 
 import os
-from PyQt4 import QtGui
+from qgis.PyQt.QtGui import QIcon
 from processing.gui.ToolboxAction import ToolboxAction
 from processing.modeler.ModelerDialog import ModelerDialog
+from processing.core.alglist import algList
+
+pluginPath = os.path.split(os.path.dirname(__file__))[0]
+
 
 class CreateNewModelAction(ToolboxAction):
 
     def __init__(self):
-        self.name = self.tr('Create new model', 'CreateNewModelAction')
-        self.group = self.tr('Tools', 'CreateNewModelAction')
+        self.name, self.i18n_name = self.trAction('Create new model')
+        self.group, self.i18n_group = self.trAction('Tools')
 
     def getIcon(self):
-        return QtGui.QIcon(os.path.dirname(__file__) + '/../images/model.png')
+        return QIcon(os.path.join(pluginPath, 'images', 'model.png'))
 
     def execute(self):
         dlg = ModelerDialog()
         dlg.exec_()
         if dlg.update:
-            self.toolbox.updateProvider('model')
+            algList.reloadProvider('model')
