@@ -30,8 +30,8 @@ __revision__ = '$Format:%H$'
 
 
 # import functions for easier access
-import installer
-from installer import initPluginInstaller
+from . import installer
+from .installer import initPluginInstaller  # NOQA
 
 
 def instance():
