@@ -1,4 +1,29 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
+
+"""
+***************************************************************************
+    generate_test_mask_image.py
+    ---------------------
+    Date                 : February 2015
+    Copyright            : (C) 2015 by Nyall Dawson
+    Email                : nyall dot dawson at gmail dot com
+***************************************************************************
+*                                                                         *
+*   This program is free software; you can redistribute it and/or modify  *
+*   it under the terms of the GNU General Public License as published by  *
+*   the Free Software Foundation; either version 2 of the License, or     *
+*   (at your option) any later version.                                   *
+*                                                                         *
+***************************************************************************
+"""
+
+__author__ = 'Nyall Dawson'
+__date__ = 'February 2015'
+__copyright__ = '(C) 2015, Nyall Dawson'
+# This will get replaced with a git SHA1 when you do a git archive
+__revision__ = '$Format:%H$'
+
 
 # Generates (or updates) a unit test image mask, which is used to specify whether
 # a pixel in the control image should be checked (black pixel in mask) or not (white
@@ -8,8 +33,7 @@
 import os
 import sys
 import argparse
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from PyQt4.QtGui import QImage, QColor, qRed, qBlue, qGreen, qAlpha, qRgb
 import struct
 import urllib2
 import glob
@@ -29,7 +53,7 @@ def colorDiff(c1, c2):
 
 
 def imageFromPath(path):
-    if (path[:7] == 'http://' or path[:7] == 'file://'):
+    if (path[:7] == 'http://' or path[:8] == 'https://' or path[:7] == 'file://'):
         #fetch remote image
         data = urllib2.urlopen(path).read()
         image = QImage()
