@@ -25,8 +25,8 @@
 
 /**
  * \ingroup networkanalysis
- * \class QgsEdgeProperter
- * \brief QgsEdgeProperter is a strategy pattern.
+ * \class QgsArcProperter
+ * \brief QgsArcProperter is a strategy pattern.
  * You can use it for customize arc property. For example look at QgsDistanceArcProperter or src/plugins/roadgraph/speedproperter.h
  */
 class ANALYSIS_EXPORT QgsArcProperter
@@ -37,6 +37,8 @@ class ANALYSIS_EXPORT QgsArcProperter
      */
     QgsArcProperter() {}
 
+    virtual ~QgsArcProperter() {}
+
     /**
      * QgsGraphDirector call this method for fetching attribute from source layer
      * \return required attributes list
