@@ -16,7 +16,7 @@
 #include "qgsalignraster.h"
 
 #include <gdalwarper.h>
-#include <ogr_spatialref.h>
+#include <ogr_srs_api.h>
 #include <cpl_conv.h>
 #include <limits>
 
@@ -193,7 +193,7 @@ bool QgsAlignRaster::setParametersFromRaster( const RasterInfo& rasterInfo, cons
     else
     {
       mGridOffsetX = customGridOffset.x();
-      mGridOffsetY = customGridOffset.x();
+      mGridOffsetY = customGridOffset.y();
     }
   }
   else
@@ -253,8 +253,7 @@ bool QgsAlignRaster::checkInputParameters()
   }
 
   mXSize = mYSize = 0;
-  for ( int i = 0; i < 6; ++i )
-    mGeoTransform[i] = 0;
+  std::fill_n( mGeoTransform, 6, 0 );
 
   double finalExtent[4] = { 0, 0, 0, 0 };
 
@@ -432,7 +431,7 @@ bool QgsAlignRaster::createAndWarp( const Item& raster )
   GDALDatasetH hSrcDS = GDALOpen( raster.inputFilename.toLocal8Bit().constData(), GA_ReadOnly );
   if ( !hSrcDS )
   {
-    mErrorMessage = QObject::tr( "Unable to open input file: " ) + raster.inputFilename;
+    mErrorMessage = QObject::tr( "Unable to open input file: %1" ).arg( raster.inputFilename );
     return false;
   }
 
@@ -448,13 +447,13 @@ bool QgsAlignRaster::createAndWarp( const Item& raster )
   if ( !hDstDS )
   {
     GDALClose( hSrcDS );
-    mErrorMessage = QObject::tr( "Unable to create output file: " ) + raster.outputFilename;
+    mErrorMessage = QObject::tr( "Unable to create output file: %1" ).arg( raster.outputFilename );
     return false;
   }
 
   // Write out the projection definition.
   GDALSetProjection( hDstDS, mCrsWkt.toAscii().constData() );
-  GDALSetGeoTransform( hDstDS, ( double* )mGeoTransform );
+  GDALSetGeoTransform( hDstDS, mGeoTransform );
 
   // Copy the color table, if required.
   GDALColorTableH hCT = GDALGetRasterColorTable( GDALGetRasterBand( hSrcDS, 1 ) );
@@ -477,7 +476,7 @@ bool QgsAlignRaster::createAndWarp( const Item& raster )
     psWarpOptions->panDstBands[i] = i + 1;
   }
 
-  psWarpOptions->eResampleAlg = ( GDALResampleAlg ) raster.resampleMethod;
+  psWarpOptions->eResampleAlg = static_cast< GDALResampleAlg >( raster.resampleMethod );
 
   // our progress function
   psWarpOptions->pfnProgress = _progress;
