@@ -23,6 +23,11 @@
 #include <QMap>
 #include <QString>
 
+class QgsRasterBlock;
+
+/** \ingroup analysis
+ * \class QgsRasterCalcNode
+ */
 class ANALYSIS_EXPORT QgsRasterCalcNode
 {
   public:
@@ -31,7 +36,8 @@ class ANALYSIS_EXPORT QgsRasterCalcNode
     {
       tOperator = 1,
       tNumber,
-      tRasterRef
+      tRasterRef,
+      tMatrix
     };
 
     //! possible operators
@@ -58,11 +64,14 @@ class ANALYSIS_EXPORT QgsRasterCalcNode
       opAND,
       opOR,
       opSIGN,       // change sign
+      opLOG,
+      opLOG10,
       opNONE,
     };
 
     QgsRasterCalcNode();
     QgsRasterCalcNode( double number );
+    QgsRasterCalcNode( QgsRasterMatrix* matrix );
     QgsRasterCalcNode( Operator op, QgsRasterCalcNode* left, QgsRasterCalcNode* right );
     QgsRasterCalcNode( const QString& rasterName );
     ~QgsRasterCalcNode();
@@ -73,8 +82,19 @@ class ANALYSIS_EXPORT QgsRasterCalcNode
     void setLeft( QgsRasterCalcNode* left ) { delete mLeft; mLeft = left; }
     void setRight( QgsRasterCalcNode* right ) { delete mRight; mRight = right; }
 
-    /**Calculates result (might be real matrix or single number)*/
-    bool calculate( QMap<QString, QgsRasterMatrix*>& rasterData, QgsRasterMatrix& result ) const;
+    /** Calculates result of raster calculation (might be real matrix or single number).
+     * @param rasterData input raster data references, map of raster name to raster data block
+     * @param result destination raster matrix for calculation results
+     * @param row optional row number to calculate for calculating result by rows, or -1 to
+     * calculate entire result
+     * @note added in QGIS 2.10
+     * @note not available in Python bindings
+     */
+    bool calculate( QMap<QString, QgsRasterBlock* >& rasterData, QgsRasterMatrix& result, int row = -1 ) const;
+
+    /** @deprecated use method which accepts QgsRasterBlocks instead
+     */
+    Q_DECL_DEPRECATED bool calculate( QMap<QString, QgsRasterMatrix*>& rasterData, QgsRasterMatrix& result ) const;
 
     static QgsRasterCalcNode* parseRasterCalcString( const QString& str, QString& parserErrorMsg );
 
@@ -84,7 +104,11 @@ class ANALYSIS_EXPORT QgsRasterCalcNode
     QgsRasterCalcNode* mRight;
     double mNumber;
     QString mRasterName;
+    QgsRasterMatrix* mMatrix;
     Operator mOperator;
+
+    QgsRasterCalcNode( const QgsRasterCalcNode& rh );
+    QgsRasterCalcNode& operator=( const QgsRasterCalcNode& rh );
 };
 
 
