@@ -1,3 +1,17 @@
+/***************************************************************************
+    qgstransectsample.cpp
+    ---------------------
+    begin                : July 2013
+    copyright            : (C) 2013 by Marco Hugentobler
+    email                : marco dot hugentobler at sourcepole dot ch
+ ***************************************************************************
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ ***************************************************************************/
 #include "qgstransectsample.h"
 #include "qgsdistancearea.h"
 #include "qgsgeometry.h"
@@ -15,10 +29,21 @@
 QgsTransectSample::QgsTransectSample( QgsVectorLayer* strataLayer, const QString& strataIdAttribute, const QString& minDistanceAttribute, const QString& nPointsAttribute, DistanceUnits minDistUnits,
                                       QgsVectorLayer* baselineLayer, bool shareBaseline, const QString& baselineStrataId, const QString& outputPointLayer,
                                       const QString& outputLineLayer, const QString& usedBaselineLayer, double minTransectLength,
-                                      double baselineBufferDistance, double baselineSimplificationTolerance ): mStrataLayer( strataLayer ),
-    mStrataIdAttribute( strataIdAttribute ), mMinDistanceAttribute( minDistanceAttribute ), mNPointsAttribute( nPointsAttribute ), mBaselineLayer( baselineLayer ), mShareBaseline( shareBaseline ),
-    mBaselineStrataId( baselineStrataId ), mOutputPointLayer( outputPointLayer ), mOutputLineLayer( outputLineLayer ), mUsedBaselineLayer( usedBaselineLayer ),
-    mMinDistanceUnits( minDistUnits ), mMinTransectLength( minTransectLength ), mBaselineBufferDistance( baselineBufferDistance ), mBaselineSimplificationTolerance( baselineSimplificationTolerance )
+                                      double baselineBufferDistance, double baselineSimplificationTolerance )
+    : mStrataLayer( strataLayer )
+    , mStrataIdAttribute( strataIdAttribute )
+    , mMinDistanceAttribute( minDistanceAttribute )
+    , mNPointsAttribute( nPointsAttribute )
+    , mBaselineLayer( baselineLayer )
+    , mShareBaseline( shareBaseline )
+    , mBaselineStrataId( baselineStrataId )
+    , mOutputPointLayer( outputPointLayer )
+    , mOutputLineLayer( outputLineLayer )
+    , mUsedBaselineLayer( usedBaselineLayer )
+    , mMinDistanceUnits( minDistUnits )
+    , mMinTransectLength( minTransectLength )
+    , mBaselineBufferDistance( baselineBufferDistance )
+    , mBaselineSimplificationTolerance( baselineSimplificationTolerance )
 {
 }
 
