@@ -1,3 +1,17 @@
+/***************************************************************************
+    qgstransectsample.h
+    ---------------------
+    begin                : July 2013
+    copyright            : (C) 2013 by Marco Hugentobler
+    email                : marco dot hugentobler at sourcepole dot ch
+ ***************************************************************************
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ ***************************************************************************/
 #ifndef QGSTRANSECTSAMPLE_H
 #define QGSTRANSECTSAMPLE_H
 
@@ -12,7 +26,8 @@ class QgsVectorLayer;
 class QgsPoint;
 class QProgressDialog;
 
-/** A class for the creation of transect sample lines based on a set of strata polygons and baselines*/
+/** \ingroup analysis
+ * A class for the creation of transect sample lines based on a set of strata polygons and baselines*/
 class ANALYSIS_EXPORT QgsTransectSample
 {
   public:
