@@ -19,6 +19,7 @@
 #include "qgsgeometry.h"
 #include "qgsvectordataprovider.h"
 #include "qgsvectorlayer.h"
+#include "qmath.h"
 #include "gdal.h"
 #include "cpl_string.h"
 #include <QProgressDialog>
@@ -30,25 +31,22 @@
 #define TO8F(x) QFile::encodeName( x ).constData()
 #endif
 
-QgsZonalStatistics::QgsZonalStatistics( QgsVectorLayer* polygonLayer, const QString& rasterFile, const QString& attributePrefix, int rasterBand )
+QgsZonalStatistics::QgsZonalStatistics( QgsVectorLayer* polygonLayer, const QString& rasterFile, const QString& attributePrefix, int rasterBand, const Statistics& stats )
     : mRasterFilePath( rasterFile )
     , mRasterBand( rasterBand )
     , mPolygonLayer( polygonLayer )
     , mAttributePrefix( attributePrefix )
     , mInputNodataValue( -1 )
+    , mStatistics( stats )
 {
 
 }
 
 QgsZonalStatistics::QgsZonalStatistics()
     : mRasterBand( 0 )
-    , mPolygonLayer( 0 )
+    , mPolygonLayer( nullptr )
     , mInputNodataValue( -1 )
-{
-
-}
-
-QgsZonalStatistics::~QgsZonalStatistics()
+    , mStatistics( QgsZonalStatistics::All )
 {
 
 }
@@ -69,7 +67,7 @@ int QgsZonalStatistics::calculateStatistics( QProgressDialog* p )
   //open the raster layer and the raster band
   GDALAllRegister();
   GDALDatasetH inputDataset = GDALOpen( TO8F( mRasterFilePath ), GA_ReadOnly );
-  if ( inputDataset == NULL )
+  if ( !inputDataset )
   {
     return 3;
   }
@@ -81,12 +79,12 @@ int QgsZonalStatistics::calculateStatistics( QProgressDialog* p )
   }
 
   GDALRasterBandH rasterBand = GDALGetRasterBand( inputDataset, mRasterBand );
-  if ( rasterBand == NULL )
+  if ( !rasterBand )
   {
     GDALClose( inputDataset );
     return 5;
   }
-  mInputNodataValue = GDALGetRasterNoDataValue( rasterBand, NULL );
+  mInputNodataValue = GDALGetRasterNoDataValue( rasterBand, nullptr );
 
   //get geometry info about raster layer
   int nCellsXGDAL = GDALGetRasterXSize( inputDataset );
@@ -110,26 +108,114 @@ int QgsZonalStatistics::calculateStatistics( QProgressDialog* p )
   QgsRectangle rasterBBox( geoTransform[0], geoTransform[3] - ( nCellsYGDAL * cellsizeY ),
                            geoTransform[0] + ( nCellsXGDAL * cellsizeX ), geoTransform[3] );
 
-  //add the new count, sum, mean fields to the provider
+  //add the new fields to the provider
   QList<QgsField> newFieldList;
-  QString countFieldName = getUniqueFieldName( mAttributePrefix + "count" );
-  QString sumFieldName = getUniqueFieldName( mAttributePrefix + "sum" );
-  QString meanFieldName = getUniqueFieldName( mAttributePrefix + "mean" );
-  QgsField countField( countFieldName, QVariant::Double, "double precision" );
-  QgsField sumField( sumFieldName, QVariant::Double, "double precision" );
-  QgsField meanField( meanFieldName, QVariant::Double, "double precision" );
-  newFieldList.push_back( countField );
-  newFieldList.push_back( sumField );
-  newFieldList.push_back( meanField );
+  QString countFieldName;
+  if ( mStatistics & QgsZonalStatistics::Count )
+  {
+    countFieldName = getUniqueFieldName( mAttributePrefix + "count" );
+    QgsField countField( countFieldName, QVariant::Double, "double precision" );
+    newFieldList.push_back( countField );
+  }
+  QString sumFieldName;
+  if ( mStatistics & QgsZonalStatistics::Sum )
+  {
+    sumFieldName = getUniqueFieldName( mAttributePrefix + "sum" );
+    QgsField sumField( sumFieldName, QVariant::Double, "double precision" );
+    newFieldList.push_back( sumField );
+  }
+  QString meanFieldName;
+  if ( mStatistics & QgsZonalStatistics::Mean )
+  {
+    meanFieldName = getUniqueFieldName( mAttributePrefix + "mean" );
+    QgsField meanField( meanFieldName, QVariant::Double, "double precision" );
+    newFieldList.push_back( meanField );
+  }
+  QString medianFieldName;
+  if ( mStatistics & QgsZonalStatistics::Median )
+  {
+    medianFieldName = getUniqueFieldName( mAttributePrefix + "median" );
+    QgsField medianField( medianFieldName, QVariant::Double, "double precision" );
+    newFieldList.push_back( medianField );
+  }
+  QString stdevFieldName;
+  if ( mStatistics & QgsZonalStatistics::StDev )
+  {
+    stdevFieldName = getUniqueFieldName( mAttributePrefix + "stdev" );
+    QgsField stdField( stdevFieldName, QVariant::Double, "double precision" );
+    newFieldList.push_back( stdField );
+  }
+  QString minFieldName;
+  if ( mStatistics & QgsZonalStatistics::Min )
+  {
+    minFieldName = getUniqueFieldName( mAttributePrefix + "min" );
+    QgsField minField( minFieldName, QVariant::Double, "double precision" );
+    newFieldList.push_back( minField );
+  }
+  QString maxFieldName;
+  if ( mStatistics & QgsZonalStatistics::Max )
+  {
+    maxFieldName = getUniqueFieldName( mAttributePrefix + "max" );
+    QgsField maxField( maxFieldName, QVariant::Double, "double precision" );
+    newFieldList.push_back( maxField );
+  }
+  QString rangeFieldName;
+  if ( mStatistics & QgsZonalStatistics::Range )
+  {
+    rangeFieldName = getUniqueFieldName( mAttributePrefix + "range" );
+    QgsField rangeField( rangeFieldName, QVariant::Double, "double precision" );
+    newFieldList.push_back( rangeField );
+  }
+  QString minorityFieldName;
+  if ( mStatistics & QgsZonalStatistics::Minority )
+  {
+    minorityFieldName = getUniqueFieldName( mAttributePrefix + "minority" );
+    QgsField minorityField( minorityFieldName, QVariant::Double, "double precision" );
+    newFieldList.push_back( minorityField );
+  }
+  QString majorityFieldName;
+  if ( mStatistics & QgsZonalStatistics::Majority )
+  {
+    majorityFieldName = getUniqueFieldName( mAttributePrefix + "majority" );
+    QgsField majField( majorityFieldName, QVariant::Double, "double precision" );
+    newFieldList.push_back( majField );
+  }
+  QString varietyFieldName;
+  if ( mStatistics & QgsZonalStatistics::Variety )
+  {
+    varietyFieldName = getUniqueFieldName( mAttributePrefix + "variety" );
+    QgsField varietyField( varietyFieldName, QVariant::Int, "int" );
+    newFieldList.push_back( varietyField );
+  }
   vectorProvider->addAttributes( newFieldList );
 
   //index of the new fields
-  int countIndex = vectorProvider->fieldNameIndex( countFieldName );
-  int sumIndex = vectorProvider->fieldNameIndex( sumFieldName );
-  int meanIndex = vectorProvider->fieldNameIndex( meanFieldName );
+  int countIndex = mStatistics & QgsZonalStatistics::Count ? vectorProvider->fieldNameIndex( countFieldName ) : -1;
+  int sumIndex = mStatistics & QgsZonalStatistics::Sum ? vectorProvider->fieldNameIndex( sumFieldName ) : -1;
+  int meanIndex = mStatistics & QgsZonalStatistics::Mean ? vectorProvider->fieldNameIndex( meanFieldName ) : -1;
+  int medianIndex = mStatistics & QgsZonalStatistics::Median ? vectorProvider->fieldNameIndex( medianFieldName ) : -1;
+  int stdevIndex = mStatistics & QgsZonalStatistics::StDev ? vectorProvider->fieldNameIndex( stdevFieldName ) : -1;
+  int minIndex = mStatistics & QgsZonalStatistics::Min ? vectorProvider->fieldNameIndex( minFieldName ) : -1;
+  int maxIndex = mStatistics & QgsZonalStatistics::Max ? vectorProvider->fieldNameIndex( maxFieldName ) : -1;
+  int rangeIndex = mStatistics & QgsZonalStatistics::Range ? vectorProvider->fieldNameIndex( rangeFieldName ) : -1;
+  int minorityIndex = mStatistics & QgsZonalStatistics::Minority ? vectorProvider->fieldNameIndex( minorityFieldName ) : -1;
+  int majorityIndex = mStatistics & QgsZonalStatistics::Majority ? vectorProvider->fieldNameIndex( majorityFieldName ) : -1;
+  int varietyIndex = mStatistics & QgsZonalStatistics::Variety ? vectorProvider->fieldNameIndex( varietyFieldName ) : -1;
 
-  if ( countIndex == -1 || sumIndex == -1 || meanIndex == -1 )
+  if (( mStatistics & QgsZonalStatistics::Count && countIndex == -1 )
+      || ( mStatistics & QgsZonalStatistics::Sum && sumIndex == -1 )
+      || ( mStatistics & QgsZonalStatistics::Mean && meanIndex == -1 )
+      || ( mStatistics & QgsZonalStatistics::Median && medianIndex == -1 )
+      || ( mStatistics & QgsZonalStatistics::StDev && stdevIndex == -1 )
+      || ( mStatistics & QgsZonalStatistics::Min && minIndex == -1 )
+      || ( mStatistics & QgsZonalStatistics::Max && maxIndex == -1 )
+      || ( mStatistics & QgsZonalStatistics::Range && rangeIndex == -1 )
+      || ( mStatistics & QgsZonalStatistics::Minority && minorityIndex == -1 )
+      || ( mStatistics & QgsZonalStatistics::Majority && majorityIndex == -1 )
+      || ( mStatistics & QgsZonalStatistics::Variety && varietyIndex == -1 )
+     )
   {
+    //failed to create a required field
     return 8;
   }
 
@@ -146,11 +232,16 @@ int QgsZonalStatistics::calculateStatistics( QProgressDialog* p )
   request.setSubsetOfAttributes( QgsAttributeList() );
   QgsFeatureIterator fi = vectorProvider->getFeatures( request );
   QgsFeature f;
-  double count = 0;
-  double sum = 0;
-  double mean = 0;
+
+  bool statsStoreValues = ( mStatistics & QgsZonalStatistics::Median ) ||
+                          ( mStatistics & QgsZonalStatistics::StDev );
+  bool statsStoreValueCount = ( mStatistics & QgsZonalStatistics::Minority ) ||
+                              ( mStatistics & QgsZonalStatistics::Majority );
+
+  FeatureStats featureStats( statsStoreValues, statsStoreValueCount );
   int featureCounter = 0;
 
+  QgsChangedAttributesMap changeMap;
   while ( fi.nextFeature( f ) )
   {
     if ( p )
@@ -163,12 +254,12 @@ int QgsZonalStatistics::calculateStatistics( QProgressDialog* p )
       break;
     }
 
-    QgsGeometry* featureGeometry = f.geometry();
-    if ( !featureGeometry )
+    if ( !f.constGeometry() )
     {
       ++featureCounter;
       continue;
     }
+    const QgsGeometry* featureGeometry = f.constGeometry();
 
     QgsRectangle featureRect = featureGeometry->boundingBox().intersect( &rasterBBox );
     if ( featureRect.isEmpty() )
@@ -195,37 +286,84 @@ int QgsZonalStatistics::calculateStatistics( QProgressDialog* p )
     }
 
     statisticsFromMiddlePointTest( rasterBand, featureGeometry, offsetX, offsetY, nCellsX, nCellsY, cellsizeX, cellsizeY,
-                                   rasterBBox, sum, count );
+                                   rasterBBox, featureStats );
 
-    if ( count <= 1 )
+    if ( featureStats.count <= 1 )
     {
       //the cell resolution is probably larger than the polygon area. We switch to precise pixel - polygon intersection in this case
       statisticsFromPreciseIntersection( rasterBand, featureGeometry, offsetX, offsetY, nCellsX, nCellsY, cellsizeX, cellsizeY,
-                                         rasterBBox, sum, count );
-    }
-
-
-    if ( count == 0 )
-    {
-      mean = 0;
-    }
-    else
-    {
-      mean = sum / count;
+                                         rasterBBox, featureStats );
     }
 
     //write the statistics value to the vector data provider
-    QgsChangedAttributesMap changeMap;
     QgsAttributeMap changeAttributeMap;
-    changeAttributeMap.insert( countIndex, QVariant( count ) );
-    changeAttributeMap.insert( sumIndex, QVariant( sum ) );
-    changeAttributeMap.insert( meanIndex, QVariant( mean ) );
-    changeMap.insert( f.id(), changeAttributeMap );
-    vectorProvider->changeAttributeValues( changeMap );
+    if ( mStatistics & QgsZonalStatistics::Count )
+      changeAttributeMap.insert( countIndex, QVariant( featureStats.count ) );
+    if ( mStatistics & QgsZonalStatistics::Sum )
+      changeAttributeMap.insert( sumIndex, QVariant( featureStats.sum ) );
+    if ( featureStats.count > 0 )
+    {
+      double mean = featureStats.sum / featureStats.count;
+      if ( mStatistics & QgsZonalStatistics::Mean )
+        changeAttributeMap.insert( meanIndex, QVariant( mean ) );
+      if ( mStatistics & QgsZonalStatistics::Median )
+      {
+        qSort( featureStats.values.begin(), featureStats.values.end() );
+        int size =  featureStats.values.count();
+        bool even = ( size % 2 ) < 1;
+        double medianValue;
+        if ( even )
+        {
+          medianValue = ( featureStats.values.at( size / 2 - 1 ) + featureStats.values.at( size / 2 ) ) / 2;
+        }
+        else //odd
+        {
+          medianValue = featureStats.values.at(( size + 1 ) / 2 - 1 );
+        }
+        changeAttributeMap.insert( medianIndex, QVariant( medianValue ) );
+      }
+      if ( mStatistics & QgsZonalStatistics::StDev )
+      {
+        double sumSquared = 0;
+        for ( int i = 0; i < featureStats.values.count(); ++i )
+        {
+          double diff = featureStats.values.at( i ) - mean;
+          sumSquared += diff * diff;
+        }
+        double stdev = qPow( sumSquared / featureStats.values.count(), 0.5 );
+        changeAttributeMap.insert( stdevIndex, QVariant( stdev ) );
+      }
+      if ( mStatistics & QgsZonalStatistics::Min )
+        changeAttributeMap.insert( minIndex, QVariant( featureStats.min ) );
+      if ( mStatistics & QgsZonalStatistics::Max )
+        changeAttributeMap.insert( maxIndex, QVariant( featureStats.max ) );
+      if ( mStatistics & QgsZonalStatistics::Range )
+        changeAttributeMap.insert( rangeIndex, QVariant( featureStats.max - featureStats.min ) );
+      if ( mStatistics & QgsZonalStatistics::Minority || mStatistics & QgsZonalStatistics::Majority )
+      {
+        QList<int> vals = featureStats.valueCount.values();
+        qSort( vals.begin(), vals.end() );
+        if ( mStatistics & QgsZonalStatistics::Minority )
+        {
+          float minorityKey = featureStats.valueCount.key( vals.first() );
+          changeAttributeMap.insert( minorityIndex, QVariant( minorityKey ) );
+        }
+        if ( mStatistics & QgsZonalStatistics::Majority )
+        {
+          float majKey = featureStats.valueCount.key( vals.last() );
+          changeAttributeMap.insert( majorityIndex, QVariant( majKey ) );
+        }
+      }
+      if ( mStatistics & QgsZonalStatistics::Variety )
+        changeAttributeMap.insert( varietyIndex, QVariant( featureStats.valueCount.count() ) );
+    }
 
+    changeMap.insert( f.id(), changeAttributeMap );
     ++featureCounter;
   }
 
+  vectorProvider->changeAttributeValues( changeMap );
+
   if ( p )
   {
     p->setValue( featureCount );
@@ -249,7 +387,10 @@ int QgsZonalStatistics::cellInfoForBBox( const QgsRectangle& rasterBBox, const Q
   QgsRectangle intersectBox = rasterBBox.intersect( &featureBBox );
   if ( intersectBox.isEmpty() )
   {
-    nCellsX = 0; nCellsY = 0; offsetX = 0; offsetY = 0;
+    nCellsX = 0;
+    nCellsY = 0;
+    offsetX = 0;
+    offsetY = 0;
     return 0;
   }
 
@@ -266,15 +407,14 @@ int QgsZonalStatistics::cellInfoForBBox( const QgsRectangle& rasterBBox, const Q
   return 0;
 }
 
-void QgsZonalStatistics::statisticsFromMiddlePointTest( void* band, QgsGeometry* poly, int pixelOffsetX,
-    int pixelOffsetY, int nCellsX, int nCellsY, double cellSizeX, double cellSizeY, const QgsRectangle& rasterBBox, double& sum, double& count )
+void QgsZonalStatistics::statisticsFromMiddlePointTest( void* band, const QgsGeometry* poly, int pixelOffsetX,
+    int pixelOffsetY, int nCellsX, int nCellsY, double cellSizeX, double cellSizeY, const QgsRectangle& rasterBBox, FeatureStats &stats )
 {
   double cellCenterX, cellCenterY;
 
   float* scanLine = ( float * ) CPLMalloc( sizeof( float ) * nCellsX );
   cellCenterY = rasterBBox.yMaximum() - pixelOffsetY * cellSizeY - cellSizeY / 2;
-  count = 0;
-  sum = 0;
+  stats.reset();
 
   const GEOSGeometry* polyGeos = poly->asGeos();
   if ( !polyGeos )
@@ -289,8 +429,8 @@ void QgsZonalStatistics::statisticsFromMiddlePointTest( void* band, QgsGeometry*
     return;
   }
 
-  GEOSCoordSequence* cellCenterCoords = 0;
-  GEOSGeometry* currentCellCenter = 0;
+  GEOSCoordSequence* cellCenterCoords = nullptr;
+  GEOSGeometry* currentCellCenter = nullptr;
 
   for ( int i = 0; i < nCellsY; ++i )
   {
@@ -311,8 +451,7 @@ void QgsZonalStatistics::statisticsFromMiddlePointTest( void* band, QgsGeometry*
         currentCellCenter = GEOSGeom_createPoint_r( geosctxt, cellCenterCoords );
         if ( GEOSPreparedContains_r( geosctxt, polyGeosPrepared, currentCellCenter ) )
         {
-          sum += scanLine[j];
-          ++count;
+          stats.addValue( scanLine[j] );
         }
       }
       cellCenterX += cellSizeX;
@@ -324,14 +463,14 @@ void QgsZonalStatistics::statisticsFromMiddlePointTest( void* band, QgsGeometry*
   GEOSPreparedGeom_destroy_r( geosctxt, polyGeosPrepared );
 }
 
-void QgsZonalStatistics::statisticsFromPreciseIntersection( void* band, QgsGeometry* poly, int pixelOffsetX,
-    int pixelOffsetY, int nCellsX, int nCellsY, double cellSizeX, double cellSizeY, const QgsRectangle& rasterBBox, double& sum, double& count )
+void QgsZonalStatistics::statisticsFromPreciseIntersection( void* band, const QgsGeometry* poly, int pixelOffsetX,
+    int pixelOffsetY, int nCellsX, int nCellsY, double cellSizeX, double cellSizeY, const QgsRectangle& rasterBBox, FeatureStats &stats )
 {
-  sum = 0;
-  count = 0;
+  stats.reset();
+
   double currentY = rasterBBox.yMaximum() - pixelOffsetY * cellSizeY - cellSizeY / 2;
   float* pixelData = ( float * ) CPLMalloc( sizeof( float ) );
-  QgsGeometry* pixelRectGeometry = 0;
+  QgsGeometry* pixelRectGeometry = nullptr;
 
   double hCellSizeX = cellSizeX / 2.0;
   double hCellSizeY = cellSizeY / 2.0;
@@ -343,7 +482,11 @@ void QgsZonalStatistics::statisticsFromPreciseIntersection( void* band, QgsGeome
     double currentX = rasterBBox.xMinimum() + cellSizeX / 2.0 + pixelOffsetX * cellSizeX;
     for ( int col = 0; col < nCellsX; ++col )
     {
-      GDALRasterIO( band, GF_Read, pixelOffsetX + col, pixelOffsetY + row, nCellsX, 1, pixelData, 1, 1, GDT_Float32, 0, 0 );
+      if ( GDALRasterIO( band, GF_Read, pixelOffsetX + col, pixelOffsetY + row, nCellsX, 1, pixelData, 1, 1, GDT_Float32, 0, 0 ) != CE_None )
+      {
+        QgsDebugMsg( "Raster IO Error" );
+      }
+
       if ( !validPixel( *pixelData ) )
         continue;
 
@@ -358,13 +501,12 @@ void QgsZonalStatistics::statisticsFromPreciseIntersection( void* band, QgsGeome
           if ( intersectionArea >= 0.0 )
           {
             weight = intersectionArea / pixelArea;
-            count += weight;
-            sum += *pixelData * weight;
+            stats.addValue( *pixelData, weight );
           }
           delete intersectGeometry;
         }
         delete pixelRectGeometry;
-        pixelRectGeometry = 0;
+        pixelRectGeometry = nullptr;
       }
       currentX += cellSizeX;
     }
@@ -382,7 +524,7 @@ bool QgsZonalStatistics::validPixel( float value ) const
   return true;
 }
 
-QString QgsZonalStatistics::getUniqueFieldName( QString fieldName )
+QString QgsZonalStatistics::getUniqueFieldName( const QString& fieldName )
 {
   QgsVectorDataProvider* dp = mPolygonLayer->dataProvider();
 
