@@ -19,10 +19,12 @@
 #include "ui_qgscomposerbase.h"
 #include "qgscomposermap.h"
 #include "qgscontexthelp.h"
-#include <QDockWidget>
+#include "qgsdockwidget.h"
 
 class QgisApp;
 class QgsComposerArrow;
+class QgsComposerPolygon;
+class QgsComposerPolyline;
 class QgsComposerFrame;
 class QgsComposerHtml;
 class QgsComposerLabel;
@@ -53,7 +55,7 @@ class QLabel;
 class QTreeView;
 class QPrinter;
 
-/** \ingroup MapComposer
+/** \ingroup app
  * \brief A gui for composing a printable map.
  */
 class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
@@ -83,13 +85,13 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     void zoomFull();
 
     //! Return pointer to map canvas
-    QgsMapCanvas *mapCanvas( void );
+    QgsMapCanvas* mapCanvas();
 
     //! Return pointer to composer view
-    QgsComposerView *view( void );
+    QgsComposerView* view();
 
     //! Return current composition
-    QgsComposition* composition( void ) { return mComposition; }
+    QgsComposition* composition() { return mComposition; }
 
     //! Restore the window and toolbar state
     void restoreWindowState();
@@ -191,6 +193,11 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
 
     void on_mActionAddEllipse_triggered();
 
+    //! Nodes based shape
+    void on_mActionEditNodesItem_triggered();
+    void on_mActionAddPolygon_triggered();
+    void on_mActionAddPolyline_triggered();
+
     //! Add attribute table
     void on_mActionAddTable_triggered();
 
@@ -379,6 +386,12 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     /** Add a composer arrow to the item/widget map and creates a configuration widget for it*/
     void addComposerArrow( QgsComposerArrow* arrow );
 
+    /** Add a composer polygon to the item/widget map and creates a configuration widget for it*/
+    void addComposerPolygon( QgsComposerPolygon* polygon );
+
+    /** Add a composer polyline to the item/widget map and creates a configuration widget for it*/
+    void addComposerPolyline( QgsComposerPolyline* polyline );
+
     /** Add a composer map to the item/widget map and creates a configuration widget for it*/
     void addComposerMap( QgsComposerMap* map );
 
@@ -582,11 +595,11 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     //! We load composer map content from project xml only on demand. Therefore we need to store the real preview mode type
     QMap< QgsComposerMap*, int > mMapsToRestore;
 
-    QDockWidget* mItemDock;
-    QDockWidget* mUndoDock;
-    QDockWidget* mGeneralDock;
-    QDockWidget* mAtlasDock;
-    QDockWidget* mItemsDock;
+    QgsDockWidget* mItemDock;
+    QgsDockWidget* mUndoDock;
+    QgsDockWidget* mGeneralDock;
+    QgsDockWidget* mAtlasDock;
+    QgsDockWidget* mItemsDock;
 
     QTreeView* mItemsTreeView;
 
@@ -606,7 +619,10 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
 
     struct PanelStatus
     {
-      PanelStatus( bool visible = true, bool active = false ) : isVisible( visible ), isActive( active ) {}
+      PanelStatus( bool visible = true, bool active = false )
+          : isVisible( visible )
+          , isActive( active )
+      {}
       bool isVisible;
       bool isActive;
     };
