@@ -83,23 +83,9 @@ QgsCompositionWidget::QgsCompositionWidget( QWidget* parent, QgsComposition* c )
     mGenerateWorldFileCheckBox->setChecked( mComposition->generateWorldFile() );
 
     // populate the map list
-    mWorldFileMapComboBox->clear();
-    QList<const QgsComposerMap*> availableMaps = mComposition->composerMapItems();
-    QList<const QgsComposerMap*>::const_iterator mapItemIt = availableMaps.constBegin();
-    for ( ; mapItemIt != availableMaps.constEnd(); ++mapItemIt )
-    {
-      mWorldFileMapComboBox->addItem( tr( "Map %1" ).arg(( *mapItemIt )->id() ), qVariantFromValue(( void* )*mapItemIt ) );
-    }
-
-    int idx = mWorldFileMapComboBox->findData( qVariantFromValue(( void* )mComposition->worldFileMap() ) );
-    if ( idx != -1 )
-    {
-      mWorldFileMapComboBox->setCurrentIndex( idx );
-    }
-
-    // Connect to addition / removal of maps
-    connect( mComposition, SIGNAL( composerMapAdded( QgsComposerMap* ) ), this, SLOT( onComposerMapAdded( QgsComposerMap* ) ) );
-    connect( mComposition, SIGNAL( itemRemoved( QgsComposerItem* ) ), this, SLOT( onItemRemoved( QgsComposerItem* ) ) );
+    mWorldFileMapComboBox->setComposition( mComposition );
+    mWorldFileMapComboBox->setItemType( QgsComposerItem::ComposerMap );
+    mWorldFileMapComboBox->setItem( mComposition->worldFileMap() );
 
     mSnapToleranceSpinBox->setValue( mComposition->snapTolerance() );
 
@@ -139,6 +125,8 @@ QgsCompositionWidget::QgsCompositionWidget( QWidget* parent, QgsComposition* c )
   connect( mPaperOrientationDDBtn, SIGNAL( dataDefinedActivated( bool ) ), this, SLOT( updateDataDefinedProperty() ) );
   connect( mPaperOrientationDDBtn, SIGNAL( dataDefinedActivated( bool ) ), mPaperOrientationComboBox, SLOT( setDisabled( bool ) ) );
 
+  connect( mWorldFileMapComboBox, SIGNAL( itemChanged( QgsComposerItem* ) ), this, SLOT( worldFileMapChanged( QgsComposerItem* ) ) );
+
   //initialize data defined buttons
   populateDataDefinedButtons();
 
@@ -500,6 +488,7 @@ void QgsCompositionWidget::applyWidthHeight()
     return;
 
   mComposition->setPaperSize( width, height );
+  mComposition->refreshItems();
 }
 
 void QgsCompositionWidget::on_mPaperWidthDoubleSpinBox_editingFinished()
@@ -682,60 +671,17 @@ void QgsCompositionWidget::on_mGenerateWorldFileCheckBox_toggled( bool state )
   }
 
   mComposition->setGenerateWorldFile( state );
-  mWorldFileMapComboBox->setEnabled( state );
 }
 
-void QgsCompositionWidget::onComposerMapAdded( QgsComposerMap* map )
+void QgsCompositionWidget::worldFileMapChanged( QgsComposerItem* item )
 {
   if ( !mComposition )
   {
     return;
   }
 
-  mWorldFileMapComboBox->addItem( tr( "Map %1" ).arg( map->id() ), qVariantFromValue(( void* )map ) );
-  if ( mWorldFileMapComboBox->count() == 1 )
-  {
-    mComposition->setWorldFileMap( map );
-  }
-}
-
-void QgsCompositionWidget::onItemRemoved( QgsComposerItem* item )
-{
-  if ( !mComposition )
-  {
-    return;
-  }
-
-  QgsComposerMap* map = dynamic_cast<QgsComposerMap*>( item );
-  if ( map )
-  {
-    int idx = mWorldFileMapComboBox->findData( qVariantFromValue(( void* )map ) );
-    if ( idx != -1 )
-    {
-      mWorldFileMapComboBox->removeItem( idx );
-    }
-  }
-  if ( mWorldFileMapComboBox->count() == 0 )
-  {
-    mComposition->setWorldFileMap( nullptr );
-  }
-}
-
-void QgsCompositionWidget::on_mWorldFileMapComboBox_currentIndexChanged( int index )
-{
-  if ( !mComposition )
-  {
-    return;
-  }
-  if ( index == -1 )
-  {
-    mComposition->setWorldFileMap( nullptr );
-  }
-  else
-  {
-    QgsComposerMap* map = reinterpret_cast<QgsComposerMap*>( mWorldFileMapComboBox->itemData( index ).value<void*>() );
-    mComposition->setWorldFileMap( map );
-  }
+  QgsComposerMap* map = dynamic_cast< QgsComposerMap* >( item );
+  mComposition->setWorldFileMap( map );
 }
 
 void QgsCompositionWidget::on_mGridResolutionSpinBox_valueChanged( double d )
@@ -785,5 +731,7 @@ void QgsCompositionWidget::blockSignals( bool block )
   mOffsetXSpinBox->blockSignals( block );
   mOffsetYSpinBox->blockSignals( block );
   mSnapToleranceSpinBox->blockSignals( block );
+  mGenerateWorldFileCheckBox->blockSignals( block );
+  mWorldFileMapComboBox->blockSignals( block );
 }
 
