@@ -1048,7 +1048,7 @@ int QwtPolarGrid::marginHint() const
 */
 const QwtScaleDraw *QwtPolarGrid::scaleDraw( int axisId ) const
 {
-    if ( axisId >= QwtPolar::AxisLeft || axisId <= QwtPolar::AxisBottom )
+    if ( axisId >= QwtPolar::AxisLeft && axisId <= QwtPolar::AxisBottom )
         return static_cast<QwtScaleDraw *>( d_data->axisData[axisId].scaleDraw );
 
     return NULL;
@@ -1063,7 +1063,7 @@ const QwtScaleDraw *QwtPolarGrid::scaleDraw( int axisId ) const
 */
 QwtScaleDraw *QwtPolarGrid::scaleDraw( int axisId )
 {
-    if ( axisId >= QwtPolar::AxisLeft || axisId <= QwtPolar::AxisBottom )
+    if ( axisId >= QwtPolar::AxisLeft && axisId <= QwtPolar::AxisBottom )
         return static_cast<QwtScaleDraw *>( d_data->axisData[axisId].scaleDraw );
 
     return NULL;
