@@ -7,20 +7,20 @@
  *****************************************************************************/
 
 #include "qwt_polar_plot.h"
-#include "qwt_legend.h"
-#include "qwt_legend_item.h"
 #include "qwt_polar_item.h"
+#include <qwt_scale_div.h>
+#include <qwt_legend.h>
+#include <qwt_legend_item.h>
+#include <qpainter.h>
 
 class QwtPolarItem::PrivateData
 {
-  public:
+public:
     PrivateData():
         plot( NULL ),
         isVisible( true ),
         attributes( 0 ),
-#if QT_VERSION >= 0x040000
         renderHints( 0 ),
-#endif
         z( 0.0 )
     {
     }
@@ -28,10 +28,8 @@ class QwtPolarItem::PrivateData
     mutable QwtPolarPlot *plot;
 
     bool isVisible;
-    int attributes;
-#if QT_VERSION >= 0x040000
-    int renderHints;
-#endif
+    QwtPolarItem::ItemAttributes attributes;
+    QwtPolarItem::RenderHints renderHints;
     double z;
 
     QwtText title;
@@ -46,15 +44,15 @@ class QwtPolarItem::PrivateData
 */
 QwtPolarItem::QwtPolarItem( const QwtText &title )
 {
-  d_data = new PrivateData;
-  d_data->title = title;
+    d_data = new PrivateData;
+    d_data->title = title;
 }
 
 //! Destroy the QwtPolarItem
 QwtPolarItem::~QwtPolarItem()
 {
-  attach( NULL );
-  delete d_data;
+    attach( NULL );
+    delete d_data;
 }
 
 /*!
@@ -72,28 +70,31 @@ QwtPolarItem::~QwtPolarItem()
 */
 void QwtPolarItem::attach( QwtPolarPlot *plot )
 {
-  if ( plot == d_data->plot )
-    return;
+    if ( plot == d_data->plot )
+        return;
 
-  // remove the item from the previous plot
+    // remove the item from the previous plot
 
-  if ( d_data->plot )
-  {
-    d_data->plot->attachItem( this, false );
+    if ( d_data->plot )
+    {
+        if ( d_data->plot->legend() )
+            d_data->plot->legend()->remove( this );
 
-    if ( d_data->plot->autoReplot() )
-      d_data->plot->update();
-  }
+        d_data->plot->attachItem( this, false );
 
-  d_data->plot = plot;
+        if ( d_data->plot->autoReplot() )
+            d_data->plot->update();
+    }
 
-  if ( d_data->plot )
-  {
-    // insert the item into the current plot
+    d_data->plot = plot;
 
-    d_data->plot->attachItem( this, true );
-    itemChanged();
-  }
+    if ( d_data->plot )
+    {
+        // insert the item into the current plot
+
+        d_data->plot->attachItem( this, true );
+        itemChanged();
+    }
 }
 
 /*!
@@ -110,23 +111,24 @@ void QwtPolarItem::attach( QwtPolarPlot *plot )
 */
 int QwtPolarItem::rtti() const
 {
-  return Rtti_PolarItem;
+    return Rtti_PolarItem;
 }
 
-//! Return attached plot
+//! \return Attached plot
 QwtPolarPlot *QwtPolarItem::plot() const
 {
-  return d_data->plot;
+    return d_data->plot;
 }
 
 /*!
    Plot items are painted in increasing z-order.
 
-   \return setZ(), QwtPolarItemDict::itemList()
+   \return Z value
+   \sa setZ(), QwtPolarItemDict::itemList()
 */
 double QwtPolarItem::z() const
 {
-  return d_data->z;
+    return d_data->z;
 }
 
 /*!
@@ -139,17 +141,18 @@ double QwtPolarItem::z() const
 */
 void QwtPolarItem::setZ( double z )
 {
-  if ( d_data->z != z )
-  {
-    d_data->z = z;
-    if ( d_data->plot )
+    if ( d_data->z != z )
     {
-      // update the z order
-      d_data->plot->attachItem( this, false );
-      d_data->plot->attachItem( this, true );
+        if ( d_data->plot )
+            d_data->plot->attachItem( this, false );
+
+        d_data->z = z;
+
+        if ( d_data->plot )
+            d_data->plot->attachItem( this, true );
+
+        itemChanged();
     }
-    itemChanged();
-  }
 }
 
 /*!
@@ -160,7 +163,7 @@ void QwtPolarItem::setZ( double z )
 */
 void QwtPolarItem::setTitle( const QString &title )
 {
-  setTitle( QwtText( title ) );
+    setTitle( QwtText( title ) );
 }
 
 /*!
@@ -171,11 +174,11 @@ void QwtPolarItem::setTitle( const QString &title )
 */
 void QwtPolarItem::setTitle( const QwtText &title )
 {
-  if ( d_data->title != title )
-  {
-    d_data->title = title;
-    itemChanged();
-  }
+    if ( d_data->title != title )
+    {
+        d_data->title = title;
+        itemChanged();
+    }
 }
 
 /*!
@@ -184,7 +187,7 @@ void QwtPolarItem::setTitle( const QwtText &title )
 */
 const QwtText &QwtPolarItem::title() const
 {
-  return d_data->title;
+    return d_data->title;
 }
 
 /*!
@@ -197,15 +200,15 @@ const QwtText &QwtPolarItem::title() const
 */
 void QwtPolarItem::setItemAttribute( ItemAttribute attribute, bool on )
 {
-  if ( bool( d_data->attributes & attribute ) != on )
-  {
-    if ( on )
-      d_data->attributes |= attribute;
-    else
-      d_data->attributes &= ~attribute;
+    if ( bool( d_data->attributes & attribute ) != on )
+    {
+        if ( on )
+            d_data->attributes |= attribute;
+        else
+            d_data->attributes &= ~attribute;
 
-    itemChanged();
-  }
+        itemChanged();
+    }
 }
 
 /*!
@@ -217,11 +220,9 @@ void QwtPolarItem::setItemAttribute( ItemAttribute attribute, bool on )
 */
 bool QwtPolarItem::testItemAttribute( ItemAttribute attribute ) const
 {
-  return d_data->attributes & attribute;
+    return d_data->attributes & attribute;
 }
 
-#if QT_VERSION >= 0x040000
-
 /*!
    Toggle an render hint
 
@@ -232,15 +233,15 @@ bool QwtPolarItem::testItemAttribute( ItemAttribute attribute ) const
 */
 void QwtPolarItem::setRenderHint( RenderHint hint, bool on )
 {
-  if ((( d_data->renderHints & hint ) != 0 ) != on )
-  {
-    if ( on )
-      d_data->renderHints |= hint;
-    else
-      d_data->renderHints &= ~hint;
+    if ( ( ( d_data->renderHints & hint ) != 0 ) != on )
+    {
+        if ( on )
+            d_data->renderHints |= hint;
+        else
+            d_data->renderHints &= ~hint;
 
-    itemChanged();
-  }
+        itemChanged();
+    }
 }
 
 /*!
@@ -252,21 +253,19 @@ void QwtPolarItem::setRenderHint( RenderHint hint, bool on )
 */
 bool QwtPolarItem::testRenderHint( RenderHint hint ) const
 {
-  return ( d_data->renderHints & hint );
+    return ( d_data->renderHints & hint );
 }
 
-#endif
-
 //! Show the item
 void QwtPolarItem::show()
 {
-  setVisible( true );
+    setVisible( true );
 }
 
 //! Hide the item
 void QwtPolarItem::hide()
 {
-  setVisible( false );
+    setVisible( false );
 }
 
 /*!
@@ -277,11 +276,11 @@ void QwtPolarItem::hide()
 */
 void QwtPolarItem::setVisible( bool on )
 {
-  if ( on != d_data->isVisible )
-  {
-    d_data->isVisible = on;
-    itemChanged();
-  }
+    if ( on != d_data->isVisible )
+    {
+        d_data->isVisible = on;
+        itemChanged();
+    }
 }
 
 /*!
@@ -290,7 +289,7 @@ void QwtPolarItem::setVisible( bool on )
 */
 bool QwtPolarItem::isVisible() const
 {
-  return d_data->isVisible;
+    return d_data->isVisible;
 }
 
 /*!
@@ -301,13 +300,13 @@ bool QwtPolarItem::isVisible() const
 */
 void QwtPolarItem::itemChanged()
 {
-  if ( d_data->plot )
-  {
-    if ( d_data->plot->legend() )
-      updateLegend( d_data->plot->legend() );
+    if ( d_data->plot )
+    {
+        if ( d_data->plot->legend() )
+            updateLegend( d_data->plot->legend() );
 
-    d_data->plot->autoRefresh();
-  }
+        d_data->plot->autoRefresh();
+    }
 }
 
 /*!
@@ -318,10 +317,13 @@ void QwtPolarItem::itemChanged()
    sense an invalid interval is returned.
 
    \param scaleId Scale id ( QwtPolar::Scale )
+   \return Bounding interval of the plot item for a specific scale
 */
-QwtDoubleInterval QwtPolarItem::boundingInterval( int /*scaleId*/ ) const
+QwtInterval QwtPolarItem::boundingInterval( int scaleId ) const
 {
-  return QwtDoubleInterval(); // invalid
+    Q_UNUSED( scaleId );
+
+    return QwtInterval(); // invalid
 }
 
 /*!
@@ -339,10 +341,12 @@ QwtDoubleInterval QwtPolarItem::boundingInterval( int /*scaleId*/ ) const
 
    \sa QwtPolarPlot::updateAxes()
 */
-void QwtPolarItem::updateScaleDiv( const QwtScaleDiv & /* azimuthScaleDiv */,
-                                   const QwtScaleDiv & /* radialScaleDiv */,
-                                   const QwtDoubleInterval & /*interval */ )
+void QwtPolarItem::updateScaleDiv( const QwtScaleDiv &azimuthScaleDiv,
+    const QwtScaleDiv &radialScaleDiv, const QwtInterval &interval )
 {
+    Q_UNUSED( azimuthScaleDiv );
+    Q_UNUSED( radialScaleDiv );
+    Q_UNUSED( interval );
 }
 
 /*!
@@ -359,46 +363,60 @@ void QwtPolarItem::updateScaleDiv( const QwtScaleDiv & /* azimuthScaleDiv */,
 */
 void QwtPolarItem::updateLegend( QwtLegend *legend ) const
 {
-  if ( !legend )
-    return;
+    if ( legend == NULL )
+        return;
 
-  QWidget *lgdItem = legend->find( this );
-  if ( testItemAttribute( QwtPolarItem::Legend ) )
-  {
-    if ( lgdItem == NULL )
+    QWidget *lgdItem = legend->find( this );
+    if ( testItemAttribute( QwtPolarItem::Legend ) )
     {
-      lgdItem = legendItem();
-      if ( lgdItem )
-      {
-        if ( lgdItem->inherits( "QwtLegendItem" ) )
+        if ( lgdItem == NULL )
         {
-          QwtLegendItem *label = ( QwtLegendItem * )lgdItem;
-          label->setItemMode( legend->itemMode() );
-
-          if ( d_data->plot )
-          {
-            QObject::connect( label, SIGNAL( clicked() ),
-                              d_data->plot, SLOT( legendItemClicked() ) );
-            QObject::connect( label, SIGNAL( checked( bool ) ),
-                              d_data->plot, SLOT( legendItemChecked( bool ) ) );
-          }
+            lgdItem = legendItem();
+            if ( lgdItem )
+                legend->insert( this, lgdItem );
         }
-        legend->insert( this, lgdItem );
-      }
-    }
-    if ( lgdItem && lgdItem->inherits( "QwtLegendItem" ) )
-    {
-      QwtLegendItem* label = ( QwtLegendItem* )lgdItem;
-      if ( label )
-        label->setText( d_data->title );
-    }
-  }
-  else
-  {
-    delete lgdItem;
-  }
-}
 
+        QwtLegendItem* label = qobject_cast<QwtLegendItem *>( lgdItem );
+        if ( label )
+        {
+            // paint the identifier
+            const QSize sz = label->identifierSize();
+
+            QPixmap identifier( sz.width(), sz.height() );
+            identifier.fill( Qt::transparent );
+
+            QPainter painter( &identifier );
+            painter.setRenderHint( QPainter::Antialiasing,
+                testRenderHint( QwtPolarItem::RenderAntialiased ) );
+
+            drawLegendIdentifier( &painter,
+                QRect( 0, 0, sz.width(), sz.height() ) );
+
+            painter.end();
+
+            const bool doUpdate = label->updatesEnabled();
+            if ( doUpdate )
+                label->setUpdatesEnabled( false );
+
+            label->setText( title() );
+            label->setIdentifier( identifier );
+            label->setItemMode( legend->itemMode() );
+
+            if ( doUpdate )
+                label->setUpdatesEnabled( true );
+
+            label->update();
+        }
+    }
+    else
+    {
+        if ( lgdItem )
+        {
+            lgdItem->hide();
+            lgdItem->deleteLater();
+        }
+    }
+}
 /*!
    \brief Allocate the widget that represents the item on the legend
 
@@ -411,19 +429,17 @@ void QwtPolarItem::updateLegend( QwtLegend *legend ) const
 */
 QWidget *QwtPolarItem::legendItem() const
 {
-#if QWT_VERSION >= 0x050300
-  return new QwtLegendCurveItem;
-#else
-  return new QwtLegendItem;
-#endif
+    return new QwtLegendItem;
 }
 
 /*!
    Some items like to display something (f.e. the azimuth axis) outside
    of the area of the interval of the radial scale.
    The default implementation returns 0 pixels
+
+   \return Hint for the margin
 */
 int QwtPolarItem::marginHint() const
 {
-  return 0;
+    return 0;
 }
