@@ -23,7 +23,6 @@ class QwtScaleDiv;
 class QwtTextLabel;
 class QwtPolarCanvas;
 class QwtPolarLayout;
-class QwtAbstractLegend;
 
 /*!
   \brief A plotting widget, displaying a polar coordinate system
@@ -34,7 +33,7 @@ class QwtAbstractLegend;
   from QwtPolarItem.
 
   The coordinate system is defined by a radial and a azimuth scale.
-  The scales at the axes can be explicitly set (QwtScaleDiv), or
+  The scales at the axes can be explicitely set (QwtScaleDiv), or
   are calculated from the plot items, using algorithms (QwtScaleEngine) which
   can be configured separately for each axis. Autoscaling is supported
   for the radial scale.
@@ -141,14 +140,11 @@ public:
 
     // Legend
 
-    void insertLegend( QwtAbstractLegend *,
+    void insertLegend( QwtLegend *,
         LegendPosition = RightLegend, double ratio = -1.0 );
 
-    QwtAbstractLegend *legend();
-    const QwtAbstractLegend *legend() const;
-
-    void updateLegend();
-    void updateLegend( const QwtPolarItem * );
+    QwtLegend *legend();
+    const QwtLegend *legend() const;
 
     // Layout
     QwtPolarLayout *plotLayout();
@@ -160,28 +156,31 @@ public:
 
     int plotMarginHint() const;
 
-    virtual QVariant itemToInfo( QwtPolarItem * ) const;
-    virtual QwtPolarItem *infoToItem( const QVariant & ) const;
-
 Q_SIGNALS:
     /*!
-      A signal indicating, that an item has been attached/detached
+      A signal which is emitted when the user has clicked on
+      a legend item, which is in QwtLegend::ClickableItem mode.
 
-      \param plotItem Plot item
-      \param on Attached/Detached
-     */
-    void itemAttached( QwtPolarItem *plotItem, bool on );
+      \param plotItem Corresponding plot item of the
+                 selected legend item
 
-    /*! 
-      A signal with the attributes how to update 
-      the legend entries for a plot item.
-                
-      \param itemInfo Info about a plot, build from itemToInfo()
-    
-      \sa itemToInfo(), infoToItem(), QwtAbstractLegend::updateLegend()
+      \note clicks are disabled as default
+      \sa QwtLegend::setItemMode, QwtLegend::itemMode
      */
-    void legendDataChanged( const QVariant &itemInfo,
-        const QList<QwtLegendData> &data );
+    void legendClicked( QwtPolarItem *plotItem );
+
+    /*!
+      A signal which is emitted when the user has clicked on
+      a legend item, which is in QwtLegend::CheckableItem mode
+
+      \param plotItem Corresponding plot item of the
+                 selected legend item
+      \param on True when the legen item is checked
+
+      \note clicks are disabled as default
+      \sa QwtLegend::setItemMode, QwtLegend::itemMode
+     */
+    void legendChecked( QwtPolarItem *plotItem, bool on );
 
     /*!
       A signal that is emitted, whenever the layout of the plot
@@ -194,6 +193,10 @@ public Q_SLOTS:
     void autoRefresh();
     void setAzimuthOrigin( double );
 
+protected Q_SLOTS:
+    virtual void legendItemClicked();
+    virtual void legendItemChecked( bool );
+
 protected:
     virtual bool event( QEvent * );
     virtual void resizeEvent( QResizeEvent * );
@@ -206,9 +209,6 @@ protected:
         const QRectF &canvasRect ) const;
 
 private:
-    friend class QwtPolarItem;
-    void attachItem( QwtPolarItem *, bool );
-
     void initPlot( const QwtText & );
 
     class ScaleData;
