@@ -16,6 +16,7 @@
  ***************************************************************************/
 #include <QSettings>
 #include <QMessageBox>
+#include <QRegExpValidator>
 
 #include "qgsnewogrconnection.h"
 #include "qgscontexthelp.h"
@@ -32,8 +33,8 @@
 #endif
 
 QgsNewOgrConnection::QgsNewOgrConnection( QWidget *parent, const QString& connType, const QString& connName, Qt::WindowFlags fl )
-    : QDialog( parent, fl ),
-    mOriginalConnName( connName )
+    : QDialog( parent, fl )
+    , mOriginalConnName( connName )
 {
   setupUi( this );
 
@@ -41,11 +42,11 @@ QgsNewOgrConnection::QgsNewOgrConnection( QWidget *parent, const QString& connTy
   restoreGeometry( settings.value( "/Windows/OGRDatabaseConnection/geometry" ).toByteArray() );
 
   //add database drivers
-  QStringList dbDrivers = QgsProviderRegistry::instance()->databaseDrivers().split( ";" );
+  QStringList dbDrivers = QgsProviderRegistry::instance()->databaseDrivers().split( ';' );
   for ( int i = 0; i < dbDrivers.count(); i++ )
   {
     QString dbDrive = dbDrivers.at( i );
-    cmbDatabaseTypes->addItem( dbDrive.split( "," ).at( 0 ) );
+    cmbDatabaseTypes->addItem( dbDrive.split( ',' ).at( 0 ) );
   }
   txtName->setEnabled( true );
   cmbDatabaseTypes->setEnabled( true );
@@ -53,7 +54,7 @@ QgsNewOgrConnection::QgsNewOgrConnection( QWidget *parent, const QString& connTy
   {
     // populate the dialog with the information stored for the connection
     // populate the fields with the stored setting parameters
-    QString key = "/" + connType + "/connections/" + connName;
+    QString key = '/' + connType + "/connections/" + connName;
     txtHost->setText( settings.value( key + "/host" ).toString() );
     txtDatabase->setText( settings.value( key + "/database" ).toString() );
     QString port = settings.value( key + "/port" ).toString();
@@ -69,6 +70,7 @@ QgsNewOgrConnection::QgsNewOgrConnection( QWidget *parent, const QString& connTy
     txtName->setEnabled( false );
     cmbDatabaseTypes->setEnabled( false );
   }
+  txtName->setValidator( new QRegExpValidator( QRegExp( "[^\\/]+" ), txtName ) );
 }
 
 QgsNewOgrConnection::~QgsNewOgrConnection()
@@ -89,7 +91,7 @@ void QgsNewOgrConnection::testConnection()
   OGRSFDriverH         pahDriver;
   CPLErrorReset();
   poDS = OGROpen( TO8F( uri ), false, &pahDriver );
-  if ( poDS == NULL )
+  if ( !poDS )
   {
     QMessageBox::information( this, tr( "Test connection" ), tr( "Connection failed - Check settings and try again.\n\nExtended error information:\n%1" ).arg( QString::fromUtf8( CPLGetLastErrorMsg() ) ) );
   }
@@ -104,7 +106,7 @@ void QgsNewOgrConnection::testConnection()
 void QgsNewOgrConnection::accept()
 {
   QSettings settings;
-  QString baseKey = "/" + cmbDatabaseTypes->currentText() + "/connections/";
+  QString baseKey = '/' + cmbDatabaseTypes->currentText() + "/connections/";
   settings.setValue( baseKey + "selected", txtName->text() );
 
   // warn if entry was renamed to an existing connection
@@ -140,4 +142,4 @@ void QgsNewOgrConnection::on_btnConnect_clicked()
   testConnection();
 }
 
-/** end  Autoconnected SLOTS **/
+/** End  Autoconnected SLOTS **/
