@@ -114,6 +114,14 @@ QgsPluginManager::QgsPluginManager( QWidget * parent, bool pluginsAreEnabled, Qt
   buttonUninstall->hide();
   frameSettings->setHidden( true );
 
+  voteRating->hide();
+  voteLabel->hide();
+  voteSlider->hide();
+  voteSubmit->hide();
+#ifndef WITH_QTWEBKIT
+  connect( voteSubmit, SIGNAL( clicked() ), this, SLOT( submitVote() ) );
+#endif
+
   // Init the message bar instance
   msgBar = new QgsMessageBar( this );
   msgBar->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Fixed );
@@ -343,7 +351,7 @@ void QgsPluginManager::getCppPluginsMetadata()
       }
       else
       {
-        QgsDebugMsg( "dlopen suceeded for " + lib );
+        QgsDebugMsg( "dlopen succeeded for " + lib );
         dlclose( handle );
       }
 #endif //#ifndef Q_OS_WIN && Q_OS_MACX
@@ -491,7 +499,7 @@ void QgsPluginManager::reloadModelData()
       QString status = it->value( "status" );
       QString error = it->value( "error" );
 
-      QStandardItem * mypDetailItem = new QStandardItem( pluginName.left( 32 ) );
+      QStandardItem * mypDetailItem = new QStandardItem( pluginName );
 
       mypDetailItem->setData( baseName, PLUGIN_BASE_NAME_ROLE );
       mypDetailItem->setData( status, PLUGIN_STATUS_ROLE );
@@ -502,6 +510,7 @@ void QgsPluginManager::reloadModelData()
       mypDetailItem->setData( it->value( "downloads" ).rightJustified( 10, '0' ), PLUGIN_DOWNLOADS_ROLE );
       mypDetailItem->setData( it->value( "zip_repository" ), PLUGIN_REPOSITORY_ROLE );
       mypDetailItem->setData( it->value( "average_vote" ), PLUGIN_VOTE_ROLE );
+      mypDetailItem->setData( it->value( "trusted" ), PLUGIN_TRUSTED_ROLE );
 
       if ( QFileInfo( iconPath ).isFile() )
       {
@@ -597,7 +606,10 @@ void QgsPluginManager::showPluginDetails( QStandardItem * item )
 {
   const QMap<QString, QString> * metadata = pluginMetadata( item->data( PLUGIN_BASE_NAME_ROLE ).toString() );
 
-  if ( ! metadata ) return;
+  if ( !metadata )
+  {
+    return;
+  }
 
   QString html = "";
   html += "<style>"
@@ -605,7 +617,7 @@ void QgsPluginManager::showPluginDetails( QStandardItem * item )
           "    padding:0px;"
           "    margin:0px;"
           "    font-family:verdana;"
-          "    font-size: 12px;"
+          "    font-size: 10pt;"
           "  }"
           "  div#votes {"
           "    width:360px;"
@@ -614,8 +626,9 @@ void QgsPluginManager::showPluginDetails( QStandardItem * item )
           "  }"
           "</style>";
 
-  if ( ! metadata->value( "plugin_id" ).isEmpty() )
+  if ( !metadata->value( "plugin_id" ).isEmpty() )
   {
+#ifdef WITH_QTWEBKIT
     html += QString(
               "<style>"
               "  div#stars_bg {"
@@ -670,9 +683,31 @@ void QgsPluginManager::showPluginDetails( QStandardItem * item )
               "    document.getElementById('send_vote_trigger').dispatchEvent(ev);"
               "  }"
               "</script>" ).arg( metadata->value( "plugin_id" ) );
+#else
+    voteRating->show();
+    voteLabel->show();
+    voteSlider->show();
+    voteSubmit->show();
+    QgsDebugMsg( QString( "vote slider:%1" ).arg( qRound( metadata->value( "average_vote" ).toFloat() ) ) );
+    voteSlider->setValue( qRound( metadata->value( "average_vote" ).toFloat() ) );
+    mCurrentPluginId = metadata->value( "plugin_id" ).toInt();
+  }
+  else
+  {
+    voteRating->hide();
+    voteLabel->hide();
+    voteSlider->hide();
+    voteSubmit->hide();
+    mCurrentPluginId = -1;
+#endif
   }
 
+#ifdef WITH_QTWEBKIT
   html += "<body onload='ready()'>";
+#else
+  html += "<body>";
+#endif
+
 
   // First prepare message box(es)
   if ( ! metadata->value( "error" ).isEmpty() )
@@ -694,24 +729,28 @@ void QgsPluginManager::showPluginDetails( QStandardItem * item )
                      "  <tr><td width=\"100%\" style=\"color:#CC0000\">%1</td></tr>"
                      "</table>" ).arg( errorMsg );
   }
+
   if ( metadata->value( "status" ) == "upgradeable" )
   {
     html += QString( "<table bgcolor=\"#FFFFAA\" cellspacing=\"2\" cellpadding=\"6\" width=\"100%\">"
                      "  <tr><td width=\"100%\" style=\"color:#880000\"><b>%1</b></td></tr>"
                      "</table>" ).arg( tr( "There is a new version available" ) );
   }
+
   if ( metadata->value( "status" ) == "new" )
   {
     html += QString( "<table bgcolor=\"#CCFFCC\" cellspacing=\"2\" cellpadding=\"6\" width=\"100%\">"
                      "  <tr><td width=\"100%\" style=\"color:#008800\"><b>%1</b></td></tr>"
                      "</table>" ).arg( tr( "This is a new plugin" ) );
   }
+
   if ( metadata->value( "status" ) == "newer" )
   {
     html += QString( "<table bgcolor=\"#FFFFCC\" cellspacing=\"2\" cellpadding=\"6\" width=\"100%\">"
                      "  <tr><td width=\"100%\" style=\"color:#550000\"><b>%1</b></td></tr>"
                      "</table>" ).arg( tr( "Installed version of this plugin is higher than any version found in repository" ) );
   }
+
   if ( metadata->value( "experimental" ) == "true" )
   {
     html += QString( "<table bgcolor=\"#EEEEBB\" cellspacing=\"2\" cellpadding=\"2\" width=\"100%\">"
@@ -720,6 +759,7 @@ void QgsPluginManager::showPluginDetails( QStandardItem * item )
                      "  </td></tr>"
                      "</table>" ).arg( tr( "This plugin is experimental" ) );
   }
+
   if ( metadata->value( "deprecated" ) == "true" )
   {
     html += QString( "<table bgcolor=\"#EEBBCC\" cellspacing=\"2\" cellpadding=\"2\" width=\"100%\">"
@@ -729,6 +769,15 @@ void QgsPluginManager::showPluginDetails( QStandardItem * item )
                      "</table>" ).arg( tr( "This plugin is deprecated" ) );
   }
 
+  if ( metadata->value( "trusted" ) == "true" )
+  {
+    html += QString( "<table bgcolor=\"#90EE90\" cellspacing=\"2\" cellpadding=\"2\" width=\"100%\">"
+                     "  <tr><td width=\"100%\" style=\"color:#660000\">"
+                     "    <img src=\"qrc:/images/themes/default/mIconSuccess.png\" width=\"32\"><b>%1</b>"
+                     "  </td></tr>"
+                     "</table>" ).arg( tr( "This plugin is trusted" ) );
+  }
+
   // Now the metadata
 
   html += "<table cellspacing=\"4\" width=\"100%\"><tr><td>";
@@ -758,23 +807,33 @@ void QgsPluginManager::showPluginDetails( QStandardItem * item )
   }
 
   html += "<br/><br/>";
-  html += "<div id='stars_bg'/><div id='stars'/>";
-  html += "<div id='votes'>";
+
+  QString votes;
+#ifndef WITH_QTWEBKIT
+  votes += tr( "Average rating %1" ).arg( metadata->value( "average_vote" ).toFloat(), 0, 'f', 1 );
+#endif
   if ( ! metadata->value( "rating_votes" ).isEmpty() )
   {
-    html += tr( "%1 rating vote(s)" ).arg( metadata->value( "rating_votes" ) );
-  }
-  if ( !( metadata->value( "rating_votes" ).isEmpty() || metadata->value( "downloads" ).isEmpty() ) )
-  {
-    html += ", ";
+    if ( !votes.isEmpty() )
+      votes += ", ";
+    votes += tr( "%1 rating vote(s)" ).arg( metadata->value( "rating_votes" ) );
   }
   if ( ! metadata->value( "downloads" ).isEmpty() )
   {
-    html += tr( "%1 downloads" ).arg( metadata->value( "downloads" ) );
+    if ( !votes.isEmpty() )
+      votes += ", ";
+    votes += tr( "%1 downloads" ).arg( metadata->value( "downloads" ) );
   }
+
+#ifdef WITH_QTWEBKIT
+  html += "<div id='stars_bg'/><div id='stars'/>";
+  html += "<div id='votes'>";
+  html += votes;
   html += "</div>";
   html += "<div><a id='send_vote_trigger'/></div>";
-
+#else
+  voteRating->setText( votes );
+#endif
   html += "</td></tr><tr><td>";
   html += "<br/>";
 
@@ -795,7 +854,7 @@ void QgsPluginManager::showPluginDetails( QStandardItem * item )
     }
     if ( ! metadata->value( "tracker" ).isEmpty() )
     {
-      html += QString( "<a href='%1'>%2</a> &nbsp; " ).arg( metadata->value( "tracker" ), tr( "tracker" ) );
+      html += QString( "<a href='%1'>%2</a> &nbsp; " ).arg( metadata->value( "tracker" ), tr( "bug_tracker" ) );
     }
     if ( ! metadata->value( "code_repository" ).isEmpty() )
     {
@@ -1074,11 +1133,11 @@ void QgsPluginManager::setCurrentTab( int idx )
     if ( it != mTabDescriptions.constEnd() )
     {
       tabInfoHTML += "<style>"
-                     "body, table {"
-                     "margin:4px;"
-                     "font-family:verdana;"
-                     "font-size: 12px;"
-                     "}"
+                     "  body, p {"
+                     "      margin: 2px;"
+                     "      font-family: verdana;"
+                     "      font-size: 10pt;"
+                     "  }"
                      "</style>";
       // tabInfoHTML += "<style>" + QgsApplication::reportStyleSheet() + "</style>";
       tabInfoHTML += it.value();
@@ -1149,7 +1208,29 @@ void QgsPluginManager::on_vwPlugins_doubleClicked( const QModelIndex & theIndex
   }
 }
 
+#ifndef WITH_QTWEBKIT
+void QgsPluginManager::submitVote()
+{
+  if ( mCurrentPluginId < 0 )
+    return;
 
+  sendVote( mCurrentPluginId, voteSlider->value() );
+}
+#endif
+
+void QgsPluginManager::sendVote( int pluginId, int vote )
+{
+  QString response;
+  QgsPythonRunner::eval( QString( "pyplugin_installer.instance().sendVote('%1', '%2')" ).arg( pluginId ).arg( vote ), response );
+  if ( response == "True" )
+  {
+    pushMessage( tr( "Vote sent successfully" ), QgsMessageBar::INFO );
+  }
+  else
+  {
+    pushMessage( tr( "Sending vote to the plugin repository failed." ), QgsMessageBar::WARNING );
+  }
+}
 
 void QgsPluginManager::on_wvDetails_linkClicked( const QUrl & url )
 {
@@ -1159,17 +1240,7 @@ void QgsPluginManager::on_wvDetails_linkClicked( const QUrl & url )
     {
       QString params = url.path();
       QString response;
-      QgsPythonRunner::eval( QString( "pyplugin_installer.instance().sendVote('%1', '%2')" )
-                             .arg( params.split( '/' )[1],
-                                   params.split( '/' )[2] ), response );
-      if ( response == "True" )
-      {
-        pushMessage( tr( "Vote sent successfully" ), QgsMessageBar::INFO );
-      }
-      else
-      {
-        pushMessage( tr( "Sending vote to the plugin repository failed." ), QgsMessageBar::WARNING );
-      }
+      sendVote( params.split( '/' )[1].toInt(), params.split( '/' )[2].toInt() );
     }
   }
   else
@@ -1179,7 +1250,6 @@ void QgsPluginManager::on_wvDetails_linkClicked( const QUrl & url )
 }
 
 
-
 void QgsPluginManager::on_leFilter_textChanged( QString theText )
 {
   if ( theText.startsWith( "tag:", Qt::CaseInsensitive ) )
