@@ -45,8 +45,11 @@ class QgsComposer;
 class QgsComposerManager;
 class QgsComposerView;
 class QgsStatusBarCoordinatesWidget;
+class QgsStatusBarMagnifierWidget;
+class QgsStatusBarScaleWidget;
 class QgsContrastEnhancement;
 class QgsCustomLayerOrderWidget;
+class QgsDockWidget;
 class QgsDoubleSpinBox;
 class QgsFeature;
 class QgsGeometry;
@@ -54,6 +57,7 @@ class QgsLayerTreeMapCanvasBridge;
 class QgsLayerTreeView;
 class QgsMapCanvas;
 class QgsMapLayer;
+class QgsMapLayerConfigWidgetFactory;
 class QgsMapTip;
 class QgsMapTool;
 class QgsMapToolAdvancedDigitizing;
@@ -78,6 +82,7 @@ class QNetworkProxy;
 class QAuthenticator;
 
 class QgsBrowserDockWidget;
+class QgsBrowserModel;
 class QgsAdvancedDigitizingDockWidget;
 class QgsSnappingDialog;
 class QgsGPSInformationWidget;
@@ -89,11 +94,13 @@ class QgsDecorationItem;
 class QgsMessageLogViewer;
 class QgsMessageBar;
 
-class QgsScaleComboBox;
-
 class QgsDataItem;
 class QgsTileScaleWidget;
 
+class QgsLabelingWidget;
+class QgsLayerStylingWidget;
+class QgsDiagramProperties;
+
 #include <QMainWindow>
 #include <QToolBar>
 #include <QAbstractSocket>
@@ -112,6 +119,8 @@ class QgsTileScaleWidget;
 #include "qgsmessagebar.h"
 #include "qgsbookmarks.h"
 #include "qgswelcomepageitemsmodel.h"
+#include "qgsruntimeprofiler.h"
+
 
 #include "ui_qgisapp.h"
 
@@ -241,7 +250,7 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
      * After adding the dock widget to the ui (by delegating to the QMainWindow
      * parent class, it will also add it to the View menu list of docks.*/
     void addDockWidget( Qt::DockWidgetArea area, QDockWidget *dockwidget );
-    void removeDockWidget( QDockWidget *dockwidget );
+    void removeDockWidget( QDockWidget* dockwidget );
     /** Add a toolbar to the main window. Overloaded from QMainWindow.
      * After adding the toolbar to the ui (by delegating to the QMainWindow
      * parent class, it will also add it to the View menu list of toolbars.*/
@@ -354,6 +363,8 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     QAction *actionAddWmsLayer() { return mActionAddWmsLayer; }
     QAction *actionAddWcsLayer() { return mActionAddWcsLayer; }
     QAction *actionAddWfsLayer() { return mActionAddWfsLayer; }
+    QAction* actionAddAfsLayer() { return mActionAddAfsLayer; }
+    QAction* actionAddAmsLayer() { return mActionAddAmsLayer; }
     QAction *actionCopyLayerStyle() { return mActionCopyStyle; }
     QAction *actionPasteLayerStyle() { return mActionPasteStyle; }
     QAction *actionOpenTable() { return mActionOpenTable; }
@@ -496,6 +507,12 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
 
     void parseVersionInfo( QNetworkReply* reply, int& latestVersion, QStringList& versionInfo );
 
+    /** Register a new tab in the layer properties dialog */
+    void registerMapLayerPropertiesFactory( QgsMapLayerConfigWidgetFactory* factory );
+
+    /** Unregister a previously registered tab in the layer properties dialog */
+    void unregisterMapLayerPropertiesFactory( QgsMapLayerConfigWidgetFactory* factory );
+
   public slots:
     void layerTreeViewDoubleClicked( const QModelIndex& index );
     //! Make sure the insertion point for new layers is up-to-date with the current item in layer tree view
@@ -610,7 +627,6 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
 
     //! copies features to internal clipboard
     void copyFeatures( QgsFeatureStore & featureStore );
-    void loadOGRSublayers( const QString& layertype, const QString& uri, const QStringList& list );
     void loadGDALSublayers( const QString& uri, const QStringList& list );
 
     /** Deletes the selected attributes for the currently selected vector layer*/
@@ -643,7 +659,7 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     void toolButtonActionTriggered( QAction * );
 
     //! layer selection changed
-    void legendLayerSelectionChanged( void );
+    void legendLayerSelectionChanged();
 
     //! Watch for QFileOpenEvent.
     virtual bool event( QEvent *event ) override;
@@ -656,6 +672,10 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
 
     void addWfsLayer( const QString& uri, const QString& typeName );
 
+    void addAfsLayer( const QString& uri, const QString& typeName );
+
+    void addAmsLayer( const QString& uri, const QString& typeName );
+
     void versionReplyFinished();
 
     QgsMessageLogViewer *logViewer() { return mLogViewer; }
@@ -672,9 +692,6 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
 
     QMenu *panelMenu() { return mPanelMenu; }
 
-    bool autoTransaction() const;
-    void setAutoTransaction( bool state );
-
   protected:
 
     //! Handle state changes (WindowTitleChange)
@@ -717,10 +734,10 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     void addSelectedVectorLayer( const QString& uri, const QString& layerName, const QString& provider );
     //! Replace the selected layer by a vector layer defined by uri, layer name, data source uri
     void replaceSelectedVectorLayer( const QString& oldId, const QString& uri, const QString& layerName, const QString& provider );
-    //#ifdef HAVE_MSSQL
     //! Add a MSSQL layer to the map
     void addMssqlLayer();
-    //#endif
+    //! Add a DB2 layer to the map
+    void addDb2Layer();
     //#ifdef HAVE_ORACLE
     //! Add a Oracle layer to the map
     void addOracleLayer();
@@ -733,8 +750,6 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     void saveLastMousePosition( const QgsPoint & );
     //! Slot to show current map scale;
     void showScale( double theScale );
-    //! Slot to handle user scale input;
-    void userScale();
     //! Slot to handle user rotation input;
     //! @note added in 2.8
     void userRotation();
@@ -742,8 +757,10 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     void removeLayer();
     //! Duplicate map layer(s) in legend
     void duplicateLayers( const QList<QgsMapLayer *>& lyrList = QList<QgsMapLayer *>() );
-    //! Set Scale visibility of selected layers
+    //! Set scale visibility of selected layers
     void setLayerScaleVisibility();
+    //! Zoom to nearest scale such that current layer is visible
+    void zoomToLayerScale();
     //! Set CRS of a layer
     void setLayerCRS();
     //! Assign layer CRS to project
@@ -906,6 +923,8 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     void fileSaveAs();
     //! Export project in dxf format
     void dxfExport();
+    //! Import layers in dwg format
+    void dwgImport();
     //! Open the project file corresponding to the
     //! text)= of the given action.
     void openProject( QAction *action );
@@ -944,6 +963,8 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     void newMemoryLayer();
     //! Create a new empty spatialite layer
     void newSpatialiteLayer();
+    //! Create a new empty GeoPackage layer
+    void newGeoPackageLayer();
     //! Print the current map view frame
     void newPrintComposer();
     void showComposerManager();
@@ -1028,10 +1049,14 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     void mergeSelectedFeatures();
     //! merges the attributes of selected features
     void mergeAttributesOfSelectedFeatures();
+    //! Modifies the attributes of selected features via feature form
+    void modifyAttributesOfSelectedFeatures();
     //! provides operations with nodes
     void nodeTool();
     //! activates the rotate points tool
     void rotatePointSymbols();
+    //! activates the offset point symbol tool
+    void offsetPointSymbol();
     //! shows the snapping Options
     void snappingOptions();
 
@@ -1059,6 +1084,9 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     //! select features by expression
     void selectByExpression();
 
+    //! select features by form
+    void selectByForm();
+
     //! refresh map canvas
     void refreshMapCanvas();
 
@@ -1088,6 +1116,9 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     /** Called when some layer's editing mode was toggled on/off */
     void layerEditStateChanged();
 
+    /** Update the label toolbar buttons */
+    void updateLabelToolButtons();
+
     /** Activates or deactivates actions depending on the current maplayer type.
     Is called from the legend when the current legend item has changed*/
     void activateDeactivateLayerRelatedActions( QgsMapLayer *layer );
@@ -1117,6 +1148,10 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     void addWcsLayer();
     //! Add a WFS layer to the map
     void addWfsLayer();
+    //! Add a ArcGIS FeatureServer layer to the map
+    void addAfsLayer();
+    //! Add a ArcGIS MapServer layer to the map
+    void addAmsLayer();
     //! Set map tool to Zoom out
     void zoomOut();
     //! Set map tool to Zoom in
@@ -1163,6 +1198,12 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     //! shows label settings dialog (for labeling-ng)
     void labeling();
 
+    //! shows the map styling dock
+    void mapStyleDock( bool enabled );
+
+    //! diagrams properties
+    void diagramProperties();
+
     //! set the CAD dock widget visible
     void setCadDockVisible( bool visible );
 
@@ -1184,7 +1225,7 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     void oldProjectVersionWarning( const QString& );
 
     //! Toggle map tips on/off
-    void toggleMapTips();
+    void toggleMapTips( bool enabled );
 
     //! Show the map tip
     void showMapTip();
@@ -1281,6 +1322,12 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     /** Pushes a layer error to the message bar */
     void onLayerError( const QString& msg );
 
+    /** Set the layer for the map style dock. Doesn't show the style dock */
+    void setMapStyleDockLayer( QgsMapLayer *layer );
+
+    //! Handles processing of dropped mimedata
+    void dropEventTimeout();
+
   signals:
     /** Emitted when a key is pressed and we want non widget sublasses to be able
       to pick up on this (e.g. maplayer) */
@@ -1330,6 +1377,12 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     void layerSavedAs( QgsMapLayer* l, const QString& path );
 
   private:
+    void startProfile( const QString &name );
+    void endProfile();
+    void functionProfile( void ( QgisApp::*fnc )(), QgisApp *instance, QString name );
+
+    QgsRuntimeProfiler* mProfiler;
+
     /** This method will open a dialog so the user can select GDAL sublayers to load
      * @returns true if any items were loaded
      */
@@ -1420,6 +1473,9 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     /** Copy a vector style from a layer to another one, if they have the same geometry type */
     void duplicateVectorStyle( QgsVectorLayer* srcLayer, QgsVectorLayer* destLayer );
 
+    //! Loads the list of recent projects from settings
+    void readRecentProjects();
+
     QgisAppStyleSheet *mStyleSheetBuilder;
 
     // actions for menus and toolbars -----------------
@@ -1450,11 +1506,11 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     QMenu *mToolbarMenu;
 
     // docks ------------------------------------------
-    QDockWidget *mLayerTreeDock;
-    QDockWidget *mLayerOrderDock;
-    QDockWidget *mOverviewDock;
-    QDockWidget *mpGpsDock;
-    QDockWidget *mLogDock;
+    QgsDockWidget *mLayerTreeDock;
+    QgsDockWidget *mLayerOrderDock;
+    QgsDockWidget *mOverviewDock;
+    QgsDockWidget *mpGpsDock;
+    QgsDockWidget *mLogDock;
 
 #ifdef Q_OS_MAC
     //! Window menu action to select this window
@@ -1501,6 +1557,7 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
             , mDeletePart( nullptr )
             , mNodeTool( nullptr )
             , mRotatePointSymbolsTool( nullptr )
+            , mOffsetPointSymbolTool( nullptr )
             , mAnnotation( nullptr )
             , mFormAnnotation( nullptr )
             , mHtmlAnnotation( nullptr )
@@ -1549,6 +1606,7 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
         QgsMapTool *mDeletePart;
         QgsMapTool *mNodeTool;
         QgsMapTool *mRotatePointSymbolsTool;
+        QgsMapTool *mOffsetPointSymbolTool;
         QgsMapTool *mAnnotation;
         QgsMapTool *mFormAnnotation;
         QgsMapTool *mHtmlAnnotation;
@@ -1564,12 +1622,10 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
 
     QgsMapTool *mNonEditMapTool;
 
-    //! Widget that will live on the statusbar to display "scale 1:"
-    QLabel *mScaleLabel;
-    //! Widget that will live on the statusbar to display scale value
-    QgsScaleComboBox *mScaleEdit;
-    //! The validator for the mScaleEdit
-    QValidator * mScaleEditValidator;
+    QgsStatusBarScaleWidget* mScaleWidget;
+
+    //! zoom widget
+    QgsStatusBarMagnifierWidget *mMagnifierWidget;
 
     //! Widget that will live in the statusbar to display and edit coords
     QgsStatusBarCoordinatesWidget *mCoordsEdit;
@@ -1667,7 +1723,9 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     static QgisApp *smInstance;
 
     QgsUndoWidget *mUndoWidget;
+    QgsDockWidget *mUndoDock;
 
+    QgsBrowserModel *mBrowserModel;
     QgsBrowserDockWidget *mBrowserWidget;
     QgsBrowserDockWidget *mBrowserWidget2;
 
@@ -1678,12 +1736,11 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     QgsSnappingDialog *mSnappingDialog;
 
     QgsPluginManager *mPluginManager;
+    QgsDockWidget *mMapStylingDock;
+    QgsLayerStylingWidget* mMapStyleWidget;
 
     QgsComposerManager *mComposerManager;
 
-    //! map of transaction group: QPair( providerKey, connString ) -> transactionGroup
-    QMap< QPair< QString, QString>, QgsTransactionGroup*> mTransactionGroups;
-
     //! Persistent tile scale slider
     QgsTileScaleWidget *mpTileScaleWidget;
 
@@ -1719,11 +1776,14 @@ class APP_EXPORT QgisApp : public QMainWindow, private Ui::MainWindow
     QgsMapCanvasTracer* mTracer;
 
     QAction* mActionFilterLegend;
+    QAction* mActionStyleDock;
 
     QgsLegendFilterButton* mLegendExpressionFilterButton;
 
     QgsSnappingUtils* mSnappingUtils;
 
+    QList<QgsMapLayerConfigWidgetFactory*> mMapLayerPanelFactories;
+
     QDateTime mProjectLastModified;
 
     QgsWelcomePage* mWelcomePage;
