@@ -14,29 +14,31 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <QDockWidget>
 #include <QMessageBox>
+#include <QGridLayout>
+#include <QDialogButtonBox>
 
 #include "qgsattributetabledialog.h"
 #include "qgsattributetablemodel.h"
 #include "qgsattributetablefiltermodel.h"
 #include "qgsattributetableview.h"
 
-#include <qgsapplication.h>
-#include <qgsvectordataprovider.h>
-#include <qgsvectorlayer.h>
-#include <qgsexpression.h>
-
+#include "qgsapplication.h"
+#include "qgsvectorlayer.h"
+#include "qgsvectordataprovider.h"
+#include "qgsexpression.h"
+#include "qgsexpressionbuilderwidget.h"
 #include "qgisapp.h"
 #include "qgsaddattrdialog.h"
 #include "qgsdelattrdialog.h"
+#include "qgsdockwidget.h"
 #include "qgssearchquerybuilder.h"
 #include "qgslogger.h"
 #include "qgsmapcanvas.h"
 #include "qgsproject.h"
 #include "qgsfieldcalculator.h"
 #include "qgsfeatureaction.h"
-#include "qgsattributeaction.h"
+#include "qgsactionmanager.h"
 #include "qgsexpressionbuilderdialog.h"
 #include "qgsmessagebar.h"
 #include "qgsexpressionselectiondialog.h"
@@ -44,6 +46,7 @@
 #include "qgsrubberband.h"
 #include "qgsfield.h"
 #include "qgseditorwidgetregistry.h"
+#include "qgsfieldproxymodel.h"
 
 static QgsExpressionContext _getExpressionContext( const void* context )
 {
@@ -62,6 +65,19 @@ static QgsExpressionContext _getExpressionContext( const void* context )
   return expContext;
 }
 
+void QgsAttributeTableDialog::updateMultiEditButtonState()
+{
+  if ( mLayer->editFormConfig()->layout() == QgsEditFormConfig::UiFileLayout )
+    return;
+
+  mActionToggleMultiEdit->setEnabled( mLayer->isEditable() );
+
+  if ( !mLayer->isEditable() || ( mLayer->isEditable() && mMainView->view() != QgsDualView::AttributeEditor ) )
+  {
+    mActionToggleMultiEdit->setChecked( false );
+  }
+}
+
 QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWidget *parent, Qt::WindowFlags flags )
     : QDialog( parent, flags )
     , mDock( nullptr )
@@ -71,30 +87,54 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWid
 {
   setupUi( this );
 
+  Q_FOREACH ( const QgsField& field, mLayer->fields() )
+  {
+    mVisibleFields.append( field.name() );
+  }
+
   // Fix selection color on loosing focus (Windows)
   setStyleSheet( QgisApp::instance()->styleSheet() );
 
   setAttribute( Qt::WA_DeleteOnClose );
 
+  layout()->setMargin( 0 );
+  layout()->setContentsMargins( 0, 0, 0, 0 );
+  static_cast< QGridLayout* >( layout() )->setVerticalSpacing( 0 );
+
   QSettings settings;
 
+  int size = settings.value( "/IconSize", 16 ).toInt();
+  if ( size > 32 )
+  {
+    size -= 16;
+  }
+  else if ( size == 32 )
+  {
+    size = 24;
+  }
+  else
+  {
+    size = 16;
+  }
+  mToolbar->setIconSize( QSize( size, size ) );
+
   // Initialize the window geometry
   restoreGeometry( settings.value( "/Windows/BetterAttributeTable/geometry" ).toByteArray() );
 
-  QgsAttributeEditorContext context;
-
   myDa = new QgsDistanceArea();
 
   myDa->setSourceCrs( mLayer->crs() );
   myDa->setEllipsoidalMode( QgisApp::instance()->mapCanvas()->mapSettings().hasCrsTransformEnabled() );
   myDa->setEllipsoid( QgsProject::instance()->readEntry( "Measure", "/Ellipsoid", GEO_NONE ) );
 
-  context.setDistanceArea( *myDa );
-  context.setVectorLayerTools( QgisApp::instance()->vectorLayerTools() );
+  mEditorContext.setDistanceArea( *myDa );
+  mEditorContext.setVectorLayerTools( QgisApp::instance()->vectorLayerTools() );
 
   QgsFeatureRequest r;
+  bool needsGeom = false;
+  QgsAttributeTableFilterModel::FilterMode initialMode = static_cast< QgsAttributeTableFilterModel::FilterMode>( settings.value( QString( "/qgis/attributeTableBehaviour" ), QgsAttributeTableFilterModel::ShowAll ).toInt() );
   if ( mLayer->geometryType() != QGis::NoGeometry &&
-       settings.value( "/qgis/attributeTableBehaviour", QgsAttributeTableFilterModel::ShowAll ).toInt() == QgsAttributeTableFilterModel::ShowVisible )
+       initialMode == QgsAttributeTableFilterModel::ShowVisible )
   {
     QgsMapCanvas *mc = QgisApp::instance()->mapCanvas();
     QgsRectangle extent( mc->mapSettings().mapToLayerCoordinates( theLayer, mc->extent() ) );
@@ -106,10 +146,21 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWid
     delete g;
 
     mActionShowAllFilter->setText( tr( "Show All Features In Initial Canvas Extent" ) );
+    needsGeom = true;
   }
+  else if ( initialMode == QgsAttributeTableFilterModel::ShowSelected )
+  {
+    if ( theLayer->selectedFeatureCount() > 0 )
+      r.setFilterFids( theLayer->selectedFeaturesIds() );
+  }
+  if ( !needsGeom )
+    r.setFlags( QgsFeatureRequest::NoGeometry );
 
   // Initialize dual view
-  mMainView->init( mLayer, QgisApp::instance()->mapCanvas(), r, context );
+  mMainView->init( mLayer, QgisApp::instance()->mapCanvas(), r, mEditorContext, false );
+
+  QgsAttributeTableConfig config = mLayer->attributeTableConfig();
+  mMainView->setAttributeTableConfig( config );
 
   // Initialize filter gui elements
   mFilterActionMapper = new QSignalMapper( this );
@@ -134,7 +185,7 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWid
   connect( mFilterActionMapper, SIGNAL( mapped( QObject* ) ), SLOT( filterColumnChanged( QObject* ) ) );
   connect( mFilterQuery, SIGNAL( returnPressed() ), SLOT( filterQueryAccepted() ) );
   connect( mActionApplyFilter, SIGNAL( triggered() ), SLOT( filterQueryAccepted() ) );
-  connect( mSetStyles, SIGNAL( pressed() ), SLOT( openConditionalStyles() ) );
+  connect( mActionSetStyles, SIGNAL( triggered() ), SLOT( openConditionalStyles() ) );
 
   // info from layer to table
   connect( mLayer, SIGNAL( editingStarted() ), this, SLOT( editingToggled() ) );
@@ -148,6 +199,8 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWid
 
   // connect table info to window
   connect( mMainView, SIGNAL( filterChanged() ), this, SLOT( updateTitle() ) );
+  connect( mMainView, SIGNAL( filterExpressionSet( QString, QgsAttributeForm::FilterType ) ), this, SLOT( formFilterSet( QString, QgsAttributeForm::FilterType ) ) );
+  connect( mMainView, SIGNAL( formModeChanged( QgsAttributeForm::Mode ) ), this, SLOT( viewModeChanged( QgsAttributeForm::Mode ) ) );
 
   // info from table to application
   connect( this, SIGNAL( saveEdits( QgsMapLayer * ) ), QgisApp::instance(), SLOT( saveEdits( QgsMapLayer * ) ) );
@@ -164,23 +217,24 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWid
   columnBoxInit();
   updateTitle();
 
-  mRemoveSelectionButton->setIcon( QgsApplication::getThemeIcon( "/mActionUnselectAttributes.png" ) );
-  mSelectAllButton->setIcon( QgsApplication::getThemeIcon( "/mActionSelectAll.svg" ) );
-  mSelectedToTopButton->setIcon( QgsApplication::getThemeIcon( "/mActionSelectedToTop.png" ) );
-  mCopySelectedRowsButton->setIcon( QgsApplication::getThemeIcon( "/mActionCopySelected.png" ) );
-  mZoomMapToSelectedRowsButton->setIcon( QgsApplication::getThemeIcon( "/mActionZoomToSelected.svg" ) );
-  mPanMapToSelectedRowsButton->setIcon( QgsApplication::getThemeIcon( "/mActionPanToSelected.svg" ) );
-  mInvertSelectionButton->setIcon( QgsApplication::getThemeIcon( "/mActionInvertSelection.svg" ) );
-  mToggleEditingButton->setIcon( QgsApplication::getThemeIcon( "/mActionToggleEditing.svg" ) );
-  mSaveEditsButton->setIcon( QgsApplication::getThemeIcon( "/mActionSaveEdits.svg" ) );
-  mDeleteSelectedButton->setIcon( QgsApplication::getThemeIcon( "/mActionDeleteSelected.svg" ) );
-  mOpenFieldCalculator->setIcon( QgsApplication::getThemeIcon( "/mActionCalculateField.png" ) );
-  mAddAttribute->setIcon( QgsApplication::getThemeIcon( "/mActionNewAttribute.png" ) );
-  mRemoveAttribute->setIcon( QgsApplication::getThemeIcon( "/mActionDeleteAttribute.png" ) );
+  mActionRemoveSelection->setIcon( QgsApplication::getThemeIcon( "/mActionDeselectAll.svg" ) );
+  mActionSelectAll->setIcon( QgsApplication::getThemeIcon( "/mActionSelectAll.svg" ) );
+  mActionSelectedToTop->setIcon( QgsApplication::getThemeIcon( "/mActionSelectedToTop.svg" ) );
+  mActionCopySelectedRows->setIcon( QgsApplication::getThemeIcon( "/mActionEditCopy.svg" ) );
+  mActionPasteFeatures->setIcon( QgsApplication::getThemeIcon( "/mActionEditPaste.svg" ) );
+  mActionZoomMapToSelectedRows->setIcon( QgsApplication::getThemeIcon( "/mActionZoomToSelected.svg" ) );
+  mActionPanMapToSelectedRows->setIcon( QgsApplication::getThemeIcon( "/mActionPanToSelected.svg" ) );
+  mActionInvertSelection->setIcon( QgsApplication::getThemeIcon( "/mActionInvertSelection.svg" ) );
+  mActionToggleEditing->setIcon( QgsApplication::getThemeIcon( "/mActionToggleEditing.svg" ) );
+  mActionSaveEdits->setIcon( QgsApplication::getThemeIcon( "/mActionSaveEdits.svg" ) );
+  mActionDeleteSelected->setIcon( QgsApplication::getThemeIcon( "/mActionDeleteSelected.svg" ) );
+  mActionOpenFieldCalculator->setIcon( QgsApplication::getThemeIcon( "/mActionCalculateField.svg" ) );
+  mActionAddAttribute->setIcon( QgsApplication::getThemeIcon( "/mActionNewAttribute.svg" ) );
+  mActionRemoveAttribute->setIcon( QgsApplication::getThemeIcon( "/mActionDeleteAttribute.svg" ) );
   mTableViewButton->setIcon( QgsApplication::getThemeIcon( "/mActionOpenTable.svg" ) );
-  mAttributeViewButton->setIcon( QgsApplication::getThemeIcon( "/mActionPropertyItem.png" ) );
-  mExpressionSelectButton->setIcon( QgsApplication::getThemeIcon( "/mIconExpressionSelect.svg" ) );
-  mAddFeature->setIcon( QgsApplication::getThemeIcon( "/mActionNewTableRow.png" ) );
+  mAttributeViewButton->setIcon( QgsApplication::getThemeIcon( "/mActionFormView.svg" ) );
+  mActionExpressionSelect->setIcon( QgsApplication::getThemeIcon( "/mIconExpressionSelect.svg" ) );
+  mActionAddFeature->setIcon( QgsApplication::getThemeIcon( "/mActionNewTableRow.svg" ) );
 
   // toggle editing
   bool canChangeAttributes = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::ChangeAttributeValues;
@@ -189,18 +243,25 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWid
   bool canDeleteAttributes = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteAttributes;
   bool canAddFeatures = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::AddFeatures;
 
-  mToggleEditingButton->blockSignals( true );
-  mToggleEditingButton->setCheckable( true );
-  mToggleEditingButton->setChecked( mLayer->isEditable() );
-  mToggleEditingButton->setEnabled(( canChangeAttributes || canDeleteFeatures || canAddAttributes || canDeleteAttributes || canAddFeatures ) && !mLayer->isReadOnly() );
-  mToggleEditingButton->blockSignals( false );
+  mActionToggleEditing->blockSignals( true );
+  mActionToggleEditing->setCheckable( true );
+  mActionToggleEditing->setChecked( mLayer->isEditable() );
+  mActionToggleEditing->setEnabled(( canChangeAttributes || canDeleteFeatures || canAddAttributes || canDeleteAttributes || canAddFeatures ) && !mLayer->readOnly() );
+  mActionToggleEditing->blockSignals( false );
 
-  mSaveEditsButton->setEnabled( mToggleEditingButton->isEnabled() && mLayer->isEditable() );
-  mReloadButton->setEnabled( ! mLayer->isEditable() );
-  mAddAttribute->setEnabled(( canChangeAttributes || canAddAttributes ) && mLayer->isEditable() );
-  mDeleteSelectedButton->setEnabled( canDeleteFeatures && mLayer->isEditable() );
-  mAddFeature->setEnabled( canAddFeatures && mLayer->isEditable() );
-  mAddFeature->setHidden( !canAddFeatures );
+  mActionSaveEdits->setEnabled( mActionToggleEditing->isEnabled() && mLayer->isEditable() );
+  mActionReload->setEnabled( ! mLayer->isEditable() );
+  mActionAddAttribute->setEnabled(( canChangeAttributes || canAddAttributes ) && mLayer->isEditable() );
+  mActionRemoveAttribute->setEnabled( canDeleteAttributes && mLayer->isEditable() );
+  mActionDeleteSelected->setEnabled( canDeleteFeatures && mLayer->isEditable() );
+  if ( !canDeleteFeatures )
+    mToolbar->removeAction( mActionDeleteSelected );
+  mActionAddFeature->setEnabled( canAddFeatures && mLayer->isEditable() );
+  if ( !canAddFeatures )
+    mToolbar->removeAction( mActionAddFeature );
+
+  if ( canDeleteFeatures || canAddFeatures )
+    mToolbar->insertSeparator( mActionExpressionSelect );
 
   mMainViewButtonGroup->setId( mTableViewButton, QgsDualView::AttributeTable );
   mMainViewButtonGroup->setId( mAttributeViewButton, QgsDualView::AttributeEditor );
@@ -225,10 +286,9 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWid
   }
 
   mUpdateExpressionText->registerGetExpressionContextCallback( &_getExpressionContext, mLayer );
+  mFieldCombo->setFilters( QgsFieldProxyModel::All | QgsFieldProxyModel::HideReadOnly );
+  mFieldCombo->setLayer( mLayer );
 
-  mFieldModel = new QgsFieldModel( this );
-  mFieldModel->setLayer( mLayer );
-  mFieldCombo->setModel( mFieldModel );
   connect( mRunFieldCalc, SIGNAL( clicked() ), this, SLOT( updateFieldFromExpression() ) );
   connect( mRunFieldCalcSelected, SIGNAL( clicked() ), this, SLOT( updateFieldFromExpressionSelected() ) );
   // NW TODO Fix in 2.6 - Doesn't work with field model for some reason.
@@ -237,7 +297,26 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWid
   mUpdateExpressionText->setLayer( mLayer );
   mUpdateExpressionText->setLeftHandButtonStyle( true );
 
-  mMainView->setView( QgsDualView::AttributeTable );
+  int initialView = settings.value( "/qgis/attributeTableView", -1 ).toInt();
+  if ( initialView < 0 )
+  {
+    initialView = settings.value( "/qgis/attributeTableLastView", QgsDualView::AttributeTable ).toInt();
+  }
+  mMainView->setView( static_cast< QgsDualView::ViewMode >( initialView ) );
+  mMainViewButtonGroup->button( initialView )->setChecked( true );
+
+  connect( mActionToggleMultiEdit, SIGNAL( toggled( bool ) ), mMainView, SLOT( setMultiEditEnabled( bool ) ) );
+  connect( mActionSearchForm, SIGNAL( toggled( bool ) ), mMainView, SLOT( toggleSearchMode( bool ) ) );
+  updateMultiEditButtonState();
+
+  if ( mLayer->editFormConfig()->layout() == QgsEditFormConfig::UiFileLayout )
+  {
+    //not supported with custom UI
+    mActionToggleMultiEdit->setEnabled( false );
+    mActionToggleMultiEdit->setToolTip( tr( "Multiedit is not supported when using custom UI forms" ) );
+    mActionSearchForm->setEnabled( false );
+    mActionSearchForm->setToolTip( tr( "Search is not supported when using custom UI forms" ) );
+  }
 
   editingToggled();
 }
@@ -253,7 +332,7 @@ void QgsAttributeTableDialog::updateTitle()
   QWidget *w = mDock ? qobject_cast<QWidget*>( mDock ) : qobject_cast<QWidget*>( this );
   w->setWindowTitle( tr( " %1 :: Features total: %2, filtered: %3, selected: %4%5" )
                      .arg( mLayer->name() )
-                     .arg( mMainView->featureCount() )
+                     .arg( qMax( static_cast< long >( mMainView->featureCount() ), mLayer->featureCount() ) ) // layer count may be estimated, so use larger of the two
                      .arg( mMainView->filteredFeatureCount() )
                      .arg( mLayer->selectedFeatureCount() )
                      .arg( mRubberBand ? tr( ", spatially limited" ) : "" )
@@ -289,7 +368,7 @@ void QgsAttributeTableDialog::keyPressEvent( QKeyEvent* event )
 {
   QDialog::keyPressEvent( event );
 
-  if (( event->key() == Qt::Key_Backspace || event->key() == Qt::Key_Delete ) && mDeleteSelectedButton->isEnabled() )
+  if (( event->key() == Qt::Key_Backspace || event->key() == Qt::Key_Delete ) && mActionDeleteSelected->isEnabled() )
   {
     QgisApp::instance()->deleteSelected( mLayer, this );
   }
@@ -342,13 +421,24 @@ void QgsAttributeTableDialog::updateFieldFromExpression()
 {
   bool filtered = mMainView->filterMode() != QgsAttributeTableFilterModel::ShowAll;
   QgsFeatureIds filteredIds = filtered ? mMainView->filteredFeatures() : QgsFeatureIds();
-  runFieldCalculation( mLayer, mFieldCombo->currentText(), mUpdateExpressionText->asExpression(), filteredIds );
+  runFieldCalculation( mLayer, mFieldCombo->currentField(), mUpdateExpressionText->asExpression(), filteredIds );
 }
 
 void QgsAttributeTableDialog::updateFieldFromExpressionSelected()
 {
   QgsFeatureIds filteredIds = mLayer->selectedFeaturesIds();
-  runFieldCalculation( mLayer, mFieldCombo->currentText(), mUpdateExpressionText->asExpression(), filteredIds );
+  runFieldCalculation( mLayer, mFieldCombo->currentField(), mUpdateExpressionText->asExpression(), filteredIds );
+}
+
+void QgsAttributeTableDialog::viewModeChanged( QgsAttributeForm::Mode mode )
+{
+  if ( mode != QgsAttributeForm::SearchMode )
+    mActionSearchForm->setChecked( false );
+}
+
+void QgsAttributeTableDialog::formFilterSet( const QString& filter, QgsAttributeForm::FilterType type )
+{
+  setFilterExpression( filter, type, true );
 }
 
 void QgsAttributeTableDialog::runFieldCalculation( QgsVectorLayer* layer, const QString& fieldName, const QString& expression, const QgsFeatureIds& filteredIds )
@@ -357,8 +447,7 @@ void QgsAttributeTableDialog::runFieldCalculation( QgsVectorLayer* layer, const
 
   mLayer->beginEditCommand( "Field calculator" );
 
-  QModelIndex modelindex = mFieldModel->indexFromName( fieldName );
-  int fieldindex = modelindex.data( QgsFieldModel::FieldIndexRole ).toInt();
+  int fieldindex = layer->fields().indexFromName( fieldName );
 
   bool calculationSuccess = true;
   QString error;
@@ -419,10 +508,17 @@ void QgsAttributeTableDialog::runFieldCalculation( QgsVectorLayer* layer, const
   {
     QMessageBox::critical( nullptr, tr( "Error" ), tr( "An error occurred while evaluating the calculation string:\n%1" ).arg( error ) );
     mLayer->destroyEditCommand();
-    return;
   }
+  else
+  {
+    mLayer->endEditCommand();
 
-  mLayer->endEditCommand();
+    // refresh table with updated values
+    // fixes https://issues.qgis.org/issues/17312
+    QgsAttributeTableModel* masterModel = mMainView->masterModel();
+    int modelColumn = masterModel->fieldCol( fieldindex );
+    masterModel->reload( masterModel->index( 0, modelColumn ), masterModel->index( masterModel->rowCount() - 1, modelColumn ) );
+  }
 }
 
 void QgsAttributeTableDialog::replaceSearchWidget( QWidget* oldw, QWidget* neww )
@@ -452,7 +548,7 @@ void QgsAttributeTableDialog::filterColumnChanged( QObject* filterAction )
   const QString widgetType = mLayer->editFormConfig()->widgetType( fldIdx );
   const QgsEditorWidgetConfig widgetConfig = mLayer->editFormConfig()->widgetConfig( fldIdx );
   mCurrentSearchWidgetWrapper = QgsEditorWidgetRegistry::instance()->
-                                createSearchWidget( widgetType, mLayer, fldIdx, widgetConfig, mFilterContainer );
+                                createSearchWidget( widgetType, mLayer, fldIdx, widgetConfig, mFilterContainer, mEditorContext );
   if ( mCurrentSearchWidgetWrapper->applyDirectly() )
   {
     connect( mCurrentSearchWidgetWrapper, SIGNAL( expressionChanged( QString ) ), SLOT( filterQueryChanged( QString ) ) );
@@ -487,7 +583,7 @@ void QgsAttributeTableDialog::filterExpressionBuilder()
 
   if ( dlg.exec() == QDialog::Accepted )
   {
-    setFilterExpression( dlg.expressionText() );
+    setFilterExpression( dlg.expressionText(), QgsAttributeForm::ReplaceFilter, true );
   }
 }
 
@@ -496,6 +592,7 @@ void QgsAttributeTableDialog::filterShowAll()
   mFilterButton->setDefaultAction( mActionShowAllFilter );
   mFilterButton->setPopupMode( QToolButton::InstantPopup );
   mFilterQuery->setVisible( false );
+  mFilterQuery->setText( QString() );
   if ( mCurrentSearchWidgetWrapper )
   {
     mCurrentSearchWidgetWrapper->widget()->setVisible( false );
@@ -537,9 +634,9 @@ void QgsAttributeTableDialog::filterEdited()
   mMainView->setFilterMode( QgsAttributeTableFilterModel::ShowEdited );
 }
 
-void QgsAttributeTableDialog::on_mSelectedToTopButton_toggled()
+void QgsAttributeTableDialog::on_mActionSelectedToTop_toggled( bool checked )
 {
-  if ( mSelectedToTopButton->isChecked() )
+  if ( checked )
   {
     mMainView->setSelectedOnTop( true );
   }
@@ -549,7 +646,7 @@ void QgsAttributeTableDialog::on_mSelectedToTopButton_toggled()
   }
 }
 
-void QgsAttributeTableDialog::on_mOpenFieldCalculator_clicked()
+void QgsAttributeTableDialog::on_mActionOpenFieldCalculator_triggered()
 {
   QgsAttributeTableModel* masterModel = mMainView->masterModel();
 
@@ -565,17 +662,17 @@ void QgsAttributeTableDialog::on_mOpenFieldCalculator_clicked()
   }
 }
 
-void QgsAttributeTableDialog::on_mSaveEditsButton_clicked()
+void QgsAttributeTableDialog::on_mActionSaveEdits_triggered()
 {
   QgisApp::instance()->saveEdits( mLayer, true, true );
 }
 
-void QgsAttributeTableDialog::on_mReloadButton_clicked()
+void QgsAttributeTableDialog::on_mActionReload_triggered()
 {
   mMainView->masterModel()->layer()->dataProvider()->forceReload();
 }
 
-void QgsAttributeTableDialog::on_mAddFeature_clicked()
+void QgsAttributeTableDialog::on_mActionAddFeature_triggered()
 {
   if ( !mLayer->isEditable() )
     return;
@@ -590,50 +687,50 @@ void QgsAttributeTableDialog::on_mAddFeature_clicked()
   }
 }
 
-void QgsAttributeTableDialog::on_mExpressionSelectButton_clicked()
+void QgsAttributeTableDialog::on_mActionExpressionSelect_triggered()
 {
   QgsExpressionSelectionDialog* dlg = new QgsExpressionSelectionDialog( mLayer );
   dlg->setAttribute( Qt::WA_DeleteOnClose );
   dlg->show();
 }
 
-void QgsAttributeTableDialog::on_mCopySelectedRowsButton_clicked()
+void QgsAttributeTableDialog::on_mActionCopySelectedRows_triggered()
 {
   QgisApp::instance()->editCopy( mLayer );
 }
 
-void QgsAttributeTableDialog::on_mPasteFeatures_clicked()
+void QgsAttributeTableDialog::on_mActionPasteFeatures_triggered()
 {
   QgisApp::instance()->editPaste( mLayer );
 }
 
 
-void QgsAttributeTableDialog::on_mZoomMapToSelectedRowsButton_clicked()
+void QgsAttributeTableDialog::on_mActionZoomMapToSelectedRows_triggered()
 {
   QgisApp::instance()->mapCanvas()->zoomToSelected( mLayer );
 }
 
-void QgsAttributeTableDialog::on_mPanMapToSelectedRowsButton_clicked()
+void QgsAttributeTableDialog::on_mActionPanMapToSelectedRows_triggered()
 {
   QgisApp::instance()->mapCanvas()->panToSelected( mLayer );
 }
 
-void QgsAttributeTableDialog::on_mInvertSelectionButton_clicked()
+void QgsAttributeTableDialog::on_mActionInvertSelection_triggered()
 {
   mLayer->invertSelection();
 }
 
-void QgsAttributeTableDialog::on_mRemoveSelectionButton_clicked()
+void QgsAttributeTableDialog::on_mActionRemoveSelection_triggered()
 {
   mLayer->removeSelection();
 }
 
-void QgsAttributeTableDialog::on_mSelectAllButton_clicked()
+void QgsAttributeTableDialog::on_mActionSelectAll_triggered()
 {
   mLayer->selectAll();
 }
 
-void QgsAttributeTableDialog::on_mDeleteSelectedButton_clicked()
+void QgsAttributeTableDialog::on_mActionDeleteSelected_triggered()
 {
   QgisApp::instance()->deleteSelected( mLayer, this );
 }
@@ -641,9 +738,16 @@ void QgsAttributeTableDialog::on_mDeleteSelectedButton_clicked()
 void QgsAttributeTableDialog::on_mMainView_currentChanged( int viewMode )
 {
   mMainViewButtonGroup->button( viewMode )->click();
+  updateMultiEditButtonState();
+
+  if ( viewMode == 0 )
+    mActionSearchForm->setChecked( false );
+
+  QSettings s;
+  s.setValue( "/qgis/attributeTableLastView", static_cast< int >( viewMode ) );
 }
 
-void QgsAttributeTableDialog::on_mToggleEditingButton_toggled()
+void QgsAttributeTableDialog::on_mActionToggleEditing_toggled( bool )
 {
   if ( !mLayer )
     return;
@@ -656,26 +760,37 @@ void QgsAttributeTableDialog::on_mToggleEditingButton_toggled()
 
 void QgsAttributeTableDialog::editingToggled()
 {
-  mToggleEditingButton->blockSignals( true );
-  mToggleEditingButton->setChecked( mLayer->isEditable() );
-  mSaveEditsButton->setEnabled( mLayer->isEditable() );
-  mReloadButton->setEnabled( ! mLayer->isEditable() );
-  mToggleEditingButton->blockSignals( false );
+  mActionToggleEditing->blockSignals( true );
+  mActionToggleEditing->setChecked( mLayer->isEditable() );
+  mActionSaveEdits->setEnabled( mLayer->isEditable() );
+  mActionReload->setEnabled( ! mLayer->isEditable() );
+  updateMultiEditButtonState();
+  if ( mLayer->isEditable() )
+  {
+    mActionSearchForm->setChecked( false );
+  }
+  mActionToggleEditing->blockSignals( false );
 
   bool canChangeAttributes = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::ChangeAttributeValues;
   bool canDeleteFeatures = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteFeatures;
   bool canAddAttributes = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::AddAttributes;
+  bool canDeleteAttributes = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteAttributes;
   bool canAddFeatures = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::AddFeatures;
-  mAddAttribute->setEnabled(( canChangeAttributes || canAddAttributes ) && mLayer->isEditable() );
-  mDeleteSelectedButton->setEnabled( canDeleteFeatures && mLayer->isEditable() );
-  mAddFeature->setEnabled( canAddFeatures && mLayer->isEditable() );
+  mActionAddAttribute->setEnabled(( canChangeAttributes || canAddAttributes ) && mLayer->isEditable() );
+  mActionRemoveAttribute->setEnabled( canDeleteAttributes && mLayer->isEditable() );
+  mActionDeleteSelected->setEnabled( canDeleteFeatures && mLayer->isEditable() );
+  mActionAddFeature->setEnabled( canAddFeatures && mLayer->isEditable() );
 
   mUpdateExpressionBox->setVisible( mLayer->isEditable() );
+  if ( mLayer->isEditable() && mFieldCombo->currentIndex() == -1 )
+  {
+    mFieldCombo->setCurrentIndex( 0 );
+  }
   // not necessary to set table read only if layer is not editable
   // because model always reflects actual state when returning item flags
 }
 
-void QgsAttributeTableDialog::on_mAddAttribute_clicked()
+void QgsAttributeTableDialog::on_mActionAddAttribute_triggered()
 {
   if ( !mLayer )
   {
@@ -705,7 +820,7 @@ void QgsAttributeTableDialog::on_mAddAttribute_clicked()
   }
 }
 
-void QgsAttributeTableDialog::on_mRemoveAttribute_clicked()
+void QgsAttributeTableDialog::on_mActionRemoveAttribute_triggered()
 {
   if ( !mLayer )
   {
@@ -771,11 +886,42 @@ void QgsAttributeTableDialog::openConditionalStyles()
   mMainView->openConditionalStyles();
 }
 
-void QgsAttributeTableDialog::setFilterExpression( const QString& filterString )
+void QgsAttributeTableDialog::setFilterExpression( const QString& filterString, QgsAttributeForm::FilterType type,
+    bool alwaysShowFilter )
 {
-  if ( !mCurrentSearchWidgetWrapper || !mCurrentSearchWidgetWrapper->applyDirectly() )
+  QString filter;
+  if ( !mFilterQuery->text().isEmpty() && !filterString.isEmpty() )
   {
-    mFilterQuery->setText( filterString );
+    switch ( type )
+    {
+      case QgsAttributeForm::ReplaceFilter:
+        filter = filterString;
+        break;
+
+      case QgsAttributeForm::FilterAnd:
+        filter = QString( "(%1) AND (%2)" ).arg( mFilterQuery->text(), filterString );
+        break;
+
+      case QgsAttributeForm::FilterOr:
+        filter = QString( "(%1) OR (%2)" ).arg( mFilterQuery->text(), filterString );
+        break;
+    }
+  }
+  else if ( !filterString.isEmpty() )
+  {
+    filter = filterString;
+  }
+  else
+  {
+    filterShowAll();
+    return;
+  }
+
+  mFilterQuery->setText( filter );
+
+  if ( alwaysShowFilter || !mCurrentSearchWidgetWrapper || !mCurrentSearchWidgetWrapper->applyDirectly() )
+  {
+
     mFilterButton->setDefaultAction( mActionAdvancedFilter );
     mFilterButton->setPopupMode( QToolButton::MenuButtonPopup );
     mFilterQuery->setVisible( true );
@@ -795,7 +941,7 @@ void QgsAttributeTableDialog::setFilterExpression( const QString& filterString )
   myDa.setEllipsoid( QgsProject::instance()->readEntry( "Measure", "/Ellipsoid", GEO_NONE ) );
 
   // parse search string and build parsed tree
-  QgsExpression filterExpression( filterString );
+  QgsExpression filterExpression( filter );
   if ( filterExpression.hasParserError() )
   {
     QgisApp::instance()->messageBar()->pushMessage( tr( "Parsing error" ), filterExpression.parserErrorString(), QgsMessageBar::WARNING, QgisApp::instance()->messageTimeout() );
@@ -860,7 +1006,7 @@ void QgsAttributeTableDialog::setFilterExpression( const QString& filterString )
 //
 
 QgsAttributeTableDock::QgsAttributeTableDock( const QString& title, QWidget* parent, Qt::WindowFlags flags )
-    : QDockWidget( title, parent, flags )
+    : QgsDockWidget( title, parent, flags )
 {
   setObjectName( "AttributeTable" ); // set object name so the position can be saved
 }
