@@ -28,9 +28,8 @@
 
 #include "qgsattributedialog.h"
 #include "qgsvectorlayer.h" //QgsFeatureIds
-#include "qgsfieldmodel.h"
 #include "qgssearchwidgetwrapper.h"
-#include <QDockWidget>
+#include "qgsdockwidget.h"
 
 class QDialogButtonBox;
 class QPushButton;
@@ -56,83 +55,84 @@ class APP_EXPORT QgsAttributeTableDialog : public QDialog, private Ui::QgsAttrib
     QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWidget *parent = nullptr, Qt::WindowFlags flags = Qt::Window );
     ~QgsAttributeTableDialog();
 
-    /**
-     * Sets the filter expression to filter visible features
-     * @param filterString filter query string. QgsExpression compatible.
-     */
-    void setFilterExpression( const QString& filterString );
-
   public slots:
     /**
      * Toggles editing mode
      */
     void editingToggled();
 
+    /**
+     * Sets the filter expression to filter visible features
+     * @param filterString filter query string. QgsExpression compatible.
+     */
+    void setFilterExpression( const QString& filterString,
+                              QgsAttributeForm::FilterType type = QgsAttributeForm::ReplaceFilter,
+                              bool alwaysShowFilter = false );
+
   private slots:
     /**
      * Copies selected rows to the clipboard
      */
-    void on_mCopySelectedRowsButton_clicked();
+    void on_mActionCopySelectedRows_triggered();
     /**
      * Paste features from the clipboard
      */
-    void on_mPasteFeatures_clicked();
+    void on_mActionPasteFeatures_triggered();
     /**
      * Toggles editing mode
      */
-    void on_mToggleEditingButton_toggled();
+    void on_mActionToggleEditing_toggled( bool );
     /**
      * Saves edits
      */
-    void on_mSaveEditsButton_clicked();
+    void on_mActionSaveEdits_triggered();
     /**
      * Reload the data
      */
-    void on_mReloadButton_clicked();
-
+    void on_mActionReload_triggered();
     /**
      * Inverts selection
      */
-    void on_mInvertSelectionButton_clicked();
+    void on_mActionInvertSelection_triggered();
     /**
      * Clears selection
      */
-    void on_mRemoveSelectionButton_clicked();
+    void on_mActionRemoveSelection_triggered();
     /**
      * Select all
      */
-    void on_mSelectAllButton_clicked();
+    void on_mActionSelectAll_triggered();
     /**
      * Zooms to selected features
      */
-    void on_mZoomMapToSelectedRowsButton_clicked();
+    void on_mActionZoomMapToSelectedRows_triggered();
     /**
      * Pans to selected features
      */
-    void on_mPanMapToSelectedRowsButton_clicked();
+    void on_mActionPanMapToSelectedRows_triggered();
     /**
      * Moves selected lines to the top
      */
-    void on_mSelectedToTopButton_toggled();
+    void on_mActionSelectedToTop_toggled( bool );
 
     /**
      * Opens dialog to add new attribute
      */
-    void on_mAddAttribute_clicked();
+    void on_mActionAddAttribute_triggered();
 
     /**
      * Opens dialog to remove attribute
      */
-    void on_mRemoveAttribute_clicked();
+    void on_mActionRemoveAttribute_triggered();
     /**
      * Opens field calculator dialog
      */
-    void on_mOpenFieldCalculator_clicked();
+    void on_mActionOpenFieldCalculator_triggered();
 
     /**
      * deletes the selected features
      */
-    void on_mDeleteSelectedButton_clicked();
+    void on_mActionDeleteSelected_triggered();
 
     /**
      * Called when the current index changes in the main view
@@ -146,11 +146,9 @@ class APP_EXPORT QgsAttributeTableDialog : public QDialog, private Ui::QgsAttrib
     /**
      * add feature
      */
-    void on_mAddFeature_clicked();
+    void on_mActionAddFeature_triggered();
 
-    void on_mHelpButton_clicked() { QgsContextHelp::run( metaObject()->className() ); }
-
-    void on_mExpressionSelectButton_clicked();
+    void on_mActionExpressionSelect_triggered();
     void filterColumnChanged( QObject* filterAction );
     void filterExpressionBuilder();
     void filterShowAll();
@@ -206,12 +204,13 @@ class APP_EXPORT QgsAttributeTableDialog : public QDialog, private Ui::QgsAttrib
     void runFieldCalculation( QgsVectorLayer* layer, const QString& fieldName, const QString& expression, const QgsFeatureIds& filteredIds = QgsFeatureIds() );
     void updateFieldFromExpression();
     void updateFieldFromExpressionSelected();
+    void viewModeChanged( QgsAttributeForm::Mode mode );
+    void formFilterSet( const QString& filter, QgsAttributeForm::FilterType type );
 
   private:
     QMenu* mMenuActions;
-    QAction* mActionToggleEditing;
 
-    QDockWidget* mDock;
+    QgsDockWidget* mDock;
     QgsDistanceArea* myDa;
 
 
@@ -219,16 +218,18 @@ class APP_EXPORT QgsAttributeTableDialog : public QDialog, private Ui::QgsAttrib
     QSignalMapper* mFilterActionMapper;
 
     QgsVectorLayer* mLayer;
-    QgsFieldModel* mFieldModel;
-
     QgsRubberBand* mRubberBand;
     QgsSearchWidgetWrapper* mCurrentSearchWidgetWrapper;
+    QStringList mVisibleFields;
+    QgsAttributeEditorContext mEditorContext;
+
+    void updateMultiEditButtonState();
 
     friend class TestQgsAttributeTable;
 };
 
 
-class QgsAttributeTableDock : public QDockWidget
+class QgsAttributeTableDock : public QgsDockWidget
 {
     Q_OBJECT
 
