@@ -21,6 +21,7 @@
 
 #include "qgsvectorlayer.h"
 #include "qgseditorconfigwidget.h"
+#include "qgsfeature.h"
 
 class QDialog;
 class QLayout;
@@ -69,6 +70,11 @@ class APP_EXPORT QgsAttributeTypeDialog: public QDialog, private Ui::QgsAttribut
      */
     void setLabelOnTop( bool onTop );
 
+    /**
+     * Getter for checkbox for label on top of field
+     */
+    bool labelOnTop() const;
+
     /**
      * Setter for checkbox for editable state of field
      */
@@ -77,12 +83,54 @@ class APP_EXPORT QgsAttributeTypeDialog: public QDialog, private Ui::QgsAttribut
     /**
      * Getter for checkbox for editable state of field
      */
-    bool fieldEditable();
+    bool fieldEditable() const;
 
     /**
-     * Getter for checkbox for label on top of field
+     * Setter for checkbox for not null
      */
-    bool labelOnTop();
+    void setNotNull( bool notNull );
+
+    /**
+     * Getter for checkbox for not null
+     */
+    bool notNull() const;
+
+    /**
+     * Setter for constraint expression description
+     * @param desc the expression description
+     * @note added in QGIS 2.16
+     **/
+    void setConstraintExpressionDescription( const QString &desc );
+
+    /**
+     * Getter for constraint expression description
+     * @return the expression description
+     * @note added in QGIS 2.16
+     **/
+    QString constraintExpressionDescription();
+
+    /**
+     * Getter for the constraint expression
+     * @note added in QGIS 2.16
+     */
+    QString constraintExpression() const;
+
+    /**
+     * Setter for the constraint expression
+     * @note added in QGIS 2.16
+     */
+    void setConstraintExpression( const QString &str );
+
+    /**
+     * Returns the expression used for the field's default value, or
+     * an empty string if no default value expression is set.
+     */
+    QString defaultValueExpression() const;
+
+    /**
+     * Sets the expression used for the field's default value
+     */
+    void setDefaultValueExpression( const QString& expression );
 
   private slots:
     /**
@@ -91,6 +139,8 @@ class APP_EXPORT QgsAttributeTypeDialog: public QDialog, private Ui::QgsAttribut
      */
     void on_selectionListWidget_currentRowChanged( int index );
 
+    void defaultExpressionChanged();
+
   private:
     QgsVectorLayer *mLayer;
     int mFieldIdx;
@@ -99,6 +149,8 @@ class APP_EXPORT QgsAttributeTypeDialog: public QDialog, private Ui::QgsAttribut
 
     //! Cached configuration dialog (lazy loaded)
     QMap< QString, QgsEditorConfigWidget* > mEditorConfigWidgets;
+
+    QgsFeature mPreviewFeature;
 };
 
 #endif
