@@ -295,7 +295,12 @@ void QgsFieldCalculator::accept()
         if ( value.canConvert< QgsGeometry >() )
         {
           QgsGeometry geom = value.value< QgsGeometry >();
-          mVectorLayer->changeGeometry( feature.id(), &geom );
+          if ( !mVectorLayer->changeGeometry( feature.id(), &geom ) )
+          {
+            calculationSuccess = false;
+            error = tr( "Can not change geometry for feature: %1", "Field calculator" ).arg( feature.id() );
+            break;
+          }
         }
       }
       else
@@ -435,12 +440,14 @@ void QgsFieldCalculator::populateFields()
   const QgsFields& fields = mVectorLayer->fields();
   for ( int idx = 0; idx < fields.count(); ++idx )
   {
+    if ( fields.fieldOrigin( idx ) != QgsFields::OriginExpression && fields.fieldOrigin( idx ) != QgsFields::OriginJoin )
+    {
+      QString fieldName = fields.at( idx ).name();
 
-    QString fieldName = fields[idx].name();
-
-    //insert into field list and field combo box
-    mFieldMap.insert( fieldName, idx );
-    mExistingFieldComboBox->addItem( fieldName );
+      //insert into field list and field combo box
+      mFieldMap.insert( fieldName, idx );
+      mExistingFieldComboBox->addItem( fieldName );
+    }
   }
 
   if ( mVectorLayer->geometryType() != QGis::NoGeometry )
