@@ -22,36 +22,26 @@
 QgsLabelPreview::QgsLabelPreview( QWidget* parent )
     : QLabel( parent )
 {
-  mTmpLyr = new QgsPalLayerSettings();
-
   // construct a device-based render context
-  QgsMapToPixel newCoordXForm;
-  newCoordXForm.setParameters( 1, 0, 0, 0 );
-  mContext = new QgsRenderContext();
-  mContext->setMapToPixel( newCoordXForm );
+  QgsMapToPixel newCoordXForm( 1 );
+  mContext.setMapToPixel( newCoordXForm );
 }
 
-QgsLabelPreview::~QgsLabelPreview()
-{
-  delete mTmpLyr;
-  delete mContext;
-}
-
-void QgsLabelPreview::setTextColor( QColor color )
+void QgsLabelPreview::setTextColor( const QColor& color )
 {
   mTextColor = color;
   update();
 }
 
-void QgsLabelPreview::setBuffer( double size, QColor color, Qt::PenJoinStyle joinStyle, bool noFill )
+void QgsLabelPreview::setBuffer( double size, const QColor& color, Qt::PenJoinStyle joinStyle, bool noFill )
 {
-  mTmpLyr->bufferSize = size * 88 / 25.4; //assume standard dpi for preview;
-  mTmpLyr->bufferSizeInMapUnits = false;
-  mTmpLyr->bufferColor = color;
-  mTmpLyr->bufferJoinStyle = joinStyle;
-  mTmpLyr->bufferNoFill = noFill;
+  mTmpLyr.bufferSize = size * 88 / 25.4; //assume standard dpi for preview;
+  mTmpLyr.bufferSizeInMapUnits = false;
+  mTmpLyr.bufferColor = color;
+  mTmpLyr.bufferJoinStyle = joinStyle;
+  mTmpLyr.bufferNoFill = noFill;
 
-  mTmpLyr->textFont = font();
+  mTmpLyr.textFont = font();
   update();
 }
 
@@ -62,28 +52,28 @@ void QgsLabelPreview::paintEvent( QPaintEvent *e )
 
   // TODO: draw all label components when this preview is an actual map canvas
   // for now, only preview label's text and buffer
-  mTmpLyr->shadowDraw = false;
+  mTmpLyr.shadowDraw = false;
 
   p.setRenderHint( QPainter::Antialiasing );
   p.setFont( font() );
   QFontMetrics fm( font() );
 
   // otherwise thin buffers don't look like those on canvas
-  if ( mTmpLyr->bufferSize != 0 && mTmpLyr->bufferSize < 1 )
-    mTmpLyr->bufferSize = 1;
+  if ( mTmpLyr.bufferSize != 0 && mTmpLyr.bufferSize < 1 )
+    mTmpLyr.bufferSize = 1;
 
   double xtrans = 0;
-  if ( mTmpLyr->bufferSize != 0 )
-    xtrans = mTmpLyr->bufferSize / 4;
+  if ( mTmpLyr.bufferSize != 0 )
+    xtrans = mTmpLyr.bufferSize / 4;
 
   p.translate( xtrans, fm.ascent() + 4 );
 
-  if ( mTmpLyr->bufferSize != 0 )
+  if ( mTmpLyr.bufferSize != 0 )
   {
-    mContext->setPainter( &p );
+    mContext.setPainter( &p );
     QgsLabelComponent component;
     component.setText( text() );
-    QgsPalLabeling::drawLabelBuffer( *mContext, component, *mTmpLyr );
+    QgsPalLabeling::drawLabelBuffer( mContext, component, mTmpLyr );
   }
 
   QPainterPath path;
