@@ -113,9 +113,13 @@ void QgsMapToolDeleteRing::canvasReleaseEvent( QgsMapMouseEvent* e )
   if ( g->deleteRing( mPressedRingNum, mPressedPartNum ) )
   {
     vlayer->beginEditCommand( tr( "Ring deleted" ) );
-    vlayer->changeGeometry( mPressedFid, g );
+    if ( !vlayer->changeGeometry( mPressedFid, g ) )
+    {
+      vlayer->destroyEditCommand();
+      return;
+    }
     vlayer->endEditCommand();
-    mCanvas->refresh();
+    vlayer->triggerRepaint();
   }
 }
 
@@ -197,7 +201,7 @@ void QgsMapToolDeleteRing::deleteRing( QgsFeatureId fId, int beforeVertexNr, Qgs
     vlayer->beginEditCommand( tr( "Ring deleted" ) );
     vlayer->changeGeometry( fId, editableGeom );
     vlayer->endEditCommand();
-    mCanvas->refresh();
+    vlayer->triggerRepaint();
   }
 
 }
