@@ -50,13 +50,13 @@ class APP_EXPORT QgsMapToolIdentifyAction : public QgsMapToolIdentify
     ~QgsMapToolIdentifyAction();
 
     //! Overridden mouse move event
-    virtual void canvasMoveEvent( QMouseEvent * e ) override;
+    virtual void canvasMoveEvent( QgsMapMouseEvent* e ) override;
 
     //! Overridden mouse press event
-    virtual void canvasPressEvent( QMouseEvent * e ) override;
+    virtual void canvasPressEvent( QgsMapMouseEvent* e ) override;
 
     //! Overridden mouse release event
-    virtual void canvasReleaseEvent( QMouseEvent * e ) override;
+    virtual void canvasReleaseEvent( QgsMapMouseEvent* e ) override;
 
     virtual void activate() override;
 
@@ -68,11 +68,11 @@ class APP_EXPORT QgsMapToolIdentifyAction : public QgsMapToolIdentify
 
   signals:
     void identifyProgress( int, int );
-    void identifyMessage( QString );
+    void identifyMessage( const QString& );
     void copyToClipboard( QgsFeatureStore & );
 
   private slots:
-    void showAttributeTable( QgsMapLayer* layer, const QList<QgsFeature> featureList );
+    void showAttributeTable( QgsMapLayer* layer, const QList<QgsFeature>& featureList );
 
   private:
     //! Pointer to the identify results dialog for name/value pairs
@@ -80,8 +80,11 @@ class APP_EXPORT QgsMapToolIdentifyAction : public QgsMapToolIdentify
 
     QgsIdentifyResultsDialog *resultsDialog();
 
-    virtual QGis::UnitType displayUnits() override;
+    virtual QGis::UnitType displayDistanceUnits() const override;
+    virtual QgsUnitTypes::AreaUnit displayAreaUnits() const override;
+    void setClickContextScope( const QgsPoint &point );
 
+    friend class TestQgsMapToolIdentifyAction;
 };
 
 #endif
