@@ -37,23 +37,29 @@ class APP_EXPORT QgsMapToolLabel: public QgsMapTool
         @param xCol out: index of the attribute for data defined x coordinate
         @param yCol out: index of the attribute for data defined y coordinate
         @return true if labels of layer can be moved*/
-    bool labelMoveable( QgsMapLayer* ml, int& xCol, int& yCol ) const;
+    bool labelMoveable( QgsVectorLayer* vlayer, int& xCol, int& yCol ) const;
+    bool labelMoveable( QgsVectorLayer* vlayer, const QgsPalLayerSettings& settings, int& xCol, int& yCol ) const;
     /** Returns true if diagram move can be applied to a layer
         @param xCol out: index of the attribute for data defined x coordinate
         @param yCol out: index of the attribute for data defined y coordinate
         @return true if labels of layer can be moved*/
-    bool diagramMoveable( QgsMapLayer* ml, int& xCol, int& yCol ) const;
+    bool diagramMoveable( QgsVectorLayer* vlayer, int& xCol, int& yCol ) const;
     /** Returns true if layer has attribute fields set up
         @param xCol out: index of the attribute for data defined x coordinate
         @param yCol out: index of the attribute for data defined y coordinate
         @return true if layer fields set up and exist*/
-    bool layerCanPin( QgsMapLayer* ml, int& xCol, int& yCol ) const;
+    bool layerCanPin( QgsVectorLayer* vlayer, int& xCol, int& yCol ) const;
+    /** Returns true if layer has attribute field set up for diagrams
+      @param showCol out: attribute column for data defined diagram showing
+      @note added in QGIS 2.16 */
+    bool diagramCanShowHide( QgsVectorLayer* vlayer, int& showCol ) const;
     /** Returns true if layer has attribute field set up
       @param showCol out: attribute column for data defined label showing*/
-    bool layerCanShowHide( QgsMapLayer* layer, int& showCol ) const;
+    bool labelCanShowHide( QgsVectorLayer* vlayer, int& showCol ) const;
     /** Checks if labels in a layer can be rotated
       @param rotationCol out: attribute column for data defined label rotation*/
-    bool layerIsRotatable( QgsMapLayer *layer, int& rotationCol ) const;
+    bool layerIsRotatable( QgsVectorLayer *layer, int& rotationCol ) const;
+    bool labelIsRotatable( QgsVectorLayer *layer, const QgsPalLayerSettings& settings, int& rotationCol ) const;
 
   protected:
     QgsRubberBand* mLabelRubberBand;
@@ -61,8 +67,19 @@ class APP_EXPORT QgsMapToolLabel: public QgsMapTool
     /** Shows label fixpoint (left/bottom by default)*/
     QgsRubberBand* mFixPointRubberBand;
 
+    struct LabelDetails
+    {
+      LabelDetails(): valid( false ), layer( nullptr ) {}
+      explicit LabelDetails( const QgsLabelPosition& p );
+      bool valid;
+      QgsLabelPosition pos;
+      QgsVectorLayer* layer;
+      QgsPalLayerSettings settings;
+    };
+
     /** Currently dragged label position*/
-    QgsLabelPosition mCurrentLabelPos;
+    LabelDetails mCurrentLabel;
+
 
     /** Returns label position for mouse click location
       @param e mouse event
@@ -73,7 +90,7 @@ class APP_EXPORT QgsMapToolLabel: public QgsMapTool
     /** Finds out rotation point of current label position
       @param ignoreUpsideDown treat label as right-side-up
       @return true in case of success*/
-    bool rotationPoint( QgsPoint& pos, bool ignoreUpsideDown = false, bool rotatingUnpinned = false );
+    bool currentLabelRotationPoint( QgsPoint& pos, bool ignoreUpsideDown = false, bool rotatingUnpinned = false );
 
     /** Creates label / feature / fixpoint rubber bands for the current label position*/
     void createRubberBands();
@@ -81,13 +98,6 @@ class APP_EXPORT QgsMapToolLabel: public QgsMapTool
     /** Removes label / feature / fixpoint rubber bands*/
     void deleteRubberBands();
 
-    /** Returns vector layer for current label position*/
-    QgsVectorLayer* currentLayer();
-
-    /** Returns layer settings of current label position*/
-    QgsPalLayerSettings& currentLabelSettings( bool* ok );
-//    const QgsPalLayerSettings& currentLabelSettings( bool* ok ) const;
-
     /** Returns current label's text
       @param trunc number of chars to truncate to, with ... added */
     QString currentLabelText( int trunc = 0 );
@@ -99,28 +109,19 @@ class APP_EXPORT QgsMapToolLabel: public QgsMapTool
     bool currentFeature( QgsFeature& f, bool fetchGeom = false );
 
     /** Returns the font for the current feature (considering default font and data defined properties)*/
-    QFont labelFontCurrentFeature();
+    QFont currentLabelFont();
 
-//    /**Returns a data defined attribute column
-//      @return invalid QVariant if one does not exist or an expression is used instead */
-//    QVariant dataDefinedColumn( QgsPalLayerSettings::DataDefinedProperties p );
-
-//    /**Returns a data defined attribute column - overloaded variation
-//      @return invalid QVariant if one does not exist or an expression is used instead */
-//    QVariant dataDefinedColumn( QgsPalLayerSettings::DataDefinedProperties p, QgsPalLayerSettings& labelSettings );
+    /** Returns a data defined attribute column name for particular property or empty string if not defined */
+    QString dataDefinedColumnName( QgsPalLayerSettings::DataDefinedProperties p, const QgsPalLayerSettings& labelSettings ) const;
 
     /** Returns a data defined attribute column index
       @return -1 if column does not exist or an expression is used instead */
-//    int dataDefinedColumnIndex( QgsPalLayerSettings::DataDefinedProperties p, const QgsPalLayerSettings& labelSettings, const QgsVectorLayer* vlayer ) const;
-
-    int dataDefinedColumnIndex( QgsPalLayerSettings::DataDefinedProperties p, QgsVectorLayer* vlayer ) const;
+    int dataDefinedColumnIndex( QgsPalLayerSettings::DataDefinedProperties p, const QgsPalLayerSettings& labelSettings, const QgsVectorLayer* vlayer ) const;
 
     /** Returns whether to preserve predefined rotation data during label pin/unpin operations*/
-    bool preserveRotation();
+    bool currentLabelPreserveRotation();
 
-    /** Get data defined position of a feature
-      @param vlayer vector layer
-      @param featureId feature identification integer
+    /** Get data defined position of current label
       @param x out: data defined x-coordinate
       @param xSuccess out: false if attribute value is NULL
       @param y out: data defined y-coordinate
@@ -128,17 +129,16 @@ class APP_EXPORT QgsMapToolLabel: public QgsMapTool
       @param xCol out: index of the x position column
       @param yCol out: index of the y position column
       @return false if layer does not have data defined label position enabled*/
-    bool dataDefinedPosition( QgsVectorLayer* vlayer, QgsFeatureId featureId, double& x, bool& xSuccess, double& y, bool& ySuccess, int& xCol, int& yCol ) const;
+    bool currentLabelDataDefinedPosition( double& x, bool& xSuccess, double& y, bool& ySuccess, int& xCol, int& yCol ) const;
 
-    /** Returns data defined rotation of a feature.
-      @param vlayer vector layer
-      @param featureId feature identification integer
+    /** Returns data defined rotation of current label
       @param rotation out: rotation value
       @param rotationSuccess out: false if rotation value is NULL
+      @param rCol out: index of the rotation column
       @param ignoreXY ignore that x and y are required to be data-defined
       @return true if data defined rotation is enabled on the layer
       */
-    bool dataDefinedRotation( QgsVectorLayer* vlayer, QgsFeatureId featureId, double& rotation, bool& rotationSuccess, bool ignoreXY = false ) const;
+    bool currentLabelDataDefinedRotation( double& rotation, bool& rotationSuccess, int& rCol, bool ignoreXY = false ) const;
 
     /** Returns data defined show/hide of a feature.
       @param vlayer vector layer
@@ -150,11 +150,11 @@ class APP_EXPORT QgsMapToolLabel: public QgsMapTool
       */
     bool dataDefinedShowHide( QgsVectorLayer* vlayer, QgsFeatureId featureId, int& show, bool& showSuccess, int& showCol ) const;
 
-  private:
-    QgsPalLayerSettings mInvalidLabelSettings;
-
-    QgsVectorLayer* mCurrentLayer;
-    QgsPalLayerSettings mCurrentSettings;
+    /** Returns the pin status for the current label/diagram
+      @return true if the label/diagram is pinned, false otherwise
+      @note added in QGIS 2.16
+      */
+    bool isPinned();
 };
 
 #endif // QGSMAPTOOLLABEL_H
