@@ -142,7 +142,6 @@ void QgsMapToolMoveFeature::canvasPressEvent( QgsMapMouseEvent* e )
 
 void QgsMapToolMoveFeature::canvasReleaseEvent( QgsMapMouseEvent* e )
 {
-  //QgsDebugMsg("entering.");
   if ( !mRubberBand )
   {
     return;
@@ -166,8 +165,8 @@ void QgsMapToolMoveFeature::canvasReleaseEvent( QgsMapMouseEvent* e )
   }
   delete mRubberBand;
   mRubberBand = nullptr;
-  mCanvas->refresh();
   vlayer->endEditCommand();
+  vlayer->triggerRepaint();
 }
 
 //! called when map tool is being deactivated
