@@ -20,7 +20,7 @@
 
 #include "qgsmaptoollabel.h"
 
-/**A map tool for dragging label positions*/
+/** A map tool for dragging label positions*/
 class APP_EXPORT QgsMapToolMoveLabel: public QgsMapToolLabel
 {
     Q_OBJECT
@@ -29,15 +29,15 @@ class APP_EXPORT QgsMapToolMoveLabel: public QgsMapToolLabel
     QgsMapToolMoveLabel( QgsMapCanvas* canvas );
     ~QgsMapToolMoveLabel();
 
-    virtual void canvasPressEvent( QMouseEvent * e ) override;
+    virtual void canvasPressEvent( QgsMapMouseEvent* e ) override;
 
-    virtual void canvasMoveEvent( QMouseEvent * e ) override;
+    virtual void canvasMoveEvent( QgsMapMouseEvent* e ) override;
 
-    virtual void canvasReleaseEvent( QMouseEvent * e ) override;
+    virtual void canvasReleaseEvent( QgsMapMouseEvent* e ) override;
 
   protected:
 
-    /**Start point of the move in map coordinates*/
+    /** Start point of the move in map coordinates*/
     QgsPoint mStartPointMapCoords;
 
     double mClickOffsetX;
