@@ -19,14 +19,14 @@
 #include "qgsmaptoolcapture.h"
 #include "qgsmapmouseevent.h"
 
-/**A map tool that draws a line and splits the features cut by the line*/
+/** A map tool that draws a line and splits the features cut by the line*/
 class APP_EXPORT QgsMapToolSplitFeatures: public QgsMapToolCapture
 {
     Q_OBJECT
   public:
     QgsMapToolSplitFeatures( QgsMapCanvas* canvas );
     virtual ~QgsMapToolSplitFeatures();
-    void canvasMapReleaseEvent( QgsMapMouseEvent * e ) override;
+    void cadCanvasReleaseEvent( QgsMapMouseEvent * e ) override;
 };
 
 #endif
