@@ -19,14 +19,14 @@
 #include "qgsmaptoolcapture.h"
 #include "qgsmapmouseevent.h"
 
-/**A map tool that draws a line and splits the parts cut by the line*/
+/** A map tool that draws a line and splits the parts cut by the line*/
 class QgsMapToolSplitParts: public QgsMapToolCapture
 {
     Q_OBJECT
   public:
-    QgsMapToolSplitParts( QgsMapCanvas* canvas );
+    explicit QgsMapToolSplitParts( QgsMapCanvas* canvas );
     virtual ~QgsMapToolSplitParts();
-    void canvasMapReleaseEvent( QgsMapMouseEvent * e ) override;
+    void cadCanvasReleaseEvent( QgsMapMouseEvent * e ) override;
 };
 
 #endif
