@@ -20,10 +20,10 @@
 #include "qgspluginmetadata.h"
 QgsPluginMetadata::QgsPluginMetadata( const QString& _libraryPath,
                                       const QString& _name,
-                                      QgisPlugin * _plugin ):
-    m_name( _name ),
-    libraryPath( _libraryPath ),
-    m_plugin( _plugin )
+                                      QgisPlugin * _plugin )
+    : m_name( _name )
+    , libraryPath( _libraryPath )
+    , m_plugin( _plugin )
 {
 
 }
