@@ -1,4 +1,10 @@
 /***************************************************************************
+    qgssavestyletodbdialog.h
+    ---------------------
+    begin                : April 2013
+    copyright            : (C) 2013 by Emilio Loi
+    email                : loi at faunalia dot it
+ ***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -19,7 +25,7 @@ class APP_EXPORT QgsSaveStyleToDbDialog : public QDialog, private Ui::QgsSaveToD
     QString mUIFileContent;
     Q_OBJECT
   public:
-    explicit QgsSaveStyleToDbDialog( QWidget *parent = 0 );
+    explicit QgsSaveStyleToDbDialog( QWidget *parent = nullptr );
 
     ~QgsSaveStyleToDbDialog();
 
