@@ -105,18 +105,42 @@ QVariant QgsComposerAttributeTableColumnModel::data( const QModelIndex &index, i
         switch ( column->hAlignment() )
         {
           case Qt::AlignHCenter:
-            return tr( "Center" );
+            switch ( column->vAlignment() )
+            {
+              case Qt::AlignTop:
+                return tr( "Top center" );
+              case Qt::AlignBottom:
+                return tr( "Bottom center" );
+              default:
+                return tr( "Middle center" );
+            }
           case Qt::AlignRight:
-            return tr( "Right" );
+            switch ( column->vAlignment() )
+            {
+              case Qt::AlignTop:
+                return tr( "Top right" );
+              case Qt::AlignBottom:
+                return tr( "Bottom right" );
+              default:
+                return tr( "Middle right" );
+            }
           case Qt::AlignLeft:
           default:
-            return tr( "Left" );
+            switch ( column->vAlignment() )
+            {
+              case Qt::AlignTop:
+                return tr( "Top left" );
+              case Qt::AlignBottom:
+                return tr( "Bottom left" );
+              default:
+                return tr( "Middle left" );
+            }
         }
       }
       else
       {
         //edit role
-        return column->hAlignment();
+        return int( column->hAlignment() | column->vAlignment() );
       }
     }
 
@@ -198,7 +222,8 @@ bool QgsComposerAttributeTableColumnModel::setData( const QModelIndex& index, co
       emit dataChanged( index, index );
       return true;
     case 2:
-      column->setHAlignment(( Qt::AlignmentFlag )value.toInt() );
+      column->setHAlignment( Qt::AlignmentFlag( value.toInt() & Qt::AlignHorizontal_Mask ) );
+      column->setVAlignment( Qt::AlignmentFlag( value.toInt() & Qt::AlignVertical_Mask ) );
       emit dataChanged( index, index );
       return true;
     default:
@@ -362,12 +387,11 @@ bool QgsComposerAttributeTableColumnModel::moveColumnInSortRank( QgsComposerTabl
 
   //find column before this one in sort order
   QList<QgsComposerTableColumn*> sortedColumns;
-  QList<QgsComposerTableColumn*>::iterator columnIt = mComposerTable->columns()->begin();
-  for ( ; columnIt != mComposerTable->columns()->end(); ++columnIt )
+  Q_FOREACH ( QgsComposerTableColumn* currentColumn, *mComposerTable->columns() )
   {
-    if (( *columnIt )->sortByRank() > 0 )
+    if ( currentColumn->sortByRank() > 0 )
     {
-      sortedColumns.append( *columnIt );
+      sortedColumns.append( currentColumn );
     }
   }
   qStableSort( sortedColumns.begin(), sortedColumns.end(), columnsBySortRank );
@@ -578,7 +602,7 @@ bool QgsComposerTableSortColumnsProxyModel::setData( const QModelIndex& index, c
 
   if ( index.column() == 1 )
   {
-    column->setSortOrder(( Qt::SortOrder )value.toInt() );
+    column->setSortOrder( static_cast< Qt::SortOrder >( value.toInt() ) );
     emit dataChanged( index, index );
     return true;
   }
