@@ -29,25 +29,28 @@ class QgsDiagramInterpolationSettings;
 
 class QgsRenderContext;
 
+/** \ingroup core
+ * \class QgsPieDiagram
+ */
 class CORE_EXPORT QgsPieDiagram: public QgsDiagram
 {
   public:
     QgsPieDiagram();
     ~QgsPieDiagram();
 
-    virtual QgsDiagram* clone() const override;
+    virtual QgsPieDiagram* clone() const override;
 
-    void renderDiagram( const QgsFeature& feature, QgsRenderContext& c, const QgsDiagramSettings& s, const QPointF& position ) override;
+    void renderDiagram( const QgsFeature& feature, QgsRenderContext& c, const QgsDiagramSettings& s, QPointF position ) override;
 
     QSizeF diagramSize( const QgsAttributes& attributes, const QgsRenderContext& c, const QgsDiagramSettings& s ) override;
     QSizeF diagramSize( const QgsFeature& feature, const QgsRenderContext& c, const QgsDiagramSettings& s, const QgsDiagramInterpolationSettings& is ) override;
+    double legendSize( double value, const QgsDiagramSettings& s, const QgsDiagramInterpolationSettings& is ) const override;
     QString diagramName() const override { return DIAGRAM_NAME_PIE; }
 
   private:
     QBrush mCategoryBrush;
     QPen mPen;
 
-    static int sCount;
 };
 
 #endif // QGSPIEDIAGRAM_H
