@@ -29,7 +29,9 @@ class QgsDiagramInterpolationSettings;
 
 class QgsRenderContext;
 
-
+/** \ingroup core
+ * \class QgsTextDiagram
+ */
 class CORE_EXPORT QgsTextDiagram: public QgsDiagram
 {
   public:
@@ -48,12 +50,13 @@ class CORE_EXPORT QgsTextDiagram: public QgsDiagram
 
     QgsTextDiagram();
     ~QgsTextDiagram();
-    virtual QgsDiagram* clone() const override;
+    virtual QgsTextDiagram* clone() const override;
 
-    void renderDiagram( const QgsFeature& feature, QgsRenderContext& c, const QgsDiagramSettings& s, const QPointF& position ) override;
+    void renderDiagram( const QgsFeature& feature, QgsRenderContext& c, const QgsDiagramSettings& s, QPointF position ) override;
 
     QSizeF diagramSize( const QgsAttributes& attributes, const QgsRenderContext& c, const QgsDiagramSettings& s ) override;
     QSizeF diagramSize( const QgsFeature& feature, const QgsRenderContext& c, const QgsDiagramSettings& s, const QgsDiagramInterpolationSettings& is ) override;
+    double legendSize( double value, const QgsDiagramSettings& s, const QgsDiagramInterpolationSettings& is ) const override;
 
     QString diagramName() const override { return DIAGRAM_NAME_TEXT; }
 
@@ -63,9 +66,9 @@ class CORE_EXPORT QgsTextDiagram: public QgsDiagram
     QBrush mBrush; //transparent brush
     QPen mPen;
 
-    /**Calculates intersection points between a line and an ellipse
+    /** Calculates intersection points between a line and an ellipse
       @return intersection points*/
-    void lineEllipseIntersection( const QPointF& lineStart, const QPointF& lineEnd, const QPointF& ellipseMid, double r1, double r2, QList<QPointF>& result ) const;
+    void lineEllipseIntersection( QPointF lineStart, QPointF lineEnd, QPointF ellipseMid, double r1, double r2, QList<QPointF>& result ) const;
 };
 
 #endif // QGSTEXTDIAGRAM_H
