@@ -20,6 +20,13 @@ email                : marco.hugentobler at sourcepole dot com
 #include "qgslinestringv2.h"
 #include "qgspolygonv2.h"
 #include "qgscurvepolygonv2.h"
+#include "qgsmultilinestringv2.h"
+
+QgsMultiPolygonV2::QgsMultiPolygonV2()
+    : QgsMultiSurfaceV2()
+{
+  mWkbType = QgsWKBTypes::MultiPolygon;
+}
 
 QgsMultiPolygonV2 *QgsMultiPolygonV2::clone() const
 {
@@ -108,7 +115,7 @@ QString QgsMultiPolygonV2::asJSON( int precision ) const
 
 bool QgsMultiPolygonV2::addGeometry( QgsAbstractGeometryV2* g )
 {
-  if ( !dynamic_cast<QgsCurvePolygonV2*>( g ) )
+  if ( !dynamic_cast<QgsPolygonV2*>( g ) )
   {
     delete g;
     return false;
@@ -127,3 +134,38 @@ QgsAbstractGeometryV2* QgsMultiPolygonV2::toCurveType() const
   }
   return multiSurface;
 }
+
+QgsAbstractGeometryV2* QgsMultiPolygonV2::boundary() const
+{
+  QgsMultiLineStringV2* multiLine = new QgsMultiLineStringV2();
+  for ( int i = 0; i < mGeometries.size(); ++i )
+  {
+    if ( QgsPolygonV2* polygon = dynamic_cast<QgsPolygonV2*>( mGeometries.at( i ) ) )
+    {
+      QgsAbstractGeometryV2* polygonBoundary = polygon->boundary();
+
+      if ( QgsLineStringV2* lineStringBoundary = dynamic_cast< QgsLineStringV2* >( polygonBoundary ) )
+      {
+        multiLine->addGeometry( lineStringBoundary );
+      }
+      else if ( QgsMultiLineStringV2* multiLineStringBoundary = dynamic_cast< QgsMultiLineStringV2* >( polygonBoundary ) )
+      {
+        for ( int j = 0; j < multiLineStringBoundary->numGeometries(); ++j )
+        {
+          multiLine->addGeometry( multiLineStringBoundary->geometryN( j )->clone() );
+        }
+        delete multiLineStringBoundary;
+      }
+      else
+      {
+        delete polygonBoundary;
+      }
+    }
+  }
+  if ( multiLine->numGeometries() == 0 )
+  {
+    delete multiLine;
+    return nullptr;
+  }
+  return multiLine;
+}
