@@ -44,13 +44,13 @@ void QgsLayerTreeRegistryBridge::setLayerInsertionPoint( QgsLayerTreeGroup* pare
   mInsertionPointIndex = index;
 }
 
-void QgsLayerTreeRegistryBridge::layersAdded( QList<QgsMapLayer*> layers )
+void QgsLayerTreeRegistryBridge::layersAdded( const QList<QgsMapLayer*>& layers )
 {
   if ( !mEnabled )
     return;
 
   QList<QgsLayerTreeNode*> nodes;
-  foreach ( QgsMapLayer* layer, layers )
+  Q_FOREACH ( QgsMapLayer* layer, layers )
   {
     QgsLayerTreeLayer* nodeLayer = new QgsLayerTreeLayer( layer );
     nodeLayer->setVisible( mNewLayersVisible ? Qt::Checked : Qt::Unchecked );
@@ -73,7 +73,7 @@ void QgsLayerTreeRegistryBridge::layersAdded( QList<QgsMapLayer*> layers )
   emit addedLayersToLayerTree( layers );
 }
 
-void QgsLayerTreeRegistryBridge::layersWillBeRemoved( QStringList layerIds )
+void QgsLayerTreeRegistryBridge::layersWillBeRemoved( const QStringList& layerIds )
 {
   QgsDebugMsg( QString( "%1 layers will be removed, enabled:%2" ).arg( layerIds.count() ).arg( mEnabled ) );
 
@@ -84,7 +84,7 @@ void QgsLayerTreeRegistryBridge::layersWillBeRemoved( QStringList layerIds )
   // the registry _again_ in groupRemovedChildren() - this prevents it
   mRegistryRemovingLayers = true;
 
-  foreach ( QString layerId, layerIds )
+  Q_FOREACH ( const QString& layerId, layerIds )
   {
     QgsLayerTreeLayer* nodeLayer = mRoot->findLayer( layerId );
     if ( nodeLayer )
@@ -99,7 +99,7 @@ static void _collectLayerIdsInGroup( QgsLayerTreeGroup* group, int indexFrom, in
 {
   for ( int i = indexFrom; i <= indexTo; ++i )
   {
-    QgsLayerTreeNode* child = group->children()[i];
+    QgsLayerTreeNode* child = group->children().at( i );
     if ( QgsLayerTree::isLayer( child ) )
     {
       lst << QgsLayerTree::toLayer( child )->layerId();
@@ -132,7 +132,7 @@ void QgsLayerTreeRegistryBridge::groupRemovedChildren()
   // remove only those that really do not exist in the tree
   // (ignores layers that were dragged'n'dropped: 1. drop new 2. remove old)
   QStringList toRemove;
-  foreach ( QString layerId, mLayerIdsForRemoval )
+  Q_FOREACH ( const QString& layerId, mLayerIdsForRemoval )
     if ( !mRoot->findLayer( layerId ) )
       toRemove << layerId;
   mLayerIdsForRemoval.clear();
@@ -145,7 +145,7 @@ void QgsLayerTreeRegistryBridge::groupRemovedChildren()
   QMetaObject::invokeMethod( this, "removeLayersFromRegistry", Qt::QueuedConnection, Q_ARG( QStringList, toRemove ) );
 }
 
-void QgsLayerTreeRegistryBridge::removeLayersFromRegistry( QStringList layerIds )
+void QgsLayerTreeRegistryBridge::removeLayersFromRegistry( const QStringList& layerIds )
 {
   QgsMapLayerRegistry::instance()->removeMapLayers( layerIds );
 }
