@@ -45,7 +45,10 @@ bool bigger( double l, double r )
 }
 
 // O (size log size)
-PriorityQueue::PriorityQueue( int n, int maxId, bool min ) : size( 0 ), maxsize( n ), maxId( maxId )
+PriorityQueue::PriorityQueue( int n, int maxId, bool min )
+    : size( 0 )
+    , maxsize( n )
+    , maxId( maxId )
 {
   heap = new int[maxsize];
   p = new double[maxsize];
