@@ -16,6 +16,7 @@
  *                                                                         *
  ***************************************************************************/
 #include "qgscoordinatereferencesystem.h"
+#include "qgscoordinatereferencesystem_p.h"
 
 #include <cmath>
 
@@ -50,47 +51,35 @@ CUSTOM_CRS_VALIDATION QgsCoordinateReferenceSystem::mCustomSrsValidation = nullp
 //--------------------------
 
 QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem()
-    : mSrsId( 0 )
-    , mGeoFlag( false )
-    , mMapUnits( QGis::UnknownUnit )
-    , mSRID( 0 )
-    , mIsValidFlag( 0 )
-    , mValidationHint( "" )
-    , mAxisInverted( false )
 {
-  mCRS = OSRNewSpatialReference( nullptr );
+  d = new QgsCoordinateReferenceSystemPrivate();
 }
 
 QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem( const QString& theDefinition )
-    : mSrsId( 0 )
-    , mGeoFlag( false )
-    , mMapUnits( QGis::UnknownUnit )
-    , mSRID( 0 )
-    , mIsValidFlag( 0 )
-    , mValidationHint( "" )
-    , mAxisInverted( false )
 {
-  mCRS = OSRNewSpatialReference( nullptr );
+  d = new QgsCoordinateReferenceSystemPrivate();
   createFromString( theDefinition );
 }
 
-
 QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem( const long theId, CrsType theType )
-    : mSrsId( 0 )
-    , mGeoFlag( false )
-    , mMapUnits( QGis::UnknownUnit )
-    , mSRID( 0 )
-    , mIsValidFlag( 0 )
-    , mValidationHint( "" )
-    , mAxisInverted( false )
 {
-  mCRS = OSRNewSpatialReference( nullptr );
+  d = new QgsCoordinateReferenceSystemPrivate();
   createFromId( theId, theType );
 }
 
+QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem( const QgsCoordinateReferenceSystem &srs )
+    : d( srs.d )
+{
+}
+
+QgsCoordinateReferenceSystem& QgsCoordinateReferenceSystem::operator=( const QgsCoordinateReferenceSystem & srs )
+{
+  d = srs.d;
+  return *this;
+}
+
 QgsCoordinateReferenceSystem::~QgsCoordinateReferenceSystem()
 {
-  OSRDestroySpatialReference( mCRS );
 }
 
 bool QgsCoordinateReferenceSystem::createFromId( const long theId, CrsType theType )
@@ -208,101 +197,76 @@ void QgsCoordinateReferenceSystem::setupESRIWktFix()
 #endif
 }
 
-bool QgsCoordinateReferenceSystem::createFromOgcWmsCrs( QString theCrs )
+bool QgsCoordinateReferenceSystem::createFromOgcWmsCrs( const QString& theCrs )
 {
+  QString wmsCrs = theCrs;
+
   QRegExp re( "urn:ogc:def:crs:([^:]+).+([^:]+)", Qt::CaseInsensitive );
-  if ( re.exactMatch( theCrs ) )
+  if ( re.exactMatch( wmsCrs ) )
   {
-    theCrs = re.cap( 1 ) + ':' + re.cap( 2 );
+    wmsCrs = re.cap( 1 ) + ':' + re.cap( 2 );
   }
   else
   {
     re.setPattern( "(user|custom|qgis):(\\d+)" );
-    if ( re.exactMatch( theCrs ) && createFromSrsId( re.cap( 2 ).toInt() ) )
+    if ( re.exactMatch( wmsCrs ) && createFromSrsId( re.cap( 2 ).toInt() ) )
     {
       return true;
     }
   }
 
-  if ( loadFromDb( QgsApplication::srsDbFilePath(), "lower(auth_name||':'||auth_id)", theCrs.toLower() ) )
+  if ( loadFromDb( QgsApplication::srsDbFilePath(), "lower(auth_name||':'||auth_id)", wmsCrs.toLower() ) )
     return true;
 
   // NAD27
-  if ( theCrs.compare( "CRS:27", Qt::CaseInsensitive ) == 0 ||
-       theCrs.compare( "OGC:CRS27", Qt::CaseInsensitive ) == 0 )
+  if ( wmsCrs.compare( "CRS:27", Qt::CaseInsensitive ) == 0 ||
+       wmsCrs.compare( "OGC:CRS27", Qt::CaseInsensitive ) == 0 )
   {
     // TODO: verify same axis orientation
     return createFromOgcWmsCrs( "EPSG:4267" );
   }
 
   // NAD83
-  if ( theCrs.compare( "CRS:83", Qt::CaseInsensitive ) == 0 ||
-       theCrs.compare( "OGC:CRS83", Qt::CaseInsensitive ) == 0 )
+  if ( wmsCrs.compare( "CRS:83", Qt::CaseInsensitive ) == 0 ||
+       wmsCrs.compare( "OGC:CRS83", Qt::CaseInsensitive ) == 0 )
   {
     // TODO: verify same axis orientation
     return createFromOgcWmsCrs( "EPSG:4269" );
   }
 
   // WGS84
-  if ( theCrs.compare( "CRS:84", Qt::CaseInsensitive ) == 0 ||
-       theCrs.compare( "OGC:CRS84", Qt::CaseInsensitive ) == 0 )
+  if ( wmsCrs.compare( "CRS:84", Qt::CaseInsensitive ) == 0 ||
+       wmsCrs.compare( "OGC:CRS84", Qt::CaseInsensitive ) == 0 )
   {
     createFromOgcWmsCrs( "EPSG:4326" );
-    mAxisInverted = 0;
-    return mIsValidFlag;
+
+    d.detach();
+    d->mAxisInverted = false;
+    d->mAxisInvertedDirty = false;
+
+    return d->mIsValid;
   }
 
   return false;
 }
 
-QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem( const QgsCoordinateReferenceSystem &srs )
-{
-  mCRS = OSRNewSpatialReference( nullptr );
-  *this = srs;
-}
-
-// Assignment operator
-QgsCoordinateReferenceSystem& QgsCoordinateReferenceSystem::operator=( const QgsCoordinateReferenceSystem & srs )
-{
-  if ( &srs != this )
-  {
-    mSrsId = srs.mSrsId;
-    mDescription = srs.mDescription;
-    mProjectionAcronym = srs.mProjectionAcronym;
-    mEllipsoidAcronym = srs.mEllipsoidAcronym;
-    mGeoFlag = srs.mGeoFlag;
-    mAxisInverted = srs.mAxisInverted;
-    mMapUnits = srs.mMapUnits;
-    mSRID = srs.mSRID;
-    mAuthId = srs.mAuthId;
-    mIsValidFlag = srs.mIsValidFlag;
-    mValidationHint = srs.mValidationHint;
-    mWkt = srs.mWkt;
-    mProj4 = srs.mProj4;
-    if ( mIsValidFlag )
-    {
-      OSRDestroySpatialReference( mCRS );
-      mCRS = OSRClone( srs.mCRS );
-    }
-  }
-  return *this;
-}
-
 // Misc helper functions -----------------------
 
 
 void QgsCoordinateReferenceSystem::validate()
 {
-  if ( mIsValidFlag )
+  if ( d->mIsValid )
     return;
 
+  d.detach();
+
   // try to validate using custom validation routines
   if ( mCustomSrsValidation )
     mCustomSrsValidation( *this );
 
-  if ( !mIsValidFlag )
+  if ( !d->mIsValid )
   {
-    *this = QgsCRSCache::instance()->crsByAuthId( GEO_EPSG_CRS_AUTHID );
+    *this = QgsCRSCache::instance()->crsByOgcWmsCrs( GEO_EPSG_CRS_AUTHID );
   }
 }
 
@@ -320,15 +284,17 @@ bool QgsCoordinateReferenceSystem::createFromSrsId( long id )
 
 bool QgsCoordinateReferenceSystem::loadFromDb( const QString& db, const QString& expression, const QString& value )
 {
+  d.detach();
+
   QgsDebugMsgLevel( "load CRS from " + db + " where " + expression + " is " + value, 3 );
-  mIsValidFlag = false;
-  mWkt.clear();
+  d->mIsValid = false;
+  d->mWkt.clear();
 
   QFileInfo myInfo( db );
   if ( !myInfo.exists() )
   {
     QgsDebugMsg( "failed : " + db + " does not exist!" );
-    return mIsValidFlag;
+    return d->mIsValid;
   }
 
   sqlite3      *myDatabase;
@@ -340,7 +306,7 @@ bool QgsCoordinateReferenceSystem::loadFromDb( const QString& db, const QString&
   if ( myResult != SQLITE_OK )
   {
     QgsDebugMsg( "failed : " + db + " could not be opened!" );
-    return mIsValidFlag;
+    return d->mIsValid;
   }
 
   /*
@@ -364,33 +330,33 @@ bool QgsCoordinateReferenceSystem::loadFromDb( const QString& db, const QString&
   // XXX Need to free memory from the error msg if one is set
   if ( myResult == SQLITE_OK && sqlite3_step( myPreparedStatement ) == SQLITE_ROW )
   {
-    mSrsId = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text(
-                                  myPreparedStatement, 0 ) ) ).toLong();
-    mDescription = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text(
-                                        myPreparedStatement, 1 ) ) );
-    mProjectionAcronym = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 2 ) ) );
-    mEllipsoidAcronym = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 3 ) ) );
-    mProj4 = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 4 ) ) );
-    mSRID = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 5 ) ) ).toLong() ;
-    mAuthId = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 6 ) ) );
-    mGeoFlag = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 7 ) ) ).toInt() != 0;
-    mAxisInverted = -1;
+    d->mSrsId = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text(
+                                     myPreparedStatement, 0 ) ) ).toLong();
+    d->mDescription = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text(
+                                           myPreparedStatement, 1 ) ) );
+    d->mProjectionAcronym = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 2 ) ) );
+    d->mEllipsoidAcronym = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 3 ) ) );
+    d->mProj4 = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 4 ) ) );
+    d->mSRID = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 5 ) ) ).toLong() ;
+    d->mAuthId = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 6 ) ) );
+    d->mIsGeographic = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 7 ) ) ).toInt() != 0;
+    d->mAxisInvertedDirty = true;
 
-    if ( mSrsId >= USER_CRS_START_ID && mAuthId.isEmpty() )
+    if ( d->mSrsId >= USER_CRS_START_ID && d->mAuthId.isEmpty() )
     {
-      mAuthId = QString( "USER:%1" ).arg( mSrsId );
+      d->mAuthId = QString( "USER:%1" ).arg( d->mSrsId );
     }
-    else if ( mAuthId.startsWith( "EPSG:", Qt::CaseInsensitive ) )
+    else if ( d->mAuthId.startsWith( "EPSG:", Qt::CaseInsensitive ) )
     {
-      OSRDestroySpatialReference( mCRS );
-      mCRS = OSRNewSpatialReference( nullptr );
-      mIsValidFlag = OSRSetFromUserInput( mCRS, mAuthId.toLower().toAscii() ) == OGRERR_NONE;
+      OSRDestroySpatialReference( d->mCRS );
+      d->mCRS = OSRNewSpatialReference( nullptr );
+      d->mIsValid = OSRSetFromUserInput( d->mCRS, d->mAuthId.toLower().toAscii() ) == OGRERR_NONE;
       setMapUnits();
     }
 
-    if ( !mIsValidFlag )
+    if ( !d->mIsValid )
     {
-      setProj4String( mProj4 );
+      setProj4String( d->mProj4 );
     }
   }
   else
@@ -399,22 +365,22 @@ bool QgsCoordinateReferenceSystem::loadFromDb( const QString& db, const QString&
   }
   sqlite3_finalize( myPreparedStatement );
   sqlite3_close( myDatabase );
-  return mIsValidFlag;
+  return d->mIsValid;
 }
 
 bool QgsCoordinateReferenceSystem::axisInverted() const
 {
-  if ( mAxisInverted == -1 )
+  if ( d->mAxisInvertedDirty )
   {
     OGRAxisOrientation orientation;
-    OSRGetAxis( mCRS, OSRIsGeographic( mCRS ) ? "GEOGCS" : "PROJCS", 0, &orientation );
+    OSRGetAxis( d->mCRS, OSRIsGeographic( d->mCRS ) ? "GEOGCS" : "PROJCS", 0, &orientation );
 
     // If axis orientation is unknown, try again with OSRImportFromEPSGA for EPSG crs
-    if ( orientation == OAO_Other && mAuthId.startsWith( "EPSG:", Qt::CaseInsensitive ) )
+    if ( orientation == OAO_Other && d->mAuthId.startsWith( "EPSG:", Qt::CaseInsensitive ) )
     {
       OGRSpatialReferenceH crs = OSRNewSpatialReference( nullptr );
 
-      if ( OSRImportFromEPSGA( crs, mAuthId.mid( 5 ).toInt() ) == OGRERR_NONE )
+      if ( OSRImportFromEPSGA( crs, d->mAuthId.mid( 5 ).toInt() ) == OGRERR_NONE )
       {
         OSRGetAxis( crs, OSRIsGeographic( crs ) ? "GEOGCS" : "PROJCS", 0, &orientation );
       }
@@ -422,28 +388,31 @@ bool QgsCoordinateReferenceSystem::axisInverted() const
       OSRDestroySpatialReference( crs );
     }
 
-    mAxisInverted = orientation == OAO_North;
+    d->mAxisInverted = orientation == OAO_North;
+    d->mAxisInvertedDirty = false;
   }
 
-  return mAxisInverted != 0;
+  return d->mAxisInverted;
 }
 
 bool QgsCoordinateReferenceSystem::createFromWkt( const QString &theWkt )
 {
-  mIsValidFlag = false;
-  mWkt.clear();
-  mProj4.clear();
+  d.detach();
+
+  d->mIsValid = false;
+  d->mWkt.clear();
+  d->mProj4.clear();
 
   if ( theWkt.isEmpty() )
   {
     QgsDebugMsg( "theWkt is uninitialized, operation failed" );
-    return mIsValidFlag;
+    return d->mIsValid;
   }
   QgsDebugMsg( "wkt: " + theWkt );
   QByteArray ba = theWkt.toLatin1();
   const char *pWkt = ba.data();
 
-  OGRErr myInputResult = OSRImportFromWkt( mCRS, const_cast< char ** >( & pWkt ) );
+  OGRErr myInputResult = OSRImportFromWkt( d->mCRS, const_cast< char ** >( & pWkt ) );
 
   if ( myInputResult != OGRERR_NONE )
   {
@@ -452,14 +421,14 @@ bool QgsCoordinateReferenceSystem::createFromWkt( const QString &theWkt )
     QgsDebugMsg( "INPUT: " + theWkt );
     QgsDebugMsg( QString( "UNUSED WKT: %1" ).arg( pWkt ) );
     QgsDebugMsg( "---------------------------------------------------------------\n" );
-    return mIsValidFlag;
+    return d->mIsValid;
   }
 
-  if ( OSRAutoIdentifyEPSG( mCRS ) == OGRERR_NONE )
+  if ( OSRAutoIdentifyEPSG( d->mCRS ) == OGRERR_NONE )
   {
     QString authid = QString( "%1:%2" )
-                     .arg( OSRGetAuthorityName( mCRS, nullptr ),
-                           OSRGetAuthorityCode( mCRS, nullptr ) );
+                     .arg( OSRGetAuthorityName( d->mCRS, nullptr ),
+                           OSRGetAuthorityCode( d->mCRS, nullptr ) );
     QgsDebugMsg( "authid recognized as " + authid );
     return createFromOgcWmsCrs( authid );
   }
@@ -470,7 +439,7 @@ bool QgsCoordinateReferenceSystem::createFromWkt( const QString &theWkt )
 
   // create the proj4 structs needed for transforming
   char *proj4src = nullptr;
-  OSRExportToProj4( mCRS, &proj4src );
+  OSRExportToProj4( d->mCRS, &proj4src );
 
   //now that we have the proj4string, delegate to createFromProj4 so
   // that we can try to fill in the remaining class members...
@@ -480,9 +449,9 @@ bool QgsCoordinateReferenceSystem::createFromWkt( const QString &theWkt )
     CPLFree( proj4src );
 
     // try fixed up version
-    OSRFixup( mCRS );
+    OSRFixup( d->mCRS );
 
-    OSRExportToProj4( mCRS, &proj4src );
+    OSRExportToProj4( d->mCRS, &proj4src );
 
     createFromProj4( proj4src );
   }
@@ -490,7 +459,7 @@ bool QgsCoordinateReferenceSystem::createFromWkt( const QString &theWkt )
   // this behavior was changed in order to separate creation and saving.
   // Not sure if it necessary to save it here, should be checked by someone
   // familiar with the code (should also give a more descriptive name to the generated CRS)
-  if ( mSrsId == 0 )
+  if ( d->mSrsId == 0 )
   {
     QString myName = QString( " * %1 (%2)" )
                      .arg( QObject::tr( "Generated CRS", "A CRS automatically generated from layer info get this prefix for description" ),
@@ -500,17 +469,19 @@ bool QgsCoordinateReferenceSystem::createFromWkt( const QString &theWkt )
 
   CPLFree( proj4src );
 
-  return mIsValidFlag;
+  return d->mIsValid;
   //setMapunits will be called by createfromproj above
 }
 
 bool QgsCoordinateReferenceSystem::isValid() const
 {
-  return mIsValidFlag;
+  return d->mIsValid;
 }
 
 bool QgsCoordinateReferenceSystem::createFromProj4( const QString &theProj4String )
 {
+  d.detach();
+
   //
   // Examples:
   // +proj=tmerc +lat_0=0 +lon_0=-62 +k=0.999500 +x_0=400000 +y_0=0
@@ -521,29 +492,29 @@ bool QgsCoordinateReferenceSystem::createFromProj4( const QString &theProj4Strin
   //
   QString myProj4String = theProj4String.trimmed();
   QgsDebugMsg( "proj4: " + myProj4String );
-  mIsValidFlag = false;
-  mWkt.clear();
+  d->mIsValid = false;
+  d->mWkt.clear();
 
   QRegExp myProjRegExp( "\\+proj=(\\S+)" );
   int myStart = myProjRegExp.indexIn( myProj4String );
   if ( myStart == -1 )
   {
     QgsDebugMsg( "proj string supplied has no +proj argument" );
-    return mIsValidFlag;
+    return d->mIsValid;
   }
 
-  mProjectionAcronym = myProjRegExp.cap( 1 );
+  d->mProjectionAcronym = myProjRegExp.cap( 1 );
 
   QRegExp myEllipseRegExp( "\\+ellps=(\\S+)" );
   myStart = myEllipseRegExp.indexIn( myProj4String );
   if ( myStart == -1 )
   {
     QgsDebugMsg( "proj string supplied has no +ellps argument" );
-    mEllipsoidAcronym = "";
+    d->mEllipsoidAcronym.clear();
   }
   else
   {
-    mEllipsoidAcronym = myEllipseRegExp.cap( 1 );
+    d->mEllipsoidAcronym = myEllipseRegExp.cap( 1 );
   }
 
   QRegExp myAxisRegExp( "\\+a=(\\S+)" );
@@ -690,19 +661,19 @@ bool QgsCoordinateReferenceSystem::createFromProj4( const QString &theProj4Strin
     }
     else
     {
-      mIsValidFlag = false;
+      d->mIsValid = false;
     }
   }
 
   // if we failed to look up the projection in database, don't worry. we can still use it :)
-  if ( !mIsValidFlag )
+  if ( !d->mIsValid )
   {
     QgsDebugMsg( "Projection is not found in databases." );
     //setProj4String will set mIsValidFlag to true if there is no issue
     setProj4String( myProj4String );
   }
 
-  return mIsValidFlag;
+  return d->mIsValid;
 }
 
 //private method meant for internal use by this class only
@@ -823,79 +794,79 @@ QgsCoordinateReferenceSystem::RecordMap QgsCoordinateReferenceSystem::getRecord(
 
 long QgsCoordinateReferenceSystem::srsid() const
 {
-  return mSrsId;
+  return d->mSrsId;
 }
 
 long QgsCoordinateReferenceSystem::postgisSrid() const
 {
-  return mSRID;
+  return d->mSRID;
 }
 
 QString QgsCoordinateReferenceSystem::authid() const
 {
-  return mAuthId;
+  return d->mAuthId;
 }
 
 QString QgsCoordinateReferenceSystem::description() const
 {
-  if ( mDescription.isNull() )
+  if ( d->mDescription.isNull() )
   {
     return "";
   }
   else
   {
-    return mDescription;
+    return d->mDescription;
   }
 }
 
 QString QgsCoordinateReferenceSystem::projectionAcronym() const
 {
-  if ( mProjectionAcronym.isNull() )
+  if ( d->mProjectionAcronym.isNull() )
   {
     return "";
   }
   else
   {
-    return mProjectionAcronym;
+    return d->mProjectionAcronym;
   }
 }
 
 QString QgsCoordinateReferenceSystem::ellipsoidAcronym() const
 {
-  if ( mEllipsoidAcronym.isNull() )
+  if ( d->mEllipsoidAcronym.isNull() )
   {
     return "";
   }
   else
   {
-    return mEllipsoidAcronym;
+    return d->mEllipsoidAcronym;
   }
 }
 
 QString QgsCoordinateReferenceSystem::toProj4() const
 {
-  if ( !mIsValidFlag )
+  if ( !d->mIsValid )
     return "";
 
-  if ( mProj4.isEmpty() )
+  if ( d->mProj4.isEmpty() )
   {
     char *proj4src = nullptr;
-    OSRExportToProj4( mCRS, &proj4src );
-    mProj4 = proj4src;
+    OSRExportToProj4( d->mCRS, &proj4src );
+    d->mProj4 = proj4src;
     CPLFree( proj4src );
   }
   // Stray spaces at the end?
-  return mProj4.trimmed();
+  return d->mProj4.trimmed();
 }
 
 bool QgsCoordinateReferenceSystem::geographicFlag() const
 {
-  return mGeoFlag;
+  return d->mIsGeographic;
 }
 
 QGis::UnitType QgsCoordinateReferenceSystem::mapUnits() const
 {
-  return mMapUnits;
+  return d->mMapUnits;
 }
 
 
@@ -904,44 +875,51 @@ QGis::UnitType QgsCoordinateReferenceSystem::mapUnits() const
 
 void QgsCoordinateReferenceSystem::setInternalId( long theSrsId )
 {
-  mSrsId = theSrsId;
+  d.detach();
+  d->mSrsId = theSrsId;
 }
 void QgsCoordinateReferenceSystem::setAuthId( const QString& authId )
 {
-  mAuthId = authId;
+  d.detach();
+  d->mAuthId = authId;
 }
 void QgsCoordinateReferenceSystem::setSrid( long theSrid )
 {
-  mSRID = theSrid;
+  d.detach();
+  d->mSRID = theSrid;
 }
 void QgsCoordinateReferenceSystem::setDescription( const QString& theDescription )
 {
-  mDescription = theDescription;
+  d.detach();
+  d->mDescription = theDescription;
 }
 void QgsCoordinateReferenceSystem::setProj4String( const QString& theProj4String )
 {
-  mProj4 = theProj4String;
+  d.detach();
+  d->mProj4 = theProj4String;
 
   QgsLocaleNumC l;
 
-  OSRDestroySpatialReference( mCRS );
-  mCRS = OSRNewSpatialReference( nullptr );
-  mIsValidFlag = OSRImportFromProj4( mCRS, theProj4String.trimmed().toLatin1().constData() ) == OGRERR_NONE;
+  OSRDestroySpatialReference( d->mCRS );
+  d->mCRS = OSRNewSpatialReference( nullptr );
+  d->mIsValid = OSRImportFromProj4( d->mCRS, theProj4String.trimmed().toLatin1().constData() ) == OGRERR_NONE;
   // OSRImportFromProj4() may accept strings that are not valid proj.4 strings,
   // e.g if they lack a +ellps parameter, it will automatically add +ellps=WGS84, but as
   // we use the original mProj4 with QgsCoordinateTransform, it will fail to initialize
   // so better detect it now.
-  projPJ theProj = pj_init_plus( theProj4String.trimmed().toLatin1().constData() );
+  projCtx pContext = pj_ctx_alloc();
+  projPJ theProj = pj_init_plus_ctx( pContext, theProj4String.trimmed().toLatin1().constData() );
   if ( !theProj )
   {
     QgsDebugMsg( "proj.4 string rejected by pj_init_plus()" );
-    mIsValidFlag = false;
+    d->mIsValid = false;
   }
   else
   {
     pj_free( theProj );
   }
-  mWkt.clear();
+  pj_ctx_free( pContext );
+  d->mWkt.clear();
   setMapUnits();
 
 #if defined(QGISDEBUG) && QGISDEBUG>=3
@@ -950,26 +928,31 @@ void QgsCoordinateReferenceSystem::setProj4String( const QString& theProj4String
 }
 void QgsCoordinateReferenceSystem::setGeographicFlag( bool theGeoFlag )
 {
-  mGeoFlag = theGeoFlag;
+  d.detach();
+  d->mIsGeographic = theGeoFlag;
 }
 void QgsCoordinateReferenceSystem::setEpsg( long theEpsg )
 {
-  mAuthId = QString( "EPSG:%1" ).arg( theEpsg );
+  d.detach();
+  d->mAuthId = QString( "EPSG:%1" ).arg( theEpsg );
 }
 void  QgsCoordinateReferenceSystem::setProjectionAcronym( const QString& theProjectionAcronym )
 {
-  mProjectionAcronym = theProjectionAcronym;
+  d.detach();
+  d->mProjectionAcronym = theProjectionAcronym;
 }
 void  QgsCoordinateReferenceSystem::setEllipsoidAcronym( const QString& theEllipsoidAcronym )
 {
-  mEllipsoidAcronym = theEllipsoidAcronym;
+  d.detach();
+  d->mEllipsoidAcronym = theEllipsoidAcronym;
 }
 
 void QgsCoordinateReferenceSystem::setMapUnits()
 {
-  if ( !mIsValidFlag )
+  d.detach();
+  if ( !d->mIsValid )
   {
-    mMapUnits = QGis::UnknownUnit;
+    d->mMapUnits = QGis::UnknownUnit;
     return;
   }
 
@@ -977,11 +960,11 @@ void QgsCoordinateReferenceSystem::setMapUnits()
 
   // Of interest to us is that this call adds in a unit parameter if
   // one doesn't already exist.
-  OSRFixup( mCRS );
+  OSRFixup( d->mCRS );
 
-  if ( OSRIsProjected( mCRS ) )
+  if ( OSRIsProjected( d->mCRS ) )
   {
-    double toMeter = OSRGetLinearUnits( mCRS, &unitName );
+    double toMeter = OSRGetLinearUnits( d->mCRS, &unitName );
     QString unit( unitName );
 
     // If the units parameter was created during the Fixup() call
@@ -998,25 +981,25 @@ void QgsCoordinateReferenceSystem::setMapUnits()
     QgsDebugMsg( "Projection has linear units of " + unit );
 
     if ( qgsDoubleNear( toMeter, 1.0 ) ) //Unit name for meters would be "metre"
-      mMapUnits = QGis::Meters;
+      d->mMapUnits = QGis::Meters;
     else if ( unit == "Foot" )
-      mMapUnits = QGis::Feet;
+      d->mMapUnits = QGis::Feet;
     else
     {
       QgsDebugMsg( "Unsupported map units of " + unit );
-      mMapUnits = QGis::UnknownUnit;
+      d->mMapUnits = QGis::UnknownUnit;
     }
   }
   else
   {
-    OSRGetAngularUnits( mCRS, &unitName );
+    OSRGetAngularUnits( d->mCRS, &unitName );
     QString unit( unitName );
     if ( unit == "degree" )
-      mMapUnits = QGis::Degrees;
+      d->mMapUnits = QGis::Degrees;
     else
     {
       QgsDebugMsg( "Unsupported map units of " + unit );
-      mMapUnits = QGis::UnknownUnit;
+      d->mMapUnits = QGis::UnknownUnit;
     }
     QgsDebugMsgLevel( "Projection has angular units of " + unit, 3 );
   }
@@ -1033,9 +1016,8 @@ void QgsCoordinateReferenceSystem::setMapUnits()
 */
 long QgsCoordinateReferenceSystem::findMatchingProj()
 {
-  QgsDebugMsg( "entered." );
-  if ( mEllipsoidAcronym.isNull() ||  mProjectionAcronym.isNull()
-       || !mIsValidFlag )
+  if ( d->mEllipsoidAcronym.isNull() || d->mProjectionAcronym.isNull()
+       || !d->mIsValid )
   {
     QgsDebugMsg( "QgsCoordinateReferenceSystem::findMatchingProj will only "
                  "work if prj acr ellipsoid acr and proj4string are set"
@@ -1052,8 +1034,8 @@ long QgsCoordinateReferenceSystem::findMatchingProj()
   // needed to populate the list
   QString mySql = QString( "select srs_id,parameters from tbl_srs where "
                            "projection_acronym=%1 and ellipsoid_acronym=%2 order by deprecated" )
-                  .arg( quotedValue( mProjectionAcronym ),
-                        quotedValue( mEllipsoidAcronym ) );
+                  .arg( quotedValue( d->mProjectionAcronym ),
+                        quotedValue( d->mEllipsoidAcronym ) );
   // Get the full path name to the sqlite3 spatial reference database.
   QString myDatabaseFileName = QgsApplication::srsDbFilePath();
 
@@ -1136,8 +1118,8 @@ long QgsCoordinateReferenceSystem::findMatchingProj()
 
 bool QgsCoordinateReferenceSystem::operator==( const QgsCoordinateReferenceSystem &theSrs ) const
 {
-  return ( !mIsValidFlag && !theSrs.mIsValidFlag ) ||
-         ( mIsValidFlag && theSrs.mIsValidFlag && theSrs.authid() == authid() );
+  return ( !d->mIsValid && !theSrs.d->mIsValid ) ||
+         ( d->mIsValid && theSrs.d->mIsValid && theSrs.authid() == authid() );
 }
 
 bool QgsCoordinateReferenceSystem::operator!=( const QgsCoordinateReferenceSystem &theSrs ) const
@@ -1147,20 +1129,21 @@ bool QgsCoordinateReferenceSystem::operator!=( const QgsCoordinateReferenceSyste
 
 QString QgsCoordinateReferenceSystem::toWkt() const
 {
-  if ( mWkt.isEmpty() )
+  if ( d->mWkt.isEmpty() )
   {
     char *wkt;
-    if ( OSRExportToWkt( mCRS, &wkt ) == OGRERR_NONE )
+    if ( OSRExportToWkt( d->mCRS, &wkt ) == OGRERR_NONE )
     {
-      mWkt = wkt;
+      d->mWkt = wkt;
       OGRFree( wkt );
     }
   }
-  return mWkt;
+  return d->mWkt;
 }
 
 bool QgsCoordinateReferenceSystem::readXML( const QDomNode & theNode )
 {
+  d.detach();
   QgsDebugMsg( "Reading Spatial Ref Sys from xml ------------------------!" );
   bool result = true;
   QDomNode srsNode  = theNode.namedItem( "spatialrefsys" );
@@ -1178,7 +1161,7 @@ bool QgsCoordinateReferenceSystem::readXML( const QDomNode & theNode )
       myNode = srsNode.namedItem( "authid" );
       if ( !myNode.isNull() )
       {
-        operator=( QgsCRSCache::instance()->crsByAuthId( myNode.toElement().text() ) );
+        operator=( QgsCRSCache::instance()->crsByOgcWmsCrs( myNode.toElement().text() ) );
         if ( isValid() )
         {
           initialized = true;
@@ -1255,13 +1238,13 @@ bool QgsCoordinateReferenceSystem::readXML( const QDomNode & theNode )
         setMapUnits();
 
         //@TODO this srs needs to be validated!!!
-        mIsValidFlag = true; //shamelessly hard coded for now
+        d->mIsValid = true; //shamelessly hard coded for now
       }
       //TODO: createFromProj4 used to save to the user database any new CRS
       // this behavior was changed in order to separate creation and saving.
       // Not sure if it necessary to save it here, should be checked by someone
       // familiar with the code (should also give a more descriptive name to the generated CRS)
-      if ( mSrsId == 0 )
+      if ( d->mSrsId == 0 )
       {
         QString myName = QString( " * %1 (%2)" )
                          .arg( QObject::tr( "Generated CRS", "A CRS automatically generated from layer info get this prefix for description" ),
@@ -1361,7 +1344,7 @@ QString QgsCoordinateReferenceSystem::proj4FromSrsId( const int theSrsId )
     if ( !myFileInfo.exists() ) //its unlikely that this condition will ever be reached
     {
       QgsDebugMsg( "users qgis.db not found" );
-      return nullptr;
+      return QString();
     }
   }
   else //must be  a system projection then
@@ -1434,11 +1417,11 @@ CUSTOM_CRS_VALIDATION QgsCoordinateReferenceSystem::customSrsValidation()
 void QgsCoordinateReferenceSystem::debugPrint()
 {
   QgsDebugMsg( "***SpatialRefSystem***" );
-  QgsDebugMsg( "* Valid : " + ( mIsValidFlag ? QString( "true" ) : QString( "false" ) ) );
-  QgsDebugMsg( "* SrsId : " + QString::number( mSrsId ) );
+  QgsDebugMsg( "* Valid : " + ( d->mIsValid ? QString( "true" ) : QString( "false" ) ) );
+  QgsDebugMsg( "* SrsId : " + QString::number( d->mSrsId ) );
   QgsDebugMsg( "* Proj4 : " + toProj4() );
   QgsDebugMsg( "* WKT   : " + toWkt() );
-  QgsDebugMsg( "* Desc. : " + mDescription );
+  QgsDebugMsg( "* Desc. : " + d->mDescription );
   if ( mapUnits() == QGis::Meters )
   {
     QgsDebugMsg( "* Units : meters" );
@@ -1455,12 +1438,13 @@ void QgsCoordinateReferenceSystem::debugPrint()
 
 void QgsCoordinateReferenceSystem::setValidationHint( const QString& html )
 {
-  mValidationHint = html;
+  d.detach();
+  d->mValidationHint = html;
 }
 
 QString QgsCoordinateReferenceSystem::validationHint()
 {
-  return mValidationHint;
+  return d->mValidationHint;
 }
 
 /// Copied from QgsCustomProjectionDialog ///
@@ -1468,7 +1452,7 @@ QString QgsCoordinateReferenceSystem::validationHint()
 
 bool QgsCoordinateReferenceSystem::saveAsUserCRS( const QString& name )
 {
-  if ( ! mIsValidFlag )
+  if ( !d->mIsValid )
   {
     QgsDebugMsg( "Can't save an invalid CRS!" );
     return false;
@@ -1476,7 +1460,7 @@ bool QgsCoordinateReferenceSystem::saveAsUserCRS( const QString& name )
 
   QString mySql;
 
-  QString proj4String = mProj4;
+  QString proj4String = d->mProj4;
   if ( proj4String.isEmpty() )
   {
     proj4String = toProj4();
@@ -1823,7 +1807,7 @@ int QgsCoordinateReferenceSystem::syncDb()
         ellps = ellipseRegExp.cap( 1 );
       }
 
-      QString name( OSRIsGeographic( crs ) ? OSRGetAttrValue( crs, "GEOCS", 0 ) : OSRGetAttrValue( crs, "PROJCS", 0 ) );
+      QString name( OSRIsGeographic( crs ) ? OSRGetAttrValue( crs, "GEOGCS", 0 ) : OSRGetAttrValue( crs, "PROJCS", 0 ) );
       if ( name.isEmpty() )
         name = QObject::tr( "Imported from GDAL" );
 
@@ -1876,6 +1860,8 @@ int QgsCoordinateReferenceSystem::syncDb()
                sqlite3_errmsg( database ) );
   }
 
+  projCtx pContext = pj_ctx_alloc();
+
 #if !defined(PJ_VERSION) || PJ_VERSION!=470
   sql = QString( "select auth_name,auth_id,parameters from tbl_srs WHERE auth_name<>'EPSG' AND NOT deprecated AND NOT noupdate" );
   if ( sqlite3_prepare( database, sql.toAscii(), sql.size(), &select, &tail ) == SQLITE_OK )
@@ -1887,11 +1873,11 @@ int QgsCoordinateReferenceSystem::syncDb()
       const char *params    = reinterpret_cast< const char * >( sqlite3_column_text( select, 2 ) );
 
       QString input = QString( "+init=%1:%2" ).arg( QString( auth_name ).toLower(), auth_id );
-      projPJ pj = pj_init_plus( input.toAscii() );
+      projPJ pj = pj_init_plus_ctx( pContext, input.toAscii() );
       if ( !pj )
       {
         input = QString( "+init=%1:%2" ).arg( QString( auth_name ).toUpper(), auth_id );
-        pj = pj_init_plus( input.toAscii() );
+        pj = pj_init_plus_ctx( pContext, input.toAscii() );
       }
 
       if ( pj )
@@ -1954,6 +1940,7 @@ int QgsCoordinateReferenceSystem::syncDb()
 #endif
 
   OSRDestroySpatialReference( crs );
+  pj_ctx_free( pContext );
 
   if ( sqlite3_exec( database, "COMMIT", nullptr, nullptr, nullptr ) != SQLITE_OK )
   {
@@ -2169,11 +2156,11 @@ QString QgsCoordinateReferenceSystem::geographicCRSAuthId() const
 {
   if ( geographicFlag() )
   {
-    return mAuthId;
+    return d->mAuthId;
   }
-  else if ( mCRS )
+  else if ( d->mCRS )
   {
-    return OSRGetAuthorityName( mCRS, "GEOGCS" ) + QLatin1String( ":" ) + OSRGetAuthorityCode( mCRS, "GEOGCS" );
+    return OSRGetAuthorityName( d->mCRS, "GEOGCS" ) + QLatin1String( ":" ) + OSRGetAuthorityCode( d->mCRS, "GEOGCS" );
   }
   else
   {
