@@ -37,12 +37,12 @@ extern "C"
 // if defined shows all information about transform to stdout
 // #define COORDINATE_TRANSFORM_VERBOSE
 
+QThreadStorage< QgsCoordinateTransform::QgsProjContextStore* > QgsCoordinateTransform::mProjContext;
+
 QgsCoordinateTransform::QgsCoordinateTransform()
     : QObject()
     , mShortCircuit( false )
     , mInitialisedFlag( false )
-    , mSourceProjection( nullptr )
-    , mDestinationProjection( nullptr )
     , mSourceDatumTransform( -1 )
     , mDestinationDatumTransform( -1 )
 {
@@ -53,8 +53,6 @@ QgsCoordinateTransform::QgsCoordinateTransform( const QgsCoordinateReferenceSyst
     : QObject()
     , mShortCircuit( false )
     , mInitialisedFlag( false )
-    , mSourceProjection( nullptr )
-    , mDestinationProjection( nullptr )
     , mSourceDatumTransform( -1 )
     , mDestinationDatumTransform( -1 )
 {
@@ -67,10 +65,8 @@ QgsCoordinateTransform::QgsCoordinateTransform( const QgsCoordinateReferenceSyst
 QgsCoordinateTransform::QgsCoordinateTransform( long theSourceSrsId, long theDestSrsId )
     : QObject()
     , mInitialisedFlag( false )
-    , mSourceCRS( theSourceSrsId, QgsCoordinateReferenceSystem::InternalCrsId )
-    , mDestCRS( theDestSrsId, QgsCoordinateReferenceSystem::InternalCrsId )
-    , mSourceProjection( nullptr )
-    , mDestinationProjection( nullptr )
+    , mSourceCRS( QgsCRSCache::instance()->crsBySrsId( theSourceSrsId ) )
+    , mDestCRS( QgsCRSCache::instance()->crsBySrsId( theDestSrsId ) )
     , mSourceDatumTransform( -1 )
     , mDestinationDatumTransform( -1 )
 {
@@ -80,14 +76,12 @@ QgsCoordinateTransform::QgsCoordinateTransform( long theSourceSrsId, long theDes
 QgsCoordinateTransform::QgsCoordinateTransform( const QString& theSourceCRS, const QString& theDestCRS )
     : QObject()
     , mInitialisedFlag( false )
-    , mSourceProjection( nullptr )
-    , mDestinationProjection( nullptr )
     , mSourceDatumTransform( -1 )
     , mDestinationDatumTransform( -1 )
 {
   setFinder();
-  mSourceCRS.createFromWkt( theSourceCRS );
-  mDestCRS.createFromWkt( theDestCRS );
+  mSourceCRS = QgsCRSCache::instance()->crsByWkt( theSourceCRS );
+  mDestCRS = QgsCRSCache::instance()->crsByWkt( theDestCRS );
   // initialize the coordinate system data structures
   //XXX Who spells initialize initialise?
   //XXX A: Its the queen's english....
@@ -100,15 +94,13 @@ QgsCoordinateTransform::QgsCoordinateTransform( long theSourceSrid,
     QgsCoordinateReferenceSystem::CrsType theSourceCRSType )
     : QObject()
     , mInitialisedFlag( false )
-    , mSourceProjection( nullptr )
-    , mDestinationProjection( nullptr )
     , mSourceDatumTransform( -1 )
     , mDestinationDatumTransform( -1 )
 {
   setFinder();
 
   mSourceCRS.createFromId( theSourceSrid, theSourceCRSType );
-  mDestCRS.createFromWkt( theDestWkt );
+  mDestCRS = QgsCRSCache::instance()->crsByWkt( theDestWkt );
   // initialize the coordinate system data structures
   //XXX Who spells initialize initialise?
   //XXX A: Its the queen's english....
@@ -118,15 +110,7 @@ QgsCoordinateTransform::QgsCoordinateTransform( long theSourceSrid,
 
 QgsCoordinateTransform::~QgsCoordinateTransform()
 {
-  // free the proj objects
-  if ( mSourceProjection )
-  {
-    pj_free( mSourceProjection );
-  }
-  if ( mDestinationProjection )
-  {
-    pj_free( mDestinationProjection );
-  }
+  freeProj();
 }
 
 QgsCoordinateTransform* QgsCoordinateTransform::clone() const
@@ -152,7 +136,7 @@ void QgsCoordinateTransform::setDestCRS( const QgsCoordinateReferenceSystem& the
 void QgsCoordinateTransform::setDestCRSID( long theCRSID )
 {
   //!todo Add some logic here to determine if the srsid is a system or user one
-  mDestCRS.createFromSrsId( theCRSID );
+  mDestCRS = QgsCRSCache::instance()->crsBySrsId( theCRSID );
   initialise();
 }
 
@@ -175,42 +159,40 @@ void QgsCoordinateTransform::initialise()
   {
     //No destination projection is set so we set the default output projection to
     //be the same as input proj.
-    mDestCRS = QgsCRSCache::instance()->crsByAuthId( mSourceCRS.authid() );
+    mDestCRS = QgsCRSCache::instance()->crsByOgcWmsCrs( mSourceCRS.authid() );
   }
 
   bool useDefaultDatumTransform = ( mSourceDatumTransform == - 1 && mDestinationDatumTransform == -1 );
 
-  // init the projections (destination and source)
+  freeProj();
 
-  pj_free( mSourceProjection );
-  QString sourceProjString = mSourceCRS.toProj4();
+  mSourceProjString = mSourceCRS.toProj4();
   if ( !useDefaultDatumTransform )
   {
-    sourceProjString = stripDatumTransform( sourceProjString );
+    mSourceProjString = stripDatumTransform( mSourceProjString );
   }
   if ( mSourceDatumTransform != -1 )
   {
-    sourceProjString += ( ' ' + datumTransformString( mSourceDatumTransform ) );
+    mSourceProjString += ( ' ' + datumTransformString( mSourceDatumTransform ) );
   }
 
-  pj_free( mDestinationProjection );
-  QString destProjString = mDestCRS.toProj4();
+  mDestProjString = mDestCRS.toProj4();
   if ( !useDefaultDatumTransform )
   {
-    destProjString = stripDatumTransform( destProjString );
+    mDestProjString = stripDatumTransform( mDestProjString );
   }
   if ( mDestinationDatumTransform != -1 )
   {
-    destProjString += ( ' ' +  datumTransformString( mDestinationDatumTransform ) );
+    mDestProjString += ( ' ' +  datumTransformString( mDestinationDatumTransform ) );
   }
 
   if ( !useDefaultDatumTransform )
   {
-    addNullGridShifts( sourceProjString, destProjString );
+    addNullGridShifts( mSourceProjString, mDestProjString );
   }
 
-  mSourceProjection = pj_init_plus( sourceProjString.toUtf8() );
-  mDestinationProjection = pj_init_plus( destProjString.toUtf8() );
+  // create proj projections for current thread
+  QPair< projPJ, projPJ > res = threadLocalProjData();
 
 #ifdef COORDINATE_TRANSFORM_VERBOSE
   QgsDebugMsg( "From proj : " + mSourceCRS.toProj4() );
@@ -218,11 +200,11 @@ void QgsCoordinateTransform::initialise()
 #endif
 
   mInitialisedFlag = true;
-  if ( !mDestinationProjection )
+  if ( !res.second )
   {
     mInitialisedFlag = false;
   }
-  if ( !mSourceProjection )
+  if ( !res.first )
   {
     mInitialisedFlag = false;
   }
@@ -661,8 +643,12 @@ void QgsCoordinateTransform::transformCoords( int numPoints, double *x, double *
   QString dir;
   // if the source/destination projection is lat/long, convert the points to radians
   // prior to transforming
-  if (( pj_is_latlong( mDestinationProjection ) && ( direction == ReverseTransform ) )
-      || ( pj_is_latlong( mSourceProjection ) && ( direction == ForwardTransform ) ) )
+  QPair< projPJ, projPJ > projData = threadLocalProjData();
+  projPJ sourceProj = projData.first;
+  projPJ destProj = projData.second;
+
+  if (( pj_is_latlong( destProj ) && ( direction == ReverseTransform ) )
+      || ( pj_is_latlong( sourceProj ) && ( direction == ForwardTransform ) ) )
   {
     for ( int i = 0; i < numPoints; ++i )
     {
@@ -674,13 +660,13 @@ void QgsCoordinateTransform::transformCoords( int numPoints, double *x, double *
   int projResult;
   if ( direction == ReverseTransform )
   {
-    projResult = pj_transform( mDestinationProjection, mSourceProjection, numPoints, 0, x, y, z );
+    projResult = pj_transform( destProj, sourceProj, numPoints, 0, x, y, z );
   }
   else
   {
-    Q_ASSERT( mSourceProjection );
-    Q_ASSERT( mDestinationProjection );
-    projResult = pj_transform( mSourceProjection, mDestinationProjection, numPoints, 0, x, y, z );
+    Q_ASSERT( sourceProj );
+    Q_ASSERT( destProj );
+    projResult = pj_transform( sourceProj, destProj, numPoints, 0, x, y, z );
   }
 
   if ( projResult != 0 )
@@ -702,8 +688,8 @@ void QgsCoordinateTransform::transformCoords( int numPoints, double *x, double *
 
     dir = ( direction == ForwardTransform ) ? tr( "forward transform" ) : tr( "inverse transform" );
 
-    char *srcdef = pj_get_def( mSourceProjection, 0 );
-    char *dstdef = pj_get_def( mDestinationProjection, 0 );
+    char *srcdef = pj_get_def( sourceProj, 0 );
+    char *dstdef = pj_get_def( destProj, 0 );
 
     QString msg = tr( "%1 of\n"
                       "%2"
@@ -728,8 +714,8 @@ void QgsCoordinateTransform::transformCoords( int numPoints, double *x, double *
 
   // if the result is lat/long, convert the results from radians back
   // to degrees
-  if (( pj_is_latlong( mDestinationProjection ) && ( direction == ForwardTransform ) )
-      || ( pj_is_latlong( mSourceProjection ) && ( direction == ReverseTransform ) ) )
+  if (( pj_is_latlong( destProj ) && ( direction == ForwardTransform ) )
+      || ( pj_is_latlong( sourceProj ) && ( direction == ReverseTransform ) ) )
   {
     for ( int i = 0; i < numPoints; ++i )
     {
@@ -1020,11 +1006,9 @@ bool QgsCoordinateTransform::datumTransformCrsInfo( int datumTransform, int& eps
   preferred = sqlite3_column_int( stmt, 5 ) != 0;
   deprecated = sqlite3_column_int( stmt, 6 ) != 0;
 
-  QgsCoordinateReferenceSystem srcCrs;
-  srcCrs.createFromOgcWmsCrs( QString( "EPSG:%1" ).arg( srcCrsId ) );
+  QgsCoordinateReferenceSystem srcCrs = QgsCRSCache::instance()->crsByOgcWmsCrs( QString( "EPSG:%1" ).arg( srcCrsId ) );
   srcProjection = srcCrs.description();
-  QgsCoordinateReferenceSystem destCrs;
-  destCrs.createFromOgcWmsCrs( QString( "EPSG:%1" ).arg( destCrsId ) );
+  QgsCoordinateReferenceSystem destCrs = QgsCRSCache::instance()->crsByOgcWmsCrs( QString( "EPSG:%1" ).arg( destCrsId ) );
   dstProjection = destCrs.description();
 
   sqlite3_finalize( stmt );
@@ -1057,3 +1041,56 @@ void QgsCoordinateTransform::addNullGridShifts( QString& srcProjString, QString&
     destProjString += " +nadgrids=@null";
   }
 }
+
+QPair<projPJ, projPJ> QgsCoordinateTransform::threadLocalProjData() const
+{
+  mProjLock.lockForRead();
+  projCtx pContext = nullptr;
+  if ( mProjContext.hasLocalData() )
+    pContext = mProjContext.localData()->get();
+  else
+  {
+    mProjContext.setLocalData( new QgsProjContextStore() );
+    pContext = mProjContext.localData()->get();
+  }
+
+  QMap< uintptr_t, QPair< projPJ, projPJ > >::const_iterator it = mProjProjections.constFind( reinterpret_cast< uintptr_t >( pContext ) );
+  if ( it != mProjProjections.constEnd() )
+  {
+    QPair< projPJ, projPJ > res = it.value();
+    mProjLock.unlock();
+    return res;
+  }
+
+  // proj projections don't exist yet, so we need to create
+  mProjLock.unlock();
+  mProjLock.lockForWrite();
+  QPair< projPJ, projPJ > res = qMakePair( pj_init_plus_ctx( pContext, mSourceProjString.toUtf8() ),
+                                pj_init_plus_ctx( pContext, mDestProjString.toUtf8() ) );
+  mProjProjections.insert( reinterpret_cast< uintptr_t >( pContext ), res );
+  mProjLock.unlock();
+  return res;
+}
+
+void QgsCoordinateTransform::freeProj()
+{
+  mProjLock.lockForWrite();
+  QMap< uintptr_t, QPair< projPJ, projPJ > >::const_iterator it = mProjProjections.constBegin();
+  for ( ; it != mProjProjections.constEnd(); ++it )
+  {
+    pj_free( it.value().first );
+    pj_free( it.value().second );
+  }
+  mProjProjections.clear();
+  mProjLock.unlock();
+}
+
+QgsCoordinateTransform::QgsProjContextStore::QgsProjContextStore()
+{
+  context = pj_ctx_alloc();
+}
+
+QgsCoordinateTransform::QgsProjContextStore::~QgsProjContextStore()
+{
+  pj_ctx_free( context );
+}
