@@ -18,7 +18,7 @@
 
 #include <QTextStream>
 
-QgsCredentials *QgsCredentials::smInstance = 0;
+QgsCredentials *QgsCredentials::smInstance = nullptr;
 
 void QgsCredentials::setInstance( QgsCredentials *theInstance )
 {
@@ -35,7 +35,7 @@ QgsCredentials *QgsCredentials::instance()
   if ( smInstance )
     return smInstance;
 
-  return new QgsCredentialsConsole();
+  return new QgsCredentialsNone();
 }
 
 QgsCredentials::QgsCredentials()
@@ -46,14 +46,14 @@ QgsCredentials::~QgsCredentials()
 {
 }
 
-bool QgsCredentials::get( QString realm, QString &username, QString &password, QString message )
+bool QgsCredentials::get( const QString& realm, QString &username, QString &password, const QString& message )
 {
   if ( mCredentialCache.contains( realm ) )
   {
     QPair<QString, QString> credentials = mCredentialCache.take( realm );
     username = credentials.first;
     password = credentials.second;
-    QgsDebugMsg( QString( "retrieved realm:%1 username:%2 password:%3" ).arg( realm ).arg( username ).arg( password ) );
+    QgsDebugMsg( QString( "retrieved realm:%1 username:%2 password:%3" ).arg( realm, username, password ) );
 
     if ( !password.isNull() )
       return true;
@@ -61,7 +61,7 @@ bool QgsCredentials::get( QString realm, QString &username, QString &password, Q
 
   if ( request( realm, username, password, message ) )
   {
-    QgsDebugMsg( QString( "requested realm:%1 username:%2 password:%3" ).arg( realm ).arg( username ).arg( password ) );
+    QgsDebugMsg( QString( "requested realm:%1 username:%2 password:%3" ).arg( realm, username, password ) );
     return true;
   }
   else
@@ -71,12 +71,22 @@ bool QgsCredentials::get( QString realm, QString &username, QString &password, Q
   }
 }
 
-void QgsCredentials::put( QString realm, QString username, QString password )
+void QgsCredentials::put( const QString& realm, const QString& username, const QString& password )
 {
-  QgsDebugMsg( QString( "inserting realm:%1 username:%2 password:%3" ).arg( realm ).arg( username ).arg( password ) );
+  QgsDebugMsg( QString( "inserting realm:%1 username:%2 password:%3" ).arg( realm, username, password ) );
   mCredentialCache.insert( realm, QPair<QString, QString>( username, password ) );
 }
 
+bool QgsCredentials::getMasterPassword( QString &password , bool stored )
+{
+  if ( requestMasterPassword( password, stored ) )
+  {
+    QgsDebugMsg( "requested master password" );
+    return true;
+  }
+  return false;
+}
+
 void QgsCredentials::lock()
 {
   mMutex.lock();
@@ -88,6 +98,30 @@ void QgsCredentials::unlock()
 }
 
 
+////////////////////////////////
+// QgsCredentialsNone
+
+QgsCredentialsNone::QgsCredentialsNone()
+{
+  setInstance( this );
+}
+
+bool QgsCredentialsNone::request( const QString& realm, QString &username, QString &password, const QString& message )
+{
+  Q_UNUSED( realm );
+  Q_UNUSED( username );
+  Q_UNUSED( password );
+  Q_UNUSED( message );
+  return false;
+}
+
+bool QgsCredentialsNone::requestMasterPassword( QString &password, bool stored )
+{
+  Q_UNUSED( password );
+  Q_UNUSED( stored );
+  return false;
+}
+
 ////////////////////////////////
 // QgsCredentialsConsole
 
@@ -96,7 +130,7 @@ QgsCredentialsConsole::QgsCredentialsConsole()
   setInstance( this );
 }
 
-bool QgsCredentialsConsole::request( QString realm, QString &username, QString &password, QString message )
+bool QgsCredentialsConsole::request( const QString& realm, QString &username, QString &password, const QString& message )
 {
   QTextStream in( stdin, QIODevice::ReadOnly );
   QTextStream out( stdout, QIODevice::WriteOnly );
@@ -111,3 +145,18 @@ bool QgsCredentialsConsole::request( QString realm, QString &username, QString &
 
   return true;
 }
+
+bool QgsCredentialsConsole::requestMasterPassword( QString &password, bool stored )
+{
+  Q_UNUSED( stored );
+
+  QTextStream in( stdin, QIODevice::ReadOnly );
+  QTextStream out( stdout, QIODevice::WriteOnly );
+
+  QString msg( stored ? "Master password for authentication configs: " : "Set master password for authentication configs: " );
+
+  out << msg;
+  in >> password;
+
+  return true;
+}
