@@ -40,6 +40,7 @@
 #include "qgsconfig.h"
 
 // use GDAL VSI mechanism
+#define CPL_SUPRESS_CPLUSPLUS
 #include "cpl_vsi.h"
 #include "cpl_string.h"
 
@@ -81,7 +82,7 @@ void QgsAnimatedIcon::connectFrameChanged( const QObject * receiver, const char
     mCount++;
   }
   mMovie->setPaused( mCount == 0 );
-  QgsDebugMsg( QString( "mCount = %1" ).arg( mCount ) );
+  QgsDebugMsgLevel( QString( "mCount = %1" ).arg( mCount ), 3 );
 }
 
 void QgsAnimatedIcon::disconnectFrameChanged( const QObject * receiver, const char * method )
@@ -91,7 +92,7 @@ void QgsAnimatedIcon::disconnectFrameChanged( const QObject * receiver, const ch
     mCount--;
   }
   mMovie->setPaused( mCount == 0 );
-  QgsDebugMsg( QString( "mCount = %1" ).arg( mCount ) );
+  QgsDebugMsgLevel( QString( "mCount = %1" ).arg( mCount ), 3 );
 }
 
 // shared icons
@@ -130,7 +131,7 @@ const QIcon &QgsLayerItem::iconTable()
   static QIcon icon;
 
   if ( icon.isNull() )
-    icon = QgsApplication::getThemeIcon( "/mIconTableLayer.png" );
+    icon = QgsApplication::getThemeIcon( "/mIconTableLayer.svg" );
 
   return icon;
 }
@@ -202,8 +203,6 @@ const QIcon &QgsZipItem::iconZip()
   return icon;
 }
 
-QMap<QString, QIcon> QgsDataItem::mIconMap = QMap<QString, QIcon>();
-
 QgsAnimatedIcon * QgsDataItem::mPopulatingIcon = nullptr;
 
 QgsDataItem::QgsDataItem( QgsDataItem::Type type, QgsDataItem* parent, const QString& name, const QString& path )
@@ -250,7 +249,7 @@ QString QgsDataItem::pathComponent( const QString &string )
 
 void QgsDataItem::deleteLater()
 {
-  QgsDebugMsg( "path = " + path() );
+  QgsDebugMsgLevel( "path = " + path(), 3 );
   setParent( nullptr ); // also disconnects parent
   Q_FOREACH ( QgsDataItem *child, mChildren )
   {
@@ -289,7 +288,7 @@ void QgsDataItem::moveToThread( QThread * targetThread )
   {
     if ( !child ) // should not happen
       continue;
-    QgsDebugMsg( "moveToThread child " + child->path() );
+    QgsDebugMsgLevel( "moveToThread child " + child->path(), 3 );
     child->QObject::setParent( nullptr ); // to be sure
     child->moveToThread( targetThread );
   }
@@ -343,7 +342,7 @@ void QgsDataItem::emitStateChanged( QgsDataItem* item, QgsDataItem::State oldSta
 {
   if ( !item )
     return;
-  QgsDebugMsg( QString( "item %1 state changed %2 -> %3" ).arg( item->path() ).arg( oldState ).arg( item->state() ) );
+  QgsDebugMsgLevel( QString( "item %1 state changed %2 -> %3" ).arg( item->path() ).arg( oldState ).arg( item->state() ), 2 );
   emit stateChanged( item, oldState );
 }
 
@@ -357,7 +356,7 @@ void QgsDataItem::populate( bool foreground )
   if ( state() == Populated || state() == Populating )
     return;
 
-  QgsDebugMsg( "mPath = " + mPath );
+  QgsDebugMsgLevel( "mPath = " + mPath, 2 );
 
   if ( capabilities2() & QgsDataItem::Fast || foreground )
   {
@@ -379,27 +378,27 @@ void QgsDataItem::populate( bool foreground )
 // This is expected to be run in a separate thread
 QVector<QgsDataItem*> QgsDataItem::runCreateChildren( QgsDataItem* item )
 {
-  QgsDebugMsg( "path = " + item->path() );
+  QgsDebugMsgLevel( "path = " + item->path(), 2 );
   QTime time;
   time.start();
   QVector <QgsDataItem*> children = item->createChildren();
-  QgsDebugMsg( QString( "%1 children created in %2 ms" ).arg( children.size() ).arg( time.elapsed() ) );
+  QgsDebugMsgLevel( QString( "%1 children created in %2 ms" ).arg( children.size() ).arg( time.elapsed() ), 3 );
   // Children objects must be pushed to main thread.
   Q_FOREACH ( QgsDataItem* child, children )
   {
     if ( !child ) // should not happen
       continue;
-    QgsDebugMsg( "moveToThread child " + child->path() );
+    QgsDebugMsgLevel( "moveToThread child " + child->path(), 2 );
     if ( qApp )
       child->moveToThread( qApp->thread() ); // moves also children
   }
-  QgsDebugMsg( QString( "finished path %1: %2 children" ).arg( item->path() ).arg( children.size() ) );
+  QgsDebugMsgLevel( QString( "finished path %1: %2 children" ).arg( item->path() ).arg( children.size() ), 3 );
   return children;
 }
 
 void QgsDataItem::childrenCreated()
 {
-  QgsDebugMsg( QString( "path = %1 children.size() = %2" ).arg( path() ).arg( mFutureWatcher->result().size() ) );
+  QgsDebugMsgLevel( QString( "path = %1 children.size() = %2" ).arg( path() ).arg( mFutureWatcher->result().size() ), 3 );
 
   if ( deferredDelete() )
   {
@@ -422,7 +421,7 @@ void QgsDataItem::childrenCreated()
 
 void QgsDataItem::populate( const QVector<QgsDataItem*>& children )
 {
-  QgsDebugMsg( "mPath = " + mPath );
+  QgsDebugMsgLevel( "mPath = " + mPath, 3 );
 
   Q_FOREACH ( QgsDataItem *child, children )
   {
@@ -436,11 +435,11 @@ void QgsDataItem::populate( const QVector<QgsDataItem*>& children )
 
 void QgsDataItem::depopulate()
 {
-  QgsDebugMsg( "mPath = " + mPath );
+  QgsDebugMsgLevel( "mPath = " + mPath, 3 );
 
   Q_FOREACH ( QgsDataItem *child, mChildren )
   {
-    QgsDebugMsg( "remove " + child->path() );
+    QgsDebugMsgLevel( "remove " + child->path(), 3 );
     child->depopulate(); // recursive
     deleteChildItem( child );
   }
@@ -452,7 +451,7 @@ void QgsDataItem::refresh()
   if ( state() == Populating )
     return;
 
-  QgsDebugMsg( "mPath = " + mPath );
+  QgsDebugMsgLevel( "mPath = " + mPath, 3 );
 
   if ( capabilities2() & QgsDataItem::Fast )
   {
@@ -486,7 +485,7 @@ void QgsDataItem::refresh( QVector<QgsDataItem*> children )
   }
   Q_FOREACH ( QgsDataItem *child, remove )
   {
-    QgsDebugMsg( "remove " + child->path() );
+    QgsDebugMsgLevel( "remove " + child->path(), 3 );
     deleteChildItem( child );
   }
 
@@ -550,7 +549,7 @@ void QgsDataItem::setParent( QgsDataItem* parent )
 void QgsDataItem::addChildItem( QgsDataItem * child, bool refresh )
 {
   Q_ASSERT( child );
-  QgsDebugMsg( QString( "path = %1 add child #%2 - %3 - %4" ).arg( mPath ).arg( mChildren.size() ).arg( child->mName ).arg( child->mType ) );
+  QgsDebugMsgLevel( QString( "path = %1 add child #%2 - %3 - %4" ).arg( mPath ).arg( mChildren.size() ).arg( child->mName ).arg( child->mType ), 3 );
 
   //calculate position to insert child
   int i;
@@ -644,7 +643,7 @@ QgsDataItem::State QgsDataItem::state() const
 
 void QgsDataItem::setState( State state )
 {
-  QgsDebugMsg( QString( "item %1 set state %2 -> %3" ).arg( path() ).arg( this->state() ).arg( state ) );
+  QgsDebugMsgLevel( QString( "item %1 set state %2 -> %3" ).arg( path() ).arg( this->state() ).arg( state ), 3 );
   if ( state == mState )
     return;
 
@@ -698,7 +697,7 @@ QgsLayerItem::QgsLayerItem( QgsDataItem* parent, const QString& name, const QStr
       mIconName = "/mIconPolygonLayer.svg";
       break;
     case TableLayer:
-      mIconName = "/mIconTableLayer.png";
+      mIconName = "/mIconTableLayer.svg";
       break;
     case Raster:
       mIconName = "/mIconRaster.svg";
@@ -878,7 +877,6 @@ QVector<QgsDataItem*> QgsDirectoryItem::createChildren()
 
 void QgsDirectoryItem::setState( State state )
 {
-  QgsDebugMsg( "Entered" );
   QgsDataCollectionItem::setState( state );
 
   if ( state == Populated )
@@ -902,7 +900,6 @@ void QgsDirectoryItem::setState( State state )
 
 void QgsDirectoryItem::directoryChanged()
 {
-  QgsDebugMsg( "Entered" );
   if ( state() == Populating )
   {
     // schedule to refresh later, because refres() simply returns if Populating
@@ -925,11 +922,11 @@ bool QgsDirectoryItem::hiddenPath( QString path )
 
 void QgsDirectoryItem::childrenCreated()
 {
-  QgsDebugMsg( QString( "mRefreshLater = %1" ).arg( mRefreshLater ) );
+  QgsDebugMsgLevel( QString( "mRefreshLater = %1" ).arg( mRefreshLater ), 3 );
 
   if ( mRefreshLater )
   {
-    QgsDebugMsg( "directory changed during createChidren() -> refresh() again" );
+    QgsDebugMsgLevel( "directory changed during createChidren() -> refresh() again", 3 );
     mRefreshLater = false;
     setState( Populated );
     refresh();
@@ -1251,7 +1248,7 @@ void QgsZipItem::init()
     for ( i = keys.begin(); i != keys.end(); ++i )
     {
       QString k( *i );
-      QgsDebugMsg( "provider " + k );
+      QgsDebugMsgLevel( "provider " + k, 3 );
       // some providers hangs with empty uri (Postgis) etc...
       // -> using libraries directly
       QLibrary *library = QgsProviderRegistry::instance()->providerLibrary( k );
@@ -1305,7 +1302,7 @@ char **VSIReadDirRecursive1( const char *pszPath )
   char **papszFiles1 = nullptr;
   char **papszFiles2 = nullptr;
   VSIStatBufL psStatBuf;
-  CPLString osTemp1, osTemp2;
+  QString temp1, temp2;
   int i, j;
   int nCount1, nCount2;
 
@@ -1319,41 +1316,35 @@ char **VSIReadDirRecursive1( const char *pszPath )
   for ( i = 0; i < nCount1; i++ )
   {
     // build complete file name for stat
-    osTemp1.clear();
-    osTemp1.append( pszPath );
-    osTemp1.append( "/" );
-    osTemp1.append( papszFiles1[i] );
+    temp1 = QString( "%1/%2" ).arg( pszPath, papszFiles1[i] );
 
     // if is file, add it
-    if ( VSIStatL( osTemp1.c_str(), &psStatBuf ) == 0 &&
+    if ( VSIStatL( temp1.toUtf8(), &psStatBuf ) == 0 &&
          VSI_ISREG( psStatBuf.st_mode ) )
     {
       // oFiles.AddString( papszFiles1[i] );
       papszOFiles = CSLAddString( papszOFiles, papszFiles1[i] );
     }
-    else if ( VSIStatL( osTemp1.c_str(), &psStatBuf ) == 0 &&
+    else if ( VSIStatL( temp1.toUtf8(), &psStatBuf ) == 0 &&
               VSI_ISDIR( psStatBuf.st_mode ) )
     {
       // add directory entry
-      osTemp2.clear();
-      osTemp2.append( papszFiles1[i] );
-      osTemp2.append( "/" );
-      // oFiles.AddString( osTemp2.c_str() );
-      papszOFiles = CSLAddString( papszOFiles, osTemp2.c_str() );
+      temp2 = QString( "%1/" ).arg( papszFiles1[i] );
+
+      // oFiles.AddString( temp2.toUtf8() );
+      papszOFiles = CSLAddString( papszOFiles, temp2.toUtf8() );
 
       // recursively add files inside directory
-      papszFiles2 = VSIReadDirRecursive1( osTemp1.c_str() );
+      papszFiles2 = VSIReadDirRecursive1( temp1.toUtf8() );
       if ( papszFiles2 )
       {
         nCount2 = CSLCount( papszFiles2 );
         for ( j = 0; j < nCount2; j++ )
         {
-          osTemp2.clear();
-          osTemp2.append( papszFiles1[i] );
-          osTemp2.append( "/" );
-          osTemp2.append( papszFiles2[j] );
-          // oFiles.AddString( osTemp2.c_str() );
-          papszOFiles = CSLAddString( papszOFiles, osTemp2.c_str() );
+          temp2 = QString( "%1/%2" ).arg( papszFiles1[i], papszFiles2[j] );
+
+          // oFiles.AddString( temp2.toUtf8() );
+          papszOFiles = CSLAddString( papszOFiles, temp2.toUtf8() );
         }
         CSLDestroy( papszFiles2 );
       }
@@ -1446,7 +1437,6 @@ QgsDataItem* QgsZipItem::itemFromPath( QgsDataItem* parent, const QString& fileP
   QString scanZipSetting = settings.value( "/qgis/scanZipInBrowser2", "basic" ).toString();
   int zipFileCount = 0;
   QStringList zipFileList;
-  QFileInfo fileInfo( filePath );
   QString vsiPrefix = QgsZipItem::vsiPrefix( filePath );
   QgsZipItem * zipItem = nullptr;
   bool populated = false;
