@@ -61,6 +61,19 @@ class CORE_EXPORT QgsDataProvider : public QObject
       Net                 = 1 << 3  // Internet source
     };
 
+    /**
+     * Properties are used to pass custom configuration options into data providers.
+     * This enum defines a list of custom properties which can be used on different
+     * providers. It depends on the provider, which properties are supported.
+     * In addition to these default properties, providers can add their custom properties
+     * starting from CustomData.
+     */
+    enum ProviderProperty
+    {
+      EvaluateDefaultValues,       //!< Evaluate default values on provider side when calling QgsVectorDataProvider::defaultValue( int index ) rather than on commit.
+      CustomData   = 3000          //!< Custom properties for 3rd party providers or very provider-specific properties which are not expected to be of interest for other providers can be added starting from this value up.
+    };
+
     QgsDataProvider( QString const & uri = "" )
         : mDataSourceURI( uri )
     {}
@@ -330,7 +343,7 @@ class CORE_EXPORT QgsDataProvider : public QObject
      *
      * @return true in case of success (or no-op implementation), false in case of failure.
      *
-     * @note added in QGIS 2.14.4
+     * @note added in QGIS 2.16
      */
     virtual bool enterUpdateMode() { return true; }
 
@@ -348,10 +361,42 @@ class CORE_EXPORT QgsDataProvider : public QObject
      *
      * @return true in case of success (or no-op implementation), false in case of failure.
      *
-     * @note added in QGIS 2.14.4
+     * @note added in QGIS 2.16
      */
     virtual bool leaveUpdateMode() { return true; }
 
+    /**
+     * Allows setting arbitrary properties on the provider.
+     * It depends on the provider which properties are supported.
+     *
+     * @note added in 2.16
+     */
+    void setProviderProperty( ProviderProperty property, const QVariant& value );
+
+    /**
+     * Allows setting arbitrary properties on the provider.
+     * It depends on the provider which properties are supported.
+     *
+     * @note added in 2.16
+     */
+    void setProviderProperty( int property, const QVariant& value );
+
+    /**
+     * Get the current value of a certain provider property.
+     * It depends on the provider which properties are supported.
+     *
+     * @note added in 2.16
+     */
+    QVariant providerProperty( ProviderProperty property, const QVariant& defaultValue = QVariant() ) const;
+
+    /**
+     * Get the current value of a certain provider property.
+     * It depends on the provider which properties are supported.
+     *
+     * @note added in 2.16
+     */
+    QVariant providerProperty( int property , const QVariant& defaultValue ) const;
+
   signals:
 
     /**
@@ -398,6 +443,8 @@ class CORE_EXPORT QgsDataProvider : public QObject
      * This could be a file, database, or server address.
      */
     QString mDataSourceURI;
+
+    QMap< int, QVariant > mProviderProperties;
 };
 
 
