@@ -35,7 +35,7 @@
 class CORE_EXPORT QgsDataSourceURI
 {
   public:
-    enum SSLmode { SSLprefer, SSLdisable, SSLallow, SSLrequire };
+    enum SSLmode { SSLprefer, SSLdisable, SSLallow, SSLrequire, SSLverifyCA, SSLverifyFull };
 
     //! default constructor
     QgsDataSourceURI();
@@ -47,10 +47,10 @@ class CORE_EXPORT QgsDataSourceURI
     QgsDataSourceURI( const QByteArray & uri );
 
     //! return connection part of URI
-    QString connectionInfo() const;
+    QString connectionInfo( bool expandAuthConfig = true ) const;
 
     //! return complete uri
-    QString uri() const;
+    QString uri( bool expandAuthConfig = true ) const;
 
     //! return complete encoded uri (generic mode)
     QByteArray encodedUri() const;
@@ -68,6 +68,7 @@ class CORE_EXPORT QgsDataSourceURI
     //! Set generic param (generic mode)
     // \note if key exists, another is inserted
     void setParam( const QString &key, const QString &value );
+    //! @note available in python as setParamList
     void setParam( const QString &key, const QStringList &value );
 
     //! Remove generic param (generic mode)
@@ -89,14 +90,16 @@ class CORE_EXPORT QgsDataSourceURI
                         const QString& aDatabase,
                         const QString& aUsername,
                         const QString& aPassword,
-                        SSLmode sslmode = SSLprefer );
+                        SSLmode sslmode = SSLprefer,
+                        const QString& authConfigId = QString() );
 
     //! Set all connection related members at once (for the service case)
     void setConnection( const QString& aService,
                         const QString& aDatabase,
                         const QString& aUsername,
                         const QString& aPassword,
-                        SSLmode sslmode = SSLprefer );
+                        SSLmode sslmode = SSLprefer,
+                        const QString& authConfigId = QString() );
 
     //! Set database
     void setDatabase( const QString &database );
@@ -108,47 +111,100 @@ class CORE_EXPORT QgsDataSourceURI
                         const QString& aSql = QString(),
                         const QString& aKeyColumn = QString() );
 
+    //! Set authentication configuration ID
+    void setAuthConfigId( const QString& authcfg );
+
     //! set username
-    void setUsername( QString username );
+    void setUsername( const QString& username );
 
     //! set password
-    void setPassword( QString password );
+    void setPassword( const QString& password );
 
     //! Removes password element from uris
     static QString removePassword( const QString& aUri );
 
+    //! Any associated authentication configuration ID
+    QString authConfigId() const;
+
+    //! Returns the username
     QString username() const;
+
+    //! Returns the schema
     QString schema() const;
+
+    //! Returns the table
     QString table() const;
+
+    //! Returns the SQL query
     QString sql() const;
+
+    //! Return the name of the geometry column
     QString geometryColumn() const;
 
     //! set use Estimated Metadata
     void setUseEstimatedMetadata( bool theFlag );
+
+    //! Returns true if estimated metadata are used
     bool useEstimatedMetadata() const;
 
+    //! Set to true to disable selection by id
     void disableSelectAtId( bool theFlag );
+    //! Returns whether the selection by id is disabled
     bool selectAtIdDisabled() const;
 
+    //! Clears the schema
     void clearSchema();
-    void setSql( QString sql );
 
+    //! set the table schema
+    // @note added in 2.11
+    void setSchema( const QString& schema );
+
+    //! Sets the SQL query
+    void setSql( const QString& sql );
+
+    //! Returns the host
     QString host() const;
+    //! Returns the database
     QString database() const;
+    //! Returns the port
     QString port() const;
+    //! Returns the driver
+    // @note added in QGIS 2.16
+    QString driver() const;
+    //! Sets the driver name
+    // @note added in QGIS 2.16
+    void setDriver( const QString& driver );
+    //! Returns the password
     QString password() const;
+    //! Returns the SSL mode
     enum SSLmode sslMode() const;
 
+    //! Returns the service name
     QString service() const;
 
+    //! Returns the name of the (primary) key column
     QString keyColumn() const;
-    void setKeyColumn( QString column );
+    //! Sets the name of the (primary) key column
+    void setKeyColumn( const QString& column );
 
-    QGis::WkbType wkbType() const;
-    void setWkbType( QGis::WkbType type );
+    /** The (old) wkb type.
+        @deprecated Will return QgsWKBTypes::Type in 3.0. Prefer newWkbType() in the meantime */
+    Q_DECL_DEPRECATED QGis::WkbType wkbType() const;
 
+    /** The wkb type.
+        @note Will be removed in 3.0 */
+    QgsWKBTypes::Type newWkbType() const;
+
+    //! @deprecated See setWkbType( QgsWKBTypes::Type )
+    Q_DECL_DEPRECATED void setWkbType( QGis::WkbType type );
+
+    //! Sets the wkb type
+    void setWkbType( QgsWKBTypes::Type type );
+
+    //! Returns the srid
     QString srid() const;
-    void setSrid( QString srid );
+    //! Sets the srid
+    void setSrid( const QString& srid );
 
   private:
     void skipBlanks( const QString &uri, int &i );
@@ -161,6 +217,8 @@ class CORE_EXPORT QgsDataSourceURI
     QString mHost;
     //! port the database server listens on
     QString mPort;
+    //! device driver for ODBC
+    QString mDriver;
     //! service name
     QString mService;
     //! database name
@@ -173,6 +231,8 @@ class CORE_EXPORT QgsDataSourceURI
     QString mGeometryColumn;
     //! SQL query or where clause used to limit features returned from the layer
     QString mSql;
+    //! authentication configuration ID
+    QString mAuthConfigId;
     //! username
     QString mUsername;
     //! password
@@ -186,7 +246,7 @@ class CORE_EXPORT QgsDataSourceURI
     //! Disable SelectAtId capability (eg. to trigger the attribute table memory model for expensive views)
     bool mSelectAtIdDisabled;
     //! geometry type (or QGis::WKBUnknown if not specified)
-    QGis::WkbType mWkbType;
+    QgsWKBTypes::Type mWkbType;
     //! SRID or a null string if not specified
     QString mSrid;
     //! Generic params store
