@@ -20,16 +20,19 @@
 
 #include <QSortFilterProxyModel>
 
-/**A class that implements a custom filter and can be used
+/** \ingroup core
+ * A class that implements a custom filter and can be used
  as a proxy for QgsDbTableModel*/
 class CORE_EXPORT QgsDbFilterProxyModel: public QSortFilterProxyModel
 {
+    Q_OBJECT
+
   public:
-    QgsDbFilterProxyModel( QObject* parent = 0 );
+    QgsDbFilterProxyModel( QObject* parent = nullptr );
     ~QgsDbFilterProxyModel();
-    /**Calls QSortFilterProxyModel::setFilterWildcard and triggers update*/
+    /** Calls QSortFilterProxyModel::setFilterWildcard and triggers update*/
     void _setFilterWildcard( const QString& pattern );
-    /**Calls QSortFilterProxyModel::setFilterRegExp and triggers update*/
+    /** Calls QSortFilterProxyModel::setFilterRegExp and triggers update*/
     void _setFilterRegExp( const QString& pattern );
 
   protected:
