@@ -20,10 +20,13 @@
 #include "qgsrendercontext.h"
 #include "qgslayertreemodellegendnode.h"
 #include "qgsfontutils.h"
+#include "qgssymbollayerv2utils.h"
+#include "qgssymbolv2.h"
 
 #include <QDomElement>
 #include <QPainter>
 
+Q_NOWARN_DEPRECATED_PUSH // because of deprecated xform member
 QgsDiagramLayerSettings::QgsDiagramLayerSettings()
     : placement( AroundPoint )
     , placementFlags( OnLine )
@@ -36,10 +39,13 @@ QgsDiagramLayerSettings::QgsDiagramLayerSettings()
     , xform( nullptr )
     , xPosColumn( -1 )
     , yPosColumn( -1 )
+    , showColumn( -1 )
     , showAll( true )
 {
 }
+Q_NOWARN_DEPRECATED_POP
 
+Q_NOWARN_DEPRECATED_PUSH // because of deprecated xform member
 QgsDiagramLayerSettings::QgsDiagramLayerSettings( const QgsDiagramLayerSettings& rh )
     : placement( rh.placement )
     , placementFlags( rh.placementFlags )
@@ -48,15 +54,18 @@ QgsDiagramLayerSettings::QgsDiagramLayerSettings( const QgsDiagramLayerSettings&
     , obstacle( rh.obstacle )
     , dist( rh.dist )
     , renderer( rh.renderer ? rh.renderer->clone() : nullptr )
-    , ct( rh.ct )
+    , ct( rh.ct ? rh.ct->clone() : nullptr )
     , xform( rh.xform )
     , fields( rh.fields )
     , xPosColumn( rh.xPosColumn )
     , yPosColumn( rh.yPosColumn )
+    , showColumn( rh.showColumn )
     , showAll( rh.showAll )
 {
 }
+Q_NOWARN_DEPRECATED_POP
 
+Q_NOWARN_DEPRECATED_PUSH // because of deprecated xform member
 QgsDiagramLayerSettings&QgsDiagramLayerSettings::operator=( const QgsDiagramLayerSettings & rh )
 {
   placement = rh.placement;
@@ -66,18 +75,38 @@ QgsDiagramLayerSettings&QgsDiagramLayerSettings::operator=( const QgsDiagramLaye
   obstacle = rh.obstacle;
   dist = rh.dist;
   renderer = rh.renderer ? rh.renderer->clone() : nullptr;
-  ct = rh.ct;
+  ct = rh.ct ? rh.ct->clone() : nullptr;
   xform = rh.xform;
   fields = rh.fields;
   xPosColumn = rh.xPosColumn;
   yPosColumn = rh.yPosColumn;
+  showColumn = rh.showColumn;
   showAll = rh.showAll;
   return *this;
 }
+Q_NOWARN_DEPRECATED_POP
 
+Q_NOWARN_DEPRECATED_PUSH // because of deprecated fields member
 QgsDiagramLayerSettings::~QgsDiagramLayerSettings()
 {
   delete renderer;
+  delete ct;
+}
+Q_NOWARN_DEPRECATED_POP
+
+void QgsDiagramLayerSettings::setRenderer( QgsDiagramRendererV2 *diagramRenderer )
+{
+  if ( diagramRenderer == renderer )
+    return;
+
+  delete renderer;
+  renderer = diagramRenderer;
+}
+
+void QgsDiagramLayerSettings::setCoordinateTransform( QgsCoordinateTransform *transform )
+{
+  delete ct;
+  ct = transform;
 }
 
 void QgsDiagramLayerSettings::readXML( const QDomElement& elem, const QgsVectorLayer* layer )
@@ -92,6 +121,7 @@ void QgsDiagramLayerSettings::readXML( const QDomElement& elem, const QgsVectorL
   dist = elem.attribute( "dist" ).toDouble();
   xPosColumn = elem.attribute( "xPosColumn" ).toInt();
   yPosColumn = elem.attribute( "yPosColumn" ).toInt();
+  showColumn = elem.attribute( "showColumn" ).toInt();
   showAll = ( elem.attribute( "showAll", "0" ) != "0" );
 }
 
@@ -108,10 +138,30 @@ void QgsDiagramLayerSettings::writeXML( QDomElement& layerElem, QDomDocument& do
   diagramLayerElem.setAttribute( "dist", QString::number( dist ) );
   diagramLayerElem.setAttribute( "xPosColumn", xPosColumn );
   diagramLayerElem.setAttribute( "yPosColumn", yPosColumn );
+  diagramLayerElem.setAttribute( "showColumn", showColumn );
   diagramLayerElem.setAttribute( "showAll", showAll );
   layerElem.appendChild( diagramLayerElem );
 }
 
+QSet<QString> QgsDiagramLayerSettings::referencedFields( const QgsExpressionContext &context, const QgsFields& fieldsParameter ) const
+{
+  QSet< QString > referenced;
+  if ( renderer )
+    referenced = renderer->referencedFields( context, fieldsParameter );
+
+  //and the ones needed for data defined diagram positions
+  if ( xPosColumn >= 0 && xPosColumn < fieldsParameter.count() )
+    referenced << fieldsParameter.at( xPosColumn ).name();
+  if ( yPosColumn >= 0 && yPosColumn < fieldsParameter.count() )
+    referenced << fieldsParameter.at( yPosColumn ).name();
+
+  // and the ones needed for data defined diagram visibility
+  if ( showColumn >= 0 && showColumn < fieldsParameter.count() )
+    referenced << fieldsParameter.at( showColumn ).name();
+
+  return referenced;
+}
+
 void QgsDiagramSettings::readXML( const QDomElement& elem, const QgsVectorLayer* layer )
 {
   Q_UNUSED( layer );
@@ -142,15 +192,21 @@ void QgsDiagramSettings::readXML( const QDomElement& elem, const QgsVectorLayer*
     scaleBasedVisibility = minScaleDenominator >= 0 && maxScaleDenominator >= 0;
   }
 
-  //mm vs map units
-  if ( elem.attribute( "sizeType" ) == "MM" )
+  //diagram size unit type and scale
+  if ( elem.attribute( "sizeType" ) == "MapUnits" )
   {
-    sizeType = MM;
+    //compatibility with pre-2.16 project files
+    sizeType = QgsSymbolV2::MapUnit;
   }
   else
   {
-    sizeType = MapUnits;
+    sizeType = QgsSymbolLayerV2Utils::decodeOutputUnit( elem.attribute( "sizeType" ) );
   }
+  sizeScale = QgsSymbolLayerV2Utils::decodeMapUnitScale( elem.attribute( "sizeScale" ) );
+
+  //line width unit type and scale
+  lineSizeUnit = QgsSymbolLayerV2Utils::decodeOutputUnit( elem.attribute( "lineSizeType" ) );
+  lineSizeScale = QgsSymbolLayerV2Utils::decodeMapUnitScale( elem.attribute( "lineSizeScale" ) );
 
   //label placement method
   if ( elem.attribute( "labelPlacementMethod" ) == "Height" )
@@ -260,15 +316,13 @@ void QgsDiagramSettings::writeXML( QDomElement& rendererElem, QDomDocument& doc,
   categoryElem.setAttribute( "maxScaleDenominator", QString::number( maxScaleDenominator ) );
   categoryElem.setAttribute( "transparency", QString::number( transparency ) );
 
-  // site type (mm vs. map units)
-  if ( sizeType == MM )
-  {
-    categoryElem.setAttribute( "sizeType", "MM" );
-  }
-  else
-  {
-    categoryElem.setAttribute( "sizeType", "MapUnits" );
-  }
+  //diagram size unit type and scale
+  categoryElem.setAttribute( "sizeType", QgsSymbolLayerV2Utils::encodeOutputUnit( sizeType ) );
+  categoryElem.setAttribute( "sizeScale", QgsSymbolLayerV2Utils::encodeMapUnitScale( sizeScale ) );
+
+  //line width unit type and scale
+  categoryElem.setAttribute( "lineSizeType", QgsSymbolLayerV2Utils::encodeOutputUnit( lineSizeUnit ) );
+  categoryElem.setAttribute( "lineSizeScale", QgsSymbolLayerV2Utils::encodeMapUnitScale( lineSizeScale ) );
 
   // label placement method (text diagram)
   if ( labelPlacementMethod == Height )
@@ -317,7 +371,6 @@ void QgsDiagramSettings::writeXML( QDomElement& rendererElem, QDomDocument& doc,
   categoryElem.setAttribute( "minimumSize", QString::number( minimumSize ) );
   categoryElem.setAttribute( "angleOffset", QString::number( angleOffset ) );
 
-  QString colors;
   int nCats = qMin( categoryColors.size(), categoryAttributes.size() );
   for ( int i = 0; i < nCats; ++i )
   {
@@ -334,6 +387,9 @@ void QgsDiagramSettings::writeXML( QDomElement& rendererElem, QDomDocument& doc,
 
 QgsDiagramRendererV2::QgsDiagramRendererV2()
     : mDiagram( nullptr )
+    , mShowAttributeLegend( true )
+    , mShowSizeLegend( false )
+    , mSizeLegendSymbol( QgsMarkerSymbolV2::createSimple( QgsStringMap() ) )
 {
 }
 
@@ -350,10 +406,22 @@ void QgsDiagramRendererV2::setDiagram( QgsDiagram* d )
 
 QgsDiagramRendererV2::QgsDiagramRendererV2( const QgsDiagramRendererV2& other )
     : mDiagram( other.mDiagram ? other.mDiagram->clone() : nullptr )
+    , mShowAttributeLegend( other.mShowAttributeLegend )
+    , mShowSizeLegend( other.mShowSizeLegend )
+    , mSizeLegendSymbol( other.mSizeLegendSymbol.data() ? other.mSizeLegendSymbol->clone() : nullptr )
 {
 }
 
-void QgsDiagramRendererV2::renderDiagram( const QgsFeature& feature, QgsRenderContext& c, QPointF pos )
+QgsDiagramRendererV2 &QgsDiagramRendererV2::operator=( const QgsDiagramRendererV2 & other )
+{
+  mDiagram = other.mDiagram ? other.mDiagram->clone() : nullptr;
+  mShowAttributeLegend = other.mShowAttributeLegend;
+  mShowSizeLegend = other.mShowSizeLegend;
+  mSizeLegendSymbol.reset( other.mSizeLegendSymbol.data() ? other.mSizeLegendSymbol->clone() : nullptr );
+  return *this;
+}
+
+void QgsDiagramRendererV2::renderDiagram( const QgsFeature& feature, QgsRenderContext& c, QPointF pos ) const
 {
   if ( !mDiagram )
   {
@@ -369,7 +437,7 @@ void QgsDiagramRendererV2::renderDiagram( const QgsFeature& feature, QgsRenderCo
   mDiagram->renderDiagram( feature, c, s, pos );
 }
 
-QSizeF QgsDiagramRendererV2::sizeMapUnits( const QgsFeature& feature, const QgsRenderContext& c )
+QSizeF QgsDiagramRendererV2::sizeMapUnits( const QgsFeature& feature, const QgsRenderContext& c ) const
 {
   QgsDiagramSettings s;
   if ( !diagramSettings( feature, c, s ) )
@@ -378,13 +446,35 @@ QSizeF QgsDiagramRendererV2::sizeMapUnits( const QgsFeature& feature, const QgsR
   }
 
   QSizeF size = diagramSize( feature, c );
-  if ( s.sizeType == QgsDiagramSettings::MM )
+  if ( size.isValid() )
   {
-    convertSizeToMapUnits( size, c );
+    double width = QgsSymbolLayerV2Utils::convertToMapUnits( c, size.width(), s.sizeType, s.sizeScale );
+    size.rheight() *= width / size.width();
+    size.setWidth( width );
   }
   return size;
 }
 
+QSet<QString> QgsDiagramRendererV2::referencedFields( const QgsExpressionContext &context, const QgsFields &fields ) const
+{
+  Q_UNUSED( fields );
+
+  QSet< QString > referenced;
+
+  if ( !mDiagram )
+    return referenced;
+
+  Q_FOREACH ( const QString& att, diagramAttributes() )
+  {
+    QgsExpression* expression = mDiagram->getExpression( att, context );
+    Q_FOREACH ( const QString& field, expression->referencedColumns() )
+    {
+      referenced << field;
+    }
+  }
+  return referenced;
+}
+
 void QgsDiagramRendererV2::convertSizeToMapUnits( QSizeF& size, const QgsRenderContext& context ) const
 {
   if ( !size.isValid() )
@@ -432,6 +522,13 @@ void QgsDiagramRendererV2::_readXML( const QDomElement& elem, const QgsVectorLay
   {
     mDiagram = nullptr;
   }
+  mShowAttributeLegend = ( elem.attribute( "attributeLegend", "1" ) != "0" );
+  mShowSizeLegend = ( elem.attribute( "sizeLegend", "0" ) != "0" );
+  QDomElement sizeLegendSymbolElem = elem.firstChildElement( "symbol" );
+  if ( !sizeLegendSymbolElem.isNull() && sizeLegendSymbolElem.attribute( "name" ) == "sizeSymbol" )
+  {
+    mSizeLegendSymbol.reset( QgsSymbolLayerV2Utils::loadSymbol<QgsMarkerSymbolV2>( sizeLegendSymbolElem ) );
+  }
 }
 
 void QgsDiagramRendererV2::_writeXML( QDomElement& rendererElem, QDomDocument& doc, const QgsVectorLayer* layer ) const
@@ -443,6 +540,10 @@ void QgsDiagramRendererV2::_writeXML( QDomElement& rendererElem, QDomDocument& d
   {
     rendererElem.setAttribute( "diagramType", mDiagram->diagramName() );
   }
+  rendererElem.setAttribute( "attributeLegend", mShowAttributeLegend );
+  rendererElem.setAttribute( "sizeLegend", mShowSizeLegend );
+  QDomElement sizeLegendSymbolElem = QgsSymbolLayerV2Utils::saveSymbol( "sizeSymbol", mSizeLegendSymbol.data(), doc );
+  rendererElem.appendChild( sizeLegendSymbolElem );
 }
 
 QgsSingleCategoryDiagramRenderer::QgsSingleCategoryDiagramRenderer(): QgsDiagramRendererV2()
@@ -458,14 +559,14 @@ QgsSingleCategoryDiagramRenderer* QgsSingleCategoryDiagramRenderer::clone() cons
   return new QgsSingleCategoryDiagramRenderer( *this );
 }
 
-bool QgsSingleCategoryDiagramRenderer::diagramSettings( const QgsFeature&, const QgsRenderContext& c, QgsDiagramSettings& s )
+bool QgsSingleCategoryDiagramRenderer::diagramSettings( const QgsFeature&, const QgsRenderContext& c, QgsDiagramSettings& s ) const
 {
   Q_UNUSED( c );
   s = mSettings;
   return true;
 }
 
-QSizeF QgsSingleCategoryDiagramRenderer::diagramSize( const QgsFeature &feature, const QgsRenderContext &c )
+QSizeF QgsSingleCategoryDiagramRenderer::diagramSize( const QgsFeature &feature, const QgsRenderContext &c ) const
 {
   return mDiagram->diagramSize( feature.attributes(), c, mSettings );
 }
@@ -519,7 +620,7 @@ QList<QgsDiagramSettings> QgsLinearlyInterpolatedDiagramRenderer::diagramSetting
   return settingsList;
 }
 
-bool QgsLinearlyInterpolatedDiagramRenderer::diagramSettings( const QgsFeature& feature, const QgsRenderContext& c, QgsDiagramSettings& s )
+bool QgsLinearlyInterpolatedDiagramRenderer::diagramSettings( const QgsFeature& feature, const QgsRenderContext& c, QgsDiagramSettings& s ) const
 {
   s = mSettings;
   s.size = diagramSize( feature, c );
@@ -531,7 +632,25 @@ QList<QString> QgsLinearlyInterpolatedDiagramRenderer::diagramAttributes() const
   return mSettings.categoryAttributes;
 }
 
-QSizeF QgsLinearlyInterpolatedDiagramRenderer::diagramSize( const QgsFeature& feature, const QgsRenderContext& c )
+QSet<QString> QgsLinearlyInterpolatedDiagramRenderer::referencedFields( const QgsExpressionContext &context, const QgsFields& fields ) const
+{
+  QSet< QString > referenced = QgsDiagramRendererV2::referencedFields( context, fields );
+  if ( mInterpolationSettings.classificationAttributeIsExpression )
+  {
+    QgsExpression* expression = mDiagram->getExpression( mInterpolationSettings.classificationAttributeExpression, context );
+    Q_FOREACH ( const QString& field, expression->referencedColumns() )
+    {
+      referenced << field;
+    }
+  }
+  else
+  {
+    referenced << fields.at( mInterpolationSettings.classificationAttribute ).name();
+  }
+  return referenced;
+}
+
+QSizeF QgsLinearlyInterpolatedDiagramRenderer::diagramSize( const QgsFeature& feature, const QgsRenderContext& c ) const
 {
   return mDiagram->diagramSize( feature, c, mSettings, mInterpolationSettings );
 }
@@ -603,10 +722,33 @@ QList< QgsLayerTreeModelLegendNode* > QgsDiagramRendererV2::legendItems( QgsLaye
 
 QList< QgsLayerTreeModelLegendNode* > QgsSingleCategoryDiagramRenderer::legendItems( QgsLayerTreeLayer* nodeLayer ) const
 {
-  return mSettings.legendItems( nodeLayer );
+  QList< QgsLayerTreeModelLegendNode* > nodes;
+  if ( mShowAttributeLegend )
+    nodes = mSettings.legendItems( nodeLayer );
+
+  return nodes;
 }
 
 QList< QgsLayerTreeModelLegendNode* > QgsLinearlyInterpolatedDiagramRenderer::legendItems( QgsLayerTreeLayer* nodeLayer ) const
 {
-  return mSettings.legendItems( nodeLayer );
+  QList< QgsLayerTreeModelLegendNode* > nodes;
+  if ( mShowAttributeLegend )
+    nodes = mSettings.legendItems( nodeLayer );
+
+  if ( mShowSizeLegend && mDiagram && mSizeLegendSymbol.data() )
+  {
+    // add size legend
+    Q_FOREACH ( double v, QgsSymbolLayerV2Utils::prettyBreaks( mInterpolationSettings.lowerValue, mInterpolationSettings.upperValue, 4 ) )
+    {
+      double size = mDiagram->legendSize( v, mSettings, mInterpolationSettings );
+      QgsLegendSymbolItemV2 si( mSizeLegendSymbol.data(), QString::number( v ), nullptr );
+      QgsMarkerSymbolV2 * s = static_cast<QgsMarkerSymbolV2 *>( si.symbol() );
+      s->setSize( size );
+      s->setSizeUnit( mSettings.sizeType );
+      s->setSizeMapUnitScale( mSettings.sizeScale );
+      nodes << new QgsSymbolV2LegendNode( nodeLayer, si );
+    }
+  }
+
+  return nodes;
 }
