@@ -19,7 +19,19 @@
 #include "qgslogger.h"
 #include "qgsindexedfeature.h"
 
-class QgsAbstractGeometrySimplifier;
+
+/** \ingroup core
+ * Interface that can be optionaly attached to an iterator so its
+ * nextFeature() implementaton can check if it must stop as soon as possible.
+ * @note Added in QGIS 2.16
+ * @note not available in Python bindings
+ */
+class CORE_EXPORT QgsInterruptionChecker
+{
+  public:
+    //! return true if the iterator must stop as soon as possible
+    virtual bool mustStop() const = 0;
+};
 
 /** \ingroup core
  * Internal feature iterator to be implemented within data providers
@@ -27,6 +39,15 @@ class QgsAbstractGeometrySimplifier;
 class CORE_EXPORT QgsAbstractFeatureIterator
 {
   public:
+
+    //! Status of expression compilation for filter expression requests
+    enum CompileStatus
+    {
+      NoCompilation, /*!< Expression could not be compiled or not attempt was made to compile expression */
+      PartiallyCompiled, /*!< Expression was partially compiled, but extra checks need to be applied to features*/
+      Compiled, /*!< Expression was fully compiled and delegated to data provider source*/
+    };
+
     //! base class constructor - stores the iteration parameters
     QgsAbstractFeatureIterator( const QgsFeatureRequest& request );
 
@@ -41,6 +62,21 @@ class CORE_EXPORT QgsAbstractFeatureIterator
     //! end of iterating: free the resources / lock
     virtual bool close() = 0;
 
+    /** Attach an object that can be queried regularly by the iterator to check
+     * if it must stopped. This is mostly useful for iterators where a single
+     * nextFeature()/fetchFeature() iteration might be very long. A typical use case is the
+     * WFS provider. When nextFeature()/fetchFeature() is reasonably fast, it is not necessary
+     * to implement this method. The default implementation does nothing.
+     * @note added in QGIS 2.16
+     * @note not available in Python bindings
+     */
+    virtual void setInterruptionChecker( QgsInterruptionChecker* interruptionChecker );
+
+    /** Returns the status of expression compilation for filter expression requests.
+     * @note added in QGIS 2.16
+     */
+    CompileStatus compileStatus() const { return mCompileStatus; }
+
   protected:
     /**
      * If you write a feature iterator for your provider, this is the method you
@@ -101,15 +137,13 @@ class CORE_EXPORT QgsAbstractFeatureIterator
     //! Number of features already fetched by iterator
     long mFetchedCount;
 
+    //! Status of compilation of filter expression
+    CompileStatus mCompileStatus;
+
     //! Setup the simplification of geometries to fetch using the specified simplify method
     virtual bool prepareSimplification( const QgsSimplifyMethod& simplifyMethod );
 
   private:
-    //! optional object to locally simplify geometries fetched by this feature iterator
-    QgsAbstractGeometrySimplifier* mGeometrySimplifier;
-    //! this iterator runs local simplification
-    bool mLocalSimplification;
-
     bool mUseCachedFeatures;
     QList<QgsIndexedFeature> mCachedFeatures;
     QList<QgsIndexedFeature>::ConstIterator mFeatureIterator;
@@ -117,9 +151,6 @@ class CORE_EXPORT QgsAbstractFeatureIterator
     //! returns whether the iterator supports simplify geometries on provider side
     virtual bool providerCanSimplify( QgsSimplifyMethod::MethodType methodType ) const;
 
-    //! simplify the specified geometry if it was configured
-    virtual bool simplify( QgsFeature& feature );
-
     /**
      * Should be overwritten by providers which implement an own order by strategy
      * If the own order by strategy is successful, return true, if not, return false
@@ -141,7 +172,8 @@ class CORE_EXPORT QgsAbstractFeatureIterator
 
 
 
-/** Helper template that cares of two things: 1. automatic deletion of source if owned by iterator, 2. notification of open/closed iterator.
+/** \ingroup core
+ * Helper template that cares of two things: 1. automatic deletion of source if owned by iterator, 2. notification of open/closed iterator.
  * \note not available in Python bindings
 */
 template<typename T>
@@ -149,7 +181,9 @@ class QgsAbstractFeatureIteratorFromSource : public QgsAbstractFeatureIterator
 {
   public:
     QgsAbstractFeatureIteratorFromSource( T* source, bool ownSource, const QgsFeatureRequest& request )
-        : QgsAbstractFeatureIterator( request ), mSource( source ), mOwnSource( ownSource )
+        : QgsAbstractFeatureIterator( request )
+        , mSource( source )
+        , mOwnSource( ownSource )
     {
       mSource->iteratorOpened( this );
     }
@@ -168,8 +202,6 @@ class QgsAbstractFeatureIteratorFromSource : public QgsAbstractFeatureIterator
     bool mOwnSource;
 };
 
-
-
 /**
  * \ingroup core
  * Wrapper for iterator of features from vector data provider or vector layer
@@ -195,6 +227,20 @@ class CORE_EXPORT QgsFeatureIterator
     //! find out whether the iterator is still valid or closed already
     bool isClosed() const;
 
+    /** Attach an object that can be queried regularly by the iterator to check
+     * if it must stopped. This is mostly useful for iterators where a single
+     * nextFeature()/fetchFeature() iteration might be very long. A typical use case is the
+     * WFS provider.
+     * @note added in QGIS 2.16
+     * @note not available in Python bindings
+     */
+    void setInterruptionChecker( QgsInterruptionChecker* interruptionChecker );
+
+    /** Returns the status of expression compilation for filter expression requests.
+     * @note added in QGIS 2.16
+     */
+    QgsAbstractFeatureIterator::CompileStatus compileStatus() const { return mIter->compileStatus(); }
+
     friend bool operator== ( const QgsFeatureIterator &fi1, const QgsFeatureIterator &fi2 );
     friend bool operator!= ( const QgsFeatureIterator &fi1, const QgsFeatureIterator &fi2 );
 
@@ -265,4 +311,10 @@ inline bool operator!= ( const QgsFeatureIterator &fi1, const QgsFeatureIterator
   return !( fi1 == fi2 );
 }
 
+inline void QgsFeatureIterator::setInterruptionChecker( QgsInterruptionChecker* interruptionChecker )
+{
+  if ( mIter )
+    mIter->setInterruptionChecker( interruptionChecker );
+}
+
 #endif // QGSFEATUREITERATOR_H
