@@ -64,8 +64,8 @@ bool QgsTopologyPreservingSimplifier::simplifyGeometry( QgsGeometry* geometry )
 
   if ( g )
   {
-    size_t wkbSize = g->wkbSize();
-    unsigned char* wkb = ( unsigned char* )malloc( wkbSize );
+    int wkbSize = g->wkbSize();
+    unsigned char *wkb = new unsigned char[ wkbSize ];
     memcpy( wkb, g->asWkb(), wkbSize );
     geometry->fromWkb( wkb, wkbSize );
     delete g;
