@@ -27,39 +27,42 @@
 #include <labelposition.h>
 #include "qgsrectangle.h"
 
-using namespace pal;
-
-/**A class to query the labeling structure at a given point (small wraper around pal RTree class)*/
+/** \ingroup core
+ * A class to query the labeling structure at a given point (small wraper around pal RTree class)*/
 class CORE_EXPORT QgsLabelSearchTree
 {
   public:
     QgsLabelSearchTree();
     ~QgsLabelSearchTree();
 
-    /**Removes and deletes all the entries*/
+    /** Removes and deletes all the entries*/
     void clear();
 
-    /**Returns label position(s) at a given point. QgsLabelSearchTree keeps ownership, don't delete the LabelPositions
+    /** Returns label position(s) at a given point. QgsLabelSearchTree keeps ownership, don't delete the LabelPositions
      * @note not available in python bindings
      * TODO: why does this break bindings with QList<QgsLabelPosition>?
      */
     void label( const QgsPoint& p, QList<QgsLabelPosition*>& posList ) const;
 
-    /**Returns label position(s) in given rectangle. QgsLabelSearchTree keeps ownership, don't delete the LabelPositions
+    /** Returns label position(s) in given rectangle. QgsLabelSearchTree keeps ownership, don't delete the LabelPositions
      * @note not available in python bindings
      * TODO: why does this break bindings with QList<QgsLabelPosition>?
      */
     void labelsInRect( const QgsRectangle& r, QList<QgsLabelPosition*>& posList ) const;
 
-    /**Inserts label position. Does not take ownership of labelPos
+    /** Inserts label position. Does not take ownership of labelPos
      * @return true in case of success
      * @note not available in python bindings
      */
-    bool insertLabel( LabelPosition* labelPos, int featureId, const QString& layerName, const QString& labeltext, const QFont& labelfont, bool diagram = false, bool pinned = false );
+    bool insertLabel( pal::LabelPosition* labelPos, int featureId, const QString& layerName, const QString& labeltext, const QFont& labelfont, bool diagram = false, bool pinned = false, const QString& providerId = QString() );
 
   private:
     // set as mutable because RTree template is not const-correct
-    mutable RTree<QgsLabelPosition*, double, 2, double> mSpatialIndex;
+    mutable pal::RTree<QgsLabelPosition*, double, 2, double> mSpatialIndex;
+    QList< QgsLabelPosition* > mOwnedPositions;
+
+    QgsLabelSearchTree( const QgsLabelSearchTree& rh );
+    QgsLabelSearchTree& operator=( const QgsLabelSearchTree& rh );
 };
 
 #endif // QGSLABELTREE_H
