@@ -3,7 +3,7 @@
      --------------------------------------
     Date                 : 13.5.2014
     Copyright            : (C) 2014 Matthias Kuhn
-    Email                : matthias dot kuhn at gmx dot ch
+    Email                : matthias at opengis dot ch
  ***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -19,6 +19,7 @@
 
 const QString QgsLegacyHelpers::convertEditType( QgsVectorLayer::EditType editType, QgsEditorWidgetConfig& cfg, QgsVectorLayer* vl, const QString& name, const QDomElement &editTypeElement )
 {
+  Q_NOWARN_DEPRECATED_PUSH
   QString widgetType = "TextEdit"; // Fallback
 
   switch ( editType )
@@ -85,8 +86,8 @@ const QString QgsLegacyHelpers::convertEditType( QgsVectorLayer::EditType editTy
            editTypeElement.hasAttribute( "filterAttributeValue" ) )
       {
         filterExpression = QString( "\"%1\"='%2'" )
-                           .arg( editTypeElement.attribute( "filterAttributeColumn" ) )
-                           .arg( editTypeElement.attribute( "filterAttributeValue" ) );
+                           .arg( editTypeElement.attribute( "filterAttributeColumn" ),
+                                 editTypeElement.attribute( "filterAttributeValue" ) );
       }
       else
       {
@@ -140,7 +141,7 @@ const QString QgsLegacyHelpers::convertEditType( QgsVectorLayer::EditType editTy
     {
       widgetType = "TextEdit";
       cfg.insert( "IsMultiline", false );
-      vl->setFieldEditable( vl->pendingFields().fieldNameIndex( name ), false );
+      vl->setFieldEditable( vl->fields().fieldNameIndex( name ), false );
       break;
     }
 
@@ -202,10 +203,13 @@ const QString QgsLegacyHelpers::convertEditType( QgsVectorLayer::EditType editTy
   }
 
   return widgetType;
+  Q_NOWARN_DEPRECATED_POP
 }
 
 QgsVectorLayer::EditType QgsLegacyHelpers::convertEditType( const QString& editType, const QgsEditorWidgetConfig& cfg, QgsVectorLayer* vl, const QString& name )
 {
+  Q_NOWARN_DEPRECATED_PUSH
+
   int idx = vl->fieldNameIndex( name );
 
   if ( !vl->fieldEditable( idx ) )
@@ -310,4 +314,5 @@ QgsVectorLayer::EditType QgsLegacyHelpers::convertEditType( const QString& editT
   }
 
   return QgsVectorLayer::EditorWidgetV2;
+  Q_NOWARN_DEPRECATED_POP
 }
