@@ -20,6 +20,8 @@
 
 #include <QMutex>
 
+/** \ingroup core
+ */
 class CORE_EXPORT QgsLocaleNumC
 {
     char *mOldlocale;
@@ -29,6 +31,11 @@ class CORE_EXPORT QgsLocaleNumC
     QgsLocaleNumC();
     ~QgsLocaleNumC();
 
+  private:
+
+    QgsLocaleNumC( const QgsLocaleNumC& rh );
+    QgsLocaleNumC& operator=( const QgsLocaleNumC& rh );
+
 };
 
 #endif // QGSLOCALENUMC_H
