@@ -27,7 +27,9 @@
 #include "qgscrscache.h"
 
 QgsMapHitTest::QgsMapHitTest( const QgsMapSettings& settings, const QgsGeometry& polygon, const LayerFilterExpression& layerFilterExpression )
-    : mSettings( settings ), mLayerFilterExpression( layerFilterExpression ), mOnlyExpressions( false )
+    : mSettings( settings )
+    , mLayerFilterExpression( layerFilterExpression )
+    , mOnlyExpressions( false )
 {
   if ( !polygon.isEmpty() && polygon.type() == QGis::Polygon )
   {
@@ -36,7 +38,9 @@ QgsMapHitTest::QgsMapHitTest( const QgsMapSettings& settings, const QgsGeometry&
 }
 
 QgsMapHitTest::QgsMapHitTest( const QgsMapSettings& settings, const LayerFilterExpression& layerFilterExpression )
-    : mSettings( settings ), mLayerFilterExpression( layerFilterExpression ), mOnlyExpressions( true )
+    : mSettings( settings )
+    , mLayerFilterExpression( layerFilterExpression )
+    , mOnlyExpressions( true )
 {
 }
 
@@ -59,7 +63,7 @@ void QgsMapHitTest::run()
 
     if ( !mOnlyExpressions )
     {
-      if ( vl->hasScaleBasedVisibility() && ( mSettings.scale() < vl->minimumScale() || mSettings.scale() > vl->maximumScale() ) )
+      if ( !vl->isInScaleRange( mSettings.scale() ) )
       {
         mHitTest[vl] = SymbolV2Set(); // no symbols -> will not be shown
         mHitTestRuleKey[vl] = SymbolV2Set();
