@@ -63,12 +63,8 @@ QgsMapLayer::QgsMapLayer( QgsMapLayer::LayerType type,
     , mLegend( nullptr )
     , mStyleManager( new QgsMapLayerStyleManager( this ) )
 {
-  mCRS = new QgsCoordinateReferenceSystem();
-
   // Set the display name = internal name
-  QgsDebugMsg( "original name: '" + mLayerOrigName + '\'' );
   mLayerName = capitaliseLayerName( mLayerOrigName );
-  QgsDebugMsg( "display name: '" + mLayerName + '\'' );
 
   mShortName = "";
   //mShortName.replace( QRegExp( "[\\W]" ), "_" );
@@ -88,11 +84,14 @@ QgsMapLayer::QgsMapLayer( QgsMapLayer::LayerType type,
   mMinScale = 0;
   mMaxScale = 100000000;
   mScaleBasedVisibility = false;
+
+  connect( this, SIGNAL( nameChanged() ), this, SIGNAL( layerNameChanged() ) );
+
+  connect( mStyleManager, SIGNAL( currentStyleChanged( QString ) ), this, SIGNAL( styleChanged() ) );
 }
 
 QgsMapLayer::~QgsMapLayer()
 {
-  delete mCRS;
   delete mLegend;
   delete mStyleManager;
 }
@@ -108,16 +107,21 @@ QString QgsMapLayer::id() const
   return mID;
 }
 
-/** Write property of QString layerName. */
 void QgsMapLayer::setLayerName( const QString & name )
 {
-  QgsDebugMsg( "new original name: '" + name + '\'' );
+  setName( name );
+}
+
+void QgsMapLayer::setName( const QString& name )
+{
   QString newName = capitaliseLayerName( name );
-  QgsDebugMsg( "new display name: '" + name + '\'' );
-  if ( name == mLayerOrigName && newName == mLayerName ) return;
+  if ( name == mLayerOrigName && newName == mLayerName )
+    return;
+
   mLayerOrigName = name; // store the new original name
   mLayerName = newName;
-  emit layerNameChanged();
+
+  emit nameChanged();
 }
 
 /** Read property of QString layerName. */
@@ -150,6 +154,7 @@ void QgsMapLayer::setBlendMode( QPainter::CompositionMode blendMode )
 {
   mBlendMode = blendMode;
   emit blendModeChanged( blendMode );
+  emit styleChanged();
 }
 
 /** Read blend mode for layer */
@@ -169,10 +174,8 @@ void QgsMapLayer::drawLabels( QgsRenderContext& rendererContext )
   Q_UNUSED( rendererContext );
 }
 
-bool QgsMapLayer::readLayerXML( const QDomElement& layerElement )
+bool QgsMapLayer::readLayerXML( const QDomElement& layerElement, const QString& relativeBasePath )
 {
-  QgsCoordinateReferenceSystem savedCRS;
-  CUSTOM_CRS_VALIDATION savedValidation;
   bool layerError;
 
   QDomNode mnl;
@@ -202,19 +205,19 @@ bool QgsMapLayer::readLayerXML( const QDomElement& layerElement )
   if ( provider == "spatialite" )
   {
     QgsDataSourceURI uri( mDataSource );
-    uri.setDatabase( QgsProject::instance()->readPath( uri.database() ) );
+    uri.setDatabase( QgsProject::instance()->readPath( uri.database(), relativeBasePath ) );
     mDataSource = uri.uri();
   }
   else if ( provider == "ogr" )
   {
     QStringList theURIParts = mDataSource.split( '|' );
-    theURIParts[0] = QgsProject::instance()->readPath( theURIParts[0] );
+    theURIParts[0] = QgsProject::instance()->readPath( theURIParts[0], relativeBasePath );
     mDataSource = theURIParts.join( "|" );
   }
   else if ( provider == "gpx" )
   {
     QStringList theURIParts = mDataSource.split( '?' );
-    theURIParts[0] = QgsProject::instance()->readPath( theURIParts[0] );
+    theURIParts[0] = QgsProject::instance()->readPath( theURIParts[0], relativeBasePath );
     mDataSource = theURIParts.join( "?" );
   }
   else if ( provider == "delimitedtext" )
@@ -228,7 +231,7 @@ bool QgsMapLayer::readLayerXML( const QDomElement& layerElement )
       urlSource.setPath( file.path() );
     }
 
-    QUrl urlDest = QUrl::fromLocalFile( QgsProject::instance()->readPath( urlSource.toLocalFile() ) );
+    QUrl urlDest = QUrl::fromLocalFile( QgsProject::instance()->readPath( urlSource.toLocalFile(), relativeBasePath ) );
     urlDest.setQueryItems( urlSource.queryItems() );
     mDataSource = QString::fromAscii( urlDest.toEncoded() );
   }
@@ -245,7 +248,9 @@ bool QgsMapLayer::readLayerXML( const QDomElement& layerElement )
     // This is modified version of old QgsWmsProvider::parseUri
     // The new format has always params crs,format,layers,styles and that params
     // should not appear in old format url -> use them to identify version
-    if ( !mDataSource.contains( "crs=" ) && !mDataSource.contains( "format=" ) )
+    // XYZ tile layers do not need to contain crs,format params, but they have type=xyz
+    if ( !mDataSource.contains( "type=" ) &&
+         !mDataSource.contains( "crs=" ) && !mDataSource.contains( "format=" ) )
     {
       QgsDebugMsg( "Old WMS URI format detected -> converting to new format" );
       QgsDataSourceURI uri;
@@ -324,7 +329,7 @@ bool QgsMapLayer::readLayerXML( const QDomElement& layerElement )
           QString filename = r.cap( 1 );
           if ( filename.startsWith( '"' ) && filename.endsWith( '"' ) )
             filename = filename.mid( 1, filename.length() - 2 );
-          mDataSource = "NETCDF:\"" + QgsProject::instance()->readPath( filename ) + "\":" + r.cap( 2 );
+          mDataSource = "NETCDF:\"" + QgsProject::instance()->readPath( filename, relativeBasePath ) + "\":" + r.cap( 2 );
           handled = true;
         }
       }
@@ -338,7 +343,7 @@ bool QgsMapLayer::readLayerXML( const QDomElement& layerElement )
           QString filename = r.cap( 2 );
           if ( filename.startsWith( '"' ) && filename.endsWith( '"' ) )
             filename = filename.mid( 1, filename.length() - 2 );
-          mDataSource = "HDF4_SDS:" + r.cap( 1 ) + ":\"" + QgsProject::instance()->readPath( filename ) + "\":" + r.cap( 3 );
+          mDataSource = "HDF4_SDS:" + r.cap( 1 ) + ":\"" + QgsProject::instance()->readPath( filename, relativeBasePath ) + "\":" + r.cap( 3 );
           handled = true;
         }
       }
@@ -352,7 +357,7 @@ bool QgsMapLayer::readLayerXML( const QDomElement& layerElement )
           QString filename = r.cap( 1 );
           if ( filename.startsWith( '"' ) && filename.endsWith( '"' ) )
             filename = filename.mid( 1, filename.length() - 2 );
-          mDataSource = "HDF5:\"" + QgsProject::instance()->readPath( filename ) + "\":" + r.cap( 2 );
+          mDataSource = "HDF5:\"" + QgsProject::instance()->readPath( filename, relativeBasePath ) + "\":" + r.cap( 2 );
           handled = true;
         }
       }
@@ -363,14 +368,14 @@ bool QgsMapLayer::readLayerXML( const QDomElement& layerElement )
         QRegExp r( "([^:]+):([^:]+):(.+)" );
         if ( r.exactMatch( mDataSource ) )
         {
-          mDataSource = r.cap( 1 ) + ':' + r.cap( 2 ) + ':' + QgsProject::instance()->readPath( r.cap( 3 ) );
+          mDataSource = r.cap( 1 ) + ':' + r.cap( 2 ) + ':' + QgsProject::instance()->readPath( r.cap( 3 ), relativeBasePath );
           handled = true;
         }
       }
     }
 
     if ( !handled )
-      mDataSource = QgsProject::instance()->readPath( mDataSource );
+      mDataSource = QgsProject::instance()->readPath( mDataSource, relativeBasePath );
   }
 
   // Set the CRS from project file, asking the user if necessary.
@@ -380,11 +385,14 @@ bool QgsMapLayer::readLayerXML( const QDomElement& layerElement )
   mnl = layerElement.namedItem( "layername" );
   mne = mnl.toElement();
 
+  QgsCoordinateReferenceSystem savedCRS;
+  CUSTOM_CRS_VALIDATION savedValidation;
+
   QDomNode srsNode = layerElement.namedItem( "srs" );
-  mCRS->readXML( srsNode );
-  mCRS->setValidationHint( tr( "Specify CRS for layer %1" ).arg( mne.text() ) );
-  mCRS->validate();
-  savedCRS = *mCRS;
+  mCRS.readXML( srsNode );
+  mCRS.setValidationHint( tr( "Specify CRS for layer %1" ).arg( mne.text() ) );
+  mCRS.validate();
+  savedCRS = mCRS;
 
   // Do not validate any projections in children, they will be overwritten anyway.
   // No need to ask the user for a projections when it is overwritten, is there?
@@ -398,7 +406,7 @@ bool QgsMapLayer::readLayerXML( const QDomElement& layerElement )
   // file readnig functions changed it. They will if projections is specfied in the file.
   // FIXME: is this necessary?
   QgsCoordinateReferenceSystem::setCustomSrsValidation( savedValidation );
-  *mCRS = savedCRS;
+  mCRS = savedCRS;
 
   // Abort if any error in layer, such as not found.
   if ( layerError )
@@ -423,8 +431,8 @@ bool QgsMapLayer::readLayerXML( const QDomElement& layerElement )
 
   // use scale dependent visibility flag
   setScaleBasedVisibility( layerElement.attribute( "hasScaleBasedVisibilityFlag" ).toInt() == 1 );
-  setMinimumScale( layerElement.attribute( "minimumScale" ).toFloat() );
-  setMaximumScale( layerElement.attribute( "maximumScale" ).toFloat() );
+  setMinimumScale( layerElement.attribute( "minimumScale" ).toDouble() );
+  setMaximumScale( layerElement.attribute( "maximumScale" ).toDouble() );
 
   QDomNode extentNode = layerElement.namedItem( "extent" );
   if ( !extentNode.isNull() )
@@ -435,7 +443,7 @@ bool QgsMapLayer::readLayerXML( const QDomElement& layerElement )
   // set name
   mnl = layerElement.namedItem( "layername" );
   mne = mnl.toElement();
-  setLayerName( mne.text() );
+  setName( mne.text() );
 
   //short name
   QDomElement shortNameElem = layerElement.firstChildElement( "shortname" );
@@ -774,7 +782,7 @@ bool QgsMapLayer::writeLayerXML( QDomElement& layerElement, QDomDocument& docume
 
   // spatial reference system id
   QDomElement mySrsElement = document.createElement( "srs" );
-  mCRS->writeXML( mySrsElement, document );
+  mCRS.writeXML( mySrsElement, document );
   layerElement.appendChild( mySrsElement );
 
 #if 0
@@ -809,7 +817,7 @@ QDomDocument QgsMapLayer::asLayerDefinition( const QList<QgsMapLayer *>& layers,
   return doc;
 }
 
-QList<QgsMapLayer*> QgsMapLayer::fromLayerDefinition( QDomDocument& document, bool addToRegistry, bool addToLegend )
+QList<QgsMapLayer*> QgsMapLayer::fromLayerDefinition( QDomDocument& document, bool addToRegistry, bool addToLegend, const QString& relativeBasePath )
 {
   QList<QgsMapLayer*> layers;
   QDomNodeList layernodes = document.elementsByTagName( "maplayer" );
@@ -839,7 +847,7 @@ QList<QgsMapLayer*> QgsMapLayer::fromLayerDefinition( QDomDocument& document, bo
     if ( !layer )
       continue;
 
-    bool ok = layer->readLayerXML( layerElem );
+    bool ok = layer->readLayerXML( layerElem, relativeBasePath );
     if ( ok )
     {
       layers << layer;
@@ -945,6 +953,10 @@ void QgsMapLayer::connectNotify( const char * signal )
 } //  QgsMapLayer::connectNotify
 #endif
 
+bool QgsMapLayer::isInScaleRange( double scale ) const
+{
+  return !mScaleBasedVisibility || ( mMinScale * QGis::SCALE_PRECISION < scale && scale < mMaxScale );
+}
 
 void QgsMapLayer::toggleScaleBasedVisibility( bool theVisibilityFlag )
 {
@@ -956,18 +968,18 @@ bool QgsMapLayer::hasScaleBasedVisibility() const
   return mScaleBasedVisibility;
 }
 
-void QgsMapLayer::setMinimumScale( const float theMinScale )
+void QgsMapLayer::setMinimumScale( double theMinScale )
 {
   mMinScale = theMinScale;
 }
 
-float QgsMapLayer::minimumScale() const
+double QgsMapLayer::minimumScale() const
 {
   return mMinScale;
 }
 
 
-void QgsMapLayer::setMaximumScale( const float theMaxScale )
+void QgsMapLayer::setMaximumScale( double theMaxScale )
 {
   mMaxScale = theMaxScale;
 }
@@ -977,7 +989,7 @@ void QgsMapLayer::setScaleBasedVisibility( const bool enabled )
   mScaleBasedVisibility = enabled;
 }
 
-float QgsMapLayer::maximumScale() const
+double QgsMapLayer::maximumScale() const
 {
   return mMaxScale;
 }
@@ -1002,17 +1014,17 @@ void QgsMapLayer::setSubLayerVisibility( const QString& name, bool vis )
 
 const QgsCoordinateReferenceSystem& QgsMapLayer::crs() const
 {
-  return *mCRS;
+  return mCRS;
 }
 
 void QgsMapLayer::setCrs( const QgsCoordinateReferenceSystem& srs, bool emitSignal )
 {
-  *mCRS = srs;
+  mCRS = srs;
 
-  if ( !mCRS->isValid() )
+  if ( !mCRS.isValid() )
   {
-    mCRS->setValidationHint( tr( "Specify CRS for layer %1" ).arg( name() ) );
-    mCRS->validate();
+    mCRS.setValidationHint( tr( "Specify CRS for layer %1" ).arg( name() ) );
+    mCRS.validate();
   }
 
   if ( emitSignal )
@@ -1028,8 +1040,8 @@ QString QgsMapLayer::capitaliseLayerName( const QString& name )
 
   QString layerName( name );
 
-  if ( capitaliseLayerName )
-    layerName = layerName.left( 1 ).toUpper() + layerName.mid( 1 );
+  if ( capitaliseLayerName && !layerName.isEmpty() )
+    layerName = layerName.at( 0 ).toUpper() + layerName.mid( 1 );
 
   return layerName;
 }
@@ -1225,8 +1237,8 @@ bool QgsMapLayer::importNamedStyle( QDomDocument& myDocument, QString& myErrorMe
 
   // use scale dependent visibility flag
   setScaleBasedVisibility( myRoot.attribute( "hasScaleBasedVisibilityFlag" ).toInt() == 1 );
-  setMinimumScale( myRoot.attribute( "minimumScale" ).toFloat() );
-  setMaximumScale( myRoot.attribute( "maximumScale" ).toFloat() );
+  setMinimumScale( myRoot.attribute( "minimumScale" ).toDouble() );
+  setMaximumScale( myRoot.attribute( "maximumScale" ).toDouble() );
 
 #if 0
   //read transparency level
@@ -1250,7 +1262,7 @@ void QgsMapLayer::exportNamedStyle( QDomDocument &doc, QString &errorMsg )
   QDomDocument myDocument( documentType );
 
   QDomElement myRootNode = myDocument.createElement( "qgis" );
-  myRootNode.setAttribute( "version", QString( "%1" ).arg( QGis::QGIS_VERSION ) );
+  myRootNode.setAttribute( "version", QGis::QGIS_VERSION );
   myDocument.appendChild( myRootNode );
 
   myRootNode.setAttribute( "hasScaleBasedVisibilityFlag", hasScaleBasedVisibility() ? 1 : 0 );
@@ -1612,12 +1624,32 @@ QString QgsMapLayer::loadSldStyle( const QString &theURI, bool &theResultFlag )
   return "";
 }
 
+bool QgsMapLayer::readStyle( const QDomNode& node, QString& errorMessage )
+{
+  Q_UNUSED( node );
+  Q_UNUSED( errorMessage );
+  return false;
+}
+
+bool QgsMapLayer::writeStyle( QDomNode& node, QDomDocument& doc, QString& errorMessage ) const
+{
+  Q_UNUSED( node );
+  Q_UNUSED( doc );
+  Q_UNUSED( errorMessage );
+  return false;
+}
+
 
 QUndoStack* QgsMapLayer::undoStack()
 {
   return &mUndoStack;
 }
 
+QUndoStack* QgsMapLayer::undoStackStyles()
+{
+  return &mUndoStackStyles;
+}
+
 
 void QgsMapLayer::setCustomProperty( const QString& key, const QVariant& value )
 {
@@ -1690,6 +1722,11 @@ QString QgsMapLayer::metadata()
   return QString();
 }
 
+void QgsMapLayer::emitStyleChanged()
+{
+  emit styleChanged();
+}
+
 void QgsMapLayer::setExtent( const QgsRectangle &r )
 {
   mExtent = r;
