@@ -50,6 +50,8 @@ class CORE_EXPORT QgsMapLayer : public QObject
 {
     Q_OBJECT
 
+    Q_PROPERTY( QString name READ name WRITE setName NOTIFY nameChanged )
+
   public:
     /** Layers enum defining the types of layers that can be added to a map */
     enum LayerType
@@ -64,7 +66,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * @param lyrname Display Name of the layer
      * @param source datasource of layer
      */
-    QgsMapLayer( QgsMapLayer::LayerType type = VectorLayer, QString lyrname = QString::null, QString source = QString::null );
+    QgsMapLayer( QgsMapLayer::LayerType type = VectorLayer, const QString& lyrname = QString::null, const QString& source = QString::null );
 
     /** Destructor */
     virtual ~QgsMapLayer();
@@ -79,59 +81,178 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
     /** Set the display name of the layer
      * @param name New name for the layer
+     * @deprecated Since 2.16, use setName instead
      */
-    void setLayerName( const QString & name );
+    Q_DECL_DEPRECATED void setLayerName( const QString & name );
+
+    /**
+     * Set the display name of the layer
+     * @param name New name for the layer
+     *
+     * @note added in 2.16
+     */
+    void setName( const QString& name );
 
     /** Get the display name of the layer
      * @return the layer name
      */
-    const QString & name() const;
+    QString name() const;
 
-    /** Get the original name of the layer */
-    const QString & originalName() const { return mLayerOrigName; }
+    /** Get the original name of the layer
+     * @return the original layer name
+     */
+    QString originalName() const { return mLayerOrigName; }
 
+    /** Set the short name of the layer
+     *  used by QGIS Server to identify the layer
+     * @return the layer short name
+     */
+    void setShortName( const QString& shortName ) { mShortName = shortName; }
+    /** Get the short name of the layer
+     *  used by QGIS Server to identify the layer
+     * @return the layer short name
+     */
+    QString shortName() const { return mShortName; }
+
+    /** Set the title of the layer
+     *  used by QGIS Server in GetCapabilities request
+     * @return the layer title
+     */
     void setTitle( const QString& title ) { mTitle = title; }
-    const QString& title() const { return mTitle; }
+    /** Get the title of the layer
+     *  used by QGIS Server in GetCapabilities request
+     * @return the layer title
+     */
+    QString title() const { return mTitle; }
 
+    /** Set the abstract of the layer
+     *  used by QGIS Server in GetCapabilities request
+     * @return the layer abstract
+     */
     void setAbstract( const QString& abstract ) { mAbstract = abstract; }
-    const QString& abstract() const { return mAbstract; }
+    /** Get the abstract of the layer
+     *  used by QGIS Server in GetCapabilities request
+     * @return the layer abstract
+     */
+    QString abstract() const { return mAbstract; }
 
+    /** Set the keyword list of the layer
+     *  used by QGIS Server in GetCapabilities request
+     * @return the layer keyword list
+     */
     void setKeywordList( const QString& keywords ) { mKeywordList = keywords; }
-    const QString& keywordList() const { return mKeywordList; }
+    /** Get the keyword list of the layer
+     *  used by QGIS Server in GetCapabilities request
+     * @return the layer keyword list
+     */
+    QString keywordList() const { return mKeywordList; }
 
     /* Layer dataUrl information */
+    /** Set the DataUrl of the layer
+     *  used by QGIS Server in GetCapabilities request
+     *  DataUrl is a a link to the underlying data represented by a particular layer
+     * @return the layer DataUrl
+     */
     void setDataUrl( const QString& dataUrl ) { mDataUrl = dataUrl; }
-    const QString& dataUrl() const { return mDataUrl; }
+    /** Get the DataUrl of the layer
+     *  used by QGIS Server in GetCapabilities request
+     *  DataUrl is a a link to the underlying data represented by a particular layer
+     * @return the layer DataUrl
+     */
+    QString dataUrl() const { return mDataUrl; }
+    /** Set the DataUrl format of the layer
+     *  used by QGIS Server in GetCapabilities request
+     *  DataUrl is a a link to the underlying data represented by a particular layer
+     * @return the layer DataUrl format
+     */
     void setDataUrlFormat( const QString& dataUrlFormat ) { mDataUrlFormat = dataUrlFormat; }
-    const QString& dataUrlFormat() const { return mDataUrlFormat; }
+    /** Get the DataUrl format of the layer
+     *  used by QGIS Server in GetCapabilities request
+     *  DataUrl is a a link to the underlying data represented by a particular layer
+     * @return the layer DataUrl format
+     */
+    QString dataUrlFormat() const { return mDataUrlFormat; }
 
     /* Layer attribution information */
+    /** Set the attribution of the layer
+     *  used by QGIS Server in GetCapabilities request
+     *  Attribution indicates the provider of a Layer or collection of Layers.
+     * @return the layer attribution
+     */
     void setAttribution( const QString& attrib ) { mAttribution = attrib; }
-    const QString& attribution() const { return mAttribution; }
+    /** Get the attribution of the layer
+     *  used by QGIS Server in GetCapabilities request
+     *  Attribution indicates the provider of a Layer or collection of Layers.
+     * @return the layer attribution
+     */
+    QString attribution() const { return mAttribution; }
+    /** Set the attribution URL of the layer
+     *  used by QGIS Server in GetCapabilities request
+     *  Attribution indicates the provider of a Layer or collection of Layers.
+     * @return the layer attribution URL
+     */
     void setAttributionUrl( const QString& attribUrl ) { mAttributionUrl = attribUrl; }
-    const QString& attributionUrl() const { return mAttributionUrl; }
+    /** Get the attribution URL of the layer
+     *  used by QGIS Server in GetCapabilities request
+     *  Attribution indicates the provider of a Layer or collection of Layers.
+     * @return the layer attribution URL
+     */
+    QString attributionUrl() const { return mAttributionUrl; }
 
     /* Layer metadataUrl information */
+    /** Set the metadata URL of the layer
+     *  used by QGIS Server in GetCapabilities request
+     *  MetadataUrl is a a link to the detailed, standardized metadata about the data.
+     * @return the layer metadata URL
+     */
     void setMetadataUrl( const QString& metaUrl ) { mMetadataUrl = metaUrl; }
-    const QString& metadataUrl() const { return mMetadataUrl; }
+    /** Get the metadata URL of the layer
+     *  used by QGIS Server in GetCapabilities request
+     *  MetadataUrl is a a link to the detailed, standardized metadata about the data.
+     * @return the layer metadata URL
+     */
+    QString metadataUrl() const { return mMetadataUrl; }
+    /** Set the metadata type of the layer
+     *  used by QGIS Server in GetCapabilities request
+     *  MetadataUrlType indicates the standard to which the metadata complies.
+     * @return the layer metadata type
+     */
     void setMetadataUrlType( const QString& metaUrlType ) { mMetadataUrlType = metaUrlType; }
-    const QString& metadataUrlType() const { return mMetadataUrlType; }
+    /** Get the metadata type of the layer
+     *  used by QGIS Server in GetCapabilities request
+     *  MetadataUrlType indicates the standard to which the metadata complies.
+     * @return the layer metadata type
+     */
+    QString metadataUrlType() const { return mMetadataUrlType; }
+    /** Set the metadata format of the layer
+     *  used by QGIS Server in GetCapabilities request
+     *  MetadataUrlType indicates how the metadata is structured.
+     * @return the layer metadata format
+     */
     void setMetadataUrlFormat( const QString& metaUrlFormat ) { mMetadataUrlFormat = metaUrlFormat; }
-    const QString& metadataUrlFormat() const { return mMetadataUrlFormat; }
+    /** Get the metadata format of the layer
+     *  used by QGIS Server in GetCapabilities request
+     *  MetadataUrlType indicates how the metadata is structured.
+     * @return the layer metadata format
+     */
+    QString metadataUrlFormat() const { return mMetadataUrlFormat; }
 
-    /* Set the blending mode used for rendering a layer */
-    void setBlendMode( const QPainter::CompositionMode &blendMode );
-    /* Returns the current blending mode for a layer */
+    /** Set the blending mode used for rendering a layer */
+    void setBlendMode( QPainter::CompositionMode blendMode );
+    /** Returns the current blending mode for a layer */
     QPainter::CompositionMode blendMode() const;
 
-    /**Synchronises with changes in the datasource
+    /** Returns if this layer is read only. */
+    bool readOnly() const { return isReadOnly(); }
+
+    /** Synchronises with changes in the datasource
         */
     virtual void reload() {}
 
     /** Return new instance of QgsMapLayerRenderer that will be used for rendering of given context
      * @note added in 2.4
      */
-    virtual QgsMapLayerRenderer* createMapRenderer( QgsRenderContext& rendererContext ) { Q_UNUSED( rendererContext ); return 0; }
+    virtual QgsMapLayerRenderer* createMapRenderer( QgsRenderContext& rendererContext ) { Q_UNUSED( rendererContext ); return nullptr; }
 
     /** This is the method that does the actual work of
      * drawing the layer onto a paint device.
@@ -149,20 +270,20 @@ class CORE_EXPORT QgsMapLayer : public QObject
     /** Return the extent of the layer */
     virtual QgsRectangle extent();
 
-    /*! Return the status of the layer. An invalid layer is one which has a bad datasource
+    /** Return the status of the layer. An invalid layer is one which has a bad datasource
      * or other problem. Child classes set this flag when intialized
      * @return True if the layer is valid and can be accessed
      */
     bool isValid();
 
-    /*! Gets a version of the internal layer definition that has sensitive
+    /** Gets a version of the internal layer definition that has sensitive
       *  bits removed (for example, the password). This function should
       * be used when displaying the source name for general viewing.
      */
     QString publicSource() const;
 
     /** Returns the source for the layer */
-    const QString &source() const;
+    QString source() const;
 
     /**
      * Returns the sublayers of this layer
@@ -177,13 +298,19 @@ class CORE_EXPORT QgsMapLayer : public QObject
     virtual void setLayerOrder( const QStringList &layers );
 
     /** Set the visibility of the given sublayer name */
-    virtual void setSubLayerVisibility( QString name, bool vis );
+    virtual void setSubLayerVisibility( const QString& name, bool vis );
 
     /** True if the layer can be edited */
     virtual bool isEditable() const;
 
-    /** sets state from Dom document
+    /** Returns true if the layer is considered a spatial layer, ie it has some form of geometry associated with it.
+     * @note added in QGIS 2.16
+     */
+    virtual bool isSpatial() const { return true; }
+
+    /** Sets state from Dom document
        @param layerElement The Dom element corresponding to ``maplayer'' tag
+       @param relativeBasePath base path for relative paths
        @note
 
        The Dom node corresponds to a Dom document project file XML element read
@@ -196,37 +323,37 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
        @returns true if successful
      */
-    bool readLayerXML( const QDomElement& layerElement );
+    bool readLayerXML( const QDomElement& layerElement, const QString& relativeBasePath = QString() );
 
 
-    /** stores state in Dom node
-       @param layerElement is a Dom element corresponding to ``maplayer'' tag
-       @param document is a the dom document being written
-       @param relativeBasePath base path for relative paths
-       @note
-
-       The Dom node corresponds to a Dom document project file XML element to be
-       written by QgsProject.
-
-       This, in turn, calls writeXml(), which is over-rideable by sub-classes so
-       that they can write their own specific state to the given Dom node.
-
-       Invoked by QgsProject::write().
-
-       @returns true if successful
-    */
-    bool writeLayerXML( QDomElement& layerElement, QDomDocument& document, QString relativeBasePath = QString::null );
+    /** Stores state in Dom node
+     * @param layerElement is a Dom element corresponding to ``maplayer'' tag
+     * @param document is a the dom document being written
+     * @param relativeBasePath base path for relative paths
+     * @note
+     *
+     * The Dom node corresponds to a Dom document project file XML element to be
+     * written by QgsProject.
+     *
+     * This, in turn, calls writeXml(), which is over-rideable by sub-classes so
+     * that they can write their own specific state to the given Dom node.
+     *
+     * Invoked by QgsProject::write().
+     *
+     * @returns true if successful
+     */
+    bool writeLayerXML( QDomElement& layerElement, QDomDocument& document, const QString& relativeBasePath = QString() );
 
     /** Returns the given layer as a layer definition document
-        Layer definitions store the data source as well as styling and custom properties.
-
-        Layer definitions can be used to load a layer and styling all from a single file.
-    */
-    static QDomDocument asLayerDefinition( QList<QgsMapLayer*> layers, QString relativeBasePath = QString::null );
+     *  Layer definitions store the data source as well as styling and custom properties.
+     *
+     *  Layer definitions can be used to load a layer and styling all from a single file.
+     */
+    static QDomDocument asLayerDefinition( const QList<QgsMapLayer*>& layers, const QString& relativeBasePath = QString() );
 
     /** Creates a new layer from a layer defininition document
-    */
-    static QList<QgsMapLayer*> fromLayerDefinition( QDomDocument& document );
+     */
+    static QList<QgsMapLayer*> fromLayerDefinition( QDomDocument& document, bool addToRegistry = false, bool addToLegend = false, const QString& relativeBasePath = QString() );
     static QList<QgsMapLayer*> fromLayerDefinitionFile( const QString &qlrfile );
 
     /** Set a custom property for layer. Properties are stored in a map and saved in project file. */
@@ -251,7 +378,8 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
     /** Returns layer's spatial reference system
     @note This was introduced in QGIS 1.4
-    */
+     */
+    //TODO QGIS 3.0 - return QgsCoordinateReferenceSystem object, not reference (since they are implicitly shared)
     const QgsCoordinateReferenceSystem& crs() const;
 
     /** Sets layer's spatial reference system */
@@ -354,13 +482,23 @@ class CORE_EXPORT QgsMapLayer : public QObject
     { Q_UNUSED( node ); errorMessage = QString( "Layer type %1 not supported" ).arg( type() ); return false; }
 
 
+
     /** Read the symbology for the current layer from the Dom node supplied.
      * @param node node that will contain the symbology definition for this layer.
      * @param errorMessage reference to string that will be updated with any error messages
      * @return true in case of success.
-    */
+     */
     virtual bool readSymbology( const QDomNode& node, QString& errorMessage ) = 0;
 
+    /** Read the style for the current layer from the Dom node supplied.
+     * @param node node that will contain the style definition for this layer.
+     * @param errorMessage reference to string that will be updated with any error messages
+     * @return true in case of success.
+     * @note added in 2.16
+     * @note To be implemented in subclasses. Default implementation does nothing and returns false.
+     */
+    virtual bool readStyle( const QDomNode& node, QString& errorMessage );
+
     /** Write the symbology for the layer into the docment provided.
      *  @param node the node that will have the style element added to it.
      *  @param doc the document that will have the QDomNode added.
@@ -369,17 +507,32 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     virtual bool writeSymbology( QDomNode &node, QDomDocument& doc, QString& errorMessage ) const = 0;
 
+    /** Write just the style information for the layer into the document
+     *  @param node the node that will have the style element added to it.
+     *  @param doc the document that will have the QDomNode added.
+     *  @param errorMessage reference to string that will be updated with any error messages
+     *  @return true in case of success.
+     *  @note added in 2.16
+     *  @note To be implemented in subclasses. Default implementation does nothing and returns false.
+     */
+    virtual bool writeStyle( QDomNode& node, QDomDocument& doc, QString& errorMessage ) const;
+
     /** Return pointer to layer's undo stack */
     QUndoStack *undoStack();
 
+    /** Return pointer to layer's style undo stack
+     *  @note added in 2.16
+     */
+    QUndoStack *undoStackStyles();
+
     /* Layer legendUrl information */
     void setLegendUrl( const QString& legendUrl ) { mLegendUrl = legendUrl; }
-    const QString& legendUrl() const { return mLegendUrl; }
+    QString legendUrl() const { return mLegendUrl; }
     void setLegendUrlFormat( const QString& legendUrlFormat ) { mLegendUrlFormat = legendUrlFormat; }
-    const QString& legendUrlFormat() const { return mLegendUrlFormat; }
+    QString legendUrlFormat() const { return mLegendUrlFormat; }
 
-    /** @deprecated since 2.4 - returns NULL */
-    Q_DECL_DEPRECATED QImage *cacheImage() { return 0; }
+    /** @deprecated since 2.4 - returns nullptr */
+    Q_DECL_DEPRECATED QImage *cacheImage() { return nullptr; }
     /** @deprecated since 2.4 - caches listen to repaintRequested() signal to invalidate the cached image */
     Q_DECL_DEPRECATED void setCacheImage( QImage * );
     /** @deprecated since 2.4 - does nothing */
@@ -403,30 +556,43 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     QgsMapLayerStyleManager* styleManager() const;
 
-    /**Returns the minimum scale denominator at which the layer is visible.
+    /** Tests whether the layer should be visible at the specified scale.
+     * @param scale scale denominator to test
+     * @returns true if the layer is visible at the given scale.
+     * @note added in QGIS 2.16
+     * @see minimumScale()
+     * @see maximumScale()
+     * @see hasScaleBasedVisibility()
+     */
+    bool isInScaleRange( double scale ) const;
+
+    /** Returns the minimum scale denominator at which the layer is visible.
      * Scale based visibility is only used if hasScaleBasedVisibility is true.
      * @returns minimum scale denominator at which the layer will render
-     * @see setMinimumScale
-     * @see maximumScale
-     * @see hasScaleBasedVisibility
+     * @see setMinimumScale()
+     * @see maximumScale()
+     * @see hasScaleBasedVisibility()
+     * @see isInScaleRange()
      */
-    float minimumScale() const;
+    double minimumScale() const;
 
-    /**Returns the maximum scale denominator at which the layer is visible.
+    /** Returns the maximum scale denominator at which the layer is visible.
      * Scale based visibility is only used if hasScaleBasedVisibility is true.
      * @returns minimum scale denominator at which the layer will render
-     * @see setMaximumScale
-     * @see minimumScale
-     * @see hasScaleBasedVisibility
+     * @see setMaximumScale()
+     * @see minimumScale()
+     * @see hasScaleBasedVisibility()
+     * @see isInScaleRange()
      */
-    float maximumScale() const;
+    double maximumScale() const;
 
-    /**Returns whether scale based visibility is enabled for the layer.
+    /** Returns whether scale based visibility is enabled for the layer.
      * @returns true if scale based visibility is enabled
-     * @see minimumScale
-     * @see maximumScale
-     * @see setScaleBasedVisibility
-    */
+     * @see minimumScale()
+     * @see maximumScale()
+     * @see setScaleBasedVisibility()
+     * @see isInScaleRange()
+     */
     bool hasScaleBasedVisibility() const;
 
   public slots:
@@ -434,25 +600,25 @@ class CORE_EXPORT QgsMapLayer : public QObject
     /** Event handler for when a coordinate transform fails due to bad vertex error */
     virtual void invalidTransformInput();
 
-    /**Sets the minimum scale denominator at which the layer will be visible.
+    /** Sets the minimum scale denominator at which the layer will be visible.
      * Scale based visibility is only used if setScaleBasedVisibility is set to true.
      * @param theMinScale minimum scale denominator at which the layer should render
      * @see minimumScale
      * @see setMaximumScale
      * @see setScaleBasedVisibility
      */
-    void setMinimumScale( const float theMinScale );
+    void setMinimumScale( double theMinScale );
 
-    /**Sets the maximum scale denominator at which the layer will be visible.
+    /** Sets the maximum scale denominator at which the layer will be visible.
      * Scale based visibility is only used if setScaleBasedVisibility is set to true.
      * @param theMaxScale maximum scale denominator at which the layer should render
      * @see maximumScale
      * @see setMinimumScale
      * @see setScaleBasedVisibility
      */
-    void setMaximumScale( const float theMaxScale );
+    void setMaximumScale( double theMaxScale );
 
-    /**Sets whether scale based visibility is enabled for the layer.
+    /** Sets whether scale based visibility is enabled for the layer.
      * @param enabled set to true to enable scale based visibility
      * @see setMinimumScale
      * @see setMaximumScale
@@ -460,13 +626,14 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     void setScaleBasedVisibility( const bool enabled );
 
-    /**Accessor for the scale based visilibility flag
+    /** Accessor for the scale based visilibility flag
      * @deprecated use setScaleBasedVisibility instead
-    */
+     */
     Q_DECL_DEPRECATED void toggleScaleBasedVisibility( bool theVisibilityFlag );
 
     /** Clear cached image
-     *  @deprecated in 2.4 - use triggerRepaint() - caches automatically listen to repaintRequested() signal to invalidate the cached image */
+     *  @deprecated in 2.4 - use triggerRepaint() - caches automatically listen to repaintRequested() signal to invalidate the cached image
+     */
     Q_DECL_DEPRECATED void clearCacheImage();
 
     /**
@@ -483,16 +650,30 @@ class CORE_EXPORT QgsMapLayer : public QObject
     /** Time stamp of data source in the moment when data/metadata were loaded by provider */
     virtual QDateTime timestamp() const { return QDateTime() ; }
 
+    /** Triggers an emission of the styleChanged() signal.
+     * @note added in QGIS 2.16
+     */
+    void emitStyleChanged();
+
   signals:
 
     //! @deprecated in 2.4 - not emitted anymore
-    void drawingProgress( int theProgress, int theTotalSteps );
+    Q_DECL_DEPRECATED void drawingProgress( int theProgress, int theTotalSteps );
 
     /** Emit a signal with status (e.g. to be caught by QgisApp and display a msg on status bar) */
-    void statusChanged( QString theStatus );
+    void statusChanged( const QString& theStatus );
 
-    /** Emit a signal that the layer name has been changed */
-    void layerNameChanged();
+    /** Emit a signal that the layer name has been changed
+     * @deprecated since 2.16 use nameChanged() instead
+     */
+    Q_DECL_DEPRECATED void layerNameChanged();
+
+    /**
+     * Emitted when the name has been changed
+     *
+     * @note added in 2.16
+     */
+    void nameChanged();
 
     /** Emit a signal that layer's CRS has been reset */
     void layerCrsChanged();
@@ -508,36 +689,52 @@ class CORE_EXPORT QgsMapLayer : public QObject
     /** This is used to send a request that any mapcanvas using this layer update its extents */
     void recalculateExtents();
 
-    /** data of layer changed */
+    /** Data of layer changed */
     void dataChanged();
 
     /** Signal emitted when the blend mode is changed, through QgsMapLayer::setBlendMode() */
-    void blendModeChanged( const QPainter::CompositionMode &blendMode );
+    void blendModeChanged( QPainter::CompositionMode blendMode );
 
-    /** Signal emitted when renderer is changed */
+    /** Signal emitted when renderer is changed.
+     * @see styleChanged()
+    */
     void rendererChanged();
 
+    /** Signal emitted whenever a change affects the layer's style. Ie this may be triggered
+     * by renderer changes, label style changes, or other style changes such as blend
+     * mode or layer opacity changes.
+     * @note added in QGIS 2.16
+     * @see rendererChanged()
+    */
+    void styleChanged();
+
     /**
      * Signal emitted when legend of the layer has changed
      * @note added in 2.6
      */
     void legendChanged();
 
+    /**
+     * Emitted whenever the configuration is changed. The project listens to this signal
+     * to be marked as dirty.
+     */
+    void configChanged();
+
   protected:
     /** Set the extent */
     virtual void setExtent( const QgsRectangle &rect );
 
-    /** set whether layer is valid or not - should be used in constructor. */
+    /** Set whether layer is valid or not - should be used in constructor. */
     void setValid( bool valid );
 
-    /** called by readLayerXML(), used by children to read state specific to them from
-        project files.
-    */
+    /** Called by readLayerXML(), used by children to read state specific to them from
+     *  project files.
+     */
     virtual bool readXml( const QDomNode& layer_node );
 
-    /** called by writeLayerXML(), used by children to write state specific to them to
-        project files.
-    */
+    /** Called by writeLayerXML(), used by children to write state specific to them to
+     *  project files.
+     */
     virtual bool writeXml( QDomNode & layer_node, QDomDocument & document );
 
 
@@ -555,7 +752,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     void writeStyleManager( QDomNode& layerNode, QDomDocument& doc ) const;
 
 #if 0
-    /** debugging member - invoked when a connect() is made to this object */
+    /** Debugging member - invoked when a connect() is made to this object */
     void connectNotify( const char * signal ) override;
 #endif
 
@@ -570,7 +767,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     /** Indicates if the layer is valid and can be drawn */
     bool mValid;
 
-    /** data source description string, varies by layer type */
+    /** Data source description string, varies by layer type */
     QString mDataSource;
 
     /** Name of the layer - used for display */
@@ -580,26 +777,27 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     QString mLayerOrigName;
 
+    QString mShortName;
     QString mTitle;
 
-    /**Description of the layer*/
+    /** Description of the layer*/
     QString mAbstract;
     QString mKeywordList;
 
-    /**DataUrl of the layer*/
+    /** DataUrl of the layer*/
     QString mDataUrl;
     QString mDataUrlFormat;
 
-    /**Attribution of the layer*/
+    /** Attribution of the layer*/
     QString mAttribution;
     QString mAttributionUrl;
 
-    /**MetadataUrl of the layer*/
+    /** MetadataUrl of the layer*/
     QString mMetadataUrl;
     QString mMetadataUrlType;
     QString mMetadataUrlFormat;
 
-    /**WMS legend*/
+    /** WMS legend*/
     QString mLegendUrl;
     QString mLegendUrlFormat;
 
@@ -607,14 +805,20 @@ class CORE_EXPORT QgsMapLayer : public QObject
     QgsError mError;
 
   private:
-    /** layer's spatial reference system.
-        private to make sure setCrs must be used and layerCrsChanged() is emitted */
-    QgsCoordinateReferenceSystem* mCRS;
+    /**
+     * This method returns true by default but can be overwritten to specify
+     * that a certain layer is writable.
+     */
+    virtual bool isReadOnly() const { return true; }
 
-    /** private copy constructor - QgsMapLayer not copyable */
+    /** Layer's spatial reference system.
+        private to make sure setCrs must be used and layerCrsChanged() is emitted */
+    QgsCoordinateReferenceSystem mCRS;
+
+    /** Private copy constructor - QgsMapLayer not copyable */
     QgsMapLayer( QgsMapLayer const & );
 
-    /** private assign operator - QgsMapLayer not copyable */
+    /** Private assign operator - QgsMapLayer not copyable */
     QgsMapLayer & operator=( QgsMapLayer const & );
 
     /** Unique ID of this layer - used to refer to this layer in map layer registry */
@@ -630,15 +834,17 @@ class CORE_EXPORT QgsMapLayer : public QObject
     QString mTag;
 
     /** Minimum scale denominator at which this layer should be displayed */
-    float mMinScale;
+    double mMinScale;
     /** Maximum scale denominator at which this layer should be displayed */
-    float mMaxScale;
+    double mMaxScale;
     /** A flag that tells us whether to use the above vars to restrict layer visibility */
     bool mScaleBasedVisibility;
 
     /** Collection of undoable operations for this layer. **/
     QUndoStack mUndoStack;
 
+    QUndoStack mUndoStackStyles;
+
     //! Layer's persistent storage of additional properties (may be used by plugins)
     QgsObjectCustomProperties mCustomProperties;
 
@@ -649,4 +855,6 @@ class CORE_EXPORT QgsMapLayer : public QObject
     QgsMapLayerStyleManager* mStyleManager;
 };
 
+Q_DECLARE_METATYPE( QgsMapLayer* )
+
 #endif
