@@ -22,15 +22,16 @@
 
 QgsMapRendererSequentialJob::QgsMapRendererSequentialJob( const QgsMapSettings& settings )
     : QgsMapRendererQImageJob( settings )
-    , mInternalJob( 0 )
-    , mPainter( 0 )
-    , mLabelingResults( 0 )
+    , mInternalJob( nullptr )
+    , mPainter( nullptr )
+    , mLabelingResults( nullptr )
 {
   QgsDebugMsg( "SEQUENTIAL construct" );
 
   mImage = QImage( mSettings.outputSize(), mSettings.outputImageFormat() );
   mImage.setDotsPerMeterX( 1000 * settings.outputDpi() / 25.4 );
   mImage.setDotsPerMeterY( 1000 * settings.outputDpi() / 25.4 );
+  mImage.fill( Qt::transparent );
 }
 
 QgsMapRendererSequentialJob::~QgsMapRendererSequentialJob()
@@ -43,10 +44,10 @@ QgsMapRendererSequentialJob::~QgsMapRendererSequentialJob()
     cancel();
   }
 
-  Q_ASSERT( mInternalJob == 0 && mPainter == 0 );
+  Q_ASSERT( !mInternalJob && !mPainter );
 
   delete mLabelingResults;
-  mLabelingResults = 0;
+  mLabelingResults = nullptr;
 }
 
 
@@ -61,7 +62,7 @@ void QgsMapRendererSequentialJob::start()
 
   QgsDebugMsg( "SEQUENTIAL START" );
 
-  Q_ASSERT( mInternalJob == 0  && mPainter == 0 );
+  Q_ASSERT( !mInternalJob && !mPainter );
 
   mPainter = new QPainter( &mImage );
 
@@ -82,7 +83,16 @@ void QgsMapRendererSequentialJob::cancel()
   QgsDebugMsg( "sequential - cancel internal" );
   mInternalJob->cancel();
 
-  Q_ASSERT( mInternalJob == 0 && mPainter == 0 );
+  Q_ASSERT( !mInternalJob && !mPainter );
+}
+
+void QgsMapRendererSequentialJob::cancelWithoutBlocking()
+{
+  if ( !isActive() )
+    return;
+
+  QgsDebugMsg( "sequential - cancel internal" );
+  mInternalJob->cancelWithoutBlocking();
 }
 
 void QgsMapRendererSequentialJob::waitForFinished()
@@ -95,13 +105,13 @@ void QgsMapRendererSequentialJob::waitForFinished()
 
 bool QgsMapRendererSequentialJob::isActive() const
 {
-  return mInternalJob != 0;
+  return nullptr != mInternalJob;
 }
 
 QgsLabelingResults* QgsMapRendererSequentialJob::takeLabelingResults()
 {
   QgsLabelingResults* tmp = mLabelingResults;
-  mLabelingResults = 0;
+  mLabelingResults = nullptr;
   return tmp;
 }
 
@@ -122,7 +132,7 @@ void QgsMapRendererSequentialJob::internalFinished()
 
   mPainter->end();
   delete mPainter;
-  mPainter = 0;
+  mPainter = nullptr;
 
   mLabelingResults = mInternalJob->takeLabelingResults();
 
@@ -131,7 +141,7 @@ void QgsMapRendererSequentialJob::internalFinished()
   // now we are in a slot called from mInternalJob - do not delete it immediately
   // so the class is still valid when the execution returns to the class
   mInternalJob->deleteLater();
-  mInternalJob = 0;
+  mInternalJob = nullptr;
 
   mRenderingTime = mRenderingStart.elapsed();
 
