@@ -30,7 +30,7 @@
 */
 class CORE_EXPORT QgsMessageLog : public QObject
 {
-    Q_OBJECT;
+    Q_OBJECT
 
   public:
     static QgsMessageLog *instance();
@@ -43,21 +43,23 @@ class CORE_EXPORT QgsMessageLog : public QObject
     };
 
     //! add a message to the instance (and create it if necessary)
-    static void logMessage( QString message, QString tag = QString::null, MessageLevel level = WARNING );
+    static void logMessage( const QString& message, const QString& tag = QString::null, MessageLevel level = WARNING );
 
   signals:
-    void messageReceived( QString message, QString tag, QgsMessageLog::MessageLevel level );
+    void messageReceived( const QString& message, const QString& tag, QgsMessageLog::MessageLevel level );
+
+    void messageReceived( bool received );
 
   private:
     QgsMessageLog();
 
-    void emitMessage( QString message, QString tag, QgsMessageLog::MessageLevel level );
+    void emitMessage( const QString& message, const QString& tag, QgsMessageLog::MessageLevel level );
 
     static QgsMessageLog *sInstance;
 };
 
 
-/**
+/** \ingroup core
 \brief Default implementation of message logging interface
 
 This class outputs log messages to the standard output. Therefore it might
@@ -65,13 +67,13 @@ be the right choice for apps without GUI.
 */
 class CORE_EXPORT QgsMessageLogConsole : public QObject
 {
-    Q_OBJECT;
+    Q_OBJECT
 
   public:
     QgsMessageLogConsole();
 
   public slots:
-    void logMessage( QString message, QString tag, QgsMessageLog::MessageLevel level );
+    void logMessage( const QString& message, const QString& tag, QgsMessageLog::MessageLevel level );
 };
 
 #endif
