@@ -21,7 +21,7 @@
 #include "qgspluginlayer.h"
 #include "qgsmaplayerregistry.h"
 
-QgsPluginLayerType::QgsPluginLayerType( QString name )
+QgsPluginLayerType::QgsPluginLayerType( const QString& name )
     : mName( name )
 {
 }
@@ -37,7 +37,13 @@ QString QgsPluginLayerType::name()
 
 QgsPluginLayer* QgsPluginLayerType::createLayer()
 {
-  return NULL;
+  return nullptr;
+}
+
+QgsPluginLayer* QgsPluginLayerType::createLayer( const QString& uri )
+{
+  Q_UNUSED( uri );
+  return nullptr;
 }
 
 bool QgsPluginLayerType::showLayerProperties( QgsPluginLayer *layer )
@@ -49,10 +55,10 @@ bool QgsPluginLayerType::showLayerProperties( QgsPluginLayer *layer )
 //=============================================================================
 
 /** Static calls to enforce singleton behaviour */
-QgsPluginLayerRegistry* QgsPluginLayerRegistry::_instance = NULL;
+QgsPluginLayerRegistry* QgsPluginLayerRegistry::_instance = nullptr;
 QgsPluginLayerRegistry* QgsPluginLayerRegistry::instance()
 {
-  if ( _instance == NULL )
+  if ( !_instance )
   {
     _instance = new QgsPluginLayerRegistry();
   }
@@ -69,8 +75,11 @@ QgsPluginLayerRegistry::~QgsPluginLayerRegistry()
   if ( !mPluginLayerTypes.isEmpty() )
   {
     QgsDebugMsg( "QgsPluginLayerRegistry::~QgsPluginLayerRegistry(): creator list not empty" );
-    foreach ( QString typeName, mPluginLayerTypes.keys() )
-      removePluginLayerType( typeName );
+    PluginLayerTypes::const_iterator it = mPluginLayerTypes.constBegin();
+    for ( ; it != mPluginLayerTypes.constEnd(); ++it )
+    {
+      removePluginLayerType( it.key() );
+    }
   }
 }
 
@@ -81,8 +90,9 @@ QStringList QgsPluginLayerRegistry::pluginLayerTypes()
 
 bool QgsPluginLayerRegistry::addPluginLayerType( QgsPluginLayerType* type )
 {
-  if ( type == NULL )
+  if ( !type )
     return false;
+
   if ( mPluginLayerTypes.contains( type->name() ) )
     return false;
 
@@ -91,14 +101,14 @@ bool QgsPluginLayerRegistry::addPluginLayerType( QgsPluginLayerType* type )
 }
 
 
-bool QgsPluginLayerRegistry::removePluginLayerType( QString typeName )
+bool QgsPluginLayerRegistry::removePluginLayerType( const QString& typeName )
 {
   if ( !mPluginLayerTypes.contains( typeName ) )
     return false;
 
   // remove all remaining layers of this type - to avoid invalid behaviour
   QList<QgsMapLayer*> layers = QgsMapLayerRegistry::instance()->mapLayers().values();
-  foreach ( QgsMapLayer* layer, layers )
+  Q_FOREACH ( QgsMapLayer* layer, layers )
   {
     if ( layer->type() == QgsMapLayer::PluginLayer )
     {
@@ -115,20 +125,23 @@ bool QgsPluginLayerRegistry::removePluginLayerType( QString typeName )
   return true;
 }
 
-QgsPluginLayerType* QgsPluginLayerRegistry::pluginLayerType( QString typeName )
+QgsPluginLayerType* QgsPluginLayerRegistry::pluginLayerType( const QString& typeName )
 {
-  return mPluginLayerTypes.value( typeName, NULL );
+  return mPluginLayerTypes.value( typeName, nullptr );
 }
 
 
-QgsPluginLayer* QgsPluginLayerRegistry::createLayer( QString typeName )
+QgsPluginLayer* QgsPluginLayerRegistry::createLayer( const QString& typeName, const QString& uri )
 {
   QgsPluginLayerType* type = pluginLayerType( typeName );
   if ( !type )
   {
     QgsDebugMsg( "Unknown plugin layer type: " + typeName );
-    return NULL;
+    return nullptr;
   }
 
-  return type->createLayer();
+  if ( !uri.isEmpty() )
+    return type->createLayer( uri );
+  else
+    return type->createLayer();
 }
