@@ -28,22 +28,26 @@
 // QgsVector
 //
 
-QgsVector::QgsVector() : m_x( 0.0 ), m_y( 0.0 )
+QgsVector::QgsVector()
+    : mX( 0.0 )
+    , mY( 0.0 )
 {
 }
 
-QgsVector::QgsVector( double x, double y ) : m_x( x ), m_y( y )
+QgsVector::QgsVector( double x, double y )
+    : mX( x )
+    , mY( y )
 {
 }
 
-QgsVector QgsVector::operator-( void ) const
+QgsVector QgsVector::operator-() const
 {
-  return QgsVector( -m_x, -m_y );
+  return QgsVector( -mX, -mY );
 }
 
 QgsVector QgsVector::operator*( double scalar ) const
 {
-  return QgsVector( m_x * scalar, m_y * scalar );
+  return QgsVector( mX * scalar, mY * scalar );
 }
 
 QgsVector QgsVector::operator/( double scalar ) const
@@ -53,33 +57,32 @@ QgsVector QgsVector::operator/( double scalar ) const
 
 double QgsVector::operator*( QgsVector v ) const
 {
-  return m_x * v.m_x + m_y * v.m_y;
+  return mX * v.mX + mY * v.mY;
 }
 
 double QgsVector::length() const
 {
-  return sqrt( m_x * m_x + m_y * m_y );
+  return sqrt( mX * mX + mY * mY );
 }
 
 double QgsVector::x() const
 {
-  return m_x;
+  return mX;
 }
 
 double QgsVector::y() const
 {
-  return m_y;
+  return mY;
 }
 
-// perpendicular vector (rotated 90 degrees counter-clockwise)
 QgsVector QgsVector::perpVector() const
 {
-  return QgsVector( -m_y, m_x );
+  return QgsVector( -mY, mX );
 }
 
-double QgsVector::angle( void ) const
+double QgsVector::angle() const
 {
-  double ang = atan2( m_y, m_x );
+  double ang = atan2( mY, mX );
   return ang < 0.0 ? ang + 2.0 * M_PI : ang;
 }
 
@@ -90,18 +93,23 @@ double QgsVector::angle( QgsVector v ) const
 
 QgsVector QgsVector::rotateBy( double rot ) const
 {
-  double ang = atan2( m_y, m_x ) + rot;
+  double ang = atan2( mY, mX ) + rot;
   double len = length();
   return QgsVector( len * cos( ang ), len * sin( ang ) );
 }
 
 QgsVector QgsVector::normal() const
+{
+  return normalized();
+}
+
+QgsVector QgsVector::normalized() const
 {
   double len = length();
 
   if ( len == 0.0 )
   {
-    throw QgsException( "normal vector of null vector undefined" );
+    throw QgsException( "normalized vector of null vector undefined" );
   }
 
   return *this / len;
@@ -352,6 +360,16 @@ double QgsPoint::sqrDist( const QgsPoint& other ) const
   return sqrDist( other.x(), other.y() );
 }
 
+double QgsPoint::distance( double x, double y ) const
+{
+  return sqrt( sqrDist( x, y ) );
+}
+
+double QgsPoint::distance( const QgsPoint& other ) const
+{
+  return sqrt( sqrDist( other ) );
+}
+
 double QgsPoint::azimuth( const QgsPoint& other ) const
 {
   double dx = other.x() - m_x;
@@ -359,6 +377,14 @@ double QgsPoint::azimuth( const QgsPoint& other ) const
   return ( atan2( dx, dy ) * 180.0 / M_PI );
 }
 
+QgsPoint QgsPoint::project( double distance, double bearing ) const
+{
+  double rads = bearing * M_PI / 180.0;
+  double dx = distance * sin( rads );
+  double dy = distance * cos( rads );
+  return QgsPoint( m_x + dx, m_y + dy );
+}
+
 bool QgsPoint::compare( const QgsPoint &other, double epsilon ) const
 {
   return ( qgsDoubleNear( m_x, other.x(), epsilon ) && qgsDoubleNear( m_y, other.y(), epsilon ) );
