@@ -20,8 +20,8 @@
 #include "qgis.h"
 
 QgsProviderCountCalcEvent::QgsProviderCountCalcEvent( long featuresCounted )
-    : QEvent( static_cast<QEvent::Type>( QGis::ProviderCountCalcEvent ) ),
-    n( featuresCounted )
+    : QEvent( static_cast<QEvent::Type>( QGis::ProviderCountCalcEvent ) )
+    , n( featuresCounted )
 {
   // NO-OP
 }
