@@ -3,7 +3,7 @@
      --------------------------------------
     Date                 : 29.4.2013
     Copyright            : (C) 2013 Matthias Kuhn
-    Email                : matthias dot kuhn at gmx dot ch
+    Email                : matthias at opengis dot ch
  ***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -25,15 +25,19 @@
 
 class QgsVectorLayer;
 
+/** \ingroup core
+ * \class QgsRelation
+ */
 class CORE_EXPORT QgsRelation
 {
   public:
     /**
-     * Defines a relation between matchin fields of the two involved tables of a relation.
+     * \ingroup core
+     * Defines a relation between matching fields of the two involved tables of a relation.
      * Often, a relation is only defined by just one FieldPair with the name of the foreign key
-     * column of the referencing table as first element and the name of the primkary key column
-     * of the referenced table as the second element.
-     *
+     * column of the referencing (child) table as first element and the name of the primary key column
+     * of the referenced (parent) table as the second element.
+     * @note not available in Python bindings
      */
     class FieldPair : public QPair< QString, QString >
     {
@@ -43,13 +47,13 @@ class CORE_EXPORT QgsRelation
             : QPair< QString, QString >() {}
 
         //! Constructor which takes two fields
-        FieldPair( QString referencingField, QString referencedField )
+        FieldPair( const QString& referencingField, const QString& referencedField )
             : QPair< QString, QString >( referencingField, referencedField ) {}
 
-        //! Get the name of the referencing field
-        const QString& referencingField() const { return first; }
-        //! Get the name of the referenced field
-        const QString& referencedField() const { return second; }
+        //! Get the name of the referencing (child) field
+        QString referencingField() const { return first; }
+        //! Get the name of the referenced (parent) field
+        QString referencedField() const { return second; }
     };
 
     /**
@@ -79,48 +83,48 @@ class CORE_EXPORT QgsRelation
      *
      * @param id
      */
-    void setRelationId( QString id );
+    void setRelationId( const QString& id );
 
     /**
      * Set a name for this relation
      *
      * @param name
      */
-    void setRelationName( QString name );
+    void setRelationName( const QString& name );
 
     /**
-     * Set the referencing layer id. This layer will be searched in the registry.
+     * Set the referencing (child) layer id. This layer will be searched in the registry.
      *
      * @param id
      */
-    void setReferencingLayer( QString id );
+    void setReferencingLayer( const QString& id );
 
     /**
-     * Set the referenced layer id. This layer will be searched in the registry.
+     * Set the referenced (parent) layer id. This layer will be searched in the registry.
      *
      * @param id
      */
-    void setReferencedLayer( QString id );
+    void setReferencedLayer( const QString& id );
 
     /**
      * Add a field pairs which is part of this relation
-     * The first element of each pair are the field names fo the foreign key.
+     * The first element of each pair are the field names of the foreign key.
      * The second element of each pair are the field names of the matching primary key.
      *
-     * @param referencingField  The field name on the referencing layer (FK)
-     * @param referencedField   The field name on the referenced layer  (PK)
+     * @param referencingField  The field name on the referencing (child) layer (FK)
+     * @param referencedField   The field name on the referenced (parent) layer  (PK)
      */
-    void addFieldPair( QString referencingField, QString referencedField );
+    void addFieldPair( const QString& referencingField, const QString& referencedField );
 
     /**
      * Add a field pairs which is part of this relation
-     * The first element of each pair are the field names fo the foreign key.
+     * The first element of each pair are the field names of the foreign key.
      * The second element of each pair are the field names of the matching primary key.
      *
      * @param fieldPair A pair of two strings
      * @note not available in python bindings
      */
-    void addFieldPair( FieldPair fieldPair );
+    void addFieldPair( const FieldPair& fieldPair );
 
     /**
      * Creates an iterator which returns all the features on the referencing (child) layer
@@ -129,6 +133,8 @@ class CORE_EXPORT QgsRelation
      * @param feature A feature from the referenced (parent) layer
      *
      * @return An iterator with all the referenced features
+     * @see getRelatedFeaturesRequest()
+     * @see getRelatedFeaturesFilter()
      */
     QgsFeatureIterator getRelatedFeatures( const QgsFeature& feature ) const;
 
@@ -138,17 +144,68 @@ class CORE_EXPORT QgsRelation
      *
      * @param feature A feature from the referenced (parent) layer
      *
-     * @return An request for all the referenced features
+     * @return A request for all the referencing features
+     * @see getRelatedFeatures()
+     * @see getRelatedFeaturesFilter()
      */
     QgsFeatureRequest getRelatedFeaturesRequest( const QgsFeature& feature ) const;
 
-    const QString name() const;
+    /** Returns a filter expression which returns all the features on the referencing (child) layer
+     * which have a foreign key pointing to the provided feature.
+     * @param feature A feature from the referenced (parent) layer
+     * @return expression filter string for all the referencing features
+     * @note added in QGIS 2.16
+     * @see getRelatedFeatures()
+     * @see getRelatedFeaturesRequest()
+     */
+    QString getRelatedFeaturesFilter( const QgsFeature& feature ) const;
 
     /**
-     * The id
-     * @return
+     * Creates a request to return the feature on the referenced (parent) layer
+     * which is referenced by the provided feature.
+     *
+     * @param attributes An attribute vector containing the foreign key
+     *
+     * @return A request the referenced feature
+     * @note not available in python bindings
      */
-    const QString& id() const;
+    QgsFeatureRequest getReferencedFeatureRequest( const QgsAttributes& attributes ) const;
+
+    /**
+     * Creates a request to return the feature on the referenced (parent) layer
+     * which is referenced by the provided feature.
+     *
+     * @param feature A feature from the referencing (child) layer
+     *
+     * @return A request the referenced feature
+     */
+    QgsFeatureRequest getReferencedFeatureRequest( const QgsFeature& feature ) const;
+
+    /**
+     * Creates a request to return the feature on the referenced (parent) layer
+     * which is referenced by the provided feature.
+     *
+     * @param feature A feature from the referencing (child) layer
+     *
+     * @return A request the referenced feature
+     */
+    QgsFeature getReferencedFeature( const QgsFeature& feature ) const;
+
+    /**
+     * Returns a human readable name for this relation. Mostly used as title for the children.
+     *
+     * @see id()
+     *
+     * @return A name
+     */
+    QString name() const;
+
+    /**
+     * A (project-wide) unique id for this relation
+     *
+     * @return The id
+     */
+    QString id() const;
 
     /**
      * Access the referencing (child) layer's id
@@ -182,13 +239,29 @@ class CORE_EXPORT QgsRelation
 
     /**
      * Returns the field pairs which form this relation
-     * The first element of each pair are the field names fo the foreign key.
+     * The first element of each pair are the field names of the foreign key.
      * The second element of each pair are the field names of the matching primary key.
      *
      * @return The fields forming the relation
      */
     QList< FieldPair > fieldPairs() const;
 
+    /**
+     * Returns a list of attributes used to form the referenced fields
+     * (most likely primary key) on the referenced (parent) layer.
+     *
+     * @return A list of attributes
+     */
+    QgsAttributeList referencedFields() const;
+
+    /**
+     * Returns a list of attributes used to form the referencing fields
+     * (foreign key) on the referencing (child) layer.
+     *
+     * @return A list of attributes
+     */
+    QgsAttributeList referencingFields() const;
+
     /**
      * Returns the validity of this relation. Don't use the information if it's not valid.
      *
@@ -197,8 +270,11 @@ class CORE_EXPORT QgsRelation
     bool isValid() const;
 
   protected:
+    /**
+     * Updates the validity status of this relation.
+     * Will be called internally whenever a member is changed.
+     */
     void updateRelationStatus();
-    void runChecks();
 
   private:
     /** Unique Id */
