@@ -16,6 +16,7 @@
 #include "qgsstringutils.h"
 #include <QVector>
 #include <QRegExp>
+#include <QTextDocument> // for Qt::escape
 #include <QStringList>
 #include <QTextBoundaryFinder>
 
@@ -344,3 +345,130 @@ QString QgsStringUtils::soundex( const QString& string )
 
   return tmp;
 }
+
+QString QgsStringUtils::insertLinks( const QString& string, bool *foundLinks )
+{
+  QString converted = string;
+
+  // http://alanstorm.com/url_regex_explained
+  // note - there's more robust implementations available, but we need one which works within the limitation of QRegExp
+  static QRegExp urlRegEx( "(\\b(([\\w-]+://?|www[.])[^\\s()<>]+(?:\\([\\w\\d]+\\)|([^!\"#$%&'()*+,\\-./:;<=>?@[\\\\\\]^_`{|}~\\s]|/))))" );
+  static QRegExp protoRegEx( "^(?:f|ht)tps?://" );
+  static QRegExp emailRegEx( "([\\w._%+-]+@[\\w.-]+\\.[A-Za-z]+)" );
+
+  int offset = 0;
+  bool found = false;
+  while ( urlRegEx.indexIn( converted, offset ) != -1 )
+  {
+    found = true;
+    QString url = urlRegEx.cap( 1 );
+    QString protoUrl = url;
+    if ( protoRegEx.indexIn( protoUrl ) == -1 )
+    {
+      protoUrl.prepend( "http://" );
+    }
+    QString anchor = QString( "<a href=\"%1\">%2</a>" ).arg( Qt::escape( protoUrl ) ).arg( Qt::escape( url ) );
+    converted.replace( urlRegEx.pos( 1 ), url.length(), anchor );
+    offset = urlRegEx.pos( 1 ) + anchor.length();
+  }
+  offset = 0;
+  while ( emailRegEx.indexIn( converted, offset ) != -1 )
+  {
+    found = true;
+    QString email = emailRegEx.cap( 1 );
+    QString anchor = QString( "<a href=\"mailto:%1\">%1</a>" ).arg( Qt::escape( email ) ).arg( Qt::escape( email ) );
+    converted.replace( emailRegEx.pos( 1 ), email.length(), anchor );
+    offset = emailRegEx.pos( 1 ) + anchor.length();
+  }
+
+  if ( foundLinks )
+    *foundLinks = found;
+
+  return converted;
+}
+
+QgsStringReplacement::QgsStringReplacement( const QString& match, const QString& replacement, bool caseSensitive, bool wholeWordOnly )
+    : mMatch( match )
+    , mReplacement( replacement )
+    , mCaseSensitive( caseSensitive )
+    , mWholeWordOnly( wholeWordOnly )
+{
+  if ( mWholeWordOnly )
+    mRx = QRegExp( QString( "\\b%1\\b" ).arg( mMatch ),
+                   mCaseSensitive ? Qt::CaseSensitive : Qt::CaseInsensitive );
+}
+
+QString QgsStringReplacement::process( const QString& input ) const
+{
+  QString result = input;
+  if ( !mWholeWordOnly )
+  {
+    return result.replace( mMatch, mReplacement, mCaseSensitive ? Qt::CaseSensitive : Qt::CaseInsensitive );
+  }
+  else
+  {
+    return result.replace( mRx, mReplacement );
+  }
+}
+
+QgsStringMap QgsStringReplacement::properties() const
+{
+  QgsStringMap map;
+  map.insert( "match", mMatch );
+  map.insert( "replace", mReplacement );
+  map.insert( "caseSensitive", mCaseSensitive ? "1" : "0" );
+  map.insert( "wholeWord", mWholeWordOnly ? "1" : "0" );
+  return map;
+}
+
+QgsStringReplacement QgsStringReplacement::fromProperties( const QgsStringMap& properties )
+{
+  return QgsStringReplacement( properties.value( "match" ),
+                               properties.value( "replace" ),
+                               properties.value( "caseSensitive", "0" ) == "1",
+                               properties.value( "wholeWord", "0" ) == "1" );
+}
+
+QString QgsStringReplacementCollection::process( const QString& input ) const
+{
+  QString result = input;
+  Q_FOREACH ( const QgsStringReplacement& r, mReplacements )
+  {
+    result = r.process( result );
+  }
+  return result;
+}
+
+void QgsStringReplacementCollection::writeXml( QDomElement& elem, QDomDocument& doc ) const
+{
+  Q_FOREACH ( const QgsStringReplacement& r, mReplacements )
+  {
+    QgsStringMap props = r.properties();
+    QDomElement propEl = doc.createElement( "replacement" );
+    QgsStringMap::const_iterator it = props.constBegin();
+    for ( ; it != props.constEnd(); ++it )
+    {
+      propEl.setAttribute( it.key(), it.value() );
+    }
+    elem.appendChild( propEl );
+  }
+}
+
+void QgsStringReplacementCollection::readXml( const QDomElement& elem )
+{
+  mReplacements.clear();
+  QDomNodeList nodelist = elem.elementsByTagName( "replacement" );
+  for ( int i = 0;i < nodelist.count(); i++ )
+  {
+    QDomElement replacementElem = nodelist.at( i ).toElement();
+    QDomNamedNodeMap nodeMap = replacementElem.attributes();
+
+    QgsStringMap props;
+    for ( int j = 0; j < nodeMap.count(); ++j )
+    {
+      props.insert( nodeMap.item( j ).nodeName(), nodeMap.item( j ).nodeValue() );
+    }
+    mReplacements << QgsStringReplacement::fromProperties( props );
+  }
+
+}
