@@ -16,6 +16,7 @@
 
 #include "qgsgeometry.h"
 #include "qgslogger.h"
+#include <qgsmessagelog.h>
 #include "qgsvectorlayerundocommand.h"
 #include "qgsvectordataprovider.h"
 #include "qgsvectorlayer.h"
@@ -65,12 +66,18 @@ void QgsVectorLayerEditBuffer::updateFields( QgsFields& fields )
   // delete attributes from the higher indices to lower indices
   for ( int i = mDeletedAttributeIds.count() - 1; i >= 0; --i )
   {
-    fields.remove( mDeletedAttributeIds[i] );
+    fields.remove( mDeletedAttributeIds.at( i ) );
   }
   // add new fields
   for ( int i = 0; i < mAddedAttributes.count(); ++i )
   {
-    fields.append( mAddedAttributes[i], QgsFields::OriginEdit, i );
+    fields.append( mAddedAttributes.at( i ), QgsFields::OriginEdit, i );
+  }
+  // rename fields
+  QgsFieldNameMap::const_iterator renameIt = mRenamedAttributes.constBegin();
+  for ( ; renameIt != mRenamedAttributes.constEnd(); ++renameIt )
+  {
+    fields[ renameIt.key()].setName( renameIt.value() );
   }
 }
 
@@ -106,25 +113,25 @@ void QgsVectorLayerEditBuffer::updateChangedAttributes( QgsFeature &f )
   f.setAttributes( attrs );
 }
 
-
-
-
 bool QgsVectorLayerEditBuffer::addFeature( QgsFeature& f )
 {
-  if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::AddFeatures ) )
+  QgsVectorDataProvider* provider = L->dataProvider();
+
+  if ( !( provider->capabilities() & QgsVectorDataProvider::AddFeatures ) )
   {
     return false;
   }
   if ( L->mUpdatedFields.count() != f.attributes().count() )
     return false;
 
-  // TODO: check correct geometry type
+  // if not then try to convert to a compatible geometry type
+  if ( !adaptGeometry( f.geometry() ) )
+    return false;
 
   L->undoStack()->push( new QgsVectorLayerUndoCommandAddFeature( this, f ) );
   return true;
 }
 
-
 bool QgsVectorLayerEditBuffer::addFeatures( QgsFeatureList& features )
 {
   if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::AddFeatures ) )
@@ -132,7 +139,8 @@ bool QgsVectorLayerEditBuffer::addFeatures( QgsFeatureList& features )
 
   for ( QgsFeatureList::iterator iter = features.begin(); iter != features.end(); ++iter )
   {
-    addFeature( *iter );
+    if ( !addFeature( *iter ) )
+      return false;
   }
 
   L->updateExtents();
@@ -172,6 +180,32 @@ bool QgsVectorLayerEditBuffer::deleteFeatures( const QgsFeatureIds& fids )
   return true;
 }
 
+bool QgsVectorLayerEditBuffer::adaptGeometry( QgsGeometry* geometry )
+{
+  /* this routine was introduced to fi the following issue:
+   * https://issues.qgis.org/issues/15741
+   * and later used in changeGeometry and addFeature to
+   * fix also the following issue
+   * https://issues.qgis.org/issues/16948
+   * https://issues.qgis.org/issues/15741
+   */
+  QgsVectorDataProvider* provider = L->dataProvider();
+  if ( geometry && geometry->geometry() &&
+       !geometry->isEmpty() &&
+       geometry->wkbType() != provider->geometryType() )
+  {
+    // check if provider do strict geometry control
+    // otherwise leave to the commit to rise provider errors
+    if ( provider->doesStrictFeatureTypeCheck() )
+    {
+      QgsGeometry* newGeom = provider->convertToProviderType( geometry );
+      if ( !newGeom )
+        return false;
+      geometry = newGeom;
+    }
+  }
+  return true;
+}
 
 bool QgsVectorLayerEditBuffer::changeGeometry( QgsFeatureId fid, QgsGeometry* geom )
 {
@@ -188,7 +222,11 @@ bool QgsVectorLayerEditBuffer::changeGeometry( QgsFeatureId fid, QgsGeometry* ge
   else if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::ChangeGeometries ) )
     return false;
 
-  // TODO: check compatible geometry
+  // if not then try to convert to a compatible geometry type
+  if ( !adaptGeometry( geom ) )
+  {
+    return false;
+  }
 
   L->undoStack()->push( new QgsVectorLayerUndoCommandChangeGeometry( this, fid, geom ) );
   return true;
@@ -225,10 +263,9 @@ bool QgsVectorLayerEditBuffer::addAttribute( const QgsField &field )
   if ( field.name().isEmpty() )
     return false;
 
-  const QgsFields& updatedFields = L->fields();
-  for ( int idx = 0; idx < updatedFields.count(); ++idx )
+  Q_FOREACH ( const QgsField& updatedField, L->fields() )
   {
-    if ( updatedFields[idx].name() == field.name() )
+    if ( updatedField.name() == field.name() )
       return false;
   }
 
@@ -262,6 +299,27 @@ bool QgsVectorLayerEditBuffer::deleteAttribute( int index )
   return true;
 }
 
+bool QgsVectorLayerEditBuffer::renameAttribute( int index, const QString& newName )
+{
+  if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::RenameAttributes ) )
+    return false;
+
+  if ( newName.isEmpty() )
+    return false;
+
+  if ( index < 0 || index >= L->fields().count() )
+    return false;
+
+  Q_FOREACH ( const QgsField& updatedField, L->fields() )
+  {
+    if ( updatedField.name() == newName )
+      return false;
+  }
+
+  L->undoStack()->push( new QgsVectorLayerUndoCommandRenameAttribute( this, index, newName ) );
+  return true;
+}
+
 
 bool QgsVectorLayerEditBuffer::commitChanges( QStringList& commitErrors )
 {
@@ -356,6 +414,25 @@ bool QgsVectorLayerEditBuffer::commitChanges( QStringList& commitErrors )
     }
   }
 
+  // rename attributes
+  if ( !mRenamedAttributes.isEmpty() )
+  {
+    if (( cap & QgsVectorDataProvider::RenameAttributes ) && provider->renameAttributes( mRenamedAttributes ) )
+    {
+      commitErrors << tr( "SUCCESS: %n attribute(s) renamed.", "renamed attributes count", mRenamedAttributes.size() );
+
+      emit committedAttributesRenamed( L->id(), mRenamedAttributes );
+
+      mRenamedAttributes.clear();
+      attributesChanged = true;
+    }
+    else
+    {
+      commitErrors << tr( "ERROR: %n attribute(s) not renamed", "not renamed attributes count", mRenamedAttributes.size() );
+      success = false;
+    }
+  }
+
   //
   // check that addition/removal went as expected
   //
@@ -621,6 +698,20 @@ void QgsVectorLayerEditBuffer::handleAttributeAdded( int index )
     attrs.insert( index, QVariant() );
     featureIt->setAttributes( attrs );
   }
+
+  // go through renamed attributes and adapt
+  QList< int > sortedRenamedIndices = mRenamedAttributes.keys();
+  //sort keys
+  qSort( sortedRenamedIndices.begin(), sortedRenamedIndices.end(), qGreater< int >() );
+  Q_FOREACH ( int renameIndex, sortedRenamedIndices )
+  {
+    if ( renameIndex >= index )
+    {
+      mRenamedAttributes[ renameIndex + 1 ] = mRenamedAttributes.value( renameIndex );
+    }
+  }
+  //remove last
+  mRenamedAttributes.remove( index );
 }
 
 void QgsVectorLayerEditBuffer::handleAttributeDeleted( int index )
@@ -646,6 +737,24 @@ void QgsVectorLayerEditBuffer::handleAttributeDeleted( int index )
     attrs.remove( index );
     featureIt->setAttributes( attrs );
   }
+
+  // go through rename attributes and adapt
+  QList< int > sortedRenamedIndices = mRenamedAttributes.keys();
+  //sort keys
+  qSort( sortedRenamedIndices.begin(), sortedRenamedIndices.end() );
+  int last = -1;
+  mRenamedAttributes.remove( index );
+  Q_FOREACH ( int renameIndex, sortedRenamedIndices )
+  {
+    if ( renameIndex > index )
+    {
+      mRenamedAttributes.insert( renameIndex - 1, mRenamedAttributes.value( renameIndex ) );
+      last = renameIndex;
+    }
+  }
+  //remove last
+  if ( last > -1 )
+    mRenamedAttributes.remove( last );
 }
 
 
