@@ -19,6 +19,9 @@
 
 class QgsVectorLayer;
 
+/** \ingroup core
+ * \class QgsVectorLayerEditPassthrough
+ */
 class CORE_EXPORT QgsVectorLayerEditPassthrough : public QgsVectorLayerEditBuffer
 {
     Q_OBJECT
@@ -33,6 +36,7 @@ class CORE_EXPORT QgsVectorLayerEditPassthrough : public QgsVectorLayerEditBuffe
     bool changeAttributeValue( QgsFeatureId fid, int field, const QVariant &newValue, const QVariant &oldValue = QVariant() ) override;
     bool addAttribute( const QgsField &field ) override;
     bool deleteAttribute( int attr ) override;
+    bool renameAttribute( int attr, const QString& newName ) override;
     bool commitChanges( QStringList& commitErrors ) override;
     void rollBack() override;
 
