@@ -19,6 +19,8 @@
 
 QgsVectorSimplifyMethod::QgsVectorSimplifyMethod()
     : mSimplifyHints( QGis::DEFAULT_MAPTOPIXEL_THRESHOLD > 1 ? QgsVectorSimplifyMethod::FullSimplification : QgsVectorSimplifyMethod::GeometrySimplification )
+    , mSimplifyAlgorithm( QgsVectorSimplifyMethod::Distance )
+    , mTolerance( 1 )
     , mThreshold( QGis::DEFAULT_MAPTOPIXEL_THRESHOLD )
     , mLocalOptimization( true )
     , mMaximumScale( 1 )
