@@ -29,7 +29,7 @@ class CORE_EXPORT QgsCubicRasterResampler: public QgsRasterResampler
   public:
     QgsCubicRasterResampler();
     ~QgsCubicRasterResampler();
-    QgsRasterResampler * clone() const override;
+    QgsCubicRasterResampler * clone() const override;
     void resample( const QImage& srcImage, QImage& dstImage ) override;
     QString type() const override { return "cubic"; }
 
@@ -42,7 +42,7 @@ class CORE_EXPORT QgsCubicRasterResampler: public QgsRasterResampler
                                  double* xDerivativeMatrixAlpha, double* yDerivativeMatrixRed, double* yDerivativeMatrixGreen, double* yDerivativeMatrixBlue,
                                  double* yDerivativeMatrixAlpha );
 
-    /**Use cubic curve interpoation at the borders of the raster*/
+    /** Use cubic curve interpoation at the borders of the raster*/
     QRgb curveInterpolation( QRgb pt1, QRgb pt2, double t, double d1red, double d1green, double d1blue, double d1alpha, double d2red, double d2green,
                              double d2blue, double d2alpha );
 
@@ -55,17 +55,73 @@ class CORE_EXPORT QgsCubicRasterResampler: public QgsRasterResampler
     //control points
 
     //red
-    double cRed00; double cRed10; double cRed20; double cRed30; double cRed01; double cRed11; double cRed21; double cRed31;
-    double cRed02; double cRed12; double cRed22; double cRed32; double cRed03; double cRed13; double cRed23; double cRed33;
+    double cRed00;
+    double cRed10;
+    double cRed20;
+    double cRed30;
+    double cRed01;
+    double cRed11;
+    double cRed21;
+    double cRed31;
+    double cRed02;
+    double cRed12;
+    double cRed22;
+    double cRed32;
+    double cRed03;
+    double cRed13;
+    double cRed23;
+    double cRed33;
     //green
-    double cGreen00; double cGreen10; double cGreen20; double cGreen30; double cGreen01; double cGreen11; double cGreen21; double cGreen31;
-    double cGreen02; double cGreen12; double cGreen22; double cGreen32; double cGreen03; double cGreen13; double cGreen23; double cGreen33;
+    double cGreen00;
+    double cGreen10;
+    double cGreen20;
+    double cGreen30;
+    double cGreen01;
+    double cGreen11;
+    double cGreen21;
+    double cGreen31;
+    double cGreen02;
+    double cGreen12;
+    double cGreen22;
+    double cGreen32;
+    double cGreen03;
+    double cGreen13;
+    double cGreen23;
+    double cGreen33;
     //blue
-    double cBlue00; double cBlue10; double cBlue20; double cBlue30; double cBlue01; double cBlue11; double cBlue21; double cBlue31;
-    double cBlue02; double cBlue12; double cBlue22; double cBlue32; double cBlue03; double cBlue13; double cBlue23; double cBlue33;
+    double cBlue00;
+    double cBlue10;
+    double cBlue20;
+    double cBlue30;
+    double cBlue01;
+    double cBlue11;
+    double cBlue21;
+    double cBlue31;
+    double cBlue02;
+    double cBlue12;
+    double cBlue22;
+    double cBlue32;
+    double cBlue03;
+    double cBlue13;
+    double cBlue23;
+    double cBlue33;
     //alpha
-    double cAlpha00; double cAlpha10; double cAlpha20; double cAlpha30; double cAlpha01; double cAlpha11; double cAlpha21; double cAlpha31;
-    double cAlpha02; double cAlpha12; double cAlpha22; double cAlpha32; double cAlpha03; double cAlpha13; double cAlpha23; double cAlpha33;
+    double cAlpha00;
+    double cAlpha10;
+    double cAlpha20;
+    double cAlpha30;
+    double cAlpha01;
+    double cAlpha11;
+    double cAlpha21;
+    double cAlpha31;
+    double cAlpha02;
+    double cAlpha12;
+    double cAlpha22;
+    double cAlpha32;
+    double cAlpha03;
+    double cAlpha13;
+    double cAlpha23;
+    double cAlpha33;
 
 
 };
