@@ -37,16 +37,17 @@ class CORE_EXPORT QgsSingleBandGrayRenderer: public QgsRasterRenderer
 
     QgsSingleBandGrayRenderer( QgsRasterInterface* input, int grayBand );
     ~QgsSingleBandGrayRenderer();
-    QgsRasterInterface * clone() const override;
+    QgsSingleBandGrayRenderer * clone() const override;
 
     static QgsRasterRenderer* create( const QDomElement& elem, QgsRasterInterface* input );
 
     QgsRasterBlock *block( int bandNo, QgsRectangle  const & extent, int width, int height ) override;
+    QgsRasterBlock *block2( int bandNo, QgsRectangle  const & extent, int width, int height, QgsRasterBlockFeedback* feedback = nullptr ) override;
 
     int grayBand() const { return mGrayBand; }
     void setGrayBand( int band ) { mGrayBand = band; }
     const QgsContrastEnhancement* contrastEnhancement() const { return mContrastEnhancement; }
-    /**Takes ownership*/
+    /** Takes ownership*/
     void setContrastEnhancement( QgsContrastEnhancement* ce );
 
     void setGradient( Gradient theGradient ) { mGradient = theGradient; }
@@ -62,6 +63,9 @@ class CORE_EXPORT QgsSingleBandGrayRenderer: public QgsRasterRenderer
     int mGrayBand;
     Gradient mGradient;
     QgsContrastEnhancement* mContrastEnhancement;
+
+    QgsSingleBandGrayRenderer( const QgsSingleBandGrayRenderer& );
+    const QgsSingleBandGrayRenderer& operator=( const QgsSingleBandGrayRenderer& );
 };
 
 #endif // QGSSINGLEBANDGRAYRENDERER_H
