@@ -445,12 +445,6 @@ void QgsCategorizedSymbolRendererV2::startRender( QgsRenderContext& context, con
       mTempSymbols[ cat.symbol()] = tempSymbol;
     }
   }
-
-  Q_FOREACH ( QgsSymbolV2 *symbol, mSymbolHash.values() )
-  {
-    symbol->startRender( context, &fields );
-  }
-
   return;
 }
 
@@ -461,11 +455,6 @@ void QgsCategorizedSymbolRendererV2::stopRender( QgsRenderContext& context )
     cat.symbol()->stopRender( context );
   }
 
-  Q_FOREACH ( QgsSymbolV2 *symbol, mSymbolHash.values() )
-  {
-    symbol->stopRender( context );
-  }
-
   // cleanup mTempSymbols
   QHash<QgsSymbolV2*, QgsSymbolV2*>::const_iterator it2 = mTempSymbols.constBegin();
   for ( ; it2 != mTempSymbols.constEnd(); ++it2 )
@@ -739,31 +728,34 @@ QDomElement QgsCategorizedSymbolRendererV2::save( QDomDocument& doc )
   rendererElem.setAttribute( "attr", mAttrName );
 
   // categories
-  int i = 0;
-  QgsSymbolV2Map symbols;
-  QDomElement catsElem = doc.createElement( "categories" );
-  QgsCategoryList::const_iterator it = mCategories.constBegin();
-  for ( ; it != mCategories.constEnd(); ++it )
+  if ( !mCategories.isEmpty() )
   {
-    const QgsRendererCategoryV2& cat = *it;
-    QString symbolName = QString::number( i );
-    symbols.insert( symbolName, cat.symbol() );
+    int i = 0;
+    QgsSymbolV2Map symbols;
+    QDomElement catsElem = doc.createElement( "categories" );
+    QgsCategoryList::const_iterator it = mCategories.constBegin();
+    for ( ; it != mCategories.constEnd(); ++it )
+    {
+      const QgsRendererCategoryV2& cat = *it;
+      QString symbolName = QString::number( i );
+      symbols.insert( symbolName, cat.symbol() );
+
+      QDomElement catElem = doc.createElement( "category" );
+      catElem.setAttribute( "value", cat.value().toString() );
+      catElem.setAttribute( "symbol", symbolName );
+      catElem.setAttribute( "label", cat.label() );
+      catElem.setAttribute( "render", cat.renderState() ? "true" : "false" );
+      catsElem.appendChild( catElem );
+      i++;
+    }
+    rendererElem.appendChild( catsElem );
+
+    // save symbols
+    QDomElement symbolsElem = QgsSymbolLayerV2Utils::saveSymbols( symbols, "symbols", doc );
+    rendererElem.appendChild( symbolsElem );
 
-    QDomElement catElem = doc.createElement( "category" );
-    catElem.setAttribute( "value", cat.value().toString() );
-    catElem.setAttribute( "symbol", symbolName );
-    catElem.setAttribute( "label", cat.label() );
-    catElem.setAttribute( "render", cat.renderState() ? "true" : "false" );
-    catsElem.appendChild( catElem );
-    i++;
   }
 
-  rendererElem.appendChild( catsElem );
-
-  // save symbols
-  QDomElement symbolsElem = QgsSymbolLayerV2Utils::saveSymbols( symbols, "symbols", doc );
-  rendererElem.appendChild( symbolsElem );
-
   // save source symbol
   if ( mSourceSymbol.data() )
   {
