@@ -15,12 +15,15 @@
 #ifndef QGSELLIPSESYMBOLLAYERV2_H
 #define QGSELLIPSESYMBOLLAYERV2_H
 
+#define DEFAULT_ELLIPSE_JOINSTYLE    Qt::MiterJoin
+
 #include "qgsmarkersymbollayerv2.h"
 #include <QPainterPath>
 
 class QgsExpression;
 
-/** A symbol layer for rendering objects with major and minor axis (e.g. ellipse, rectangle )*/
+/** \ingroup core
+ * A symbol layer for rendering objects with major and minor axis (e.g. ellipse, rectangle )*/
 class CORE_EXPORT QgsEllipseSymbolLayerV2: public QgsMarkerSymbolLayerV2
 {
   public:
@@ -55,6 +58,13 @@ class CORE_EXPORT QgsEllipseSymbolLayerV2: public QgsMarkerSymbolLayerV2
     Qt::PenStyle outlineStyle() const { return mOutlineStyle; }
     void setOutlineStyle( Qt::PenStyle outlineStyle ) { mOutlineStyle = outlineStyle; }
 
+    /** Get outline join style.
+     * @note added in 2.16 */
+    Qt::PenJoinStyle penJoinStyle() const { return mPenJoinStyle; }
+    /** Set outline join style.
+     * @note added in 2.16 */
+    void setPenJoinStyle( Qt::PenJoinStyle style ) { mPenJoinStyle = style; }
+
     void setOutlineWidth( double w ) { mOutlineWidth = w; }
     double outlineWidth() const { return mOutlineWidth; }
 
@@ -100,6 +110,7 @@ class CORE_EXPORT QgsEllipseSymbolLayerV2: public QgsMarkerSymbolLayerV2
     QgsMapUnitScale mSymbolHeightMapUnitScale;
     QColor mOutlineColor;
     Qt::PenStyle mOutlineStyle;
+    Qt::PenJoinStyle mPenJoinStyle;
     double mOutlineWidth;
     QgsSymbolV2::OutputUnit mOutlineWidthUnit;
     QgsMapUnitScale mOutlineWidthMapUnitScale;
