@@ -197,6 +197,20 @@ QSet<QString> QgsRuleBasedRendererV2::Rule::usedAttributes() const
   return attrs;
 }
 
+bool QgsRuleBasedRendererV2::Rule::needsGeometry() const
+{
+  if ( mFilter && mFilter->needsGeometry() )
+    return true;
+
+  Q_FOREACH ( Rule* rule, mChildren )
+  {
+    if ( rule->needsGeometry() )
+      return true;
+  }
+
+  return false;
+}
+
 QgsSymbolV2List QgsRuleBasedRendererV2::Rule::symbols( const QgsRenderContext& context ) const
 {
   QgsSymbolV2List lst;
@@ -793,7 +807,8 @@ QgsRuleBasedRendererV2::Rule* QgsRuleBasedRendererV2::Rule::createFromSld( QDomE
 /////////////////////
 
 QgsRuleBasedRendererV2::QgsRuleBasedRendererV2( QgsRuleBasedRendererV2::Rule* root )
-    : QgsFeatureRendererV2( "RuleRenderer" ), mRootRule( root )
+    : QgsFeatureRendererV2( "RuleRenderer" )
+    , mRootRule( root )
 {
 }
 
@@ -905,7 +920,12 @@ QString QgsRuleBasedRendererV2::filter( const QgsFields& )
 QList<QString> QgsRuleBasedRendererV2::usedAttributes()
 {
   QSet<QString> attrs = mRootRule->usedAttributes();
-  return attrs.values();
+  return attrs.toList();
+}
+
+bool QgsRuleBasedRendererV2::filterNeedsGeometry() const
+{
+  return mRootRule->needsGeometry();
 }
 
 QgsRuleBasedRendererV2* QgsRuleBasedRendererV2::clone() const
@@ -1191,12 +1211,12 @@ QSet< QString > QgsRuleBasedRendererV2::legendKeysForFeature( QgsFeature& featur
 
 QgsRuleBasedRendererV2* QgsRuleBasedRendererV2::convertFromRenderer( const QgsFeatureRendererV2* renderer )
 {
+  QgsRuleBasedRendererV2* r = nullptr;
   if ( renderer->type() == "RuleRenderer" )
   {
-    return dynamic_cast<QgsRuleBasedRendererV2*>( renderer->clone() );
+    r = dynamic_cast<QgsRuleBasedRendererV2*>( renderer->clone() );
   }
-
-  if ( renderer->type() == "singleSymbol" )
+  else if ( renderer->type() == "singleSymbol" )
   {
     const QgsSingleSymbolRendererV2* singleSymbolRenderer = dynamic_cast<const QgsSingleSymbolRendererV2*>( renderer );
     if ( !singleSymbolRenderer )
@@ -1204,10 +1224,9 @@ QgsRuleBasedRendererV2* QgsRuleBasedRendererV2::convertFromRenderer( const QgsFe
 
     QgsSymbolV2* origSymbol = singleSymbolRenderer->symbol()->clone();
     convertToDataDefinedSymbology( origSymbol, singleSymbolRenderer->sizeScaleField() );
-    return new QgsRuleBasedRendererV2( origSymbol );
+    r = new QgsRuleBasedRendererV2( origSymbol );
   }
-
-  if ( renderer->type() == "categorizedSymbol" )
+  else if ( renderer->type() == "categorizedSymbol" )
   {
     const QgsCategorizedSymbolRendererV2* categorizedRenderer = dynamic_cast<const QgsCategorizedSymbolRendererV2*>( renderer );
     if ( !categorizedRenderer )
@@ -1268,10 +1287,9 @@ QgsRuleBasedRendererV2* QgsRuleBasedRendererV2::convertFromRenderer( const QgsFe
       rootrule->appendChild( rule );
     }
 
-    return new QgsRuleBasedRendererV2( rootrule );
+    r = new QgsRuleBasedRendererV2( rootrule );
   }
-
-  if ( renderer->type() == "graduatedSymbol" )
+  else if ( renderer->type() == "graduatedSymbol" )
   {
     const QgsGraduatedSymbolRendererV2* graduatedRenderer = dynamic_cast<const QgsGraduatedSymbolRendererV2*>( renderer );
     if ( !graduatedRenderer )
@@ -1325,23 +1343,28 @@ QgsRuleBasedRendererV2* QgsRuleBasedRendererV2::convertFromRenderer( const QgsFe
       rootrule->appendChild( rule );
     }
 
-    return new QgsRuleBasedRendererV2( rootrule );
+    r = new QgsRuleBasedRendererV2( rootrule );
   }
-
-  if ( renderer->type() == "pointDisplacement" )
+  else if ( renderer->type() == "pointDisplacement" )
   {
     const QgsPointDisplacementRenderer* pointDisplacementRenderer = dynamic_cast<const QgsPointDisplacementRenderer*>( renderer );
     if ( pointDisplacementRenderer )
-      return convertFromRenderer( pointDisplacementRenderer->embeddedRenderer() );
+      r = convertFromRenderer( pointDisplacementRenderer->embeddedRenderer() );
   }
-  if ( renderer->type() == "invertedPolygonRenderer" )
+  else if ( renderer->type() == "invertedPolygonRenderer" )
   {
     const QgsInvertedPolygonRenderer* invertedPolygonRenderer = dynamic_cast<const QgsInvertedPolygonRenderer*>( renderer );
     if ( invertedPolygonRenderer )
-      return convertFromRenderer( invertedPolygonRenderer->embeddedRenderer() );
+      r = convertFromRenderer( invertedPolygonRenderer->embeddedRenderer() );
   }
 
-  return nullptr;
+  if ( r )
+  {
+    r->setOrderBy( renderer->orderBy() );
+    r->setOrderByEnabled( renderer->orderByEnabled() );
+  }
+
+  return r;
 }
 
 void QgsRuleBasedRendererV2::convertToDataDefinedSymbology( QgsSymbolV2* symbol, const QString& sizeScaleField, const QString& rotationField )
