@@ -27,7 +27,7 @@ class QgsExpression;
 class QgsCategorizedSymbolRendererV2;
 class QgsGraduatedSymbolRendererV2;
 
-/**
+/** \ingroup core
 When drawing a vector layer with rule-based renderer, it goes through
 the rules and draws features with symbols from rules that match.
  */
@@ -41,7 +41,10 @@ class CORE_EXPORT QgsRuleBasedRendererV2 : public QgsFeatureRendererV2
     // feature for rendering: QgsFeature and some flags
     struct FeatureToRender
     {
-      FeatureToRender( QgsFeature& _f, int _flags ) : feat( _f ), flags( _flags ) {}
+      FeatureToRender( QgsFeature& _f, int _flags )
+          : feat( _f )
+          , flags( _flags )
+      {}
       QgsFeature feat;
       int flags; // selected and/or draw markers
     };
@@ -50,7 +53,10 @@ class CORE_EXPORT QgsRuleBasedRendererV2 : public QgsFeatureRendererV2
     // (both f, symbol are _not_ owned by this class)
     struct RenderJob
     {
-      RenderJob( FeatureToRender& _ftr, QgsSymbolV2* _s ) : ftr( _ftr ), symbol( _s ) {}
+      RenderJob( FeatureToRender& _ftr, QgsSymbolV2* _s )
+          : ftr( _ftr )
+          , symbol( _s )
+      {}
       FeatureToRender& ftr;
       QgsSymbolV2* symbol;
     };
@@ -93,7 +99,7 @@ class CORE_EXPORT QgsRuleBasedRendererV2 : public QgsFeatureRendererV2
     class Rule;
     typedef QList<Rule*> RuleList;
 
-    /**
+    /** \ingroup core
       This class keeps data about a rules for rule-based renderer.
       A rule consists of a symbol, filter expression and range of scales.
       If filter is empty, it matches all features.
@@ -130,6 +136,11 @@ class CORE_EXPORT QgsRuleBasedRendererV2 : public QgsFeatureRendererV2
          */
         QSet<QString> usedAttributes() const;
 
+        /**
+         * Returns true if this rule or one of its chilren needs the geometry to be applied.
+         */
+        bool needsGeometry() const;
+
         //! @note available in python bindings as symbol2
         QgsSymbolV2List symbols( const QgsRenderContext& context = QgsRenderContext() ) const;
 
@@ -427,6 +438,8 @@ class CORE_EXPORT QgsRuleBasedRendererV2 : public QgsFeatureRendererV2
 
     virtual QList<QString> usedAttributes() override;
 
+    virtual bool filterNeedsGeometry() const override;
+
     virtual QgsRuleBasedRendererV2* clone() const override;
 
     //! Writes the SLD element following the SLD v1.1 specs
