@@ -16,17 +16,19 @@
 #include "qplugin.h"
 
 #include "qgiscustomwidgets.h"
-
 #include "qgscollapsiblegroupboxplugin.h"
 #include "qgscolorbuttonplugin.h"
 #include "qgscolorbuttonv2plugin.h"
 #include "qgsdatadefinedbuttonplugin.h"
 #include "qgsdatetimeeditplugin.h"
+#include "qgsdockwidgetplugin.h"
 #include "qgsdoublespinboxplugin.h"
 #include "qgsexpressionbuilderwidgetplugin.h"
 #include "qgsextentgroupboxplugin.h"
+#include "qgsexternalresourcewidgetplugin.h"
 #include "qgsfieldcomboboxplugin.h"
 #include "qgsfieldexpressionwidgetplugin.h"
+#include "qgsfilewidgetplugin.h"
 #include "qgsfilterlineeditplugin.h"
 #include "qgsmaplayercomboboxplugin.h"
 #include "qgsprojectionselectionwidgetplugin.h"
@@ -45,11 +47,14 @@ QgisCustomWidgets::QgisCustomWidgets( QObject *parent )
   mWidgets.append( new QgsColorButtonV2Plugin( this ) );
   mWidgets.append( new QgsDataDefinedButtonPlugin( this ) );
   mWidgets.append( new QgsDateTimeEditPlugin( this ) );
+  mWidgets.append( new QgsDockWidgetPlugin( this ) );
   mWidgets.append( new QgsDoubleSpinBoxPlugin( this ) );
   mWidgets.append( new QgsExpressionBuilderWidgetPlugin( this ) );
   mWidgets.append( new QgsExtentGroupBoxPlugin( this ) );
+  mWidgets.append( new QgsExternalResourceWidgetPlugin( this ) );
   mWidgets.append( new QgsFieldComboBoxPlugin( this ) );
   mWidgets.append( new QgsFieldExpressionWidgetPlugin( this ) );
+  mWidgets.append( new QgsFileWidgetPlugin( this ) );
   mWidgets.append( new QgsFilterLineEditPlugin( this ) );
   mWidgets.append( new QgsMapLayerComboBoxPlugin( this ) );
   mWidgets.append( new QgsProjectionSelectionWidgetPlugin( this ) );
@@ -68,5 +73,3 @@ QList<QDesignerCustomWidgetInterface*> QgisCustomWidgets::customWidgets() const
 #if QT_VERSION < 0x050000
 Q_EXPORT_PLUGIN2( customwidgetsplugin, QgisCustomWidgets )
 #endif
-
-
