@@ -42,7 +42,7 @@ QString QgsCollapsibleGroupBoxPlugin::includeFile() const
 
 QIcon QgsCollapsibleGroupBoxPlugin::icon() const
 {
-  return QIcon();
+  return QIcon( ":/images/icons/qgis-icon-60x60.png" );
 }
 
 bool QgsCollapsibleGroupBoxPlugin::isContainer() const
