@@ -28,22 +28,47 @@ class QgsVectorLayerCache;
 class QgsMapCanvas;
 class QItemSelectionModel;
 
+/** \ingroup gui
+ * \class QgsAttributeTableFilterModel
+ */
 class GUI_EXPORT QgsAttributeTableFilterModel: public QSortFilterProxyModel, public QgsFeatureModel
 {
     Q_OBJECT
 
   public:
+    /**
+     * The filter mode defines how the rows should be filtered.
+     */
     enum FilterMode
     {
-      ShowAll,
-      ShowSelected,
-      ShowVisible,
-      ShowFilteredList,
-      ShowEdited
+      ShowAll,          //!< Show all features
+      ShowSelected,     //!< Show only selected features
+      ShowVisible,      //!< Show only visible features (depends on the map canvas)
+      ShowFilteredList, //!< Show only features whose ids are on the filter list. {@see setFilteredFeatures}
+      ShowEdited        //!< Show only features which have unsaved changes
     };
 
     /**
-     *
+     * The type of a column.
+     */
+    enum ColumnType
+    {
+      ColumnTypeField,       //!< This column shows a field
+      ColumnTypeActionButton //!< This column shows action buttons
+    };
+
+    /**
+     * The additional roles defined by this filter model.
+     * The values of these roles start just after the roles defined by
+     * QgsAttributeTableModel so they do not conflict.
+     */
+    enum Role
+    {
+      TypeRole = QgsAttributeTableModel::UserRole //!< The type of a given column
+    };
+
+
+    /**
      * Make sure, the master model is already loaded, so the selection will get synchronized.
      *
      * @param parent parent object (owner)
@@ -100,8 +125,6 @@ class GUI_EXPORT QgsAttributeTableFilterModel: public QSortFilterProxyModel, pub
 
     /**
      * The current filterModel
-     *
-     * @return Mode
      */
     FilterMode filterMode() { return mFilterMode; }
 
@@ -136,11 +159,18 @@ class GUI_EXPORT QgsAttributeTableFilterModel: public QSortFilterProxyModel, pub
     QgsFeatureId rowToId( const QModelIndex& row );
 
     QModelIndex fidToIndex( QgsFeatureId fid ) override;
+
     QModelIndexList fidToIndexList( QgsFeatureId fid );
 
-    virtual QModelIndex mapToMaster( const QModelIndex &proxyIndex ) const;
+    inline QModelIndex mapToMaster( const QModelIndex& proxyIndex ) const { return mapToSource( proxyIndex ); }
 
-    virtual QModelIndex mapFromMaster( const QModelIndex &sourceIndex ) const;
+    inline QModelIndex mapFromMaster( const QModelIndex& sourceIndex ) const { return mapFromSource( sourceIndex ); }
+
+    virtual QModelIndex mapToSource( const QModelIndex& proxyIndex ) const override;
+
+    virtual QModelIndex mapFromSource( const QModelIndex& sourceIndex ) const override;
+
+    virtual Qt::ItemFlags flags( const QModelIndex &index ) const override;
 
     /**
      * Sort by the given column using the given order.
@@ -151,9 +181,50 @@ class GUI_EXPORT QgsAttributeTableFilterModel: public QSortFilterProxyModel, pub
      */
     virtual void sort( int column, Qt::SortOrder order = Qt::AscendingOrder ) override;
 
+    /**
+     * Sort by the given expression using the given order.
+     * Prefetches all the data from the layer to speed up sorting.
+     *
+     * @param expression The expression which should be used for sorting
+     * @param order      The order ( Qt::AscendingOrder or Qt::DescendingOrder )
+     */
+    void sort( QString expression, Qt::SortOrder order = Qt::AscendingOrder );
+
+    /**
+     * The expression which is used to sort the attribute table.
+     */
+    QString sortExpression() const;
+
     /** Returns the map canvas*/
     QgsMapCanvas* mapCanvas() const { return mCanvas; }
 
+    virtual QVariant data( const QModelIndex& index, int role ) const override;
+
+    QVariant headerData( int section, Qt::Orientation orientation, int role ) const override;
+
+    /**
+     * Get the index of the first column that contains an action widget.
+     * Returns -1 if none is defined.
+     */
+    int actionColumnIndex() const;
+
+    int columnCount( const QModelIndex &parent ) const override;
+
+    /**
+     * Set the attribute table configuration to control which fields are shown,
+     * in which order they are shown as well as if and where an action column
+     * is shown.
+     */
+    void setAttributeTableConfig( const QgsAttributeTableConfig& config );
+
+  signals:
+    /**
+     * Is emitted whenever the sort column is changed
+     * @param column The sort column
+     * @param order The sort order
+     */
+    void sortColumnChanged( int column, Qt::SortOrder order );
+
   protected:
     /**
      * Returns true if the source row will be accepted
@@ -179,12 +250,12 @@ class GUI_EXPORT QgsAttributeTableFilterModel: public QSortFilterProxyModel, pub
     /**
      * Is called upon every change of the visible extents on the map canvas.
      * When a change is signalled, the filter is updated and invalidated if needed.
-     *
      */
     void extentsChanged();
 
   private slots:
     void selectionChanged();
+    void onColumnsChanged();
 
   private:
     QgsFeatureIds mFilteredFeatures;
@@ -192,6 +263,13 @@ class GUI_EXPORT QgsAttributeTableFilterModel: public QSortFilterProxyModel, pub
     FilterMode mFilterMode;
     bool mSelectedOnTop;
     QgsAttributeTableModel* mTableModel;
+
+    QgsAttributeTableConfig mConfig;
+    QVector<int> mColumnMapping;
+    int mapColumnToSource( int column ) const;
+
+    friend class TestQgsAttributeTable;
+
 };
 
 #endif
