@@ -1,3 +1,17 @@
+/***************************************************************************
+    qgsfeaturelistmodel.cpp
+    ---------------------
+    begin                : February 2013
+    copyright            : (C) 2013 by Matthias Kuhn
+    email                : matthias at opengis dot ch
+ ***************************************************************************
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ ***************************************************************************/
 #include "qgsexception.h"
 #include "qgsvectordataprovider.h"
 #include "qgsfeaturelistmodel.h"
@@ -63,7 +77,7 @@ QVariant QgsFeatureListModel::data( const QModelIndex &index, int role ) const
     }
     else
     {
-      return QVariant( QVariant::Int );
+      return QVariant( QVariant::Invalid );
     }
   }
 
@@ -116,13 +130,24 @@ QVariant QgsFeatureListModel::data( const QModelIndex &index, int role ) const
 
     return QVariant::fromValue( feat );
   }
+  else if ( role == Qt::TextAlignmentRole )
+  {
+    return Qt::AlignLeft;
+  }
 
   return sourceModel()->data( mapToSource( index ), role );
 }
 
 Qt::ItemFlags QgsFeatureListModel::flags( const QModelIndex &index ) const
 {
-  return sourceModel()->flags( mapToSource( index ) ) & ~Qt::ItemIsEditable;
+  if ( mInjectNull && index.row() == 0 )
+  {
+    return Qt::ItemIsSelectable | Qt::ItemIsEnabled;
+  }
+  else
+  {
+    return sourceModel()->flags( mapToSource( index ) ) & ~Qt::ItemIsEditable;
+  }
 }
 
 void QgsFeatureListModel::setInjectNull( bool injectNull )
