@@ -3,7 +3,7 @@
      --------------------------------------
     Date                 : Jan 2012
     Copyright            : (C) 2013 Matthias Kuhn
-    Email                : matthias dot kuhn at gmx dot ch
+    Email                : matthias at opengis dot ch
  ***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -13,8 +13,8 @@
  *                                                                         *
  ***************************************************************************/
 
-#ifndef QGSATTRIBUTELISTVIEW_H
-#define QGSATTRIBUTELISTVIEW_H
+#ifndef QGSFEATURELISTVIEW_H
+#define QGSFEATURELISTVIEW_H
 
 #include <QListView>
 #include <qdebug.h>
@@ -25,12 +25,13 @@ class QgsAttributeTableFilterModel;
 class QgsFeatureListModel;
 class QgsFeatureSelectionModel;
 class QgsAttributeTableModel;
+class QgsIFeatureSelectionManager;
 class QgsVectorLayer;
 class QgsVectorLayerCache;
 class QgsFeatureListViewDelegate;
 class QRect;
 
-/**
+/** \ingroup gui
  * Shows a list of features and renders a edit button next to each feature.
  *
  * @brief
@@ -48,7 +49,7 @@ class GUI_EXPORT QgsFeatureListView : public QListView
      *
      * @param parent   owner
      */
-    explicit QgsFeatureListView( QWidget* parent = 0 );
+    explicit QgsFeatureListView( QWidget* parent = nullptr );
 
     /**
      * Destructor
@@ -82,7 +83,7 @@ class GUI_EXPORT QgsFeatureListView : public QListView
      *
      * @see QgsExpression
      */
-    bool setDisplayExpression( const QString displayExpression );
+    bool setDisplayExpression( const QString& displayExpression );
 
     /**
      * Returns the expression which is currently used to render the features.
@@ -114,6 +115,11 @@ class GUI_EXPORT QgsFeatureListView : public QListView
      */
     void setCurrentFeatureEdited( bool state );
 
+    /**
+     * @brief setFeatureSelectionManager
+     * @param featureSelectionManager We will take ownership
+     */
+    void setFeatureSelectionManager( QgsIFeatureSelectionManager* featureSelectionManager );
   protected:
     virtual void mouseMoveEvent( QMouseEvent *event ) override;
     virtual void mousePressEvent( QMouseEvent *event ) override;
@@ -133,8 +139,9 @@ class GUI_EXPORT QgsFeatureListView : public QListView
      * Is emitted, whenever the display expression is successfully changed
      * @param expression The expression that was applied
      */
-    void displayExpressionChanged( const QString expression );
+    void displayExpressionChanged( const QString& expression );
 
+    //! @note not available in Python bindings
     void aboutToChangeEditSelection( bool& ok );
 
   public slots:
@@ -151,18 +158,18 @@ class GUI_EXPORT QgsFeatureListView : public QListView
      * @param index The selection to set
      * @param command selection update mode
      */
-    void setEditSelection( const QModelIndex& index, QItemSelectionModel::SelectionFlags command );
+    void setEditSelection( const QModelIndex& index, const QItemSelectionModel::SelectionFlags& command );
 
     /**
      * Select all currently visible features
      */
     virtual void selectAll() override;
 
-    void repaintRequested( QModelIndexList indexes );
+    void repaintRequested( const QModelIndexList& indexes );
     void repaintRequested();
 
   private slots:
-    void editSelectionChanged( QItemSelection deselected, QItemSelection selected );
+    void editSelectionChanged( const QItemSelection& deselected, const QItemSelection& selected );
 
   private:
     void selectRow( const QModelIndex &index, bool anchor );
@@ -170,6 +177,7 @@ class GUI_EXPORT QgsFeatureListView : public QListView
     QgsFeatureListModel *mModel;
     QItemSelectionModel* mCurrentEditSelectionModel;
     QgsFeatureSelectionModel* mFeatureSelectionModel;
+    QgsIFeatureSelectionManager* mFeatureSelectionManager;
     QgsFeatureListViewDelegate* mItemDelegate;
     bool mEditSelectionDrag; // Is set to true when the user initiated a left button click over an edit button and still keeps pressing /**< TODO */
     int mRowAnchor;
