@@ -3,7 +3,7 @@
      --------------------------------------
     Date                 : 6.6.2013
     Copyright            : (C) 2013 Matthias Kuhn
-    Email                : matthias dot kuhn at gmx dot ch
+    Email                : matthias at opengis dot ch
  ***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -20,12 +20,16 @@
 
 class QgsVectorLayer;
 
+/** \ingroup gui
+ * \class QgsVectorLayerSelectionManager
+ * \note not available in Python bindings
+ */
 class GUI_EXPORT QgsVectorLayerSelectionManager : public QgsIFeatureSelectionManager
 {
     Q_OBJECT
 
   public:
-    explicit QgsVectorLayerSelectionManager( QgsVectorLayer* layer, QObject *parent = 0 );
+    explicit QgsVectorLayerSelectionManager( QgsVectorLayer* layer, QObject *parent = nullptr );
     /**
      * The number of features that are selected in this layer
      *
@@ -49,7 +53,7 @@ class GUI_EXPORT QgsVectorLayerSelectionManager : public QgsIFeatureSelectionMan
 
     /**
      * Change selection to the new set of features. Dismisses the current selection.
-     * Will emit the { @link selectionChanged( QgsFeatureIds, QgsFeatureIds, bool ) } signal with the
+     * Will emit the { @link selectionChanged( const QgsFeatureIds&, const QgsFeatureIds&, bool ) } signal with the
      * clearAndSelect flag set.
      *
      * @param ids   The ids which will be the new selection
