@@ -14,12 +14,12 @@
  ***************************************************************************/
 
 #include "qgscheckboxwidgetfactory.h"
-
+#include "qgscheckboxsearchwidgetwrapper.h"
 #include "qgscheckboxwidgetwrapper.h"
 #include "qgscheckboxconfigdlg.h"
 
-QgsCheckboxWidgetFactory::QgsCheckboxWidgetFactory( const QString& name ) :
-    QgsEditorWidgetFactory( name )
+QgsCheckboxWidgetFactory::QgsCheckboxWidgetFactory( const QString& name )
+    : QgsEditorWidgetFactory( name )
 {
 }
 
@@ -28,6 +28,11 @@ QgsEditorWidgetWrapper* QgsCheckboxWidgetFactory::create( QgsVectorLayer* vl, in
   return new QgsCheckboxWidgetWrapper( vl, fieldIdx, editor, parent );
 }
 
+QgsSearchWidgetWrapper*QgsCheckboxWidgetFactory::createSearchWidget( QgsVectorLayer* vl, int fieldIdx, QWidget* parent ) const
+{
+  return new QgsCheckboxSearchWidgetWrapper( vl, fieldIdx, parent );
+}
+
 QgsEditorConfigWidget* QgsCheckboxWidgetFactory::configWidget( QgsVectorLayer* vl, int fieldIdx, QWidget* parent ) const
 {
   return new QgsCheckBoxConfigDlg( vl, fieldIdx, parent );
