@@ -14,6 +14,7 @@
  ***************************************************************************/
 
 #include "qgscolorwidgetwrapper.h"
+#include <QLayout>
 
 QgsColorWidgetWrapper::QgsColorWidgetWrapper( QgsVectorLayer* vl, int fieldIdx, QWidget* editor, QWidget* parent )
     : QgsEditorWidgetWrapper( vl, fieldIdx, editor, parent )
@@ -31,17 +32,38 @@ QVariant QgsColorWidgetWrapper::value() const
   return c.isValid() ? QVariant( c ) : QVariant( QVariant::Color );
 }
 
+void QgsColorWidgetWrapper::showIndeterminateState()
+{
+  if ( mColorButton )
+  {
+    whileBlocking( mColorButton )->setColor( QColor() );
+  }
+}
+
 QWidget* QgsColorWidgetWrapper::createWidget( QWidget* parent )
 {
-  QgsColorButtonV2* button = new QgsColorButtonV2( parent );
+  QWidget* container = new QWidget( parent );
+  QHBoxLayout* layout = new QHBoxLayout();
+  container->setLayout( layout );
+  layout->setMargin( 0 );
+  layout->setContentsMargins( 0, 0, 0, 0 );
+  QgsColorButtonV2* button = new QgsColorButtonV2();
   button->setContext( QString( "editor" ) );
-  return button;
+  layout->addWidget( button );
+  layout->addStretch();
+  container->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Minimum );
+  return container;
 }
 
 void QgsColorWidgetWrapper::initWidget( QWidget* editor )
 {
   mColorButton = qobject_cast<QgsColorButtonV2*>( editor );
+  if ( !mColorButton )
+  {
+    mColorButton = editor->findChild<QgsColorButtonV2*>();
+  }
 
+  mColorButton->setShowNull( true );
   connect( mColorButton, SIGNAL( colorChanged( QColor ) ), this, SLOT( valueChanged() ) );
 }
 
@@ -55,3 +77,8 @@ void QgsColorWidgetWrapper::setValue( const QVariant& value )
   if ( mColorButton )
     mColorButton->setColor( !value.isNull() ? QColor( value.toString() ) : QColor() );
 }
+
+void QgsColorWidgetWrapper::updateConstraintWidgetStatus( bool /*constraintValid*/ )
+{
+  // nothing
+}
