@@ -19,16 +19,21 @@
 #include "qgseditorconfigwidget.h"
 #include "ui_qgsdatetimeeditconfig.h"
 
+/** \ingroup gui
+ * \class QgsDateTimeEditConfig
+ * \note not available in Python bindings
+ */
+
 class GUI_EXPORT QgsDateTimeEditConfig : public QgsEditorConfigWidget, private Ui::QgsDateTimeEditConfig
 {
     Q_OBJECT
   public:
-    QgsDateTimeEditConfig( QgsVectorLayer* vl, int fieldIdx, QWidget* parent = 0 );
+    QgsDateTimeEditConfig( QgsVectorLayer* vl, int fieldIdx, QWidget* parent = nullptr );
 
   private slots:
     void updateDemoWidget();
     void updateFieldFormat( int idx );
-    void updateDisplayFormat( QString fieldFormat );
+    void updateDisplayFormat( const QString& fieldFormat );
     void displayFormatChanged( int idx );
     void showHelp( bool buttonChecked );
 
