@@ -16,11 +16,12 @@
 #include "qgsdatetimeeditfactory.h"
 #include "qgsdatetimeeditconfig.h"
 #include "qgsdatetimeeditwrapper.h"
+#include "qgsdatetimesearchwidgetwrapper.h"
 
 #include <QSettings>
 
-QgsDateTimeEditFactory::QgsDateTimeEditFactory( const QString& name ) :
-    QgsEditorWidgetFactory( name )
+QgsDateTimeEditFactory::QgsDateTimeEditFactory( const QString& name )
+    : QgsEditorWidgetFactory( name )
 {
 }
 
@@ -29,6 +30,11 @@ QgsEditorWidgetWrapper *QgsDateTimeEditFactory::create( QgsVectorLayer *vl, int
   return new QgsDateTimeEditWrapper( vl, fieldIdx, editor, parent );
 }
 
+QgsSearchWidgetWrapper* QgsDateTimeEditFactory::createSearchWidget( QgsVectorLayer* vl, int fieldIdx, QWidget* parent ) const
+{
+  return new QgsDateTimeSearchWidgetWrapper( vl, fieldIdx, parent );
+}
+
 QgsEditorConfigWidget *QgsDateTimeEditFactory::configWidget( QgsVectorLayer *vl, int fieldIdx, QWidget *parent ) const
 {
   return new QgsDateTimeEditConfig( vl, fieldIdx, parent );
@@ -91,6 +97,15 @@ QString QgsDateTimeEditFactory::representValue( QgsVectorLayer* vl, int fieldIdx
   return result;
 }
 
+Qt::AlignmentFlag QgsDateTimeEditFactory::alignmentFlag( QgsVectorLayer* vl, int fieldIdx, const QgsEditorWidgetConfig& config ) const
+{
+  Q_UNUSED( vl );
+  Q_UNUSED( fieldIdx );
+  Q_UNUSED( config );
+
+  return Qt::AlignLeft;
+}
+
 QMap<const char*, int> QgsDateTimeEditFactory::supportedWidgetTypes()
 {
   QMap<const char*, int> map = QMap<const char*, int>();
