@@ -3,7 +3,7 @@
      --------------------------------------
     Date                 : 5.1.2014
     Copyright            : (C) 2014 Matthias Kuhn
-    Email                : matthias dot kuhn at gmx dot ch
+    Email                : matthias at opengis dot ch
  ***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -30,5 +30,5 @@ QgsEditorWidgetWrapper* QgsHiddenWidgetFactory::create( QgsVectorLayer* vl, int
 
 QgsEditorConfigWidget* QgsHiddenWidgetFactory::configWidget( QgsVectorLayer* vl, int fieldIdx, QWidget* parent ) const
 {
-  return new QgsDummyConfigDlg( vl, fieldIdx, parent, QObject::tr( "A hidden attribute will be invisible - the user is not able to see it's contents." ) );
+  return new QgsDummyConfigDlg( vl, fieldIdx, parent, QObject::tr( "A hidden field will be invisible - the user is not able to see its contents." ) );
 }
