@@ -3,7 +3,7 @@
      --------------------------------------
     Date                 : 5.1.2014
     Copyright            : (C) 2014 Matthias Kuhn
-    Email                : matthias dot kuhn at gmx dot ch
+    Email                : matthias at opengis dot ch
  ***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -41,8 +41,8 @@ QgsEditorWidgetConfig QgsPhotoWidgetFactory::readConfig( const QDomElement& conf
 
   QgsEditorWidgetConfig cfg;
 
-  cfg.insert( "Height", configElement.attribute( "Height", 0 ).toInt() );
-  cfg.insert( "Width", configElement.attribute( "Width", 0 ).toInt() );
+  cfg.insert( "Height", configElement.attribute( "Height", nullptr ).toInt() );
+  cfg.insert( "Width", configElement.attribute( "Width", nullptr ).toInt() );
 
   return cfg;
 }
