@@ -21,6 +21,7 @@
 #include "qgspainteffectwidget.h"
 #include "qgsapplication.h"
 #include "qgssymbollayerv2utils.h"
+#include "qgspanelwidget.h"
 
 #include <QPicture>
 #include <QPainter>
@@ -93,7 +94,7 @@ class EffectItem : public QStandardItem
 //
 
 QgsEffectStackPropertiesWidget::QgsEffectStackPropertiesWidget( QgsEffectStack *stack, QWidget *parent )
-    : QWidget( parent )
+    : QgsPanelWidget( parent )
     , mStack( stack )
     , mPreviewPicture( nullptr )
 {
@@ -142,6 +143,8 @@ QgsEffectStackPropertiesWidget::QgsEffectStackPropertiesWidget( QgsEffectStack *
   }
   QModelIndex newIndex = mEffectsList->model()->index( initialRow, 0 );
   mEffectsList->setCurrentIndex( newIndex );
+
+  setPanelTitle( tr( "Effects Properties" ) );
 }
 
 QgsEffectStackPropertiesWidget::~QgsEffectStackPropertiesWidget()
@@ -229,6 +232,7 @@ void QgsEffectStackPropertiesWidget::updatePreview()
   painter.end();
 
   lblPreview->setPixmap( QPixmap::fromImage( previewImage ) );
+  emit widgetChanged();
 }
 
 EffectItem* QgsEffectStackPropertiesWidget::currentEffectItem()
@@ -387,7 +391,7 @@ void QgsEffectStackPropertiesDialog::setPreviewPicture( const QPicture &picture
 //
 
 QgsEffectStackCompactWidget::QgsEffectStackCompactWidget( QWidget *parent , QgsPaintEffect *effect )
-    : QWidget( parent )
+    : QgsPanelWidget( parent )
     , mEnabledCheckBox( nullptr )
     , mButton( nullptr )
     , mPreviewPicture( nullptr )
@@ -457,18 +461,14 @@ void QgsEffectStackCompactWidget::showDialog()
     return;
 
   QgsEffectStack* clone = static_cast<QgsEffectStack*>( mStack->clone() );
-  QgsEffectStackPropertiesDialog dialog( clone, this );
+  QgsEffectStackPropertiesWidget* widget = new QgsEffectStackPropertiesWidget( clone, nullptr );
   if ( mPreviewPicture )
   {
-    dialog.setPreviewPicture( *mPreviewPicture );
+    widget->setPreviewPicture( *mPreviewPicture );
   }
-  if ( dialog.exec() == QDialog::Accepted )
-  {
-    *mStack = *clone;
-    emit changed();
-  }
-
-  delete clone;
+  connect( widget, SIGNAL( widgetChanged() ), this, SLOT( updateEffectLive() ) );
+  connect( widget, SIGNAL( panelAccepted( QgsPanelWidget* ) ), this, SLOT( updateAcceptWidget( QgsPanelWidget* ) ) );
+  openPanel( widget );
 }
 
 void QgsEffectStackCompactWidget::enableToggled( bool checked )
@@ -482,3 +482,18 @@ void QgsEffectStackCompactWidget::enableToggled( bool checked )
   mButton->setEnabled( checked );
   emit changed();
 }
+
+void QgsEffectStackCompactWidget::updateAcceptWidget( QgsPanelWidget *panel )
+{
+  QgsEffectStackPropertiesWidget* widget = qobject_cast<QgsEffectStackPropertiesWidget*>( panel );
+  *mStack = *widget->stack();
+  emit changed();
+//    delete widget->stack();
+}
+
+void QgsEffectStackCompactWidget::updateEffectLive()
+{
+  QgsEffectStackPropertiesWidget* widget = qobject_cast<QgsEffectStackPropertiesWidget*>( sender() );
+  *mStack = *widget->stack();
+  emit changed();
+}
