@@ -25,13 +25,16 @@ class QDockWidget;
 class QMainWindow;
 class QWidget;
 
+class QgsAdvancedDigitizingDockWidget;
 class QgsAttributeDialog;
 class QgsComposerView;
 class QgsFeature;
+class QgsLayerTreeMapCanvasBridge;
 class QgsLayerTreeView;
 class QgsLegendInterface;
 class QgsMapCanvas;
 class QgsMapLayer;
+class QgsMapLayerConfigWidgetFactory;
 class QgsMessageBar;
 class QgsPluginManagerInterface;
 class QgsRasterLayer;
@@ -44,7 +47,7 @@ class QgsVectorLayerTools;
 #include <QPair>
 #include <map>
 
-#include <qgis.h>
+#include "qgis.h"
 
 
 /** \ingroup gui
@@ -95,16 +98,16 @@ class GUI_EXPORT QgisInterface : public QObject
     virtual void zoomToActiveLayer() = 0;
 
     //! Add a vector layer
-    virtual QgsVectorLayer* addVectorLayer( QString vectorLayerPath, QString baseName, QString providerKey ) = 0;
+    virtual QgsVectorLayer* addVectorLayer( const QString& vectorLayerPath, const QString& baseName, const QString& providerKey ) = 0;
 
     //! Add a raster layer given a raster layer file name
-    virtual QgsRasterLayer* addRasterLayer( QString rasterLayerPath, QString baseName = QString() ) = 0;
+    virtual QgsRasterLayer* addRasterLayer( const QString& rasterLayerPath, const QString& baseName = QString() ) = 0;
 
     //! Add a WMS layer
     virtual QgsRasterLayer* addRasterLayer( const QString& url, const QString& layerName, const QString& providerKey ) = 0;
 
     //! Add a project
-    virtual bool addProject( QString theProject ) = 0;
+    virtual bool addProject( const QString& theProject ) = 0;
     //! Start a blank project
     virtual void newProject( bool thePromptToSaveFlag = false ) = 0;
 
@@ -196,7 +199,7 @@ class GUI_EXPORT QgisInterface : public QObject
     virtual void removeWebToolBarIcon( QAction *qAction ) = 0;
 
     //! Add toolbar with specified name
-    virtual QToolBar *addToolBar( QString name ) = 0;
+    virtual QToolBar *addToolBar( const QString& name ) = 0;
 
     //! Add a toolbar
     //! @note added in 2.3
@@ -205,12 +208,25 @@ class GUI_EXPORT QgisInterface : public QObject
     /** Return a pointer to the map canvas */
     virtual QgsMapCanvas * mapCanvas() = 0;
 
+    /**
+     * Returns a pointer to the layer tree canvas bridge
+     *
+     * @note added in 2.12
+     */
+    virtual QgsLayerTreeMapCanvasBridge* layerTreeCanvasBridge() = 0;
+
     /** Return a pointer to the main window (instance of QgisApp in case of QGIS) */
     virtual QWidget * mainWindow() = 0;
 
     /** Return the message bar of the main app */
     virtual QgsMessageBar * messageBar() = 0;
 
+    /** Open the message log dock widget **/
+    virtual void openMessageLog() = 0;
+
+    /** Adds a widget to the user input tool bar.*/
+    virtual void addUserInputWidget( QWidget* widget ) = 0;
+
     /** Return mainwindows / composer views of running composer instances (currently only one) */
     virtual QList<QgsComposerView*> activeComposers() = 0;
 
@@ -219,7 +235,7 @@ class GUI_EXPORT QgisInterface : public QObject
      * @return pointer to composer's view
      * @note new composer window will be shown and activated
      */
-    virtual QgsComposerView* createNewComposer( QString title = QString( "" ) ) = 0;
+    virtual QgsComposerView* createNewComposer( const QString& title = QString() ) = 0;
 
     /** Duplicate an existing parent composer from composer view
      * @param composerView pointer to existing composer view
@@ -227,7 +243,7 @@ class GUI_EXPORT QgisInterface : public QObject
      * @return pointer to duplicate composer's view
      * @note dupicate composer window will be hidden until loaded, then shown and activated
      */
-    virtual QgsComposerView* duplicateComposer( QgsComposerView* composerView, QString title = QString( "" ) ) = 0;
+    virtual QgsComposerView* duplicateComposer( QgsComposerView* composerView, const QString& title = QString() ) = 0;
 
     /** Deletes parent composer of composer view, after closing composer window */
     virtual void deleteComposer( QgsComposerView* composerView ) = 0;
@@ -247,10 +263,10 @@ class GUI_EXPORT QgisInterface : public QObject
     virtual QFont defaultStyleSheetFont() = 0;
 
     /** Add action to the plugins menu */
-    virtual void addPluginToMenu( QString name, QAction* action ) = 0;
+    virtual void addPluginToMenu( const QString& name, QAction* action ) = 0;
 
     /** Remove action from the plugins menu */
-    virtual void removePluginMenu( QString name, QAction* action ) = 0;
+    virtual void removePluginMenu( const QString& name, QAction* action ) = 0;
 
     /** Add "add layer" action to layer menu */
     virtual void insertAddLayerAction( QAction *action ) = 0;
@@ -259,28 +275,28 @@ class GUI_EXPORT QgisInterface : public QObject
     virtual void removeAddLayerAction( QAction *action ) = 0;
 
     /** Add action to the Database menu */
-    virtual void addPluginToDatabaseMenu( QString name, QAction* action ) = 0;
+    virtual void addPluginToDatabaseMenu( const QString& name, QAction* action ) = 0;
 
     /** Remove action from the Database menu */
-    virtual void removePluginDatabaseMenu( QString name, QAction* action ) = 0;
+    virtual void removePluginDatabaseMenu( const QString& name, QAction* action ) = 0;
 
     /** Add action to the Raster menu */
-    virtual void addPluginToRasterMenu( QString name, QAction* action ) = 0;
+    virtual void addPluginToRasterMenu( const QString& name, QAction* action ) = 0;
 
     /** Remove action from the Raster menu */
-    virtual void removePluginRasterMenu( QString name, QAction* action ) = 0;
+    virtual void removePluginRasterMenu( const QString& name, QAction* action ) = 0;
 
     /** Add action to the Vector menu */
-    virtual void addPluginToVectorMenu( QString name, QAction* action ) = 0;
+    virtual void addPluginToVectorMenu( const QString& name, QAction* action ) = 0;
 
     /** Remove action from the Vector menu */
-    virtual void removePluginVectorMenu( QString name, QAction* action ) = 0;
+    virtual void removePluginVectorMenu( const QString& name, QAction* action ) = 0;
 
     /** Add action to the Web menu */
-    virtual void addPluginToWebMenu( QString name, QAction* action ) = 0;
+    virtual void addPluginToWebMenu( const QString& name, QAction* action ) = 0;
 
     /** Remove action from the Web menu */
-    virtual void removePluginWebMenu( QString name, QAction* action ) = 0;
+    virtual void removePluginWebMenu( const QString& name, QAction* action ) = 0;
 
     /** Add a dock widget to the main window */
     virtual void addDockWidget( Qt::DockWidgetArea area, QDockWidget * dockwidget ) = 0;
@@ -288,11 +304,16 @@ class GUI_EXPORT QgisInterface : public QObject
     /** Remove specified dock widget from main window (doesn't delete it). */
     virtual void removeDockWidget( QDockWidget * dockwidget ) = 0;
 
-    /** open layer properties dialog */
+    /** Advanced digitizing dock widget
+     *  @note Added in 2.12
+     */
+    virtual QgsAdvancedDigitizingDockWidget* cadDockWidget() = 0;
+
+    /** Open layer properties dialog */
     virtual void showLayerProperties( QgsMapLayer *l ) = 0;
 
-    /** open attribute table dialog */
-    virtual void showAttributeTable( QgsVectorLayer *l ) = 0;
+    /** Open attribute table dialog */
+    virtual QDialog* showAttributeTable( QgsVectorLayer *l, const QString& filterExpression = QString() ) = 0;
 
     /** Add window to Window menu. The action title is the window title
      * and the action should raise, unminimize and activate the window. */
@@ -303,11 +324,24 @@ class GUI_EXPORT QgisInterface : public QObject
     virtual void removeWindow( QAction *action ) = 0;
 
     /** Register action to the shortcuts manager so its shortcut can be changed in GUI */
-    virtual bool registerMainWindowAction( QAction* action, QString defaultShortcut ) = 0;
+    virtual bool registerMainWindowAction( QAction* action, const QString& defaultShortcut ) = 0;
 
     /** Unregister a previously registered action. (e.g. when plugin is going to be unloaded) */
     virtual bool unregisterMainWindowAction( QAction* action ) = 0;
 
+    /** Register a new tab in the vector layer properties dialog.
+     * @note added in QGIS 2.16
+     * @note Ownership of the factory is not transferred, and the factory must
+     *       be unregistered when plugin is unloaded.
+     * @see unregisterMapLayerPropertiesFactory() */
+    virtual void registerMapLayerConfigWidgetFactory( QgsMapLayerConfigWidgetFactory* factory ) = 0;
+
+    /** Unregister a previously registered tab in the vector layer properties dialog.
+     * @note added in QGIS 2.16
+     * @see registerMapLayerPropertiesFactory()
+    */
+    virtual void unregisterMapLayerConfigWidgetFactory( QgsMapLayerConfigWidgetFactory* factory ) = 0;
+
     // @todo is this deprecated in favour of QgsContextHelp?
     /** Open a url in the users browser. By default the QGIS doc directory is used
      * as the base for the URL. To open a URL that is not relative to the installed
@@ -320,7 +354,7 @@ class GUI_EXPORT QgisInterface : public QObject
 #ifndef Q_MOC_RUN
     Q_DECL_DEPRECATED
 #endif
-    virtual void openURL( QString url, bool useQgisDocDirectory = true ) = 0;
+    virtual void openURL( const QString& url, bool useQgisDocDirectory = true ) = 0;
 
 
     /** Accessors for inserting items into menus and toolbars.
@@ -429,7 +463,7 @@ class GUI_EXPORT QgisInterface : public QObject
     virtual QAction *actionZoomLast() = 0;
     //! Get access to the native zoom next action. Call trigger() on it to zoom to next.
     virtual QAction *actionZoomNext() = 0;
-    //! Get access to the native zoom actual size action. Call trigger() on it to zoom to actual size.
+    //! Get access to the native zoom resolution (100%) action. Call trigger() on it to zoom to actual size.
     virtual QAction *actionZoomActualSize() = 0;
     //! Get access to the native map tips action. Call trigger() on it to toggle map tips.
     virtual QAction *actionMapTips() = 0;
@@ -446,6 +480,10 @@ class GUI_EXPORT QgisInterface : public QObject
     virtual QAction *actionAddRasterLayer() = 0;
     virtual QAction *actionAddPgLayer() = 0;
     virtual QAction *actionAddWmsLayer() = 0;
+    /** Get access to the native Add ArcGIS FeatureServer action. */
+    virtual QAction *actionAddAfsLayer() = 0;
+    /** Get access to the native Add ArcGIS MapServer action. */
+    virtual QAction *actionAddAmsLayer() = 0;
     virtual QAction *actionCopyLayerStyle() = 0;
     virtual QAction *actionPasteLayerStyle() = 0;
     virtual QAction *actionOpenTable() = 0;
@@ -531,7 +569,7 @@ class GUI_EXPORT QgisInterface : public QObject
      *
      * More information here: http://qt-project.org/forums/viewthread/27098/
      */
-    virtual void preloadForm( QString uifile ) = 0;
+    virtual void preloadForm( const QString& uifile ) = 0;
 
     /** Return vector layers in edit mode
      * @param modified whether to return only layers that have been modified
@@ -556,41 +594,38 @@ class GUI_EXPORT QgisInterface : public QObject
      * This signal is emitted before a new composer instance is going to be removed
      */
     void composerWillBeRemoved( QgsComposerView* v );
+
+    /** This signal is emitted when a composer instance has been removed
+     * @note added in version 2.9
+     */
+    void composerRemoved( QgsComposerView* v );
+
     /**
      * This signal is emitted when the initialization is complete
      */
     void initializationCompleted();
-    /** emitted when a project file is successfully read
-        @note
-        This is useful for plug-ins that store properties with project files.  A
-        plug-in can connect to this signal.  When it is emitted, the plug-in
-        knows to then check the project properties for any relevant state.
+    /** Emitted when a project file is successfully read
+     * @note
+     * This is useful for plug-ins that store properties with project files.  A
+     * plug-in can connect to this signal.  When it is emitted, the plug-in
+     * knows to then check the project properties for any relevant state.
      */
     void projectRead();
-    /** emitted when starting an entirely new project
-        @note
-        This is similar to projectRead(); plug-ins might want to be notified
-        that they're in a new project.  Yes, projectRead() could have been
-        overloaded to be used in the case of new projects instead.  However,
-        it's probably more semantically correct to have an entirely separate
-        signal for when this happens.
-      */
+    /** Emitted when starting an entirely new project
+     * @note
+     * This is similar to projectRead(); plug-ins might want to be notified
+     * that they're in a new project.  Yes, projectRead() could have been
+     * overloaded to be used in the case of new projects instead.  However,
+     * it's probably more semantically correct to have an entirely separate
+     * signal for when this happens.
+     */
     void newProjectCreated();
 
-    /**This signal is emitted when a layer has been saved using save as
-       @note
-       added in version 2.7
-    */
-    void layerSavedAs( QgsMapLayer* l, QString path );
+    /** This signal is emitted when a layer has been saved using save as
+     * @note
+     * added in version 2.7
+     */
+    void layerSavedAs( QgsMapLayer* l, const QString& path );
 };
 
-// FIXME: also in core/qgis.h
-#ifndef QGISEXTERN
-#ifdef WIN32
-#  define QGISEXTERN extern "C" __declspec( dllexport )
-#else
-#  define QGISEXTERN extern "C"
-#endif
-#endif
-
 #endif //#ifndef QGISINTERFACE_H
