@@ -20,18 +20,18 @@
 #include "qgsfeature.h"
 #include "qgsattributeeditorcontext.h"
 #include "qgsattributeform.h"
+#include "qgstrackedvectorlayertools.h"
 
 #include <QDialog>
 #include <QMenuBar>
 #include <QGridLayout>
 
 class QgsDistanceArea;
-class QgsFeature;
-class QgsField;
 class QgsHighlight;
-class QgsVectorLayer;
-class QgsVectorLayerTools;
 
+/** \ingroup gui
+ * \class QgsAttributeDialog
+ */
 class GUI_EXPORT QgsAttributeDialog : public QDialog
 {
     Q_OBJECT
@@ -108,8 +108,16 @@ class GUI_EXPORT QgsAttributeDialog : public QDialog
      * If set to true, the dialog will add a new feature when the form is accepted.
      *
      * @param isAddDialog If set to true, turn this dialog into an add feature dialog.
+     * @deprecated use setMode() instead
      */
-    void setIsAddDialog( bool isAddDialog ) { mAttributeForm->setIsAddDialog( isAddDialog ); }
+    Q_DECL_DEPRECATED void setIsAddDialog( bool isAddDialog ) { mAttributeForm->setMode( isAddDialog ? QgsAttributeForm::AddFeatureMode : QgsAttributeForm::SingleEditMode ); }
+
+    /**
+     * Toggles the form mode.
+     * @param mode form mode. Eg if set to QgsAttributeForm::AddFeatureMode, the dialog will be editable even with an invalid feature and
+     * will add a new feature when the form is accepted.
+     */
+    void setMode( QgsAttributeForm::Mode mode ) { mAttributeForm->setMode( mode ); }
 
     /**
      * Sets the edit command message (Undo) that will be used when the dialog is accepted
@@ -129,12 +137,13 @@ class GUI_EXPORT QgsAttributeDialog : public QDialog
 
   public slots:
     void accept() override;
+    void reject() override;
 
     //! Show the dialog non-blocking. Reparents this dialog to be a child of the dialog form
     void show();
 
   private:
-    void init( QgsVectorLayer* layer, QgsFeature* feature, const QgsAttributeEditorContext& context );
+    void init( QgsVectorLayer* layer, QgsFeature* feature, const QgsAttributeEditorContext& context, bool showDialogButtons );
 
     QString mSettingsPath;
     // Used to sync multiple widgets for the same field
@@ -145,11 +154,14 @@ class GUI_EXPORT QgsAttributeDialog : public QDialog
     QgsAttributeForm* mAttributeForm;
     QgsFeature *mOwnedFeature;
 
+    QgsTrackedVectorLayerTools mTrackedVectorLayerTools;
+
     // true if this dialog is editable
     bool mEditable;
 
     static int sFormCounter;
     static QString sSettingsPath;
+
 };
 
 #endif
