@@ -23,13 +23,14 @@
  * A HTML editor based on QScintilla2. Adds syntax highlighting and
  * code autocompletion.
  * \note added in 2.6
+ * \note may not be available in Python bindings, depending on platform support
  */
 class GUI_EXPORT QgsCodeEditorHTML : public QgsCodeEditor
 {
     Q_OBJECT
 
   public:
-    QgsCodeEditorHTML( QWidget *parent = 0 );
+    QgsCodeEditorHTML( QWidget *parent = nullptr );
     ~QgsCodeEditorHTML();
 
   private:
