@@ -55,13 +55,10 @@ void QgsCodeEditorPython::setSciLexerPython()
 
   QsciLexerPython* pyLexer = new QsciLexerPython( this );
   pyLexer->setDefaultFont( font );
-  pyLexer->setFont( font, 1 ); // comment
-  pyLexer->setFont( font, 3 ); // singlequotes
-  pyLexer->setFont( font, 4 ); // doublequotes
-  pyLexer->setFont( font, 6 ); // triplequotes
-  pyLexer->setColor( Qt::red, 1 ); // comment color
-  pyLexer->setColor( Qt::darkGreen, 5 ); // keyword color
-  pyLexer->setColor( Qt::darkBlue, 15 ); // decorator color
+  pyLexer->setFont( font, -1 );
+  pyLexer->setColor( Qt::red, QsciLexerPython::Comment );
+  pyLexer->setColor( Qt::darkGreen, QsciLexerPython::Keyword );
+  pyLexer->setColor( Qt::darkBlue, QsciLexerPython::Decorator );
 
   QsciAPIs* apis = new QsciAPIs( pyLexer );
 
