@@ -19,6 +19,7 @@
 #include <QWidget>
 #include <QString>
 #include <QFont>
+#include <QLabel>
 #include <Qsci/qscilexersql.h>
 
 
@@ -39,10 +40,36 @@ QgsCodeEditorSQL::~QgsCodeEditorSQL()
 {
 }
 
+/** Internal use.
+
+   setAutoCompletionCaseSensitivity( false ) is not sufficient when installing
+   a lexer, since its caseSensitive() method is actually used, and defaults
+   to true.
+   @note not available in Python bindings
+   @ingroup gui
+*/
+class QgsCaseInsensitiveLexerSQL: public QsciLexerSQL
+{
+  public:
+    //! constructor
+    explicit QgsCaseInsensitiveLexerSQL( QObject *parent = 0 ) : QsciLexerSQL( parent ) {}
+
+    bool caseSensitive() const override { return false; }
+};
+
 void QgsCodeEditorSQL::setSciLexerSQL()
 {
-  QsciLexerSQL* sqlLexer = new QsciLexerSQL( this );
-  sqlLexer->setDefaultFont( QFont( "Sans", 10 ) );
+  QFont font = getMonospaceFont();
+#ifdef Q_OS_MAC
+  // The font size gotten from getMonospaceFont() is too small on Mac
+  font.setPointSize( QLabel().font().pointSize() );
+#endif
+  QsciLexerSQL* sqlLexer = new QgsCaseInsensitiveLexerSQL( this );
+  sqlLexer->setDefaultFont( font );
+  sqlLexer->setFont( font, -1 );
+  font.setBold( true );
+  sqlLexer->setFont( font, QsciLexerSQL::Keyword );
+  sqlLexer->setColor( Qt::darkYellow, QsciLexerSQL::DoubleQuotedString ); // fields
 
   setLexer( sqlLexer );
 }
