@@ -16,6 +16,7 @@
 #ifndef QGSCOLORWIDGETS_H
 #define QGSCOLORWIDGETS_H
 
+#include <QWidgetAction>
 #include <QWidget>
 
 class QColor;
@@ -38,7 +39,7 @@ class GUI_EXPORT QgsColorWidget : public QWidget
 
   public:
 
-    /*! Specifies the color component which the widget alters
+    /** Specifies the color component which the widget alters
      */
     enum ColorComponent
     {
@@ -52,27 +53,27 @@ class GUI_EXPORT QgsColorWidget : public QWidget
       Alpha /*!< alpha component (opacity) of color */
     };
 
-    /**Construct a new color widget.
+    /** Construct a new color widget.
      * @param parent parent QWidget for the widget
      * @param component color component the widget alters
      */
-    QgsColorWidget( QWidget* parent = 0, const ColorComponent component = Multiple );
+    QgsColorWidget( QWidget* parent = nullptr, const ColorComponent component = Multiple );
 
     virtual ~QgsColorWidget();
 
-    /**Returns the current color for the widget
+    /** Returns the current color for the widget
      * @returns current widget color
      * @see setColor
      */
     QColor color() const;
 
-    /**Returns the color component which the widget controls
+    /** Returns the color component which the widget controls
      * @returns color component for widget
      * @see setComponent
      */
     ColorComponent component() const { return mComponent; }
 
-    /**Returns the current value of the widget's color component
+    /** Returns the current value of the widget's color component
      * @returns value of color component, or -1 if widget has multiple components or an invalid color
      * set
      * @see setComponentValue
@@ -80,27 +81,27 @@ class GUI_EXPORT QgsColorWidget : public QWidget
      */
     int componentValue() const;
 
-    /**Create an icon for dragging colors
+    /** Create an icon for dragging colors
      * @param color for icon
      */
     static QPixmap createDragIcon( const QColor &color );
 
   public slots:
 
-    /**Sets the color for the widget
+    /** Sets the color for the widget
      * @param color widget color
      * @param emitSignals set to true to emit the colorChanged signal after setting color
      * @see color
      */
     virtual void setColor( const QColor &color, const bool emitSignals = false );
 
-    /**Sets the color component which the widget controls
+    /** Sets the color component which the widget controls
      * @param component color component for widget
      * @see component
      */
     virtual void setComponent( const ColorComponent component );
 
-    /**Alters the widget's color by setting the value for the widget's color component
+    /** Alters the widget's color by setting the value for the widget's color component
      * @param value value for widget's color component. This value is automatically
      * clipped to the range of valid values for the color component.
      * @see componentValue
@@ -112,33 +113,38 @@ class GUI_EXPORT QgsColorWidget : public QWidget
 
   signals:
 
-    /**Emitted when the widget's color changes
+    /** Emitted when the widget's color changes
      * @param color new widget color
      */
     void colorChanged( const QColor &color );
 
+    /** Emitted when mouse hovers over widget.
+     * @note added in QGIS 2.14
+     */
+    void hovered();
+
   protected:
 
     QColor mCurrentColor;
 
     ColorComponent mComponent;
 
-    /**QColor wipes the hue information when it is ambiguous (eg, for saturation = 0). So
+    /** QColor wipes the hue information when it is ambiguous (eg, for saturation = 0). So
      * the hue is stored in mExplicit hue to keep it around, as it is useful when modifying colors
-    */
+     */
     int mExplicitHue;
 
-    /**Returns the range of valid values for the color widget's component
+    /** Returns the range of valid values for the color widget's component
      * @returns maximum value allowed for color component, or -1 if widget has multiple components
      */
     int componentRange() const;
 
-    /**Returns the range of valid values a color component
+    /** Returns the range of valid values a color component
      * @returns maximum value allowed for color component
      */
     int componentRange( const ColorComponent component ) const;
 
-    /**Returns the value of a component of the widget's current color. This method correctly
+    /** Returns the value of a component of the widget's current color. This method correctly
      * handles hue values when the color has an ambiguous hue (eg black or white shades)
      * @param component color component to return
      * @returns value of color component, or -1 if widget has an invalid color set
@@ -146,13 +152,13 @@ class GUI_EXPORT QgsColorWidget : public QWidget
      */
     int componentValue( const ColorComponent component ) const;
 
-    /**Returns the hue for the widget. This may differ from the hue for the QColor returned by color(),
+    /** Returns the hue for the widget. This may differ from the hue for the QColor returned by color(),
      * as QColor returns a hue of -1 if the color's hue is ambiguous (eg, if the saturation is zero).
      * @returns explicitly set hue for widget
      */
     int hue() const;
 
-    /**Alters a color by modifiying the value of a specific color component
+    /** Alters a color by modifiying the value of a specific color component
      * @param color color to alter
      * @param component color component to alter
      * @param newValue new value of color component. Values are automatically clipped to a
@@ -160,7 +166,7 @@ class GUI_EXPORT QgsColorWidget : public QWidget
      */
     void alterColor( QColor& color, const QgsColorWidget::ColorComponent component, const int newValue ) const;
 
-    /**Generates a checkboard pattern pixmap for use as a background to transparent colors
+    /** Generates a checkboard pattern pixmap for use as a background to transparent colors
      * @returns checkerboard pixmap
      */
     static const QPixmap& transparentBackground();
@@ -170,9 +176,83 @@ class GUI_EXPORT QgsColorWidget : public QWidget
 
     //Reimplemented to accept dropped colors
     void dropEvent( QDropEvent *e ) override;
+
+    void mouseMoveEvent( QMouseEvent* e ) override;
+    void mousePressEvent( QMouseEvent* e ) override;
+    void mouseReleaseEvent( QMouseEvent* e ) override;
 };
 
 
+/** \ingroup gui
+ * \class QgsColorWidgetAction
+ * An action containing a color widget, which can be embedded into a menu.
+ * @see QgsColorWidget
+ * @note introduced in QGIS 2.14
+ */
+
+class GUI_EXPORT QgsColorWidgetAction: public QWidgetAction
+{
+    Q_OBJECT
+
+  public:
+
+    /** Construct a new color widget action.
+     * @param colorWidget QgsColorWidget to show in action
+     * @param menu parent menu
+     * @param parent parent widget
+     */
+    QgsColorWidgetAction( QgsColorWidget* colorWidget, QMenu* menu = nullptr, QWidget *parent = nullptr );
+
+    virtual ~QgsColorWidgetAction();
+
+    /** Returns the color widget contained in the widget action.
+     */
+    QgsColorWidget* colorWidget() { return mColorWidget; }
+
+    /** Sets whether the parent menu should be dismissed and closed when a color is selected
+     * from the action's color widget.
+     * @param dismiss set to true (default) to immediately close the menu when a color is selected
+     * from the widget. If set to false, the colorChanged signal will be emitted but the menu will
+     * stay open.
+     * @see dismissOnColorSelection()
+     */
+    void setDismissOnColorSelection( bool dismiss ) { mDismissOnColorSelection = dismiss; }
+
+    /** Returns whether the parent menu will be dismissed after a color is selected from the
+     * action's color widget.
+     * @see setDismissOnColorSelection
+     */
+    bool dismissOnColorSelection() const { return mDismissOnColorSelection; }
+
+  signals:
+
+    /** Emitted when a color has been selected from the widget
+     * @param color selected color
+     */
+    void colorChanged( const QColor &color );
+
+  private:
+    QMenu* mMenu;
+    QgsColorWidget* mColorWidget;
+
+    //used to suppress recursion with hover events
+    bool mSuppressRecurse;
+
+    bool mDismissOnColorSelection;
+
+  private slots:
+
+    /** Handles setting the active action for the menu when cursor hovers over color widget
+     */
+    void onHover();
+
+    /** Emits color changed signal and closes parent menu
+     */
+    void setColor( const QColor &color );
+};
+
+
+
 /** \ingroup gui
  * \class QgsColorWheel
  * A color wheel widget. This widget consists of an outer ring which allows for hue selection, and an
@@ -186,13 +266,14 @@ class GUI_EXPORT QgsColorWheel : public QgsColorWidget
 
   public:
 
-    /**Constructs a new color wheel widget.
+    /** Constructs a new color wheel widget.
      * @param parent parent QWidget for the widget
      */
-    QgsColorWheel( QWidget* parent = 0 );
+    QgsColorWheel( QWidget* parent = nullptr );
 
     virtual ~QgsColorWheel();
 
+    virtual QSize sizeHint() const override;
     void paintEvent( QPaintEvent* event ) override;
 
   public slots:
@@ -242,20 +323,20 @@ class GUI_EXPORT QgsColorWheel : public QgsColorWidget
     /*Conical gradient brush used for drawing hue wheel*/
     QBrush mWheelBrush;
 
-    /**Creates cache images for specified widget size
+    /** Creates cache images for specified widget size
      * @param size widget size for images
-    */
+     */
     void createImages( const QSizeF size );
 
-    /**Creates the hue wheel image*/
+    /** Creates the hue wheel image*/
     void createWheel();
 
-    /**Creates the inner triangle image*/
+    /** Creates the inner triangle image*/
     void createTriangle();
 
-    /**Sets the widget color based on a point in the widget
+    /** Sets the widget color based on a point in the widget
      * @param pos position for color
-    */
+     */
     void setColorFromPos( const QPointF pos );
 
 };
@@ -275,13 +356,13 @@ class GUI_EXPORT QgsColorBox : public QgsColorWidget
 
   public:
 
-    /**Construct a new color box widget.
+    /** Construct a new color box widget.
      * @param parent parent QWidget for the widget
      * @param component constant color component for the widget. The color components
      * which vary along the horizontal and vertical axis are automatically assigned
      * based on this constant color component.
      */
-    QgsColorBox( QWidget* parent = 0, const ColorComponent component = Value );
+    QgsColorBox( QWidget* parent = nullptr, const ColorComponent component = Value );
 
     virtual ~QgsColorBox();
 
@@ -310,40 +391,40 @@ class GUI_EXPORT QgsColorBox : public QgsColorWidget
     /*Whether the cached image requires redrawing*/
     bool mDirty;
 
-    /**Creates the color box background cached image
-    */
+    /** Creates the color box background cached image
+     */
     void createBox();
 
-    /**Returns the range of permissible values along the x axis
+    /** Returns the range of permissible values along the x axis
      * @returns maximum color component value for x axis
-    */
+     */
     int valueRangeX() const;
 
-    /**Returns the range of permissible values along the y axis
+    /** Returns the range of permissible values along the y axis
      * @returns maximum color component value for y axis
-    */
+     */
     int valueRangeY() const;
 
-    /**Returns the color component which varies along the y axis
-    */
+    /** Returns the color component which varies along the y axis
+     */
     QgsColorWidget::ColorComponent yComponent() const;
 
-    /**Returns the value of the color component which varies along the y axis
-    */
+    /** Returns the value of the color component which varies along the y axis
+     */
     int yComponentValue() const;
 
-    /**Returns the color component which varies along the x axis
-    */
+    /** Returns the color component which varies along the x axis
+     */
     QgsColorWidget::ColorComponent xComponent() const;
 
-    /**Returns the value of the color component which varies along the x axis
-    */
+    /** Returns the value of the color component which varies along the x axis
+     */
     int xComponentValue() const;
 
-    /**Updates the widget's color based on a point within the widget
+    /** Updates the widget's color based on a point within the widget
      * @param point point within the widget
-    */
-    void setColorFromPoint( const QPoint& point );
+     */
+    void setColorFromPoint( QPoint point );
 
 };
 
@@ -361,7 +442,7 @@ class GUI_EXPORT QgsColorRampWidget : public QgsColorWidget
 
   public:
 
-    /*! Specifies the orientation of a color ramp
+    /** Specifies the orientation of a color ramp
      */
     enum Orientation
     {
@@ -369,12 +450,12 @@ class GUI_EXPORT QgsColorRampWidget : public QgsColorWidget
       Vertical /*!< vertical ramp */
     };
 
-    /**Construct a new color ramp widget.
+    /** Construct a new color ramp widget.
      * @param parent parent QWidget for the widget
      * @param component color component which varies along the ramp
      * @param orientation orientation for widget
      */
-    QgsColorRampWidget( QWidget* parent = 0,
+    QgsColorRampWidget( QWidget* parent = nullptr,
                         const ColorComponent component = QgsColorWidget::Red,
                         const Orientation orientation = QgsColorRampWidget::Horizontal );
 
@@ -383,50 +464,50 @@ class GUI_EXPORT QgsColorRampWidget : public QgsColorWidget
     virtual QSize sizeHint() const override;
     void paintEvent( QPaintEvent* event ) override;
 
-    /**Sets the orientation for the color ramp
+    /** Sets the orientation for the color ramp
      * @param orientation new orientation for the ramp
      * @see orientation
      */
     void setOrientation( const Orientation orientation );
 
-    /**Fetches the orientation for the color ramp
+    /** Fetches the orientation for the color ramp
      * @returns orientation for the ramp
      * @see setOrientation
      */
     Orientation orientation() const { return mOrientation; }
 
-    /**Sets the margin between the edge of the widget and the ramp
+    /** Sets the margin between the edge of the widget and the ramp
      * @param margin margin around the ramp
      * @see interiorMargin
      */
     void setInteriorMargin( const int margin );
 
-    /**Fetches the margin between the edge of the widget and the ramp
+    /** Fetches the margin between the edge of the widget and the ramp
      * @returns margin around the ramp
      * @see setInteriorMargin
      */
     int interiorMargin() const { return mMargin; }
 
-    /**Sets whether the ramp should be drawn within a frame
+    /** Sets whether the ramp should be drawn within a frame
      * @param showFrame set to true to draw a frame around the ramp
      * @see showFrame
      */
     void setShowFrame( const bool showFrame );
 
-    /**Fetches whether the ramp is drawn within a frame
+    /** Fetches whether the ramp is drawn within a frame
      * @returns true if a frame is drawn around the ramp
      * @see setShowFrame
      */
     bool showFrame() const { return mShowFrame; }
 
-    /**Sets the size for drawing the triangular markers on the ramp
+    /** Sets the size for drawing the triangular markers on the ramp
      * @param markerSize marker size in pixels
      */
     void setMarkerSize( const int markerSize );
 
   signals:
 
-    /**Emitted when the widget's color component value changes
+    /** Emitted when the widget's color component value changes
      * @param value new value of color component
      */
     void valueChanged( const int value );
@@ -434,6 +515,7 @@ class GUI_EXPORT QgsColorRampWidget : public QgsColorWidget
   protected:
 
     virtual void mouseMoveEvent( QMouseEvent *event ) override;
+    virtual void wheelEvent( QWheelEvent* event ) override;
     virtual void mousePressEvent( QMouseEvent *event ) override;
     virtual void keyPressEvent( QKeyEvent * event ) override;
 
@@ -454,10 +536,10 @@ class GUI_EXPORT QgsColorRampWidget : public QgsColorWidget
     /*Polygon for lower triangle marker*/
     QPolygonF mBottomTriangle;
 
-    /**Updates the widget's color based on a point within the widget
+    /** Updates the widget's color based on a point within the widget
      * @param point point within the widget
-    */
-    void setColorFromPoint( const QPointF &point );
+     */
+    void setColorFromPoint( QPointF point );
 
 };
 
@@ -474,11 +556,11 @@ class GUI_EXPORT QgsColorSliderWidget : public QgsColorWidget
 
   public:
 
-    /**Construct a new color slider widget.
+    /** Construct a new color slider widget.
      * @param parent parent QWidget for the widget
      * @param component color component which is controlled by the slider
      */
-    QgsColorSliderWidget( QWidget* parent = 0, const ColorComponent component = QgsColorWidget::Red );
+    QgsColorSliderWidget( QWidget* parent = nullptr, const ColorComponent component = QgsColorWidget::Red );
 
     virtual ~QgsColorSliderWidget();
 
@@ -494,33 +576,33 @@ class GUI_EXPORT QgsColorSliderWidget : public QgsColorWidget
     /*Spin box widget*/
     QSpinBox* mSpinBox;
 
-    /**Converts the real value of a color component to a friendly display value. For instance,
+    /** Converts the real value of a color component to a friendly display value. For instance,
      * alpha values from 0-255 have little meaning to users, so we translate them to 0-100%
      * @param realValue actual value of the color component
      * @returns display value of color component
      * @see convertDisplayToReal
-    */
+     */
     int convertRealToDisplay( const int realValue ) const;
 
-    /**Converts the display value of a color component to a real value.
+    /** Converts the display value of a color component to a real value.
      * @param displayValue friendly display value of the color component
      * @returns real value of color component
      * @see convertRealToDisplay
-    */
+     */
     int convertDisplayToReal( const int displayValue ) const;
 
   private slots:
 
-    /**Called when the color for the ramp changes
-    */
+    /** Called when the color for the ramp changes
+     */
     void rampColorChanged( const QColor &color );
 
-    /**Called when the value of the spin box changes
-    */
+    /** Called when the value of the spin box changes
+     */
     void spinChanged( int value );
 
-    /**Called when the value for the ramp changes
-    */
+    /** Called when the value for the ramp changes
+     */
     void rampChanged( int value );
 
 };
@@ -539,10 +621,10 @@ class GUI_EXPORT QgsColorTextWidget : public QgsColorWidget
 
   public:
 
-    /**Construct a new color line edit widget.
+    /** Construct a new color line edit widget.
      * @param parent parent QWidget for the widget
      */
-    QgsColorTextWidget( QWidget* parent = 0 );
+    QgsColorTextWidget( QWidget* parent = nullptr );
 
     virtual ~QgsColorTextWidget();
 
@@ -553,7 +635,7 @@ class GUI_EXPORT QgsColorTextWidget : public QgsColorWidget
 
   private:
 
-    /*! Specifies the display format for a color
+    /** Specifies the display format for a color
      */
     enum ColorTextFormat
     {
@@ -571,18 +653,18 @@ class GUI_EXPORT QgsColorTextWidget : public QgsColorWidget
     /*Display format for colors*/
     ColorTextFormat mFormat;
 
-    /**Updates the text based on the current color
-    */
+    /** Updates the text based on the current color
+     */
     void updateText();
 
   private slots:
 
-    /**Called when the user enters text into the widget
-    */
+    /** Called when the user enters text into the widget
+     */
     void textChanged();
 
-    /**Called when the dropdown arrow is clicked to show the format selection menu
-    */
+    /** Called when the dropdown arrow is clicked to show the format selection menu
+     */
     void showMenu();
 };
 
@@ -599,16 +681,17 @@ class GUI_EXPORT QgsColorPreviewWidget : public QgsColorWidget
 
   public:
 
-    /**Construct a new color preview widget.
+    /** Construct a new color preview widget.
      * @param parent parent QWidget for the widget
      */
-    QgsColorPreviewWidget( QWidget* parent = 0 );
+    QgsColorPreviewWidget( QWidget* parent = nullptr );
 
     virtual ~QgsColorPreviewWidget();
 
     void paintEvent( QPaintEvent* event ) override;
+    virtual QSize sizeHint() const override;
 
-    /**Returns the secondary color for the widget
+    /** Returns the secondary color for the widget
      * @returns secondary widget color, or an invalid color if the widget
      * has no secondary color
      * @see color
@@ -618,7 +701,7 @@ class GUI_EXPORT QgsColorPreviewWidget : public QgsColorWidget
 
   public slots:
 
-    /**Sets the second color for the widget
+    /** Sets the second color for the widget
      * @param color secondary widget color. Set to an invalid color to prevent
      * drawing of a secondary color
      * @see setColor
@@ -649,7 +732,7 @@ class GUI_EXPORT QgsColorPreviewWidget : public QgsColorWidget
      * @param rect rect to draw color in
      * @param painter destination painter
      */
-    void drawColor( const QColor& color, const QRect& rect, QPainter &painter );
+    void drawColor( const QColor& color, QRect rect, QPainter &painter );
 };
 
 #endif // #ifndef QGSCOLORWIDGETS_H
