@@ -28,14 +28,6 @@
 #include <QPointer>
 #include <QGroupBox>
 
-
-QIcon QgsDataDefinedButton::mIconDataDefine;
-QIcon QgsDataDefinedButton::mIconDataDefineOn;
-QIcon QgsDataDefinedButton::mIconDataDefineError;
-QIcon QgsDataDefinedButton::mIconDataDefineExpression;
-QIcon QgsDataDefinedButton::mIconDataDefineExpressionOn;
-QIcon QgsDataDefinedButton::mIconDataDefineExpressionError;
-
 QgsDataDefinedButton::QgsDataDefinedButton( QWidget* parent,
     const QgsVectorLayer* vl,
     const QgsDataDefined* datadefined,
@@ -109,6 +101,47 @@ QgsDataDefinedButton::~QgsDataDefinedButton()
   mCheckedWidgets.clear();
 }
 
+void QgsDataDefinedButton::updateFieldLists()
+{
+  mFieldNameList.clear();
+  mFieldTypeList.clear();
+
+  if ( mVectorLayer )
+  {
+    // store just a list of fields of unknown type or those that match the expected type
+    Q_FOREACH ( const QgsField& f, mVectorLayer->fields() )
+    {
+      bool fieldMatch = false;
+      // NOTE: these are the only QVariant enums supported at this time (see QgsField)
+      QString fieldType;
+      switch ( f.type() )
+      {
+        case QVariant::String:
+          fieldMatch = mDataTypes.testFlag( String );
+          fieldType = tr( "string" );
+          break;
+        case QVariant::Int:
+          fieldMatch = mDataTypes.testFlag( Int ) || mDataTypes.testFlag( Double );
+          fieldType = tr( "integer" );
+          break;
+        case QVariant::Double:
+          fieldMatch = mDataTypes.testFlag( Double );
+          fieldType = tr( "double" );
+          break;
+        case QVariant::Invalid:
+        default:
+          fieldMatch = true; // field type is unknown
+          fieldType = tr( "unknown type" );
+      }
+      if ( fieldMatch || mDataTypes.testFlag( AnyType ) )
+      {
+        mFieldNameList << f.name();
+        mFieldTypeList << fieldType;
+      }
+    }
+  }
+}
+
 void QgsDataDefinedButton::init( const QgsVectorLayer* vl,
                                  const QgsDataDefined* datadefined,
                                  const DataTypes& datatypes,
@@ -163,46 +196,11 @@ void QgsDataDefinedButton::init( const QgsVectorLayer* vl,
     mActionDataTypes->setText( tr( "Field type: " ) + mDataTypesString );
   }
 
-  if ( mVectorLayer )
-  {
-    // store just a list of fields of unknown type or those that match the expected type
-    const QgsFields& fields = mVectorLayer->fields();
-    for ( int i = 0; i < fields.count(); ++i )
-    {
-      const QgsField& f = fields.at( i );
-      bool fieldMatch = false;
-      // NOTE: these are the only QVariant enums supported at this time (see QgsField)
-      QString fieldType;
-      switch ( f.type() )
-      {
-        case QVariant::String:
-          fieldMatch = mDataTypes.testFlag( String );
-          fieldType = tr( "string" );
-          break;
-        case QVariant::Int:
-          fieldMatch = mDataTypes.testFlag( Int ) || mDataTypes.testFlag( Double );
-          fieldType = tr( "integer" );
-          break;
-        case QVariant::Double:
-          fieldMatch = mDataTypes.testFlag( Double );
-          fieldType = tr( "double" );
-          break;
-        case QVariant::Invalid:
-        default:
-          fieldMatch = true; // field type is unknown
-          fieldType = tr( "unknown type" );
-      }
-      if ( fieldMatch || mDataTypes.testFlag( AnyType ) )
-      {
-        mFieldNameList << f.name();
-        mFieldTypeList << fieldType;
-      }
-    }
-  }
-
+  updateFieldLists();
   updateGui();
 }
 
+
 void QgsDataDefinedButton::updateDataDefined( QgsDataDefined *dd ) const
 {
   if ( !dd )
@@ -240,6 +238,8 @@ void QgsDataDefinedButton::mouseReleaseEvent( QMouseEvent *event )
 void QgsDataDefinedButton::aboutToShowMenu()
 {
   mDefineMenu->clear();
+  // update fields so that changes made to layer's fields are reflected
+  updateFieldLists();
 
   bool hasExp = !getExpression().isEmpty();
   bool hasField = !getField().isEmpty();
@@ -906,7 +906,8 @@ QString QgsDataDefinedButton::fillStyleDesc()
 
 QString QgsDataDefinedButton::markerStyleDesc()
 {
-  return trString() + QLatin1String( "[<b>circle</b>|<b>rectangle</b>|<b>cross</b>|<b>triangle</b>]" );
+  return trString() + QLatin1String( "[<b>circle</b>|<b>rectangle</b>|<b>diamond</b>|<b>cross</b>|<b>triangle"
+                                     "</b>|<b>right_half_triangle</b>|<b>left_half_triangle</b>|<b>semi_circle</b>]" );
 }
 
 QString QgsDataDefinedButton::customDashDesc()
