@@ -25,26 +25,10 @@
 #include "QStandardItemModel"
 #include "QStandardItem"
 #include "QSortFilterProxyModel"
+#include "QStringListModel"
 
-/** Search proxy used to filter the QgsExpressionBuilderWidget tree.
-  * The default search for a tree model only searches top level this will handle one
-  * level down
-  */
-class QgsExpressionItemSearchProxy : public QSortFilterProxyModel
-{
-  public:
-    QgsExpressionItemSearchProxy() { }
-
-    bool filterAcceptsRow( int source_row, const QModelIndex &source_parent ) const override
-    {
-      if ( source_parent == qobject_cast<QStandardItemModel*>( sourceModel() )->invisibleRootItem()->index() )
-        return true;
-
-      return QSortFilterProxyModel::filterAcceptsRow( source_row, source_parent );
-    }
-};
-
-/** An expression item that can be used in the QgsExpressionBuilderWidget tree.
+/** \ingroup gui
+ * An expression item that can be used in the QgsExpressionBuilderWidget tree.
   */
 class QgsExpressionItem : public QStandardItem
 {
@@ -56,59 +40,91 @@ class QgsExpressionItem : public QStandardItem
       ExpressionNode
     };
 
-    QgsExpressionItem( QString label,
-                       QString expressionText,
-                       QString helpText,
+    QgsExpressionItem( const QString& label,
+                       const QString& expressionText,
+                       const QString& helpText,
                        QgsExpressionItem::ItemType itemType = ExpressionNode )
         : QStandardItem( label )
     {
       mExpressionText = expressionText;
       mHelpText = helpText;
       mType = itemType;
+      setData( itemType, ItemTypeRole );
     }
 
-    QgsExpressionItem( QString label,
-                       QString expressionText,
+    QgsExpressionItem( const QString& label,
+                       const QString& expressionText,
                        QgsExpressionItem::ItemType itemType = ExpressionNode )
         : QStandardItem( label )
     {
       mExpressionText = expressionText;
       mType = itemType;
+      setData( itemType, ItemTypeRole );
     }
 
-    QString getExpressionText() { return mExpressionText; }
+    QString getExpressionText() const { return mExpressionText; }
 
     /** Get the help text that is associated with this expression item.
       *
       * @return The help text.
       */
-    QString getHelpText() {  return mHelpText; }
+    QString getHelpText() const { return mHelpText; }
     /** Set the help text for the current item
       *
       * @note The help text can be set as a html string.
       */
-    void setHelpText( QString helpText ) { mHelpText = helpText; }
+    void setHelpText( const QString& helpText ) { mHelpText = helpText; }
 
     /** Get the type of expression item eg header, field, ExpressionNode.
       *
       * @return The QgsExpressionItem::ItemType
       */
-    QgsExpressionItem::ItemType getItemType() { return mType; }
+    QgsExpressionItem::ItemType getItemType() const { return mType; }
+
+    //! Custom sort order role
+    static const int CustomSortRole = Qt::UserRole + 1;
+    //! Item type role
+    static const int ItemTypeRole = Qt::UserRole + 2;
 
   private:
     QString mExpressionText;
     QString mHelpText;
     QgsExpressionItem::ItemType mType;
+
 };
 
-/** A reusable widget that can be used to build a expression string.
+/** \ingroup gui
+ * Search proxy used to filter the QgsExpressionBuilderWidget tree.
+  * The default search for a tree model only searches top level this will handle one
+  * level down
+  */
+class GUI_EXPORT QgsExpressionItemSearchProxy : public QSortFilterProxyModel
+{
+    Q_OBJECT
+
+  public:
+    QgsExpressionItemSearchProxy();
+
+    bool filterAcceptsRow( int source_row, const QModelIndex &source_parent ) const override;
+
+  protected:
+
+    bool lessThan( const QModelIndex &left, const QModelIndex &right ) const override;
+};
+
+
+/** \ingroup gui
+ * A reusable widget that can be used to build a expression string.
   * See QgsExpressionBuilderDialog for exmaple of usage.
   */
 class GUI_EXPORT QgsExpressionBuilderWidget : public QWidget, private Ui::QgsExpressionBuilderWidgetBase
 {
     Q_OBJECT
   public:
-    QgsExpressionBuilderWidget( QWidget *parent );
+    /**
+     * Create a new expression builder widget with an optional parent.
+     */
+    QgsExpressionBuilderWidget( QWidget* parent = nullptr );
     ~QgsExpressionBuilderWidget();
 
     /** Sets layer in order to get the fields and values
@@ -123,6 +139,12 @@ class GUI_EXPORT QgsExpressionBuilderWidget : public QWidget, private Ui::QgsExp
 
     void loadFieldNames( const QgsFields& fields );
 
+    /** Loads field names and values from the specified map.
+     *  @note The field values must be quoted appropriately if they are strings.
+     *  @note added in QGIS 2.12
+     */
+    void loadFieldsAndValues( const QMap<QString, QStringList>& fieldValues );
+
     /** Sets geometry calculator used in distance/area calculations. */
     void setGeomCalculator( const QgsDistanceArea & da );
 
@@ -133,26 +155,52 @@ class GUI_EXPORT QgsExpressionBuilderWidget : public QWidget, private Ui::QgsExp
     /** Sets the expression string for the widget */
     void setExpressionText( const QString& expression );
 
+    /** Returns the expression context for the widget. The context is used for the expression
+     * preview result and for populating the list of available functions and variables.
+     * @see setExpressionContext
+     * @note added in QGIS 2.12
+     */
+    QgsExpressionContext expressionContext() const { return mExpressionContext; }
+
+    /** Sets the expression context for the widget. The context is used for the expression
+     * preview result and for populating the list of available functions and variables.
+     * @param context expression context
+     * @see expressionContext
+     * @note added in QGIS 2.12
+     */
+    void setExpressionContext( const QgsExpressionContext& context );
+
     /** Registers a node item for the expression builder.
       * @param group The group the item will be show in the tree view.  If the group doesn't exsit it will be created.
       * @param label The label that is show to the user for the item in the tree.
       * @param expressionText The text that is inserted into the expression area when the user double clicks on the item.
       * @param helpText The help text that the user will see when item is selected.
       * @param type The type of the expression item.
+      * @param highlightedItem set to true to make the item highlighted, which inserts a bold copy of the item at the top level
+      * @param sortOrder sort ranking for item
       */
-    void registerItem( QString group, QString label, QString expressionText,
-                       QString helpText = "",
-                       QgsExpressionItem::ItemType type = QgsExpressionItem::ExpressionNode );
+    void registerItem( const QString& group, const QString& label, const QString& expressionText,
+                       const QString& helpText = "",
+                       QgsExpressionItem::ItemType type = QgsExpressionItem::ExpressionNode,
+                       bool highlightedItem = false, int sortOrder = 1 );
 
     bool isExpressionValid();
 
-    void saveToRecent( QString key );
+    /**
+     * Adds the current expression to the given collection.
+     * By default it is saved to the collection "generic".
+     */
+    void saveToRecent( const QString& collection = "generic" );
 
-    void loadRecent( QString key );
+    /**
+     * Loads the recent expressions from the given collection.
+     * By default it is loaded from the collection "generic".
+     */
+    void loadRecent( const QString& collection = "generic" );
 
     /** Create a new file in the function editor
      */
-    void newFunctionFile( QString fileName = "scratch" );
+    void newFunctionFile( const QString& fileName = "scratch" );
 
     /** Save the current function editor text to the given file.
      */
@@ -164,30 +212,51 @@ class GUI_EXPORT QgsExpressionBuilderWidget : public QWidget, private Ui::QgsExp
 
     /** Load code into the function editor
      */
-    void loadFunctionCode( QString code );
+    void loadFunctionCode( const QString& code );
 
     /** Update the list of function files found at the given path
      */
-    void updateFunctionFileList( QString path );
+    void updateFunctionFileList( const QString& path );
 
   public slots:
+
+    /**
+     * Load sample values into the sample value area
+     */
+    void loadSampleValues();
+
+    /**
+     * Load all unique values from the set layer into the sample area
+     */
+    void loadAllValues();
+
+    /**
+     * Auto save the current Python function code.
+     */
+    void autosave();
+
+    /**
+     * Enabled or disable auto saving. When enabled Python scripts will be auto saved
+     * when text changes.
+     * @param enabled True to enable auto saving.
+     */
+    void setAutoSave( bool enabled ) { mAutoSave = enabled; }
+
+  private slots:
+    void showContextMenu( QPoint );
+    void setExpressionState( bool state );
     void currentChanged( const QModelIndex &index, const QModelIndex & );
+    void operatorButtonClicked();
     void on_btnRun_pressed();
     void on_btnNewFile_pressed();
-    void on_cmbFileNames_currentIndexChanged( int index );
-    void on_btnSaveFile_pressed();
+    void on_cmbFileNames_currentItemChanged( QListWidgetItem* item, QListWidgetItem* lastitem );
     void on_expressionTree_doubleClicked( const QModelIndex &index );
     void on_txtExpressionString_textChanged();
     void on_txtSearchEdit_textChanged();
-    void on_lblPreview_linkActivated( QString link );
-    void on_mValueListWidget_itemDoubleClicked( QListWidgetItem* item );
-    void operatorButtonClicked();
-    void showContextMenu( const QPoint & );
-    void loadSampleValues();
-    void loadAllValues();
-
-  private slots:
-    void setExpressionState( bool state );
+    void on_txtSearchEditValues_textChanged();
+    void on_lblPreview_linkActivated( const QString& link );
+    void on_mValuesListView_doubleClicked( const QModelIndex &index );
+    void on_txtPython_textChanged();
 
   signals:
     /** Emitted when the user changes the expression in the widget.
@@ -197,21 +266,38 @@ class GUI_EXPORT QgsExpressionBuilderWidget : public QWidget, private Ui::QgsExp
       */
     void expressionParsed( bool isValid );
 
+  protected:
+    void showEvent( QShowEvent *e );
+
   private:
-    void runPythonCode( QString code );
+    void runPythonCode( const QString& code );
     void updateFunctionTree();
-    void fillFieldValues( int fieldIndex, int countLimit );
+    void fillFieldValues( const QString &fieldName, int countLimit );
     QString loadFunctionHelp( QgsExpressionItem* functionName );
+    QString helpStylesheet() const;
 
-    /** Formats an expression preview result for display in the widget
-     * by truncating the string
-     * @param previewString expression preview result to format
-     */
-    QString formatPreviewString( const QString &previewString ) const;
+    void loadExpressionContext();
 
+    /** Registers a node item for the expression builder, adding multiple items when the function exists in multiple groups
+      * @param groups The groups the item will be show in the tree view.  If a group doesn't exist it will be created.
+      * @param label The label that is show to the user for the item in the tree.
+      * @param expressionText The text that is inserted into the expression area when the user double clicks on the item.
+      * @param helpText The help text that the user will see when item is selected.
+      * @param type The type of the expression item.
+      * @param highlightedItem set to true to make the item highlighted, which inserts a bold copy of the item at the top level
+      * @param sortOrder sort ranking for item
+      */
+    void registerItemForAllGroups( const QStringList& groups, const QString& label, const QString& expressionText,
+                                   const QString& helpText = "",
+                                   QgsExpressionItem::ItemType type = QgsExpressionItem::ExpressionNode,
+                                   bool highlightedItem = false, int sortOrder = 1 );
+
+    bool mAutoSave;
     QString mFunctionsPath;
     QgsVectorLayer *mLayer;
     QStandardItemModel *mModel;
+    QStringListModel *mValuesModel;
+    QSortFilterProxyModel *mProxyValues;
     QgsExpressionItemSearchProxy *mProxyModel;
     QMap<QString, QgsExpressionItem*> mExpressionGroups;
     QgsFeature mFeature;
@@ -219,7 +305,8 @@ class GUI_EXPORT QgsExpressionBuilderWidget : public QWidget, private Ui::QgsExp
     bool mExpressionValid;
     QgsDistanceArea mDa;
     QString mRecentKey;
-
+    QMap<QString, QStringList> mFieldValues;
+    QgsExpressionContext mExpressionContext;
 };
 
 #endif // QGSEXPRESSIONBUILDER_H
