@@ -25,13 +25,15 @@
 #include <QSettings>
 #include "qgsfield.h"
 
-
+/** \ingroup gui
+ * \class QgsFieldValidator
+ */
 class GUI_EXPORT QgsFieldValidator : public QValidator
 {
     Q_OBJECT
 
   public:
-    QgsFieldValidator( QObject *parent, const QgsField &field, QString defaultValue, QString dateFormat = "yyyy-MM-dd" );
+    QgsFieldValidator( QObject *parent, const QgsField &field, const QString& defaultValue, const QString& dateFormat = "yyyy-MM-dd" );
     ~QgsFieldValidator();
 
     virtual State validate( QString &, int & ) const override;
