@@ -24,24 +24,25 @@
 
 class QGraphicsProxyWidget;
 
-/**An annotation item that embedds a designer form showing the feature attribute*/
+/** \ingroup gui
+ * An annotation item that embedds a designer form showing the feature attribute*/
 class GUI_EXPORT QgsFormAnnotationItem: public QObject, public QgsAnnotationItem
 {
     Q_OBJECT
   public:
-    QgsFormAnnotationItem( QgsMapCanvas* canvas, QgsVectorLayer* vlayer = 0, bool hasFeature = false, int feature = 0 );
+    QgsFormAnnotationItem( QgsMapCanvas* canvas, QgsVectorLayer* vlayer = nullptr, bool hasFeature = false, int feature = 0 );
     ~QgsFormAnnotationItem();
 
     void paint( QPainter * painter ) override;
 
     //! paint function called by map canvas
-    void paint( QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget = 0 ) override;
+    void paint( QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget = nullptr ) override;
 
     QSizeF minimumFrameSize() const override;
-    /**Returns the optimal frame size*/
+    /** Returns the optimal frame size*/
     QSizeF preferredFrameSize() const;
 
-    /**Reimplemented from QgsAnnotationItem*/
+    /** Reimplemented from QgsAnnotationItem*/
     void setMapPosition( const QgsPoint& pos ) override;
 
     void setDesignerForm( const QString& uiFile );
@@ -53,21 +54,21 @@ class GUI_EXPORT QgsFormAnnotationItem: public QObject, public QgsAnnotationItem
     QgsVectorLayer* vectorLayer() const { return mVectorLayer; }
 
   private slots:
-    /**Sets a feature for the current map position and updates the dialog*/
+    /** Sets a feature for the current map position and updates the dialog*/
     void setFeatureForMapPosition();
-    /**Sets visibility status based on mVectorLayer visibility*/
+    /** Sets visibility status based on mVectorLayer visibility*/
     void updateVisibility();
 
   private:
     QGraphicsProxyWidget* mWidgetContainer;
     QWidget* mDesignerWidget;
-    /**Associated vectorlayer (or 0 if attributes are not supposed to be replaced)*/
+    /** Associated vectorlayer (or 0 if attributes are not supposed to be replaced)*/
     QgsVectorLayer* mVectorLayer;
-    /**True if the item is related to a vector feature*/
+    /** True if the item is related to a vector feature*/
     bool mHasAssociatedFeature;
-    /**Associated feature*/
+    /** Associated feature*/
     QgsFeatureId mFeature;
-    /**Path to (and including) the .ui file*/
+    /** Path to (and including) the .ui file*/
     QString mDesignerForm;
 
     QWidget* createDesignerWidget( const QString& filePath );
