@@ -18,7 +18,7 @@
 #include "qgsidentifymenu.h"
 
 #include "qgsapplication.h"
-#include "qgsattributeaction.h"
+#include "qgsactionmanager.h"
 #include "qgshighlight.h"
 
 
@@ -193,7 +193,7 @@ void QgsIdentifyMenu::addRasterLayer( QgsMapLayer* layer )
   }
 
   // add layer action to the top menu
-  layerAction->setIcon( QgsApplication::getThemeIcon( "/mIconRasterLayer.png" ) );
+  layerAction->setIcon( QgsApplication::getThemeIcon( "/mIconRasterLayer.svg" ) );
   layerAction->setData( QVariant::fromValue<ActionData>( ActionData( layer ) ) );
   connect( layerAction, SIGNAL( hovered() ), this, SLOT( handleMenuHover() ) );
   addAction( layerAction );
@@ -311,13 +311,13 @@ void QgsIdentifyMenu::addVectorLayer( QgsVectorLayer* layer, const QList<QgsMapT
     switch ( layer->geometryType() )
     {
       case QGis::Point:
-        layerAction->setIcon( QgsApplication::getThemeIcon( "/mIconPointLayer.png" ) );
+        layerAction->setIcon( QgsApplication::getThemeIcon( "/mIconPointLayer.svg" ) );
         break;
       case QGis::Line:
-        layerAction->setIcon( QgsApplication::getThemeIcon( "/mIconLineLayer.png" ) );
+        layerAction->setIcon( QgsApplication::getThemeIcon( "/mIconLineLayer.svg" ) );
         break;
       case QGis::Polygon:
-        layerAction->setIcon( QgsApplication::getThemeIcon( "/mIconPolygonLayer.png" ) );
+        layerAction->setIcon( QgsApplication::getThemeIcon( "/mIconPolygonLayer.svg" ) );
         break;
       default:
         break;
@@ -349,6 +349,7 @@ void QgsIdentifyMenu::addVectorLayer( QgsVectorLayer* layer, const QList<QgsMapT
     if ( mShowFeatureActions )
     {
       featureActionMenu = new QgsActionMenu( layer, result.mFeature.id(), layerMenu );
+      featureActionMenu->setExpressionContextScope( mExpressionContextScope );
     }
 
     // feature title
@@ -643,3 +644,13 @@ void QgsIdentifyMenu::removeCustomActions()
   mCustomActionRegistry.clear();
 
 }
+
+void QgsIdentifyMenu::setExpressionContextScope( const QgsExpressionContextScope &scope )
+{
+  mExpressionContextScope = scope;
+}
+
+QgsExpressionContextScope QgsIdentifyMenu::expressionContextScope() const
+{
+  return mExpressionContextScope;
+}
