@@ -35,9 +35,9 @@ class CustomActionRegistry : public QgsMapLayerActionRegistry
 };
 ///@endcond
 
-/**
+/** \ingroup gui
  * @brief The QgsIdentifyMenu class builds a menu to be used with identify results (@see QgsMapToolIdentify).
- * It is customizable and can display attribute actions (@see QgsAttributeAction) as well as map layer actions (@see QgsMapLayerAction).
+ * It is customizable and can display attribute actions (@see QgsAction) as well as map layer actions (@see QgsMapLayerAction).
  * It can also embed custom map layer actions, defined for this menu exclusively.
  * If used in a QgsMapToolIdentify, it is accessible via QgsMapToolIdentify::identifyMenu() and can be customized in the map tool sub-class.
  */
@@ -112,7 +112,7 @@ class GUI_EXPORT QgsIdentifyMenu : public QMenu
     /**
      * @brief define if attribute actions(1) and map layer actions(2) can be listed and run from the menu
      * @note custom actions will be shown in any case if they exist.
-     * @note (1) attribute actions are defined by the user in the layer properties @see QgsAttributeAction
+     * @note (1) attribute actions are defined by the user in the layer properties @see QgsAction
      * @note (2) map layer actions are built-in c++ actions or actions which are defined by a python plugin @see QgsMapLayerActionRegistry
      */
     void setShowFeatureActions( bool showFeatureActions ) { mShowFeatureActions = showFeatureActions; }
@@ -148,6 +148,20 @@ class GUI_EXPORT QgsIdentifyMenu : public QMenu
      */
     QList<QgsMapToolIdentify::IdentifyResult> exec( const QList<QgsMapToolIdentify::IdentifyResult>& idResults, QPoint pos );
 
+    /**
+     * Sets an expression context scope used to resolve underlying actions.
+     *
+     * @note Added in QGIS 2.18
+     */
+    void setExpressionContextScope( const QgsExpressionContextScope &scope );
+
+    /**
+     * Returns an expression context scope used to resolve underlying actions.
+     *
+     * @note Added in QGIS 2.18
+     */
+    QgsExpressionContextScope expressionContextScope() const;
+
   protected:
     virtual void closeEvent( QCloseEvent *e ) override;
 
@@ -178,6 +192,8 @@ class GUI_EXPORT QgsIdentifyMenu : public QMenu
     int mMaxLayerDisplay;
     int mMaxFeatureDisplay;
 
+    QgsExpressionContextScope mExpressionContextScope;
+
     // name of the action to be displayed for feature default action, if other actions are shown
     QString mDefaultActionName;
 
