@@ -22,6 +22,9 @@
 #include <QDomDocument>
 #include "ui_qgsmanageconnectionsdialogbase.h"
 
+/** \ingroup gui
+ * \class QgsManageConnectionsDialog
+ */
 class GUI_EXPORT QgsManageConnectionsDialog : public QDialog, private Ui::QgsManageConnectionsDialogBase
 {
     Q_OBJECT
@@ -39,6 +42,7 @@ class GUI_EXPORT QgsManageConnectionsDialog : public QDialog, private Ui::QgsMan
       PostGIS,
       WFS,
       MSSQL,
+      DB2,
       WCS,
       Oracle,
     };
@@ -46,7 +50,7 @@ class GUI_EXPORT QgsManageConnectionsDialog : public QDialog, private Ui::QgsMan
     // constructor
     // mode argument must be 0 for export and 1 for import
     // type argument must be 0 for WMS and 1 for PostGIS
-    QgsManageConnectionsDialog( QWidget *parent = NULL, Mode mode = Export, Type type = WMS, QString fileName = "" );
+    QgsManageConnectionsDialog( QWidget *parent = nullptr, Mode mode = Export, Type type = WMS, const QString& fileName = "" );
 
   public slots:
     void doExportImport();
@@ -62,12 +66,14 @@ class GUI_EXPORT QgsManageConnectionsDialog : public QDialog, private Ui::QgsMan
     QDomDocument savePgConnections( const QStringList & connections );
     QDomDocument saveMssqlConnections( const QStringList & connections );
     QDomDocument saveOracleConnections( const QStringList & connections );
+    QDomDocument saveDb2Connections( const QStringList & connections );
 
     void loadOWSConnections( const QDomDocument &doc, const QStringList &items, const QString &service );
     void loadWFSConnections( const QDomDocument &doc, const QStringList &items );
     void loadPgConnections( const QDomDocument &doc, const QStringList &items );
     void loadMssqlConnections( const QDomDocument &doc, const QStringList &items );
     void loadOracleConnections( const QDomDocument &doc, const QStringList &items );
+    void loadDb2Connections( const QDomDocument &doc, const QStringList &items );
 
     QString mFileName;
     Mode mDialogMode;
