@@ -24,9 +24,7 @@
 #include <QString>
 
 class QStatusBar;
-class QToolButton;
-class QShowEvent;
-class QHideEvent;
+class QCloseEvent;
 
 /** \ingroup gui
  * A generic dialog widget for displaying QGIS log messages.
@@ -41,6 +39,10 @@ class GUI_EXPORT QgsMessageLogViewer: public QDialog, private Ui::QgsMessageLogV
   public slots:
     void logMessage( QString message, QString tag, QgsMessageLog::MessageLevel level );
 
+  protected:
+    void closeEvent( QCloseEvent *e ) override;
+    void reject() override;
+
   private slots:
     void closeTab( int index );
 };
