@@ -56,14 +56,14 @@ class GUI_EXPORT QgsOptionsDialogBase : public QDialog
      * @param fl widget flags
      * @param settings custom QSettings pointer
      */
-    QgsOptionsDialogBase( QString settingsKey, QWidget* parent = 0, Qt::WindowFlags fl = 0, QSettings* settings = 0 );
+    QgsOptionsDialogBase( const QString& settingsKey, QWidget* parent = nullptr, const Qt::WindowFlags& fl = nullptr, QSettings* settings = nullptr );
     ~QgsOptionsDialogBase();
 
     /** Set up the base ui connections for vertical tabs.
      * @param restoreUi Whether to restore the base ui at this time.
      * @param title the window title
      */
-    void initOptionsBase( bool restoreUi = true, QString title = QString() );
+    void initOptionsBase( bool restoreUi = true, const QString& title = QString() );
 
     // set custom QSettings pointer if dialog used outside QGIS (in plugin)
     void setSettings( QSettings* settings );
@@ -72,9 +72,9 @@ class GUI_EXPORT QgsOptionsDialogBase : public QDialog
      * Sometimes useful to do at end of subclass's constructor.
      * @param title the window title (it does not need to be defined if previously given to initOptionsBase();
      */
-    void restoreOptionsBaseUi( QString title = QString() );
+    void restoreOptionsBaseUi( const QString& title = QString() );
 
-    /** determine if the options list is in icon only mode
+    /** Determine if the options list is in icon only mode
      */
     bool iconOnly() {return mIconOnly;}
 
