@@ -32,7 +32,8 @@
 
 QgsSearchQueryBuilder::QgsSearchQueryBuilder( QgsVectorLayer* layer,
     QWidget *parent, const Qt::WindowFlags& fl )
-    : QDialog( parent, fl ), mLayer( layer )
+    : QDialog( parent, fl )
+    , mLayer( layer )
 {
   setupUi( this );
   setupListViews();
@@ -72,7 +73,6 @@ void QgsSearchQueryBuilder::populateFields()
   if ( !mLayer )
     return;
 
-  QgsDebugMsg( "entering." );
   const QgsFields& fields = mLayer->fields();
   for ( int idx = 0; idx < fields.count(); ++idx )
   {
@@ -86,7 +86,6 @@ void QgsSearchQueryBuilder::populateFields()
 
 void QgsSearchQueryBuilder::setupListViews()
 {
-  QgsDebugMsg( "entering." );
   //Models
   mModelFields = new QStandardItemModel();
   mModelValues = new QStandardItemModel();
