@@ -118,24 +118,21 @@ void QgsSlider::valueChanged( int value )
   if ( mMin.isNull() || mMax.isNull() || mStep.isNull() )
   {
     mValue = QVariant();
-    return;
   }
-
-  if ( mMin.type() == QVariant::Int &&
-       mMax.type() == QVariant::Int &&
-       mStep.type() == QVariant::Int &&
-       mValue.type() == QVariant::Int )
+  else if ( mMin.type() == QVariant::Int &&
+            mMax.type() == QVariant::Int &&
+            mStep.type() == QVariant::Int &&
+            mValue.type() == QVariant::Int )
   {
     mValue = value;
-    return;
   }
-
-  if ( mMin.type() == QVariant::Double &&
-       mMax.type() == QVariant::Double &&
-       mStep.type() == QVariant::Double &&
-       mValue.type() == QVariant::Double )
+  else if ( mMin.type() == QVariant::Double &&
+            mMax.type() == QVariant::Double &&
+            mStep.type() == QVariant::Double &&
+            mValue.type() == QVariant::Double )
   {
     mValue = QVariant( mMin.toDouble() + value * mStep.toDouble() );
-    return;
   }
+
+  emit valueChanged( mValue );
 }
