@@ -3,7 +3,7 @@
      --------------------------------------
     Date                 : 29.5.2013
     Copyright            : (C) 2013 Matthias Kuhn
-    Email                : matthias dot kuhn at gmx dot ch
+    Email                : matthias at opengis dot ch
  ***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,7 +21,7 @@
 
 class QgsVectorLayer;
 
-/**
+/** \ingroup gui
  * Methods in this class are used to handle basic operations on vector layers.
  * With an implementation of this class, parts of the application can ask for
  * an operation to be done and the implementation will then take care of it.
@@ -33,11 +33,9 @@ class QgsVectorLayer;
 class GUI_EXPORT QgsVectorLayerTools
 {
   public:
-    QgsVectorLayerTools()
-    {}
+    QgsVectorLayerTools() {}
 
-    virtual ~QgsVectorLayerTools()
-    {}
+    virtual ~QgsVectorLayerTools() {}
 
     /**
      * This method should/will be called, whenever a new feature will be added to the layer
@@ -45,10 +43,12 @@ class GUI_EXPORT QgsVectorLayerTools
      * @param layer           The layer to which the feature should be added
      * @param defaultValues   Default values for the feature to add
      * @param defaultGeometry A default geometry to add to the feature
+     * @param feature         Updated feature after adding will be written back to this
      * @return                True in case of success, False if the operation failed/was aborted
+     *
+     * TODO QGIS 3: remove const qualifier
      */
-    virtual bool addFeature( QgsVectorLayer* layer, QgsAttributeMap defaultValues = QgsAttributeMap(), const QgsGeometry& defaultGeometry = QgsGeometry() ) const = 0;
-
+    virtual bool addFeature( QgsVectorLayer* layer, const QgsAttributeMap& defaultValues = QgsAttributeMap(), const QgsGeometry& defaultGeometry = QgsGeometry(), QgsFeature* feature = nullptr ) const = 0;
 
     /**
      * This will be called, whenever a vector layer should be switched to edit mode. Check the providers
@@ -58,6 +58,8 @@ class GUI_EXPORT QgsVectorLayerTools
      * @param layer  The layer on which to start an edit session
      *
      * @return       True, if the editing session was started
+     *
+     * TODO QGIS 3: remove const qualifier
      */
     virtual bool startEditing( QgsVectorLayer* layer ) const = 0;
 
@@ -68,6 +70,8 @@ class GUI_EXPORT QgsVectorLayerTools
      * @param layer       The layer to commit
      * @param allowCancel True if a cancel button should be offered
      * @return            True if successful
+     *
+     * TODO QGIS 3: remove const qualifier
      */
     virtual bool stopEditing( QgsVectorLayer* layer, bool allowCancel = true ) const = 0;
 
@@ -76,6 +80,8 @@ class GUI_EXPORT QgsVectorLayerTools
      *
      * @param layer       The layer to commit
      * @return            True if successful
+     *
+     * TODO QGIS 3: remove const qualifier
      */
     virtual bool saveEdits( QgsVectorLayer* layer ) const = 0;
 
