@@ -41,6 +41,7 @@ QgsPalettedRendererWidget::QgsPalettedRendererWidget( QgsRasterLayer* layer, con
     }
 
     setFromRenderer( mRasterLayer->renderer() );
+    connect( mBandComboBox, SIGNAL( currentIndexChanged( int ) ), this, SIGNAL( widgetChanged() ) );
   }
 }
 
@@ -77,6 +78,7 @@ void QgsPalettedRendererWidget::on_mTreeWidget_itemDoubleClicked( QTreeWidgetIte
     if ( c.isValid() )
     {
       item->setBackground( column, QBrush( c ) );
+      emit widgetChanged();
     }
   }
   else if ( column == 2 && item )
@@ -85,6 +87,14 @@ void QgsPalettedRendererWidget::on_mTreeWidget_itemDoubleClicked( QTreeWidgetIte
   }
 }
 
+void QgsPalettedRendererWidget::on_mTreeWidget_itemChanged( QTreeWidgetItem * item, int column )
+{
+  if ( column == 2 && item ) //palette label modified
+  {
+    emit widgetChanged();
+  }
+}
+
 void QgsPalettedRendererWidget::setFromRenderer( const QgsRasterRenderer* r )
 {
   const QgsPalettedRasterRenderer* pr = dynamic_cast<const QgsPalettedRasterRenderer*>( r );
