@@ -20,6 +20,9 @@
 
 class QgsEllipseSymbolLayerV2;
 
+/** \ingroup gui
+ * \class QgsEllipseSymbolLayerV2Widget
+ */
 class GUI_EXPORT QgsEllipseSymbolLayerV2Widget: public QgsSymbolLayerV2Widget, private Ui::WidgetEllipseBase
 {
     Q_OBJECT
@@ -56,6 +59,8 @@ class GUI_EXPORT QgsEllipseSymbolLayerV2Widget: public QgsSymbolLayerV2Widget, p
     void on_mHorizontalAnchorComboBox_currentIndexChanged( int index );
     void on_mVerticalAnchorComboBox_currentIndexChanged( int index );
 
+    void penJoinStyleChanged();
+
     void setOffset();
 };
 
