@@ -23,6 +23,9 @@
 
 class QgsPointDisplacementRenderer;
 
+/** \ingroup gui
+ * \class QgsPointDisplacementRendererWidget
+ */
 class GUI_EXPORT QgsPointDisplacementRendererWidget: public QgsRendererV2Widget, private Ui::QgsPointDisplacementRendererWidgetBase
 {
     Q_OBJECT
@@ -32,6 +35,7 @@ class GUI_EXPORT QgsPointDisplacementRendererWidget: public QgsRendererV2Widget,
     ~QgsPointDisplacementRendererWidget();
 
     QgsFeatureRendererV2* renderer() override;
+    void setMapCanvas( QgsMapCanvas* canvas ) override;
 
   private:
     QgsPointDisplacementRenderer* mRenderer;
@@ -44,10 +48,12 @@ class GUI_EXPORT QgsPointDisplacementRendererWidget: public QgsRendererV2Widget,
   private slots:
     void on_mLabelFieldComboBox_currentIndexChanged( const QString& text );
     void on_mRendererComboBox_currentIndexChanged( int index );
+    void on_mPlacementComboBox_currentIndexChanged( int index );
     void on_mLabelFontButton_clicked();
     void on_mCircleWidthSpinBox_valueChanged( double d );
     void on_mCircleColorButton_colorChanged( const QColor& newColor );
     void on_mDistanceSpinBox_valueChanged( double d );
+    void on_mDistanceUnitWidget_changed();
     void on_mLabelColorButton_colorChanged( const QColor& newColor );
     void on_mCircleModificationSpinBox_valueChanged( double d );
     void on_mScaleDependentLabelsCheckBox_stateChanged( int state );
