@@ -26,6 +26,8 @@
 #include "qgsinvertedpolygonrendererwidget.h"
 #include "qgsheatmaprendererwidget.h"
 #include "qgs25drendererwidget.h"
+#include "qgsnullsymbolrendererwidget.h"
+#include "qgspanelwidget.h"
 
 #include "qgsorderbydialog.h"
 #include "qgsapplication.h"
@@ -73,17 +75,20 @@ static void _initRendererWidgetFunctions()
   _initRenderer( "invertedPolygonRenderer", QgsInvertedPolygonRendererWidget::create, "rendererInvertedSymbol.svg" );
   _initRenderer( "heatmapRenderer", QgsHeatmapRendererWidget::create, "rendererHeatmapSymbol.svg" );
   _initRenderer( "25dRenderer", Qgs25DRendererWidget::create, "renderer25dSymbol.svg" );
+  _initRenderer( "nullSymbol", QgsNullSymbolRendererWidget::create, "rendererNullSymbol.svg" );
   initialized = true;
 }
 
-QgsRendererV2PropertiesDialog::QgsRendererV2PropertiesDialog( QgsVectorLayer* layer, QgsStyleV2* style, bool embedded )
-    : mLayer( layer )
+QgsRendererV2PropertiesDialog::QgsRendererV2PropertiesDialog( QgsVectorLayer* layer, QgsStyleV2* style, bool embedded, QWidget* parent )
+    : QDialog( parent )
+    , mLayer( layer )
     , mStyle( style )
     , mActiveWidget( nullptr )
     , mPaintEffect( nullptr )
     , mMapCanvas( nullptr )
 {
   setupUi( this );
+  mLayerRenderingGroupBox->setSettingGroup( "layerRenderingGroupBox" );
 
   // can be embedded in vector layer properties
   if ( embedded )
@@ -92,39 +97,11 @@ QgsRendererV2PropertiesDialog::QgsRendererV2PropertiesDialog( QgsVectorLayer* la
     layout()->setContentsMargins( 0, 0, 0, 0 );
   }
 
-  connect( buttonBox, SIGNAL( accepted() ), this, SLOT( onOK() ) );
-
   // initialize registry's widget functions
   _initRendererWidgetFunctions();
 
-  // Blend mode
-  mBlendModeComboBox->setBlendMode( mLayer->blendMode() );
-
-  // Feature blend mode
-  mFeatureBlendComboBox->setBlendMode( mLayer->featureBlendMode() );
-
-  // Layer transparency
-  mLayerTransparencySlider->setValue( mLayer->layerTransparency() );
-  mLayerTransparencySpnBx->setValue( mLayer->layerTransparency() );
-
-  // connect layer transparency slider and spin box
-  connect( mLayerTransparencySlider, SIGNAL( valueChanged( int ) ), mLayerTransparencySpnBx, SLOT( setValue( int ) ) );
-  connect( mLayerTransparencySpnBx, SIGNAL( valueChanged( int ) ), mLayerTransparencySlider, SLOT( setValue( int ) ) );
-
-  //paint effect widget
-  if ( mLayer->rendererV2() )
-  {
-    if ( mLayer->rendererV2()->paintEffect() )
-    {
-      mPaintEffect = mLayer->rendererV2()->paintEffect()->clone();
-      mEffectWidget->setPaintEffect( mPaintEffect );
-    }
-
-    mOrderBy = mLayer->rendererV2()->orderBy();
-  }
-
   QgsRendererV2Registry* reg = QgsRendererV2Registry::instance();
-  QStringList renderers = reg->renderersList();
+  QStringList renderers = reg->renderersList( mLayer );
   Q_FOREACH ( const QString& name, renderers )
   {
     QgsRendererV2AbstractMetadata* m = reg->rendererMetadata( name );
@@ -133,33 +110,73 @@ QgsRendererV2PropertiesDialog::QgsRendererV2PropertiesDialog( QgsVectorLayer* la
 
   cboRenderers->setCurrentIndex( -1 ); // set no current renderer
 
-  // setup slot rendererChanged()
+  connect( buttonBox, SIGNAL( accepted() ), this, SLOT( onOK() ) );
+
+  // connect layer transparency slider and spin box
+  connect( mLayerTransparencySlider, SIGNAL( valueChanged( int ) ), mLayerTransparencySpnBx, SLOT( setValue( int ) ) );
+  connect( mLayerTransparencySpnBx, SIGNAL( valueChanged( int ) ), mLayerTransparencySlider, SLOT( setValue( int ) ) );
+
   connect( cboRenderers, SIGNAL( currentIndexChanged( int ) ), this, SLOT( rendererChanged() ) );
-  //setup order by
-  if ( mLayer->rendererV2()->orderByEnabled() )
-  {
-    checkboxEnableOrderBy->setChecked( true );
-  }
-  else
-  {
-    btnOrderBy->setEnabled( false );
-    checkboxEnableOrderBy->setChecked( false );
-    lineEditOrderBy->setEnabled( false );
-  }
-  lineEditOrderBy->setReadOnly( true );
   connect( checkboxEnableOrderBy, SIGNAL( toggled( bool ) ), btnOrderBy, SLOT( setEnabled( bool ) ) );
-  connect( checkboxEnableOrderBy, SIGNAL( toggled( bool ) ), lineEditOrderBy, SLOT( setEnabled( bool ) ) );
   connect( btnOrderBy, SIGNAL( clicked( bool ) ), this, SLOT( showOrderByDialog() ) );
-  lineEditOrderBy->setText( mOrderBy.dump() );
 
-  // set current renderer from layer
-  QString rendererName = mLayer->rendererV2()->type();
+  syncToLayer();
 
-  int rendererIdx = cboRenderers->findData( rendererName );
-  cboRenderers->setCurrentIndex( rendererIdx );
+  QList<QWidget*> widgets;
+  widgets << mLayerTransparencySpnBx
+  << cboRenderers
+  << checkboxEnableOrderBy
+  << mBlendModeComboBox
+  << mFeatureBlendComboBox
+  << mEffectWidget;
 
-  // no renderer found... this mustn't happen
-  Q_ASSERT( rendererIdx != -1 && "there must be a renderer!" );
+  connectValueChanged( widgets, SIGNAL( widgetChanged() ) );
+  connect( mEffectWidget, SIGNAL( showPanel( QgsPanelWidget* ) ), this, SLOT( openPanel( QgsPanelWidget* ) ) );
+}
+
+void QgsRendererV2PropertiesDialog::connectValueChanged( QList<QWidget *> widgets, const char *slot )
+{
+  Q_FOREACH ( QWidget* widget, widgets )
+  {
+    if ( QgsDataDefinedButton* w = qobject_cast<QgsDataDefinedButton*>( widget ) )
+    {
+      connect( w, SIGNAL( dataDefinedActivated( bool ) ), this, slot );
+      connect( w, SIGNAL( dataDefinedChanged( QString ) ), this, slot );
+    }
+    else if ( QgsFieldExpressionWidget* w = qobject_cast<QgsFieldExpressionWidget*>( widget ) )
+    {
+      connect( w, SIGNAL( fieldChanged( QString ) ), this,  slot );
+    }
+    else if ( QComboBox* w =  qobject_cast<QComboBox*>( widget ) )
+    {
+      connect( w, SIGNAL( currentIndexChanged( int ) ), this, slot );
+    }
+    else if ( QSpinBox* w =  qobject_cast<QSpinBox*>( widget ) )
+    {
+      connect( w, SIGNAL( valueChanged( int ) ), this, slot );
+    }
+    else if ( QDoubleSpinBox* w =  qobject_cast<QDoubleSpinBox*>( widget ) )
+    {
+      connect( w , SIGNAL( valueChanged( double ) ), this, slot );
+    }
+    else if ( QgsColorButtonV2* w =  qobject_cast<QgsColorButtonV2*>( widget ) )
+    {
+      connect( w, SIGNAL( colorChanged( QColor ) ), this, slot );
+    }
+    else if ( QCheckBox* w =  qobject_cast<QCheckBox*>( widget ) )
+    {
+      connect( w, SIGNAL( toggled( bool ) ), this, slot );
+    }
+    else if ( QLineEdit* w =  qobject_cast<QLineEdit*>( widget ) )
+    {
+      connect( w, SIGNAL( textEdited( QString ) ), this, slot );
+      connect( w, SIGNAL( textChanged( QString ) ), this, slot );
+    }
+    else if ( QgsEffectStackCompactWidget* w = qobject_cast<QgsEffectStackCompactWidget*>( widget ) )
+    {
+      connect( w, SIGNAL( changed() ), this, slot );
+    }
+  }
 }
 
 QgsRendererV2PropertiesDialog::~QgsRendererV2PropertiesDialog()
@@ -174,10 +191,17 @@ void QgsRendererV2PropertiesDialog::setMapCanvas( QgsMapCanvas* canvas )
     mActiveWidget->setMapCanvas( mMapCanvas );
 }
 
+void QgsRendererV2PropertiesDialog::setDockMode( bool dockMode )
+{
+  mDockMode = dockMode;
+  mEffectWidget->setDockMode( dockMode );
+  if ( mActiveWidget )
+    mActiveWidget->setDockMode( mDockMode );
+}
+
 
 void QgsRendererV2PropertiesDialog::rendererChanged()
 {
-
   if ( cboRenderers->currentIndex() == -1 )
   {
     QgsDebugMsg( "No current item -- this should never happen!" );
@@ -225,6 +249,9 @@ void QgsRendererV2PropertiesDialog::rendererChanged()
       changeOrderBy( mActiveWidget->renderer()->orderBy(), mActiveWidget->renderer()->orderByEnabled() );
       connect( mActiveWidget, SIGNAL( layerVariablesChanged() ), this, SIGNAL( layerVariablesChanged() ) );
     }
+    connect( mActiveWidget, SIGNAL( widgetChanged() ), this, SIGNAL( widgetChanged() ) );
+    connect( mActiveWidget, SIGNAL( showPanel( QgsPanelWidget* ) ), this, SLOT( openPanel( QgsPanelWidget* ) ) );
+    w->setDockMode( mDockMode );
   }
   else
   {
@@ -267,6 +294,85 @@ void QgsRendererV2PropertiesDialog::onOK()
   accept();
 }
 
+void QgsRendererV2PropertiesDialog::openPanel( QgsPanelWidget *panel )
+{
+  QgsDebugMsg( "Open panel!!!" );
+  if ( mDockMode )
+  {
+    QgsDebugMsg( "DOCK MODE" );
+    emit showPanel( panel );
+  }
+  else
+  {
+    QgsDebugMsg( "DIALOG MODE" );
+    // Show the dialog version if no one is connected
+    QDialog* dlg = new QDialog();
+    QString key =  QString( "/UI/paneldialog/%1" ).arg( panel->panelTitle() );
+    QSettings settings;
+    dlg->restoreGeometry( settings.value( key ).toByteArray() );
+    dlg->setWindowTitle( panel->panelTitle() );
+    dlg->setLayout( new QVBoxLayout() );
+    dlg->layout()->addWidget( panel );
+    QDialogButtonBox* buttonBox = new QDialogButtonBox( QDialogButtonBox::Ok );
+    connect( buttonBox, SIGNAL( accepted() ), dlg, SLOT( accept() ) );
+    dlg->layout()->addWidget( buttonBox );
+    dlg->exec();
+    settings.setValue( key, dlg->saveGeometry() );
+    panel->acceptPanel();
+  }
+}
+
+void QgsRendererV2PropertiesDialog::syncToLayer()
+{
+  // Blend mode
+  mBlendModeComboBox->setBlendMode( mLayer->blendMode() );
+
+  // Feature blend mode
+  mFeatureBlendComboBox->setBlendMode( mLayer->featureBlendMode() );
+
+  // Layer transparency
+  mLayerTransparencySlider->setValue( mLayer->layerTransparency() );
+  mLayerTransparencySpnBx->setValue( mLayer->layerTransparency() );
+
+  //paint effect widget
+  if ( mLayer->rendererV2() )
+  {
+    if ( mLayer->rendererV2()->paintEffect() )
+    {
+      mPaintEffect = mLayer->rendererV2()->paintEffect()->clone();
+      mEffectWidget->setPaintEffect( mPaintEffect );
+    }
+
+    mOrderBy = mLayer->rendererV2()->orderBy();
+  }
+
+  // setup slot rendererChanged()
+  //setup order by
+  if ( mLayer->rendererV2() &&
+       mLayer->rendererV2()->orderByEnabled() )
+  {
+    checkboxEnableOrderBy->setChecked( true );
+  }
+  else
+  {
+    btnOrderBy->setEnabled( false );
+    checkboxEnableOrderBy->setChecked( false );
+  }
+
+  if ( mLayer->rendererV2() )
+  {
+    // set current renderer from layer
+    QString rendererName = mLayer->rendererV2()->type();
+
+    int rendererIdx = cboRenderers->findData( rendererName );
+    cboRenderers->setCurrentIndex( rendererIdx );
+
+    // no renderer found... this mustn't happen
+    Q_ASSERT( rendererIdx != -1 && "there must be a renderer!" );
+  }
+
+}
+
 void QgsRendererV2PropertiesDialog::showOrderByDialog()
 {
   QgsOrderByDialog dlg( mLayer, this );
@@ -275,17 +381,22 @@ void QgsRendererV2PropertiesDialog::showOrderByDialog()
   if ( dlg.exec() )
   {
     mOrderBy = dlg.orderBy();
-    lineEditOrderBy->setText( mOrderBy.dump() );
+    emit widgetChanged();
   }
 }
 
 void QgsRendererV2PropertiesDialog::changeOrderBy( const QgsFeatureRequest::OrderBy& orderBy, bool orderByEnabled )
 {
   mOrderBy = orderBy;
-  lineEditOrderBy->setText( mOrderBy.dump() );
   checkboxEnableOrderBy->setChecked( orderByEnabled );
 }
 
+void QgsRendererV2PropertiesDialog::updateUIState( bool hidden )
+{
+  mLayerRenderingGroupBox->setHidden( hidden );
+  cboRenderers->setHidden( hidden );
+}
+
 
 void QgsRendererV2PropertiesDialog::keyPressEvent( QKeyEvent * e )
 {
