@@ -191,8 +191,8 @@ class SymbolLayerItem : public QStandardItem
           }
         }
       }
-      if ( role == Qt::SizeHintRole )
-        return QVariant( QSize( 32, 32 ) );
+//      if ( role == Qt::SizeHintRole )
+//        return QVariant( QSize( 32, 32 ) );
       if ( role == Qt::CheckStateRole )
         return QVariant(); // could be true/false
       return QStandardItem::data( role );
@@ -208,8 +208,8 @@ class SymbolLayerItem : public QStandardItem
 
 //////////
 
-QgsSymbolV2SelectorDialog::QgsSymbolV2SelectorDialog( QgsSymbolV2* symbol, QgsStyleV2* style, const QgsVectorLayer* vl, QWidget* parent, bool embedded )
-    : QDialog( parent )
+QgsSymbolV2SelectorWidget::QgsSymbolV2SelectorWidget( QgsSymbolV2* symbol, QgsStyleV2* style, const QgsVectorLayer* vl, QWidget* parent )
+    : QgsPanelWidget( parent )
     , mAdvancedMenu( nullptr )
     , mVectorLayer( vl )
     , mMapCanvas( nullptr )
@@ -223,15 +223,6 @@ QgsSymbolV2SelectorDialog::QgsSymbolV2SelectorDialog( QgsSymbolV2* symbol, QgsSt
 
   setupUi( this );
 
-  QSettings settings;
-  restoreGeometry( settings.value( "/Windows/SymbolSelectorDialog/geometry" ).toByteArray() );
-
-  // can be embedded in renderer properties dialog
-  if ( embedded )
-  {
-    buttonBox->hide();
-    layout()->setContentsMargins( 0, 0, 0, 0 );
-  }
   // setup icons
   btnAddLayer->setIcon( QIcon( QgsApplication::iconPath( "symbologyAdd.svg" ) ) );
   btnRemoveLayer->setIcon( QIcon( QgsApplication::iconPath( "symbologyRemove.svg" ) ) );
@@ -262,34 +253,22 @@ QgsSymbolV2SelectorDialog::QgsSymbolV2SelectorDialog( QgsSymbolV2* symbol, QgsSt
   connect( btnRemoveLayer, SIGNAL( clicked() ), this, SLOT( removeLayer() ) );
   connect( btnLock, SIGNAL( clicked() ), this, SLOT( lockLayer() ) );
   connect( btnDuplicate, SIGNAL( clicked() ), this, SLOT( duplicateLayer() ) );
+  connect( this, SIGNAL( symbolModified() ), this, SIGNAL( widgetChanged() ) );
 
   updateUi();
 
   // set symbol as active item in the tree
   QModelIndex newIndex = layersTree->model()->index( 0, 0 );
   layersTree->setCurrentIndex( newIndex );
+
+  setPanelTitle( tr( "Symbol selector" ) );
 }
 
-QgsSymbolV2SelectorDialog::~QgsSymbolV2SelectorDialog()
+QgsSymbolV2SelectorWidget::~QgsSymbolV2SelectorWidget()
 {
-  QSettings settings;
-  settings.setValue( "/Windows/SymbolSelectorDialog/geometry", saveGeometry() );
 }
 
-void QgsSymbolV2SelectorDialog::keyPressEvent( QKeyEvent * e )
-{
-  // Ignore the ESC key to avoid close the dialog without the properties window
-  if ( !isWindow() && e->key() == Qt::Key_Escape )
-  {
-    e->ignore();
-  }
-  else
-  {
-    QDialog::keyPressEvent( e );
-  }
-}
-
-QMenu* QgsSymbolV2SelectorDialog::advancedMenu()
+QMenu* QgsSymbolV2SelectorWidget::advancedMenu()
 {
   if ( !mAdvancedMenu )
   {
@@ -300,14 +279,14 @@ QMenu* QgsSymbolV2SelectorDialog::advancedMenu()
   return mAdvancedMenu;
 }
 
-void QgsSymbolV2SelectorDialog::setExpressionContext( QgsExpressionContext *context )
+void QgsSymbolV2SelectorWidget::setExpressionContext( QgsExpressionContext *context )
 {
   mPresetExpressionContext.reset( context );
   layerChanged();
   updatePreview();
 }
 
-void QgsSymbolV2SelectorDialog::setMapCanvas( QgsMapCanvas *canvas )
+void QgsSymbolV2SelectorWidget::setMapCanvas( QgsMapCanvas *canvas )
 {
   mMapCanvas = canvas;
 
@@ -321,7 +300,13 @@ void QgsSymbolV2SelectorDialog::setMapCanvas( QgsMapCanvas *canvas )
     listWidget->setMapCanvas( canvas );
 }
 
-void QgsSymbolV2SelectorDialog::loadSymbol( QgsSymbolV2* symbol, SymbolLayerItem* parent )
+void QgsSymbolV2SelectorWidget::releaseSymbol()
+{
+  delete mSymbol;
+  mSymbol = nullptr;
+}
+
+void QgsSymbolV2SelectorWidget::loadSymbol( QgsSymbolV2* symbol, SymbolLayerItem* parent )
 {
   SymbolLayerItem* symbolItem = new SymbolLayerItem( symbol );
   QFont boldFont = symbolItem->font();
@@ -345,13 +330,13 @@ void QgsSymbolV2SelectorDialog::loadSymbol( QgsSymbolV2* symbol, SymbolLayerItem
 }
 
 
-void QgsSymbolV2SelectorDialog::loadSymbol()
+void QgsSymbolV2SelectorWidget::loadSymbol()
 {
   model->clear();
   loadSymbol( mSymbol, static_cast<SymbolLayerItem*>( model->invisibleRootItem() ) );
 }
 
-void QgsSymbolV2SelectorDialog::updateUi()
+void QgsSymbolV2SelectorWidget::updateUi()
 {
   QModelIndex currentIdx =  layersTree->currentIndex();
   if ( !currentIdx.isValid() )
@@ -378,7 +363,7 @@ void QgsSymbolV2SelectorDialog::updateUi()
   btnDuplicate->setEnabled( true );
 }
 
-void QgsSymbolV2SelectorDialog::updatePreview()
+void QgsSymbolV2SelectorWidget::updatePreview()
 {
   QImage preview = mSymbol->bigSymbolPreviewImage( mPresetExpressionContext.data() );
   lblPreview->setPixmap( QPixmap::fromImage( preview ) );
@@ -386,7 +371,7 @@ void QgsSymbolV2SelectorDialog::updatePreview()
   emit symbolModified();
 }
 
-void QgsSymbolV2SelectorDialog::updateLayerPreview()
+void QgsSymbolV2SelectorWidget::updateLayerPreview()
 {
   // get current layer item and update its icon
   SymbolLayerItem* item = currentLayerItem();
@@ -396,7 +381,7 @@ void QgsSymbolV2SelectorDialog::updateLayerPreview()
   updatePreview();
 }
 
-SymbolLayerItem* QgsSymbolV2SelectorDialog::currentLayerItem()
+SymbolLayerItem* QgsSymbolV2SelectorWidget::currentLayerItem()
 {
   QModelIndex idx = layersTree->currentIndex();
   if ( !idx.isValid() )
@@ -409,7 +394,7 @@ SymbolLayerItem* QgsSymbolV2SelectorDialog::currentLayerItem()
   return item;
 }
 
-QgsSymbolLayerV2* QgsSymbolV2SelectorDialog::currentLayer()
+QgsSymbolLayerV2* QgsSymbolV2SelectorWidget::currentLayer()
 {
   QModelIndex idx = layersTree->currentIndex();
   if ( !idx.isValid() )
@@ -422,7 +407,7 @@ QgsSymbolLayerV2* QgsSymbolV2SelectorDialog::currentLayer()
   return nullptr;
 }
 
-void QgsSymbolV2SelectorDialog::layerChanged()
+void QgsSymbolV2SelectorWidget::layerChanged()
 {
   updateUi();
 
@@ -435,6 +420,7 @@ void QgsSymbolV2SelectorDialog::layerChanged()
     SymbolLayerItem *parent = static_cast<SymbolLayerItem*>( currentItem->parent() );
     mDataDefineRestorer.reset( new DataDefinedRestorer( parent->symbol(), currentItem->layer() ) );
     QgsLayerPropertiesWidget *layerProp = new QgsLayerPropertiesWidget( currentItem->layer(), parent->symbol(), mVectorLayer );
+    layerProp->setDockMode( this->dockMode() );
     layerProp->setExpressionContext( mPresetExpressionContext.data() );
     layerProp->setMapCanvas( mMapCanvas );
     setWidget( layerProp );
@@ -442,6 +428,8 @@ void QgsSymbolV2SelectorDialog::layerChanged()
     connect( layerProp, SIGNAL( changed() ), this, SLOT( updateLayerPreview() ) );
     // This connection when layer type is changed
     connect( layerProp, SIGNAL( changeLayer( QgsSymbolLayerV2* ) ), this, SLOT( changeLayer( QgsSymbolLayerV2* ) ) );
+
+    connectChildPanel( layerProp );
   }
   else
   {
@@ -459,7 +447,7 @@ void QgsSymbolV2SelectorDialog::layerChanged()
   updateLockButton();
 }
 
-void QgsSymbolV2SelectorDialog::symbolChanged()
+void QgsSymbolV2SelectorWidget::symbolChanged()
 {
   SymbolLayerItem *currentItem = static_cast<SymbolLayerItem*>( model->itemFromIndex( layersTree->currentIndex() ) );
   if ( !currentItem || currentItem->isLayer() )
@@ -488,7 +476,7 @@ void QgsSymbolV2SelectorDialog::symbolChanged()
   connect( layersTree->selectionModel(), SIGNAL( currentChanged( const QModelIndex&, const QModelIndex& ) ), this, SLOT( layerChanged() ) );
 }
 
-void QgsSymbolV2SelectorDialog::setWidget( QWidget* widget )
+void QgsSymbolV2SelectorWidget::setWidget( QWidget* widget )
 {
   int index = stackedWidget->addWidget( widget );
   stackedWidget->setCurrentIndex( index );
@@ -497,7 +485,7 @@ void QgsSymbolV2SelectorDialog::setWidget( QWidget* widget )
   mPresentWidget = widget;
 }
 
-void QgsSymbolV2SelectorDialog::updateLockButton()
+void QgsSymbolV2SelectorWidget::updateLockButton()
 {
   QgsSymbolLayerV2* layer = currentLayer();
   if ( !layer )
@@ -505,7 +493,7 @@ void QgsSymbolV2SelectorDialog::updateLockButton()
   btnLock->setChecked( layer->isLocked() );
 }
 
-void QgsSymbolV2SelectorDialog::addLayer()
+void QgsSymbolV2SelectorWidget::addLayer()
 {
   QModelIndex idx = layersTree->currentIndex();
   if ( !idx.isValid() )
@@ -555,7 +543,7 @@ void QgsSymbolV2SelectorDialog::addLayer()
   updatePreview();
 }
 
-void QgsSymbolV2SelectorDialog::removeLayer()
+void QgsSymbolV2SelectorWidget::removeLayer()
 {
   SymbolLayerItem *item = currentLayerItem();
   int row = item->row();
@@ -577,17 +565,17 @@ void QgsSymbolV2SelectorDialog::removeLayer()
   delete tmpLayer;
 }
 
-void QgsSymbolV2SelectorDialog::moveLayerDown()
+void QgsSymbolV2SelectorWidget::moveLayerDown()
 {
   moveLayerByOffset( + 1 );
 }
 
-void QgsSymbolV2SelectorDialog::moveLayerUp()
+void QgsSymbolV2SelectorWidget::moveLayerUp()
 {
   moveLayerByOffset( -1 );
 }
 
-void QgsSymbolV2SelectorDialog::moveLayerByOffset( int offset )
+void QgsSymbolV2SelectorWidget::moveLayerByOffset( int offset )
 {
   SymbolLayerItem *item = currentLayerItem();
   if ( !item )
@@ -613,15 +601,16 @@ void QgsSymbolV2SelectorDialog::moveLayerByOffset( int offset )
   updateUi();
 }
 
-void QgsSymbolV2SelectorDialog::lockLayer()
+void QgsSymbolV2SelectorWidget::lockLayer()
 {
   QgsSymbolLayerV2* layer = currentLayer();
   if ( !layer )
     return;
   layer->setLocked( btnLock->isChecked() );
+  emit symbolModified();
 }
 
-void QgsSymbolV2SelectorDialog::duplicateLayer()
+void QgsSymbolV2SelectorWidget::duplicateLayer()
 {
   QModelIndex idx = layersTree->currentIndex();
   if ( !idx.isValid() )
@@ -658,7 +647,7 @@ void QgsSymbolV2SelectorDialog::duplicateLayer()
   updatePreview();
 }
 
-void QgsSymbolV2SelectorDialog::saveSymbol()
+void QgsSymbolV2SelectorWidget::saveSymbol()
 {
   bool ok;
   QString name = QInputDialog::getText( this, tr( "Symbol name" ),
@@ -686,7 +675,7 @@ void QgsSymbolV2SelectorDialog::saveSymbol()
   mStyle->saveSymbol( name, mSymbol->clone(), 0, QStringList() );
 }
 
-void QgsSymbolV2SelectorDialog::changeLayer( QgsSymbolLayerV2* newLayer )
+void QgsSymbolV2SelectorWidget::changeLayer( QgsSymbolLayerV2* newLayer )
 {
   SymbolLayerItem* item = currentLayerItem();
   QgsSymbolLayerV2* layer = item->layer();
@@ -714,3 +703,174 @@ void QgsSymbolV2SelectorDialog::changeLayer( QgsSymbolLayerV2* newLayer )
   // Important: This lets the layer have its own layer properties widget
   layerChanged();
 }
+
+QgsSymbolV2SelectorDialog::QgsSymbolV2SelectorDialog( QgsSymbolV2 *symbol, QgsStyleV2 *style, const QgsVectorLayer *vl, QWidget *parent, bool embedded )
+    : QDialog( parent )
+{
+  setLayout( new QVBoxLayout() );
+  mSelectorWidget = new QgsSymbolV2SelectorWidget( symbol, style, vl, this );
+  mButtonBox = new QDialogButtonBox( QDialogButtonBox::Ok | QDialogButtonBox::Cancel );
+
+  connect( mButtonBox, SIGNAL( accepted() ), this, SLOT( accept() ) );
+  connect( mButtonBox, SIGNAL( rejected() ), this, SLOT( reject() ) );
+
+  layout()->addWidget( mSelectorWidget );
+  layout()->addWidget( mButtonBox );
+
+  QSettings settings;
+  restoreGeometry( settings.value( "/Windows/SymbolSelectorWidget/geometry" ).toByteArray() );
+
+  // can be embedded in renderer properties dialog
+  if ( embedded )
+  {
+    mButtonBox->hide();
+    layout()->setContentsMargins( 0, 0, 0, 0 );
+  }
+  mSelectorWidget->setDockMode( embedded );
+}
+
+QgsSymbolV2SelectorDialog::~QgsSymbolV2SelectorDialog()
+{
+  QSettings settings;
+  settings.setValue( "/Windows/SymbolSelectorWidget/geometry", saveGeometry() );
+}
+
+QMenu *QgsSymbolV2SelectorDialog::advancedMenu()
+{
+  return mSelectorWidget->advancedMenu();
+}
+
+void QgsSymbolV2SelectorDialog::setExpressionContext( QgsExpressionContext *context )
+{
+  mSelectorWidget->setExpressionContext( context );
+}
+
+QgsExpressionContext *QgsSymbolV2SelectorDialog::expressionContext() const
+{
+  return mSelectorWidget->expressionContext();
+}
+
+void QgsSymbolV2SelectorDialog::setMapCanvas( QgsMapCanvas *canvas )
+{
+  mSelectorWidget->setMapCanvas( canvas );
+}
+
+QgsSymbolV2 *QgsSymbolV2SelectorDialog::symbol()
+{
+  return mSelectorWidget->symbol();
+}
+
+void QgsSymbolV2SelectorDialog::keyPressEvent( QKeyEvent *e )
+{
+  // Ignore the ESC key to avoid close the dialog without the properties window
+  if ( !isWindow() && e->key() == Qt::Key_Escape )
+  {
+    e->ignore();
+  }
+  else
+  {
+    QDialog::keyPressEvent( e );
+  }
+}
+
+void QgsSymbolV2SelectorDialog::loadSymbol()
+{
+  mSelectorWidget->loadSymbol();
+}
+
+void QgsSymbolV2SelectorDialog::loadSymbol( QgsSymbolV2 *symbol, SymbolLayerItem *parent )
+{
+  mSelectorWidget->loadSymbol( symbol, parent );
+}
+
+void QgsSymbolV2SelectorDialog::updateUi()
+{
+  mSelectorWidget->updateUi();
+}
+
+void QgsSymbolV2SelectorDialog::updateLockButton()
+{
+  mSelectorWidget->updateLockButton();
+}
+
+SymbolLayerItem *QgsSymbolV2SelectorDialog::currentLayerItem()
+{
+  return mSelectorWidget->currentLayerItem();
+}
+
+QgsSymbolLayerV2 *QgsSymbolV2SelectorDialog::currentLayer()
+{
+  return mSelectorWidget->currentLayer();
+}
+
+void QgsSymbolV2SelectorDialog::moveLayerByOffset( int offset )
+{
+  mSelectorWidget->moveLayerByOffset( offset );
+}
+
+void QgsSymbolV2SelectorDialog::setWidget( QWidget *widget )
+{
+  mSelectorWidget->setWidget( widget );
+}
+
+void QgsSymbolV2SelectorDialog::moveLayerDown()
+{
+  mSelectorWidget->moveLayerDown();
+}
+
+void QgsSymbolV2SelectorDialog::moveLayerUp()
+{
+  mSelectorWidget->moveLayerUp();
+}
+
+void QgsSymbolV2SelectorDialog::addLayer()
+{
+  mSelectorWidget->addLayer();
+}
+
+void QgsSymbolV2SelectorDialog::removeLayer()
+{
+  mSelectorWidget->removeLayer();
+}
+
+void QgsSymbolV2SelectorDialog::lockLayer()
+{
+  mSelectorWidget->lockLayer();
+}
+
+void QgsSymbolV2SelectorDialog::saveSymbol()
+{
+  Q_NOWARN_DEPRECATED_PUSH
+  mSelectorWidget->saveSymbol();
+  Q_NOWARN_DEPRECATED_POP
+}
+
+void QgsSymbolV2SelectorDialog::duplicateLayer()
+{
+  mSelectorWidget->duplicateLayer();
+}
+
+void QgsSymbolV2SelectorDialog::layerChanged()
+{
+  mSelectorWidget->layerChanged();
+}
+
+void QgsSymbolV2SelectorDialog::updateLayerPreview()
+{
+  mSelectorWidget->updateLayerPreview();
+}
+
+void QgsSymbolV2SelectorDialog::updatePreview()
+{
+  mSelectorWidget->updatePreview();
+}
+
+void QgsSymbolV2SelectorDialog::symbolChanged()
+{
+  mSelectorWidget->symbolChanged();
+}
+
+void QgsSymbolV2SelectorDialog::changeLayer( QgsSymbolLayerV2 *layer )
+{
+  mSelectorWidget->changeLayer( layer );
+}
