@@ -32,18 +32,18 @@ class CoordinateCaptureMapTool : public QgsMapTool
     Q_OBJECT
 
   public:
-    CoordinateCaptureMapTool( QgsMapCanvas* thepCanvas );
+    explicit CoordinateCaptureMapTool( QgsMapCanvas* thepCanvas );
 
     ~CoordinateCaptureMapTool();
 
     //! Overridden mouse move event
-    virtual void canvasMoveEvent( QMouseEvent * e ) override;
+    virtual void canvasMoveEvent( QgsMapMouseEvent* e ) override;
 
     //! Overridden mouse press event
-    virtual void canvasPressEvent( QMouseEvent * e ) override;
+    virtual void canvasPressEvent( QgsMapMouseEvent* e ) override;
 
     //! Overridden mouse release event
-    virtual void canvasReleaseEvent( QMouseEvent * e ) override;
+    virtual void canvasReleaseEvent( QgsMapMouseEvent* e ) override;
 
     //! called when map tool is being deactivated
     virtual void deactivate() override;
@@ -51,8 +51,8 @@ class CoordinateCaptureMapTool : public QgsMapTool
   public slots:
 
   signals:
-    void mouseMoved( QgsPoint );
-    void mouseClicked( QgsPoint );
+    void mouseMoved( const QgsPoint& );
+    void mouseClicked( const QgsPoint& );
   private:
 
     //! Rubber band for highlighting identified feature
