@@ -76,34 +76,34 @@ class eVisQueryDefinition
     bool autoConnect() { return mAutoConnect; }
 
     /** \brief Mutator for query description */
-    void setDescription( QString description ) { mDescription = description; }
+    void setDescription( const QString& description ) { mDescription = description; }
 
     /** \brief Mutator for query short description */
-    void setShortDescription( QString description ) { mShortDescription = description; }
+    void setShortDescription( const QString& description ) { mShortDescription = description; }
 
     /** \brief Mutator for database type */
-    void setDatabaseType( QString type ) { mDatabaseType = type; }
+    void setDatabaseType( const QString& type ) { mDatabaseType = type; }
 
     /** \brief Mutator for database host name */
-    void setDatabaseHost( QString host ) { mDatabaseHost = host; }
+    void setDatabaseHost( const QString& host ) { mDatabaseHost = host; }
 
     /** \brief Mutator for database port */
     void setDatabasePort( int port ) { mDatabasePort = port; }
 
     /** \brief Mutator for database name */
-    void setDatabaseName( QString name ) { mDatabaseName = name; }
+    void setDatabaseName( const QString& name ) { mDatabaseName = name; }
 
     /** \brief Mutator for database username */
-    void setDatabaseUsername( QString username ) { mDatabaseUsername = username; }
+    void setDatabaseUsername( const QString& username ) { mDatabaseUsername = username; }
 
     /** \brief Mutator for database password */
-    void setDatabasePassword( QString password ) { mDatabasePassword = password; }
+    void setDatabasePassword( const QString& password ) { mDatabasePassword = password; }
 
     /** \brief Mutator for SQL statement */
-    void setSqlStatement( QString statement ) { mSqlStatement = statement; }
+    void setSqlStatement( const QString& statement ) { mSqlStatement = statement; }
 
     /** \brief Mutator for auto connection flag */
-    void setAutoConnect( QString autoconnect )
+    void setAutoConnect( const QString& autoconnect )
     {
       if ( autoconnect.startsWith( "true", Qt::CaseInsensitive ) )
       {
