@@ -22,7 +22,8 @@ class QgsGeorefConfigDialog : public QDialog, private Ui::QgsGeorefConfigDialogB
 {
     Q_OBJECT
   public:
-    QgsGeorefConfigDialog( QWidget *parent = 0 );
+    explicit QgsGeorefConfigDialog( QWidget *parent = nullptr );
+    ~QgsGeorefConfigDialog();
 
   protected:
     void changeEvent( QEvent *e ) override;
