@@ -22,7 +22,7 @@ class QgsNonEditableDelegate : public QStyledItemDelegate
     Q_OBJECT
 
   public:
-    QgsNonEditableDelegate( QWidget *parent = 0 );
+    explicit QgsNonEditableDelegate( QWidget *parent = nullptr );
 
     QWidget *createEditor( QWidget *parent, const QStyleOptionViewItem &option,
                            const QModelIndex &index ) const override
@@ -30,7 +30,7 @@ class QgsNonEditableDelegate : public QStyledItemDelegate
       Q_UNUSED( parent );
       Q_UNUSED( option );
       Q_UNUSED( index );
-      return 0;
+      return nullptr;
     }
 };
 
@@ -39,7 +39,7 @@ class QgsDmsAndDdDelegate : public QStyledItemDelegate
     Q_OBJECT
 
   public:
-    QgsDmsAndDdDelegate( QWidget *parent = 0 );
+    explicit QgsDmsAndDdDelegate( QWidget *parent = nullptr );
 
     QWidget *createEditor( QWidget *parent, const QStyleOptionViewItem &option,
                            const QModelIndex &index ) const override;
@@ -52,7 +52,7 @@ class QgsDmsAndDdDelegate : public QStyledItemDelegate
                                const QModelIndex &index ) const override;
 
   private:
-    QString dmsToDD( QString dms ) const;
+    double dmsToDD( const QString& dms ) const;
 };
 
 class QgsCoordDelegate : public QStyledItemDelegate
@@ -60,17 +60,17 @@ class QgsCoordDelegate : public QStyledItemDelegate
     Q_OBJECT
 
   public:
-    QgsCoordDelegate( QWidget *parent = 0 );
+    explicit QgsCoordDelegate( QWidget *parent = nullptr );
 
     QWidget *createEditor( QWidget *parent, const QStyleOptionViewItem &option,
                            const QModelIndex &index ) const override;
 
-//  void setEditorData(QWidget *editor, const QModelIndex &index);
-//  void setModelData(QWidget *editor, QAbstractItemModel *model,
-//                    const QModelIndex &index);
-//
-//  void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option,
-//                            const QModelIndex &index);
+    void setEditorData( QWidget *editor, const QModelIndex &index ) const override;
+    void setModelData( QWidget *editor, QAbstractItemModel *model,
+                       const QModelIndex &index ) const override;
+
+    void updateEditorGeometry( QWidget *editor, const QStyleOptionViewItem &option,
+                               const QModelIndex &index ) const override;
 };
 
 #endif // QGSDELEGATES_H
