@@ -21,6 +21,7 @@
 #include <gdalwarper.h>
 #include <vector>
 #include "qgspoint.h"
+#include "qgscoordinatereferencesystem.h"
 
 class QgsGeorefTransform;
 class QProgressDialog;
@@ -28,10 +29,10 @@ class QWidget;
 
 class QgsImageWarper
 {
-    Q_DECLARE_TR_FUNCTIONS( QgsImageWarper );
+    Q_DECLARE_TR_FUNCTIONS( QgsImageWarper )
 
   public:
-    QgsImageWarper( QWidget *theParent );
+    explicit QgsImageWarper( QWidget *theParent );
 
     enum ResamplingMethod
     {
@@ -56,7 +57,7 @@ class QgsImageWarper
                   ResamplingMethod resampling,
                   bool useZeroAsTrans,
                   const QString& compression,
-                  const QString& projection,
+                  const QgsCoordinateReferenceSystem& crs,
                   double destResX = 0.0, double destResY = 0.0 );
   private:
     struct TransformChain
@@ -82,12 +83,12 @@ class QgsImageWarper
     void *addGeoToPixelTransform( GDALTransformerFunc GDALTransformer, void *GDALTransformerArg, double *padfGeotransform ) const;
     void destroyGeoToPixelTransform( void *GeoToPixelTransfomArg ) const;
 
-    bool openSrcDSAndGetWarpOpt( const QString &input, const ResamplingMethod &resampling,
-                                 const GDALTransformerFunc &pfnTransform, GDALDatasetH &hSrcDS,
+    bool openSrcDSAndGetWarpOpt( const QString &input, ResamplingMethod resampling,
+                                 const GDALTransformerFunc& pfnTransform, GDALDatasetH &hSrcDS,
                                  GDALWarpOptions *&psWarpOptions );
 
     bool createDestinationDataset( const QString &outputName, GDALDatasetH hSrcDS, GDALDatasetH &hDstDS, uint resX, uint resY,
-                                   double *adfGeoTransform, bool useZeroAsTrans, const QString& compression, const QString &projection );
+                                   double *adfGeoTransform, bool useZeroAsTrans, const QString& compression, const QgsCoordinateReferenceSystem& crs );
 
     QWidget *mParent;
     void      *createWarpProgressArg( QProgressDialog *progressDialog ) const;
