@@ -16,7 +16,6 @@
 #include <QPushButton>
 
 #include "qgsmapcanvas.h"
-#include "qgssnappingutils.h"
 
 #include "qgsgeorefvalidators.h"
 #include "qgsmapcoordsdialog.h"
@@ -46,8 +45,6 @@ QgsMapCoordsDialog::QgsMapCoordsDialog( QgsMapCanvas* qgisCanvas, const QgsPoint
   mToolEmitPoint = new QgsGeorefMapToolEmitPoint( qgisCanvas );
   mToolEmitPoint->setButton( mPointFromCanvasPushButton );
 
-  mSnapToBackgroundLayerBox->setChecked( s.value( "/Plugin-GeoReferencer/snapToBackgroundLayers", QVariant( false ) ).toBool() );
-
   connect( mPointFromCanvasPushButton, SIGNAL( clicked( bool ) ), this, SLOT( setToolEmitPoint( bool ) ) );
 
   connect( mToolEmitPoint, SIGNAL( canvasClicked( const QgsPoint&, Qt::MouseButton ) ),
@@ -91,8 +88,6 @@ void QgsMapCoordsDialog::on_buttonBox_accepted()
     y = dmsToDD( leYCoord->text() );
 
   emit pointAdded( mPixelCoords, QgsPoint( x, y ) );
-  QSettings s;
-  s.setValue( "/Plugin-GeoReferencer/snapToBackgroundLayers", mSnapToBackgroundLayerBox->isChecked() );
   close();
 }
 
@@ -102,12 +97,6 @@ void QgsMapCoordsDialog::maybeSetXY( const QgsPoint & xy, Qt::MouseButton button
   if ( Qt::LeftButton == button )
   {
     QgsPoint mapCoordPoint = xy;
-    if ( mQgisCanvas && mSnapToBackgroundLayerBox->isChecked() )
-    {
-      QgsPointLocator::Match m = mQgisCanvas->snappingUtils()->snapToMap( xy );
-      if ( m.isValid() )
-        mapCoordPoint = m.point();
-    }
 
     leXCoord->clear();
     leYCoord->clear();
