@@ -98,7 +98,6 @@ QgsGrassPlugin::QgsGrassPlugin( QgisInterface * theQgisInterFace )
 
 QgsGrassPlugin::~QgsGrassPlugin()
 {
-  QgsDebugMsg( "entered." );
   // When main app is closed, QgsGrassTools (probably because of dock widget) are destroyed before QgsGrassPlugin
   // -> do not call mTools here
   //if ( mTools )
@@ -204,11 +203,11 @@ void QgsGrassPlugin::initGui()
   mToolBarPointer->addAction( mRegionAction );
 
   // Editing
-  mAddPointAction = new QAction( QgsApplication::getThemeIcon( "/mActionCapturePoint.png" ), tr( "Add Point" ), this );
+  mAddPointAction = new QAction( QgsApplication::getThemeIcon( "/mActionCapturePoint.svg" ), tr( "Add Point" ), this );
   mAddPointAction->setObjectName( "mAddPointAction" );
   mAddPointAction->setCheckable( true );
 
-  mAddLineAction = new QAction( QgsApplication::getThemeIcon( "/mActionCaptureLine.png" ), tr( "Add Line" ), this );
+  mAddLineAction = new QAction( QgsApplication::getThemeIcon( "/mActionCaptureLine.svg" ), tr( "Add Line" ), this );
   mAddLineAction->setObjectName( "mAddLineAction" );
   mAddLineAction->setCheckable( true );
 
@@ -220,7 +219,7 @@ void QgsGrassPlugin::initGui()
   mAddCentroidAction->setObjectName( "mAddCentroidAction" );
   mAddCentroidAction->setCheckable( true );
 
-  mAddAreaAction = new QAction( QgsApplication::getThemeIcon( "/mActionCapturePolygon.png" ), tr( "Add Closed Boundary" ), this );
+  mAddAreaAction = new QAction( QgsApplication::getThemeIcon( "/mActionCapturePolygon.svg" ), tr( "Add Closed Boundary" ), this );
   mAddAreaAction->setObjectName( "mAddAreaAction" );
   mAddAreaAction->setCheckable( true );
 
@@ -294,7 +293,7 @@ void QgsGrassPlugin::initGui()
   if ( !QgsRendererV2Registry::instance()->renderersList().contains( "grassEdit" ) )
   {
     QgsRendererV2Registry::instance()->addRenderer( new QgsRendererV2Metadata( "grassEdit",
-        QObject::tr( "GRASS Edit" ),
+        QObject::tr( "GRASS edit" ),
         QgsGrassEditRenderer::create,
         QIcon( QgsApplication::defaultThemePath() + "rendererGrassSymbol.svg" ),
         QgsGrassEditRendererWidget::create ) );
@@ -306,7 +305,6 @@ void QgsGrassPlugin::initGui()
 
 void QgsGrassPlugin::onGisbaseChanged()
 {
-  QgsDebugMsg( "entered" );
   if ( !QgsGrass::init() )
   {
     // TODO: save init error and get it here more reliably
@@ -352,13 +350,11 @@ void QgsGrassPlugin::onLayerWasAdded( QgsMapLayer* theMapLayer )
 void QgsGrassPlugin::onCurrentLayerChanged( QgsMapLayer* layer )
 {
   Q_UNUSED( layer );
-  QgsDebugMsg( "Entered" );
   resetEditActions();
 }
 
 void QgsGrassPlugin::resetEditActions()
 {
-  QgsDebugMsg( "Entered" );
 
   QgsGrassProvider* grassProvider = 0;
   QgsVectorLayer *vectorLayer = qobject_cast<QgsVectorLayer *>( qGisInterface->activeLayer() );
@@ -390,7 +386,6 @@ void QgsGrassPlugin::resetEditActions()
 
 void QgsGrassPlugin::onEditingStarted()
 {
-  QgsDebugMsg( "Entered" );
   QgsVectorLayer *vectorLayer = qobject_cast<QgsVectorLayer *>( sender() );
   if ( !vectorLayer )
     return;
@@ -438,7 +433,6 @@ void QgsGrassPlugin::onEditingStarted()
 
 void QgsGrassPlugin::onEditingStopped()
 {
-  QgsDebugMsg( "entered" );
   QgsVectorLayer *vectorLayer = qobject_cast<QgsVectorLayer *>( sender() );
   if ( vectorLayer )
   {
@@ -454,7 +448,6 @@ void QgsGrassPlugin::onEditingStopped()
 
 void QgsGrassPlugin::onFieldsChanged()
 {
-  QgsDebugMsg( "entered" );
   QgsGrassProvider* grassProvider = dynamic_cast<QgsGrassProvider*>( sender() );
   if ( !grassProvider )
   {
@@ -483,7 +476,6 @@ void QgsGrassPlugin::onFieldsChanged()
 
 void QgsGrassPlugin::addFeature()
 {
-  QgsDebugMsg( "entered" );
   QgsGrassProvider* grassProvider = 0;
   QgsVectorLayer *vectorLayer = qobject_cast<QgsVectorLayer *>( qGisInterface->activeLayer() );
   if ( vectorLayer )
@@ -528,7 +520,6 @@ void QgsGrassPlugin::addFeature()
 
 void QgsGrassPlugin::onSplitFeaturesTriggered( bool checked )
 {
-  QgsDebugMsg( "entered" );
   if ( checked )
   {
     QgsGrassProvider* grassProvider = 0;
@@ -548,7 +539,6 @@ void QgsGrassPlugin::onSplitFeaturesTriggered( bool checked )
 
 void QgsGrassPlugin::mapsetChanged()
 {
-  QgsDebugMsg( "entered" );
   if ( !QgsGrass::activeMode() )
   {
     mRegionAction->setEnabled( false );
@@ -669,12 +659,10 @@ void QgsGrassPlugin::postRender( QPainter *painter )
   Q_UNUSED( painter );
   // We have to redraw rectangle, because canvas->mapRenderer()->destinationCrs is set after GRASS plugin constructor! This way it is redrawn also if canvas CRS has changed.
   displayRegion();
-// QgsDebugMsg("entered.");
 }
 
 void QgsGrassPlugin::displayRegion()
 {
-  QgsDebugMsg( "entered" );
 
   mRegionBand->reset();
   if ( !mRegionAction->isChecked() )
@@ -710,7 +698,6 @@ void QgsGrassPlugin::displayRegion()
 
 void QgsGrassPlugin::switchRegion( bool on )
 {
-// QgsDebugMsg("entered.");
 
   QSettings settings;
   settings.setValue( "/GRASS/region/on", on );
@@ -727,14 +714,12 @@ void QgsGrassPlugin::switchRegion( bool on )
 
 void QgsGrassPlugin::redrawRegion()
 {
-// QgsDebugMsg("entered.");
 
   displayRegion();
 }
 
 void QgsGrassPlugin::openMapset()
 {
-// QgsDebugMsg("entered.");
 
   QString element;
 
@@ -773,7 +758,6 @@ void QgsGrassPlugin::newMapset()
 
 void QgsGrassPlugin::projectRead()
 {
-  QgsDebugMsg( "entered." );
 
   bool ok;
   QString gisdbase = QgsProject::instance()->readPath(
@@ -819,7 +803,6 @@ void QgsGrassPlugin::projectRead()
 
 void QgsGrassPlugin::newProject()
 {
-  QgsDebugMsg( "entered" );
 }
 
 // Unload the plugin by cleaning up the GUI
