@@ -131,7 +131,8 @@ try:
 except ImportError:
     import grass
 except:
-    raise Exception ("Cannot find 'grass' Python module. Python is supported by GRASS from version >= 6.4" )
+    raise Exception("Cannot find 'grass' Python module. Python is supported by GRASS from version >= 6.4")
+
 
 def main():
     input = options['input']
@@ -147,13 +148,17 @@ def main():
 
     # Construct dsn string
     dsn = "PG:dbname=" + database
-    if host: dsn += " host=" + host
-    if port: dsn += " port=" + port
-    if user: dsn += " user=" + user
-    if password: dsn += " password=" + password
+    if host:
+        dsn += " host=" + host
+    if port:
+        dsn += " port=" + port
+    if user:
+        dsn += " user=" + user
+    if password:
+        dsn += " password=" + password
 
     if grass.run_command('v.out.ogr', flags=flags_string, input=input, layer=layer, type=type, format="PostgreSQL", dsn=dsn, olayer=olayer) != 0:
-         grass.fatal("Cannot export vector to database.")
+        grass.fatal("Cannot export vector to database.")
 
 if __name__ == "__main__":
     options, flags = grass.parser()
