@@ -31,6 +31,7 @@
 #include <qgsgraphbuilder.h>
 #include <qgsgraph.h>
 #include <qgsdistancearcproperter.h>
+#include "qgsdockwidget.h"
 
 // Road grap plugin includes
 #include "roadgraphplugin.h"
@@ -50,14 +51,13 @@
 #include <QLocale>
 #include <QToolBar>
 #include <QPushButton>
-#include <QDockWidget>
 #include <QVBoxLayout>
 #include <QDebug>
 
 // standard includes
 
 static const QString sName = QObject::tr( "Road graph plugin" );
-static const QString sDescription = QObject::tr( "It solves the shortest path problem." );
+static const QString sDescription = QObject::tr( "Solves the shortest path problem by tracing along line layers." );
 static const QString sCategory = QObject::tr( "Vector" );
 static const QString sPluginVersion = QObject::tr( "Version 0.1" );
 static const QString sPluginIcon = ":/roadgraph/road-fast.png";
@@ -102,7 +102,7 @@ void RoadGraphPlugin::initGui()
   mQGisIface->addDockWidget( Qt::LeftDockWidgetArea, mQShortestPathDock );
 
   // Create the action for tool
-  mQSettingsAction  = new QAction( QIcon( ":/roadgraph/road.png" ), tr( "Settings" ), this );
+  mQSettingsAction  = new QAction( QIcon( ":/roadgraph/road.png" ), tr( "Settings..." ), this );
   mQSettingsAction->setObjectName( "mQSettingsAction" );
 
   // Set the what's this text
@@ -113,7 +113,7 @@ void RoadGraphPlugin::initGui()
   // Connect the action to slots
   connect( mQSettingsAction, SIGNAL( triggered() ), this, SLOT( property() ) );
 
-  mQGisIface->addPluginToVectorMenu( tr( "Road graph" ), mQSettingsAction );
+  mQGisIface->addPluginToVectorMenu( tr( "Road Graph" ), mQSettingsAction );
 
   connect( mQGisIface, SIGNAL( projectRead() ), this, SLOT( projectRead() ) );
   connect( mQGisIface, SIGNAL( newProjectCreated() ), this, SLOT( newProject() ) );
@@ -122,13 +122,13 @@ void RoadGraphPlugin::initGui()
 
   // load settings
   projectRead();
-} // RoadGraphPlugin::initGui()
+}
 
 // Unload the plugin by cleaning up the GUI
 void RoadGraphPlugin::unload()
 {
   // remove the GUI
-  mQGisIface->removePluginVectorMenu( tr( "Road graph" ), mQSettingsAction );
+  mQGisIface->removePluginVectorMenu( tr( "Road Graph" ), mQSettingsAction );
 
   // disconnect
   disconnect( mQGisIface->mainWindow(), SIGNAL( projectRead() ), this, SLOT( projectRead() ) );
@@ -136,24 +136,23 @@ void RoadGraphPlugin::unload()
 
   delete mQSettingsAction;
   delete mQShortestPathDock;
-} // RoadGraphPlugin::unload()
+}
 
 void RoadGraphPlugin::setGuiElementsToDefault()
 {
 
-} // RoadGraphPlugin::setGuiElementsToDefault()
+}
 
 //method defined in interface
 void RoadGraphPlugin::help()
 {
   //implement me!
-} // RoadGraphPlugin::help()
+}
 
 void RoadGraphPlugin::onShowDirection()
 {
   mQGisIface->mapCanvas()->refresh();
-} // RoadGraphPlugin::onShowDirection()
-
+}
 
 void RoadGraphPlugin::newProject()
 {
@@ -180,7 +179,7 @@ void RoadGraphPlugin::property()
   QgsProject::instance()->writeEntry( "roadgraphplugin", "/pluginDistanceUnit", mDistanceUnitName );
   QgsProject::instance()->writeEntry( "roadgraphplugin", "/topologyToleranceFactor", mTopologyToleranceFactor );
   setGuiElementsToDefault();
-} //RoadGraphPlugin::property()
+}
 
 void RoadGraphPlugin::projectRead()
 {
@@ -190,7 +189,7 @@ void RoadGraphPlugin::projectRead()
   mTopologyToleranceFactor =
     QgsProject::instance()->readDoubleEntry( "roadgraphplugin", "/topologyToleranceFactor", 0.0 );
   setGuiElementsToDefault();
-}// RoadGraphplguin::projectRead()
+}
 
 QgisInterface* RoadGraphPlugin::iface()
 {
@@ -199,40 +198,29 @@ QgisInterface* RoadGraphPlugin::iface()
 
 const QgsGraphDirector* RoadGraphPlugin::director() const
 {
-  QString layerId;
-  QgsVectorLayer *layer = nullptr;
-  QMap< QString, QgsMapLayer* > mapLayers = QgsMapLayerRegistry::instance()->mapLayers();
-  QMap< QString, QgsMapLayer* >::const_iterator it;
-  for ( it = mapLayers.begin(); it != mapLayers.end(); ++it )
-  {
-    if ( it.value()->name() != mSettings->mLayer )
-      continue;
-    layer = dynamic_cast< QgsVectorLayer* >( it.value() );
-    break;
-  }
+  QList< QgsMapLayer* > mapLayers = QgsMapLayerRegistry::instance()->mapLayersByName( mSettings->mLayerName );
+  if ( mapLayers.isEmpty() )
+    return nullptr;
 
+  QgsVectorLayer *layer = dynamic_cast< QgsVectorLayer* >( mapLayers.at( 0 ) );
   if ( !layer )
     return nullptr;
 
   if ( layer->wkbType() == QGis::WKBLineString
        || layer->wkbType() == QGis::WKBMultiLineString )
   {
-    QgsVectorDataProvider *provider = layer->dataProvider();
-    if ( !provider )
-      return nullptr;
-
     SpeedUnit speedUnit = SpeedUnit::byName( mSettings->mSpeedUnitName );
 
     QgsLineVectorLayerDirector * director =
       new QgsLineVectorLayerDirector( layer,
-                                      provider->fieldNameIndex( mSettings->mDirection ),
+                                      layer->fields().fieldNameIndex( mSettings->mDirection ),
                                       mSettings->mFirstPointToLastPointDirectionVal,
                                       mSettings->mLastPointToFirstPointDirectionVal,
                                       mSettings->mBothDirectionVal,
                                       mSettings->mDefaultDirection
                                     );
     director->addProperter( new QgsDistanceArcProperter() );
-    director->addProperter( new RgSpeedProperter( provider->fieldNameIndex( mSettings->mSpeed ),
+    director->addProperter( new RgSpeedProperter( layer->fields().fieldNameIndex( mSettings->mSpeed ),
                             mSettings->mDefaultSpeed, speedUnit.multipler() ) );
     return director;
   }
