@@ -21,6 +21,7 @@
 #include "qgsvectordataprovider.h"
 #include "qgsfeature.h"
 #include "qgsgeometrycoordinatetransform.h"
+#include "qgsgeometryengine.h"
 #include "qgsspatialquery.h"
 
 QgsSpatialQuery::QgsSpatialQuery( MngProgressBar *pb )
@@ -207,32 +208,32 @@ void QgsSpatialQuery::setSpatialIndexReference( QgsFeatureIds &qsetIndexInvalidR
 
 void QgsSpatialQuery::execQuery( QgsFeatureIds &qsetIndexResult, QgsFeatureIds &qsetIndexInvalidTarget, int relation )
 {
-  bool ( QgsGeometry::* operation )( const QgsGeometry * ) const;
+  bool ( QgsGeometryEngine::* operation )( const QgsAbstractGeometryV2&, QString* ) const;
   switch ( relation )
   {
     case Disjoint:
-      operation = &QgsGeometry::disjoint;
+      operation = &QgsGeometryEngine::disjoint;
       break;
     case Equals:
-      operation = &QgsGeometry::equals;
+      operation = &QgsGeometryEngine::isEqual;
       break;
     case Touches:
-      operation = &QgsGeometry::touches;
+      operation = &QgsGeometryEngine::touches;
       break;
     case Overlaps:
-      operation = &QgsGeometry::overlaps;
+      operation = &QgsGeometryEngine::overlaps;
       break;
     case Within:
-      operation = &QgsGeometry::within;
+      operation = &QgsGeometryEngine::within;
       break;
     case Contains:
-      operation = &QgsGeometry::contains;
+      operation = &QgsGeometryEngine::contains;
       break;
     case Crosses:
-      operation = &QgsGeometry::crosses;
+      operation = &QgsGeometryEngine::crosses;
       break;
     case Intersects:
-      operation = &QgsGeometry::intersects;
+      operation = &QgsGeometryEngine::intersects;
       break;
     default:
       qWarning( "undefined operation" );
@@ -244,7 +245,7 @@ void QgsSpatialQuery::execQuery( QgsFeatureIds &qsetIndexResult, QgsFeatureIds &
   coordinateTransform->setCoordinateTransform( mLayerTarget, mLayerReference );
 
   // Set function for populate result
-  void ( QgsSpatialQuery::* funcPopulateIndexResult )( QgsFeatureIds&, QgsFeatureId, QgsGeometry *, bool ( QgsGeometry::* )( const QgsGeometry * ) const );
+  void ( QgsSpatialQuery::* funcPopulateIndexResult )( QgsFeatureIds&, QgsFeatureId, QgsGeometry *, bool ( QgsGeometryEngine::* )( const QgsAbstractGeometryV2&, QString* ) const );
   funcPopulateIndexResult = ( relation == Disjoint )
                             ? &QgsSpatialQuery::populateIndexResultDisjoint
                             : &QgsSpatialQuery::populateIndexResult;
@@ -273,51 +274,60 @@ void QgsSpatialQuery::execQuery( QgsFeatureIds &qsetIndexResult, QgsFeatureIds &
 
 void QgsSpatialQuery::populateIndexResult(
   QgsFeatureIds &qsetIndexResult, QgsFeatureId idTarget, QgsGeometry * geomTarget,
-  bool ( QgsGeometry::* op )( const QgsGeometry * ) const )
+  bool ( QgsGeometryEngine::* op )( const QgsAbstractGeometryV2&, QString* ) const )
 {
-  QList<QgsFeatureId> listIdReference;
-  listIdReference = mIndexReference.intersects( geomTarget->boundingBox() );
+  QgsFeatureIds listIdReference = mIndexReference.intersects( geomTarget->boundingBox() ).toSet();
   if ( listIdReference.isEmpty() )
   {
     return;
   }
+
+  //prepare geometry
+  QgsGeometryEngine* geomEngine = geomTarget->createGeometryEngine( geomTarget->geometry() );
+  geomEngine->prepareGeometry();
+
   QgsFeature featureReference;
   const QgsGeometry * geomReference;
-  QList<QgsFeatureId>::iterator iterIdReference = listIdReference.begin();
-  for ( ; iterIdReference != listIdReference.end(); ++iterIdReference )
+  QgsFeatureIterator listIt = mLayerReference->getFeatures( QgsFeatureRequest().setFilterFids( listIdReference ) );
+
+  while ( listIt.nextFeature( featureReference ) )
   {
-    mLayerReference->getFeatures( QgsFeatureRequest().setFilterFid( *iterIdReference ) ).nextFeature( featureReference );
     geomReference = featureReference.constGeometry();
-    if (( geomTarget->*op )( geomReference ) )
+
+    if (( geomEngine->*op )( *( geomReference->geometry() ), 0 ) )
     {
       qsetIndexResult.insert( idTarget );
       break;
     }
   }
 
+  delete geomEngine;
 } // void QgsSpatialQuery::populateIndexResult(...
 
 void QgsSpatialQuery::populateIndexResultDisjoint(
   QgsFeatureIds &qsetIndexResult, QgsFeatureId idTarget, QgsGeometry * geomTarget,
-  bool ( QgsGeometry::* op )( const QgsGeometry * ) const )
+  bool ( QgsGeometryEngine::* op )( const QgsAbstractGeometryV2&, QString* ) const )
 {
-  QList<QgsFeatureId> listIdReference;
-  listIdReference = mIndexReference.intersects( geomTarget->boundingBox() );
+  QgsFeatureIds listIdReference = mIndexReference.intersects( geomTarget->boundingBox() ).toSet();
   if ( listIdReference.isEmpty() )
   {
     qsetIndexResult.insert( idTarget );
     return;
   }
+
+  //prepare geometry
+  QgsGeometryEngine* geomEngine = geomTarget->createGeometryEngine( geomTarget->geometry() );
+  geomEngine->prepareGeometry();
+
   QgsFeature featureReference;
   const QgsGeometry * geomReference;
-  QList<QgsFeatureId>::iterator iterIdReference = listIdReference.begin();
-  bool addIndex = true;
-  for ( ; iterIdReference != listIdReference.end(); ++iterIdReference )
-  {
-    mLayerReference->getFeatures( QgsFeatureRequest().setFilterFid( *iterIdReference ) ).nextFeature( featureReference );
-    geomReference = featureReference.constGeometry();
+  QgsFeatureIterator listIt = mLayerReference->getFeatures( QgsFeatureRequest().setFilterFids( listIdReference ) );
 
-    if ( !( geomTarget->*op )( geomReference ) )
+  bool addIndex = true;
+  while ( listIt.nextFeature( featureReference ) )
+  {
+    geomReference = featureReference.constGeometry();
+    if (( geomEngine->*op )( *( geomReference->geometry() ), 0 ) )
     {
       addIndex = false;
       break;
@@ -327,6 +337,6 @@ void QgsSpatialQuery::populateIndexResultDisjoint(
   {
     qsetIndexResult.insert( idTarget );
   }
-
+  delete geomEngine;
 } // void QgsSpatialQuery::populateIndexResultDisjoint( ...
 
