@@ -24,6 +24,7 @@
 #include "qgsmngprogressbar.h"
 #include "qgsreaderfeatures.h"
 
+class QgsGeometryEngine;
 
 /**
 * \brief Enum with the topologic relations
@@ -137,7 +138,7 @@ class QgsSpatialQuery
      */
     void populateIndexResult(
       QgsFeatureIds &qsetIndexResult, QgsFeatureId idTarget, QgsGeometry *geomTarget,
-      bool ( QgsGeometry::* operation )( const QgsGeometry * ) const );
+      bool ( QgsGeometryEngine::* operation )( const QgsAbstractGeometryV2&, QString* ) const );
     /**
      * \brief Populate index Result Disjoint
      * \param qsetIndexResult    Reference to QSet contains the result query
@@ -145,9 +146,8 @@ class QgsSpatialQuery
      * \param geomTarget         Geometry the feature Target
      * \param operation          Pointer to function of GEOS operation
      */
-    void populateIndexResultDisjoint(
-      QgsFeatureIds &qsetIndexResult, QgsFeatureId idTarget, QgsGeometry *geomTarget,
-      bool ( QgsGeometry::* operation )( const QgsGeometry * ) const );
+    void populateIndexResultDisjoint( QgsFeatureIds &qsetIndexResult, QgsFeatureId idTarget, QgsGeometry *geomTarget,
+                                      bool ( QgsGeometryEngine::*operation )( const QgsAbstractGeometryV2&, QString* ) const );
 
     MngProgressBar *mPb;
     bool mUseReferenceSelection;
