@@ -26,6 +26,9 @@
 #include <QSettings>
 #include <QRegExp>
 #include <QUrl>
+#if QT_VERSION >= 0x050000
+#include <QUrlQuery>
+#endif
 
 #include "qgsapplication.h"
 #include "qgsdataprovider.h"
@@ -152,7 +155,12 @@ QgsDelimitedTextProvider::QgsDelimitedTextProvider( const QString& uri )
 
   if ( url.hasQueryItem( "subset" ) )
   {
+#if QT_VERSION < 0x050000
     subset = url.queryItemValue( "subset" );
+#else
+    // We need to specify FullyDecoded so that %25 is decoded as %
+    subset = QUrlQuery( url ).queryItemValue( "subset" , QUrl::FullyDecoded );
+#endif
     QgsDebugMsg( "subset is: " + subset );
   }
 
@@ -466,7 +474,7 @@ void QgsDelimitedTextProvider::scanFile( bool buildIndexes )
                 mNumberFeatures++;
                 if ( geom->isMultipart() ) mWkbType = type;
                 QgsRectangle bbox( geom->boundingBox() );
-                mExtent.combineExtentWith( &bbox );
+                mExtent.combineExtentWith( bbox );
               }
               if ( buildSpatialIndex )
               {
@@ -789,7 +797,7 @@ void QgsDelimitedTextProvider::rescanFile()
       else
       {
         QgsRectangle bbox( f.constGeometry()->boundingBox() );
-        mExtent.combineExtentWith( &bbox );
+        mExtent.combineExtentWith( bbox );
       }
       if ( buildSpatialIndex ) mSpatialIndex->insertFeature( f );
     }
