@@ -17,8 +17,11 @@
 
 #include "qgsdataitem.h"
 
+Q_NOWARN_DEPRECATED_PUSH
 class QgsGdalLayerItem : public QgsLayerItem
 {
+    Q_OBJECT
+
   private:
 
     QStringList sublayers;
@@ -26,15 +29,17 @@ class QgsGdalLayerItem : public QgsLayerItem
   public:
     QgsGdalLayerItem( QgsDataItem* parent,
                       QString name, QString path, QString uri,
-                      QStringList *theSublayers = NULL );
+                      QStringList *theSublayers = nullptr );
     ~QgsGdalLayerItem();
 
     bool setCrs( QgsCoordinateReferenceSystem crs ) override;
-    Capability capabilities() override;
+
+    Q_DECL_DEPRECATED Capability capabilities() override;
 
     QVector<QgsDataItem*> createChildren() override;
 
     QString layerName() const override;
 };
+Q_NOWARN_DEPRECATED_POP
 
 #endif // QGSGDALDATAITEMS_H
