@@ -97,9 +97,11 @@ class QgsGPXProvider : public QgsVectorDataProvider
     virtual int capabilities() const override;
 
     /**
-     * Returns the default value for field specified by @c fieldId
+     * Returns the default value for field specified by \a fieldId.
+     * If \a forceLazyEval is set to true, the provider the default value
+     * will not be evaluated on server side even if specified in the project properties.
      */
-    virtual QVariant defaultValue( int fieldId ) override;
+    virtual QVariant defaultValue( int fieldId, bool forceLazyEval = false ) override;
 
 
     /* Functions inherited from QgsDataProvider */
