@@ -217,7 +217,7 @@ int main( int argc, char **argv )
       G_suppress_masking(); // must be after G_set_window()
       fd = G_open_cell_old( rast_opt->answer, "" );
       // wait for coords from stdin
-      while ( fgets( buff, 100, stdin ) != 0 )
+      while ( fgets( buff, sizeof buff - 1, stdin ) != 0 )
       {
         if ( sscanf( buff, "%lf%lf", &x, &y ) != 2 )
         {
