@@ -121,7 +121,6 @@ QList<QAction*> QgsGrassItemActions::actions()
 
 void QgsGrassItemActions::newMapset()
 {
-  QgsDebugMsg( "entered" );
 
   QStringList existingNames = QgsGrass::mapsets( mGrassObject.gisdbase(), mGrassObject.mapsetPath() );
   QgsDebugMsg( "existingNames = " + existingNames.join( "," ) );
@@ -145,7 +144,6 @@ void QgsGrassItemActions::newMapset()
 
 void QgsGrassItemActions::openMapset()
 {
-  QgsDebugMsg( "entered" );
   QString error = QgsGrass::openMapset( mGrassObject.gisdbase(), mGrassObject.location(), mGrassObject.mapset() );
   if ( !error.isEmpty() )
   {
@@ -157,7 +155,6 @@ void QgsGrassItemActions::openMapset()
 
 void QgsGrassItemActions::addMapsetToSearchPath()
 {
-  QgsDebugMsg( "entered" );
   QString error;
   QgsGrass::instance()->addMapsetToSearchPath( mGrassObject.mapset(), error );
   if ( !error.isEmpty() )
@@ -169,7 +166,6 @@ void QgsGrassItemActions::addMapsetToSearchPath()
 
 void QgsGrassItemActions::removeMapsetFromSearchPath()
 {
-  QgsDebugMsg( "entered" );
   QString error;
   QgsGrass::instance()->removeMapsetFromSearchPath( mGrassObject.mapset(), error );
   if ( !error.isEmpty() )
@@ -181,7 +177,6 @@ void QgsGrassItemActions::removeMapsetFromSearchPath()
 
 void QgsGrassItemActions::renameGrassObject()
 {
-  QgsDebugMsg( "Entered" );
 
   QStringList existingNames = QgsGrass::grassObjects( mGrassObject, mGrassObject.type() );
   // remove current name to avoid warning that exists
@@ -225,7 +220,6 @@ void QgsGrassItemActions::renameGrassObject()
 
 void QgsGrassItemActions::deleteGrassObject()
 {
-  QgsDebugMsg( "Entered" );
 
   if ( !QgsGrass::deleteObjectDialog( mGrassObject ) )
     return;
@@ -239,7 +233,6 @@ void QgsGrassItemActions::deleteGrassObject()
 
 QString QgsGrassItemActions::newVectorMap()
 {
-  QgsDebugMsg( "entered" );
 
   QStringList existingNames = QgsGrass::grassObjects( mGrassObject, QgsGrassObject::Vector );
   QgsDebugMsg( "existingNames = " + existingNames.join( "," ) );
@@ -272,7 +265,6 @@ QString QgsGrassItemActions::newVectorMap()
 
 void QgsGrassItemActions::newLayer( QString type )
 {
-  QgsDebugMsg( "entered" );
   QString name;
   if ( mGrassObject.type() == QgsGrassObject::Mapset )
   {
@@ -307,19 +299,16 @@ void QgsGrassItemActions::newLayer( QString type )
 
 void QgsGrassItemActions::newPointLayer()
 {
-  QgsDebugMsg( "entered" );
   newLayer( "point" );
 }
 
 void QgsGrassItemActions::newLineLayer()
 {
-  QgsDebugMsg( "entered" );
   newLayer( "line" );
 }
 
 void QgsGrassItemActions::newPolygonLayer()
 {
-  QgsDebugMsg( "entered" );
   newLayer( "polygon" );
 }
 
@@ -419,7 +408,6 @@ QIcon QgsGrassMapsetItem::icon()
 
 void QgsGrassMapsetItem::setState( State state )
 {
-  QgsDebugMsg( "Entered" );
 
   // TODO: it seems to be causing strange icon switching during import, sometimes
   if ( state == Populated )
@@ -467,7 +455,6 @@ bool QgsGrassMapsetItem::objectInImports( QgsGrassObject grassObject )
 
 QVector<QgsDataItem*> QgsGrassMapsetItem::createChildren()
 {
-  QgsDebugMsg( "Entered" );
 
   QVector<QgsDataItem*> items;
 
@@ -920,7 +907,6 @@ bool QgsGrassMapsetItem::handleDrop( const QMimeData * data, Qt::DropAction )
 
 void QgsGrassMapsetItem::onImportFinished( QgsGrassImport* import )
 {
-  QgsDebugMsg( "entered" );
   if ( !import->error().isEmpty() )
   {
     QgsMessageOutput *output = QgsMessageOutput::createMessageOutput();
@@ -938,7 +924,6 @@ void QgsGrassMapsetItem::onImportFinished( QgsGrassImport* import )
 
 void QgsGrassMapsetItem::onDirectoryChanged()
 {
-  QgsDebugMsg( "entered" );
   if ( state() == Populating )
   {
     // schedule to refresh later, because refres() simply returns if Populating
@@ -988,8 +973,8 @@ bool QgsGrassObjectItem::equal( const QgsDataItem *other )
 
 //----------------------- QgsGrassVectorItem ------------------------------
 
-QgsGrassVectorItem::QgsGrassVectorItem( QgsDataItem* parent, QgsGrassObject grassObject, QString path, QString labelName, bool valid ) :
-    QgsDataCollectionItem( parent, labelName.isEmpty() ? grassObject.name() : labelName, path )
+QgsGrassVectorItem::QgsGrassVectorItem( QgsDataItem* parent, QgsGrassObject grassObject, QString path, QString labelName, bool valid )
+    : QgsDataCollectionItem( parent, labelName.isEmpty() ? grassObject.name() : labelName, path )
     , QgsGrassObjectItemBase( grassObject )
     , mValid( valid )
     , mActions( 0 )
@@ -1020,7 +1005,6 @@ QgsGrassVectorItem::~QgsGrassVectorItem()
 
 void QgsGrassVectorItem::onDirectoryChanged()
 {
-  QgsDebugMsg( "entered" );
   if ( parent() )
   {
     parent()->refresh();
@@ -1199,7 +1183,6 @@ QList<QAction*> QgsGrassImportItem::actions()
 
 QWidget * QgsGrassImportItem::paramWidget()
 {
-  QgsDebugMsg( "entered" );
   QgsGrassImportItemWidget *widget = new QgsGrassImportItemWidget();
 
   if ( mImport && mImport->progress() )
@@ -1214,7 +1197,6 @@ QWidget * QgsGrassImportItem::paramWidget()
 
 void QgsGrassImportItem::cancel()
 {
-  QgsDebugMsg( "Entered" );
   if ( !mImport ) // should not happen
   {
     QgsDebugMsg( "mImport is null" );
