@@ -225,7 +225,6 @@ QImage* QgsGrassRasterProvider::draw( QgsRectangle  const & viewExtent, int pixe
 void QgsGrassRasterProvider::readBlock( int bandNo, int xBlock, int yBlock, void *block )
 {
   Q_UNUSED( xBlock );
-  QgsDebugMsg( "Entered" );
   clearLastError();
   // TODO: optimize, see extent()
 
@@ -279,9 +278,9 @@ void QgsGrassRasterProvider::readBlock( int bandNo, int xBlock, int yBlock, void
   memcpy( block, data.data(), size );
 }
 
-void QgsGrassRasterProvider::readBlock( int bandNo, QgsRectangle  const & viewExtent, int pixelWidth, int pixelHeight, void *block )
+void QgsGrassRasterProvider::readBlock( int bandNo, QgsRectangle  const & viewExtent, int pixelWidth, int pixelHeight, void *block, QgsRasterBlockFeedback* feedback )
 {
-  QgsDebugMsg( "Entered" );
+  Q_UNUSED( feedback );
   QgsDebugMsg( "pixelWidth = "  + QString::number( pixelWidth ) );
   QgsDebugMsg( "pixelHeight = "  + QString::number( pixelHeight ) );
   QgsDebugMsg( "viewExtent: " + viewExtent.toString() );
@@ -388,7 +387,6 @@ QgsRasterBandStats QgsGrassRasterProvider::bandStatistics( int theBandNo, int th
 QList<QgsColorRampShader::ColorRampItem> QgsGrassRasterProvider::colorTable( int bandNo )const
 {
   Q_UNUSED( bandNo );
-  QgsDebugMsg( "Entered" );
   QList<QgsColorRampShader::ColorRampItem> ct;
 
   // TODO: check if color can be realy discontinuous in GRASS,
@@ -431,13 +429,11 @@ QList<QgsColorRampShader::ColorRampItem> QgsGrassRasterProvider::colorTable( int
 
 QgsCoordinateReferenceSystem QgsGrassRasterProvider::crs()
 {
-  QgsDebugMsg( "Entered" );
   return mCrs;
 }
 
 QgsRectangle QgsGrassRasterProvider::extent()
 {
-  QgsDebugMsg( "Entered" );
   // The extend can change of course so we get always fresh, to avoid running always the module
   // we should save mExtent and mLastModified and check if the map was modified
 
@@ -460,12 +456,11 @@ int QgsGrassRasterProvider::yBlockSize() const
 int QgsGrassRasterProvider::xSize() const { return mCols; }
 int QgsGrassRasterProvider::ySize() const { return mRows; }
 
-QgsRasterIdentifyResult QgsGrassRasterProvider::identify( const QgsPoint & thePoint, QgsRaster::IdentifyFormat theFormat, const QgsRectangle &theExtent, int theWidth, int theHeight )
+QgsRasterIdentifyResult QgsGrassRasterProvider::identify( const QgsPoint & thePoint, QgsRaster::IdentifyFormat theFormat, const QgsRectangle &theExtent, int theWidth, int theHeight, int /*theDpi*/ )
 {
   Q_UNUSED( theExtent );
   Q_UNUSED( theWidth );
   Q_UNUSED( theHeight );
-  QgsDebugMsg( "Entered" );
   QMap<int, QVariant> results;
   QMap<int, QVariant> noDataResults;
   noDataResults.insert( 1, QVariant() );
@@ -643,14 +638,12 @@ QDateTime QgsGrassRasterProvider::dataTimestamp() const
 
 void QgsGrassRasterProvider::freeze()
 {
-  QgsDebugMsg( "entered" );
   mRasterValue.stop();
   mValid = false;
 }
 
 void QgsGrassRasterProvider::thaw()
 {
-  QgsDebugMsg( "entered" );
   mValid = true;
 }
 
@@ -676,7 +669,6 @@ void QgsGrassRasterValue::set( const QString & gisdbase, const QString & locatio
 
 void QgsGrassRasterValue::start()
 {
-  QgsDebugMsg( "entered" );
   if ( mProcess )
   {
     QgsDebugMsg( "already running" );
@@ -701,7 +693,6 @@ void QgsGrassRasterValue::start()
 
 void QgsGrassRasterValue::stop()
 {
-  QgsDebugMsg( "entered" );
   if ( mProcess )
   {
     QgsDebugMsg( "closing process" );
