@@ -18,16 +18,17 @@
 #include <QSettings>
 #include <QInputDialog>
 #include <QMessageBox>
-
-#include <QtSql/QSqlDatabase>
-#include <QtSql/QSqlError>
+#include <QSqlDatabase>
+#include <QSqlError>
+#include <QRegExpValidator>
 
 #include "qgsmssqlnewconnection.h"
 #include "qgsmssqlprovider.h"
 #include "qgscontexthelp.h"
 
 QgsMssqlNewConnection::QgsMssqlNewConnection( QWidget *parent, const QString& connName, Qt::WindowFlags fl )
-    : QDialog( parent, fl ), mOriginalConnName( connName )
+    : QDialog( parent, fl )
+    , mOriginalConnName( connName )
 {
   setupUi( this );
 
@@ -63,6 +64,7 @@ QgsMssqlNewConnection::QgsMssqlNewConnection( QWidget *parent, const QString& co
 
     txtName->setText( connName );
   }
+  txtName->setValidator( new QRegExpValidator( QRegExp( "[^\\/]+" ), txtName ) );
   on_cb_trustedConnection_clicked();
 }
 /** Autoconnected SLOTS **/
