@@ -186,6 +186,11 @@ class QgsMssqlProvider : public QgsVectorDataProvider
      */
     bool isValid() override;
 
+    /**
+     * It returns true. Saving style to db is supported by this provider
+     */
+    virtual bool isSaveAndLoadStyleToDBSupported() override { return true; }
+
     /** Writes a list of features to the database*/
     virtual bool addFeatures( QgsFeatureList & flist ) override;
 
@@ -223,8 +228,15 @@ class QgsMssqlProvider : public QgsVectorDataProvider
     /** Convert a QgsField to work with MSSQL */
     static bool convertField( QgsField &field );
 
-    /** Returns the default value for field specified by @c fieldId */
-    QVariant defaultValue( int fieldId ) override;
+    /** Convert values to quoted values for database work **/
+    static QString quotedValue( const QVariant& value );
+
+    /**
+     * Returns the default value for field specified by \a fieldId.
+     * If \a forceLazyEval is set to true, the provider the default value
+     * will not be evaluated on server side even if specified in the project properties.
+     */
+    virtual QVariant defaultValue( int fieldId, bool forceLazyEval = false ) override;
 
     /** Import a vector layer into the database */
     static QgsVectorLayerImport::ImportError createEmptyLayer(
@@ -278,7 +290,7 @@ class QgsMssqlProvider : public QgsVectorDataProvider
     QGis::WkbType mWkbType;
 
     // The database object
-    QSqlDatabase mDatabase;
+    mutable QSqlDatabase mDatabase;
 
     // The current sql query
     QSqlQuery mQuery;
