@@ -19,6 +19,7 @@
 #include "qgsdataitem.h"
 #include "qgsogrprovider.h"
 
+Q_NOWARN_DEPRECATED_PUSH
 class QgsOgrLayerItem : public QgsLayerItem
 {
     Q_OBJECT
@@ -27,9 +28,11 @@ class QgsOgrLayerItem : public QgsLayerItem
     ~QgsOgrLayerItem();
 
     bool setCrs( QgsCoordinateReferenceSystem crs ) override;
-    Capability capabilities() override;
+
+    Q_DECL_DEPRECATED Capability capabilities() override;
     QString layerName() const override;
 };
+Q_NOWARN_DEPRECATED_POP
 
 class QgsOgrDataCollectionItem : public QgsDataCollectionItem
 {
