@@ -24,6 +24,7 @@
 #include <QMap>
 #include <QSet>
 #include <QThread>
+#include <QDateTime>
 
 #include "qgis.h"
 #include "qgsdatasourceuri.h"
@@ -164,6 +165,7 @@ class QgsOracleConn : public QObject
     static bool onlyExistingTypes( QString theConnName );
     static void deleteConnection( QString theConnName );
     static QString databaseName( QString database, QString host, QString port );
+    static QString toPoolName( const QgsDataSourceURI& uri );
 
     operator QSqlDatabase() { return mDatabase; }
 
@@ -189,6 +191,7 @@ class QgsOracleConn : public QObject
 
     static QMap<QString, QgsOracleConn *> sConnections;
     static int snConnections;
+    static QMap<QString, QDateTime> sBrokenConnections;
 };
 
 #endif
