@@ -18,6 +18,7 @@
 #include <QSettings>
 #include <QMessageBox>
 #include <QInputDialog>
+#include <QRegExpValidator>
 
 #include "qgsoraclenewconnection.h"
 #include "qgscontexthelp.h"
@@ -46,6 +47,7 @@ QgsOracleNewConnection::QgsOracleNewConnection( QWidget *parent, const QString&
     }
     txtPort->setText( port );
     txtOptions->setText( settings.value( key + "/dboptions" ).toString() );
+    txtWorkspace->setText( settings.value( key + "/dbworkspace" ).toString() );
     cb_userTablesOnly->setChecked( settings.value( key + "/userTablesOnly", false ).toBool() );
     cb_geometryColumnsOnly->setChecked( settings.value( key + "/geometryColumnsOnly", true ).toBool() );
     cb_allowGeometrylessTables->setChecked( settings.value( key + "/allowGeometrylessTables", false ).toBool() );
@@ -79,6 +81,7 @@ QgsOracleNewConnection::QgsOracleNewConnection( QWidget *parent, const QString&
 
     txtName->setText( connName );
   }
+  txtName->setValidator( new QRegExpValidator( QRegExp( "[^\\/]+" ), txtName ) );
 }
 /** Autoconnected SLOTS **/
 void QgsOracleNewConnection::accept()
@@ -130,6 +133,7 @@ void QgsOracleNewConnection::accept()
   settings.setValue( baseKey + "/saveUsername", chkStoreUsername->isChecked() ? "true" : "false" );
   settings.setValue( baseKey + "/savePassword", chkStorePassword->isChecked() ? "true" : "false" );
   settings.setValue( baseKey + "/dboptions", txtOptions->text() );
+  settings.setValue( baseKey + "/dbworkspace", txtWorkspace->text() );
 
   QDialog::accept();
 }
@@ -140,8 +144,10 @@ void QgsOracleNewConnection::on_btnConnect_clicked()
   uri.setConnection( txtHost->text(), txtPort->text(), txtDatabase->text(), txtUsername->text(), txtPassword->text() );
   if ( !txtOptions->text().isEmpty() )
     uri.setParam( "dboptions", txtOptions->text() );
+  if ( !txtWorkspace->text().isEmpty() )
+    uri.setParam( "dbworkspace", txtWorkspace->text() );
 
-  QgsOracleConn *conn = QgsOracleConnPool::instance()->acquireConnection( uri.connectionInfo() );
+  QgsOracleConn *conn = QgsOracleConnPool::instance()->acquireConnection( QgsOracleConn::toPoolName( uri ) );
 
   if ( conn )
   {
